unit acpuid;
{$MINENUMSIZE 1}
{.$DEFINE NEW_ADVAPI32} //activate this if you have WIN98+, WINNT4 SP3+
                        //or you have manually copied ADVAPI32.DLL Ver 4.80+
{
 What the heck is it?
 --------------------
 With the magic of Delphi,
 Everything is quite nice and easy to arranged. Writing
 code is much simpler and easy to understood than before.

 You might also check my other code: Absolute Read/Write
 Disk in WIN32 Platform (Win95/NT/2000).

 comments:

 CPUID instruction
 -----------------
 Starting from i486 intel built a new insruction
 (which could be executed at any privilege level)
 to extract CPU's information. This instruction
 (CPUID) will queries information about CPU, such as
 type, family, model and stepping. In more recent CPU,
 this information also includes features supported by
 CPU and cache descriptor.

 This quite useful regarding to identify specific
 capabilties you expect to be existed in CPU, so your
 application will run properly.

 This instruction may also used to queries processor
 about it's Serial Number (a relative unique number
 burnt in Pentium III class processor), but it usages
 then depreciated in IA-64 (Itanium doesn't support
 this feature anymore, some privacy concern -maybe).
 On the contrary of that, I think this features might be useful
 as a complement to Network Card ID's, I even think
 to build -or someone more capable will- some (experiment)
 network protocol based on this processor's unique ID.

 ACPUID Component
 ----------------
 note: I've built and tested this with Delphi 3,
       no hints and warning was produced.
       In Delphi 4 and up (which more restrictive)
       some exceptions may occur.

 Class: Tacpuid
   public and published members:
   properties:
   -----------
     CPUInfo: TCPUInfo
     TCPUInfo = Class(TPersistent)
       published members: properties:

         cpuClass:TcpuClass, Read-write
           TCPUClass = (x000, x086, x286, x386, x486, x586, x586p)
             Class of CPU which indicates the presence of CPUID
             instruction (supported on x486 and up).

         cpuType:TcpuType, Read-only
           TcpuType = (cpu_primary, cpu_overdrive, cpu_secondary, cpu_reserved)
             Type Of CPU based on information retrieved after
             CPUID instruction succesfully executed.

         cpuFamily:String, Read-only
         cpuModel:String, Read-only
         cpuStepping:Integer, Read-only
           Describe CPU's Family/Model/Stepping.

     CPUSerialNo: TCPUSerialNo
     TCPUSerialNo = Class(TPersistent)
       published members: properties:

         property PentiumIII:Boolean, Read-only
           Indicating support for Processor Serial Number Feature.

         PSN_ax:Integer, Read-only
         PSN_Hi:Integer, Read-only
         PSN_Lo:Integer, Read-only
         PSN_String:String, Read-only
           CPU's Processor Serial Number (if PSN is enabled).

     Registers:Tregisters
       TRegisters = Class(TPersistent)
       published members: properties:
       
           eax, ebx, ecx, edx :Integer; Read-write
           CPU's register contents after CPUID execution,
           Corresponding to x486 CPU's registers: EAX, EBX, ECX, EDX.
           It can also set to some values for the next CPUID execution.
           See also property BulkData.

     Auto:Boolean, Read-write
       Default is FALSE, if it set to TRUE then all properties
       will be populated with appropriate information.
       note: Auto is temporarily activated (regardless of current
       state) on RefereshInfo method execution, and then turned
       back to the last state after that.

     Vendor:TVendor, Read-write
       TVendor = (vn_none, vnIntel, vnAMD, vnCyrix, vnIDT,
         vnNexGen, vnUMC, vnRise);
       The CPU's vendor, currently contains only 7 major vendor.

     VendorSign:String, Read-only
       Vaguely hardcoded string in CPU.

     Level:Integer, Read-write
       Last level called by application.
       You can set this property to any value for the next CPUID execution

     MaxLevel:Integer, Read-only
       Maximum CPUID level call supported by CPU.

     Featured:Integer
       Double word representation of CPU's features.

     Features:TFeatureSet
       TFeatureSet = set of TFeature;
         TFeature = ( fe00_FPU, fe01_VME, fe02_DE, fe03_PSE,
           fe04_TSC, fe05_MSR, fe06_PAE, fe07_MCE,
           fe08_CX8, fe09_APIC, fe10_resv, fe11_SEP,
           fe12_MTRR, fe13_PGE, fe14_MCA, fe15_CMOV,
           fe16_PAT, fe17_PSE36, fe18_PPN, fe19_CLFLSH,
           fe20_resv, fe21_DTES, fe22_ACPI, fe23_MMX,
           fe24_FXSR, fe25_XMM, fe26_ISSE2, fe27_SSNOOP,
           fe28_resv, fe29_ACC, fe30_JMPE, fe31_resv )
       Provides information of the features supported by CPU.
       note: fe18_PPN stands for Physical Processor Number (new Intel's term),
       while on some descriptions and names such as PSN_Enabled or PSN_Hi/Lo,
       I used the old term of PSN (Processor Serial Number).
       Dont be confused, they're exactly refers to the same thing.

     CacheInfo:TStrings
       List of CPU's Cache Description.
       Since it has TStrings property, it can be extracted later as text.

     BulkData:pointer;
       Actually it points to 32 bytes of data resulted by the last CPUID
       instruction, in case of an application needs to retrieve them at once.
       Note that only 16 bytes of them filled with data.
       Those data could also be accesed sequentially via Registers
       published members (eax, ebx, ecx, edx).

     Property GlobalInfo:
       (see also GlobalInfoText method)
       Shows Global windows Information, see mine below for example.
       You'll see that some information (Processor Type, Level and
       Revision) are in corellation with CPUInfo's Class members.
       note that on NT Platform you must have an admin account to
       get the correct Hardware Profile Info.

         Hardware Profile Info
         ================
         Docking Info: 7 (00000007H)
         GUID: #$7BD485C050-A982-11D4-8A5E-00E07D87AF61#$7D
         Name:  aa@bitsmart.com

         System Info
         ================
         OEM ID: 0
         Page Size: 4096 (00001000H)
         Minimum Application Address: 00010000H
         Maximum Application Address: 7FFEFFFFH
         Active Processor Mask: 1
         Number Of Processors: 1
         Processor Type: 586
         Allocation Granularity: 65536 (00010000H)
         Processor Level: 6
         Processor Revision: 2051 (8038H)

         OS Version Info
         ================
         Major Version: 4 (00000004H)
         Minor Version: 0 (00000000H)
         Build Number: 1381 (00000565H)
         Platform ID: 2 (00000002H)
         CSD Version: Service Pack 6

         Memory Status
         ================
         Length: 32 (00000020H)
         Memory Load: 0 (00000000H)
         Total Physical: 133603328 (07F6A000H)
         Avail Physical: 65228800 (03E35000H)
         Total Page File: 389099520 (17313000H)
         Avail Page File: 351047680 (14EC9000H)
         Total Virtual: 2147352576 (7FFE0000H)
         Avail Virtual: 2087247872 (7C68E000H)

   methods:
   --------
     Constructor Create(Whoever:TComponent); override
     Destructor Destroy; override;
       Honestly, I want to eliminate both methods above, but I just couldn't ;)

     procedure About;
       Hi, it's me!

     procedure DoNothing(Sender:TObject)
       as it expressed, it does none thing ;-)

     OK, that's enough...

     function Execute:Integer;
       Execute CPUID instruction for current level.
       make sure that you have specified a value for Level property
       to avoid unexpected result.

     function ExecuteLevel(const nLevel:Integer):Integer
       Another way to execute CPUID instruction,
       this one will force you to enter the specified level call.

     function ExecuteCPUID(const nLevel:Integer; var eax,
       ebx, ecx, edx: Integer):Integer
       This way you have even more control of CPUID instruction flow.
       This method is merely a wrapper of the assembly form of instruction
       to a more convenient one.

     procedure ExecuteLevel0; virtual;
     procedure ExecuteLevel1; virtual;
     procedure ExecuteLevel2; virtual;
     procedure ExecuteLevel3; virtual;
       a group of methods for commonly used level call.

     procedure RefereshInfo; virtual;
       Re-enumerate all information in current state.
       It rarely needed except you had changed manually
       some of read-write properties.

     function FeaturesAvailable(fs:TFeatureSet):String;
     function FeaturesNotAvailable(fs:TFeatureSet):String;
       List of available / not available features supported by
       CPU, simply give an empty set [] as argumen, this method then
       will use current feature set as default value.

     function CacheInfoText(const datadword:Integer):String;
       Plain text representation of CPU Cache

     function GlobalInfoText:String;
     // removed: procedure ShowGlobalInfo;
       This method added (later) for additional info.
       Its a kind of greedy method, I added unit Windows, SysUtils //and Dialogs
       just for this single method alone, no kidding.
       The function itself is quite independent, you may even copy-paste it
       to your code without anymore constraint.

   events:
   -------
     OnLevelChange
       property Level Changed

     OnAutoChange
       property Auto Changed


. (this point was intended as such, since Borland's auto indent throws me apart from BOL)


notes:
  I'm running out of time!
  I just only have Intel processor (& some old cyrix also),
  never really test this code with other.

  This version can not detect Extended (or is it Extension?) Level $8XXX-XXXX.
  Actually it can, but the descriptor mechanism hasn't been prepared yet.
  You are always welcome to extend the capabilities of this component in any way.

  Look also the commented properties in this source code to see the
  Features information that can be retrieved in Extended Level
  (it appears to be applied and customized only by non-Intel's CPU),
  such as 3DNow!'s AMD and MMXX's Cyrix.
  for more details, see Christian Ludloff's http://www.sandpile.org

Final notes:
  Sorry for the mess,
  My time is up.
  my big-boss seems not too happy,
  and the salespersons btw, need to sell something.

  I've got to do my-'real-life' works now.

  Well, We'll see next month,
  If I had some time for doing some fun again.
}

{
  ===========================================================
  DISCLAIMER
  I HEREBY DISCLAIM ANY... BLAH 3x
  .....

  comments:
    If you find this code is useful for you or your company,
    please contribute US $10 (at least) or equivalent,
    for any of kidney charity/funds/organization/etc.
    in poor country to help poor people with kidney
    handicap/malfunction.

    Yes, they will otherways die indeed, but they might
    lived a little longer with your help.
  ============================================================
  CopyRight (DeleteWrong) 2000, Dadang Sofyan,
  mailto:aa@bitsmart.com   ...hope it still working ;)
}


interface

uses Classes;
//Windows, Messages, SysUtils, Graphics, Classes, Controls, Forms, Dialogs;

Type
  TWinInfo = Class(TPersistent)
  public
    Constructor Create; virtual;
    Destructor Destroy; override;
  end;

Type
  TRegisters = Class(TPersistent)
  private
    _eax, _ebx, _ecx, _edx :Integer;
  published
    property eax:Integer read _eax write _eax;
    property ebx:Integer read _ebx write _ebx;
    property ecx:Integer read _ecx write _ecx;
    property edx:Integer read _edx write _edx;
  public
    Constructor Create; virtual;
    Destructor Destroy; override;
  end;

Type
  TFeature = (
    fe00_FPU, fe01_VME, fe02_DE, fe03_PSE,
    fe04_TSC, fe05_MSR, fe06_PAE, fe07_MCE,
    fe08_CX8, fe09_APIC, fe10_resv, fe11_SEP,
    fe12_MTRR, fe13_PGE, fe14_MCA, fe15_CMOV,
    fe16_PAT, fe17_PSE36, fe18_PPN, fe19_CLFLSH,
    fe20_resv, fe21_DTES, fe22_ACPI, fe23_MMX,
    fe24_FXSR, fe25_XMM, fe26_ISSE2, fe27_SSNOOP,
    fe28_resv, fe29_ACC, fe30_JMPE, fe31_resv
  );

  TFeatureSet = set of TFeature;

const
  FeatureReserved:TFeatureSet = [fe10_resv, fe20_resv, fe28_resv, fe31_resv];

Type
  TVendor = (vn_none, vnIntel, vnAMD, vnCyrix, vnIDT, vnNexGen, vnUMC, vnRise);
  TcpuType = (cpu_primary, cpu_overdrive, cpu_secondary, cpu_reserved);
  //  TcpuFamily = (i486, i586, i686, itanium);
  TcpuClass = (x000, x086, x286, x386, x486, x586, x586p);

Type TCPUSerialNo = Class(TPersistent)
  private
    fPSN_ax, fPSN_Hi, fPSN_Lo, nullint:Integer;
    fPSNEnabled, nullb:Boolean;
    nullstr:String;
    function GetfPSN_String:String;
  published
    property PSN_ax:Integer
      read fPSN_ax write nullint stored FALSE;
    property PSN_Hi:Integer
      read fPSN_Hi write nullint stored FALSE;
    property PSN_Lo:Integer
      read fPSN_Lo write nullint stored FALSE;
    property PSN_String:String
      read GetfPSN_String write nullstr stored FALSE;
    property PentiumIII:Boolean
      read fPSNEnabled write nullb stored FALSE;
  public
    Constructor Create; virtual;
    Destructor Destroy; override;
  end;

Type
  TCPUInfo = Class(TPersistent)
  private
    fAuto:Boolean;
    fVendor:TVendor;
    fcpuClass:TcpuClass;
    fcpuFamily, fcpuModel, nullstr:String;
    fcpuStepping, nullInteger:integer;
    fcpuType, nullcpuType:TcpuType;

    function GetcpuClass:TcpuClass;
    function GetcpuClassInt:Integer;
    procedure scancpumodel(const cpu:Integer);

  published
    property cpuClass:TcpuClass
      read GetcpuClass write fcpuClass stored FALSE;
    property cpuType:TcpuType
      read fcpuType write nullcpuType stored FALSE;
    property cpuFamily:String
      read fcpuFamily write nullstr stored FALSE;
    property cpuModel:String
      read fcpuModel write nullstr stored FALSE;
    property cpuStepping:Integer
      read fcpuStepping write nullInteger stored FALSE;

  public
    Constructor Create; virtual;
    Destructor Destroy; override;
  end;

Type
  Tacpuid = class(TComponent)
  private
    nullint:Integer; nullstr:String;
  {
    //Level 0
    fVendorSign:String;
    //Level 1 :EAX result
    fcpuFamily, fcpuModel:String;
    fcpuStepping:Integer;
    fFamilyExt, fTypeExt, fFamily, fType, fModel, fStepping:Integer;

    //Level 1 - EBX result
    fAPICID, fcpuCount, fCLFlush, fBrandID:Integer;

    //Level 1 :EDX result
    fFeatured:Integer;
    fFeatures:TFeatureSet;

    ff00FPU, ff01VME, ff02DE, ff03PSE, ff04TSC, ff05MSR, ff06PAE, ff07MCE,
    ff08CX8, ff09APIC, ff11SEP, ff12MTRR, ff13PGE, ff14MCA, ff15CMOV,
    ff16PAT, ff17PSE36, ff18PSN, ff19CLFlush, ff20Resv, ff21DTES, ff22ACPI, ff23MMX,
    ff24FXSR, ff25Resv, ff26SSE2, ff27SSnoop, ff28Resv, ff29ACC, ff30JMPE,
    ff31Resv :Boolean;

    //Level 2
    //fDescriptorCount:Integer;

    fdEAH, fdEAL, fdAH, fdAL,   fdEBH, fdEBL, fdBH, fdBL,
    fdECH, fdECL, fdCH, fdCL,   fdEDH, fdEDL, fdDH, fdDL:Integer;

    //Level 3
    fPSNHi:Integer; fPSNLo:Integer;

    //Level 0 (Ext.)
    fx_Vendor:string; fxMaxLevel:Integer;

    //Level 1 (Ext.) :EAX result
    fx_Family, fx_Model, fx_Stepping:Integer;

    //Level 1 (Ext.) :EDX result
    fx_Features:Integer;
    fx00FPU, fx01VME, fx02DE, fx03PSE, fx04TSC, fx05MSR, fx06PAE, fx07MCE,
    fx08CX8, fx09APIC, fx10SEPK6, fx11SEP, fx12MTRR, fx13PGE, fx14MCA, fx15CMOV,
    fx16FCMOV, fx16PATMSR, fx17PSE36, //fx16PAT is AMDK7 specific
    fx18Resv, fx19Resv, fx20Resv, fx21Resv, fx22SSE, fx23MMX, fx24MMXX, //fx24mmxx is Cyrix specific
    fx24FXSR, fx25Resv, fx26Resv, fx27Resv, fx28Resv, fx29LM, fx303DNowExt,
    fx31Resv: Boolean;

    //Level 2 to 4 (Ext.)
    fxsEAX2, fxsEBX2, fxsECX2, fxsEDX2,
    fxsEAX3, fxsEBX3, fxsECX3, fxsEDX3,
    fxsEAX4, fxsEBX4, fxsECX4, fxsEDX4: Integer;

    //Level 5 (Ext.) :EAX result - L1 2MB TLB Config
    fxL1_2M_TLBConfig,
    fxL1_2MDTLB_assoc, fxL1_2MDTLB_entry, fxL1_2MCTLB_assoc, fxL1_2MCTLB_entry:Integer;

    //Level 5 (Ext.) :EBX result - L1 4KB TLB Config
    fxL1_4K_TLBConfig,
    fxL1_4KDTLB_assoc, fxL1_4KDTLB_entry, fxL1_4KCTLB_assoc, fxL1_4KCTLB_entry:Integer;

    //Level 5 (Ext.) :ECX result - L1 Data Cache Config
    fxL1_ADataConfig,
    fxL1_ACacheSize, fxL1_ACacheAssoc, fxL1_ACacheLnTag, fxL1_ACacheLnSize:Integer;

    //Level 5 (Ext.) :EDX result - L1 Code Cache Config
    fxL1_CodeConfig,
    fxL1_CCacheSize, fxL1_CCacheAssoc, fxL1_CCacheLnTag, fxL1_CCacheLnSize :Integer;

    //Level 6 (Ext.) :EAX result - L2 2MB TLB Config
    fxL2_2M_TLBConfig,
    fxL2_2MDTLB_assoc, fxL2_2MDTLB_entry, fxL2_2MCTLB_assoc, fxL2_2MCTLB_entry:Integer;

    //Level 6 (Ext.) :EBX result - L2 4KB TLB Config
    fxL2_4K_TLBConfig,
    fxL2_4KDTLB_assoc, fxL2_4KDTLB_entry, fxL2_4KCTLB_assoc, fxL2_4KCTLB_entry:Integer;

    //Level 6 (Ext.) :ECX result - L2 Unified Cache Config
    fxL2_CacheConfig,
    fxL2_CacheSize, fxL2_CacheAssoc, fxL2_CacheLnTag, fxL2_CacheLnSize:Integer;

    //Level 7 (Ext.) :EDX result
    fxEPM02VoltageControl, fxEPM01BusMultiplierControl:Boolean;
    //Level $8FFF-FFFE (Ext.) :EAX result = $0049-4544 (AMD only)
    fx__8FFFFFFE:Integer;

    //Level 8 (Ext.) :EAX,EBX,ECX,EDX result = 'NexGenerationAMD';
    fx__8FFFFFFF:String;
  }

  public
    Constructor Create(Whoever:TComponent); override; // I felt dominated ;)
    Destructor Destroy; override;
    procedure DoNothing(Sender:TObject);
    function Execute:Integer;
    function ExecuteLevel(const nLevel:Integer):Integer;
    function ExecuteCPUID(const nLevel:Integer; var eax, ebx, ecx, edx:Integer):Integer;
    procedure RefereshInfo; virtual;

  private
    function descriptbyte(const databyte:byte):String;
    function ExecuteCPUIDPtr(const nLevel:Integer;
      var eax, ebx, ecx, edx:pointer):Integer; stdcall;

  protected
    procedure Init; virtual;

  public
    procedure ExecuteLevel0; virtual;
    procedure ExecuteLevel1; virtual;
    procedure ExecuteLevel2; virtual;
    procedure ExecuteLevel3; virtual;

  private
    fVendor:TVendor; fVendorSign:String;
    fLevel, fMaxLevel:Integer;
//    fPSNEnabled:Boolean;
//    fPSN_ax,
//    fPSN_Lo, fPSN_Hi:Integer; // Processor Serial Number (PSN) (intel P3 doc)
                                // or Physical Processor Number (PPN) (more recent doc-IA64)
                                // do not ask me why.
    fFeatureSet:TFeatureSet;
    fcpuInfo:TCPUinfo;
    fcpuSerialNo: TCPUSerialNo;
    fRegisters:TRegisters;
    fWinInfo:TWinInfo;
    fCacheInfo, fFeatureList, fGlobalInfo:TStrings;

    fLevelChanged:TNotifyEvent;
    AutoChanged:TNotifyEvent;

    function GetRegisters:TRegisters;
    procedure SetRegisters(Registers:TRegisters);

    function GetfVendor:TVendor;
    procedure SetfVendor(Vendor:TVendor);

    function Getfcpuinfo:TCPUInfo;
    procedure Setfcpuinfo(CPUInfo:TCPUInfo);

    function GetfFeatureSet:TFeatureSet;
    procedure SetfFeatureSet(FeatureSet:TFeatureSet);

    function GetfFeatureList:TStrings;
    procedure SetfFeatureList(FeatureList:TStrings);

    function GetCacheInfo:TStrings;
    procedure SetCacheInfo(CacheInfo:TStrings);

    function GetGlobalInfo:TStrings;
    procedure SetGlobalInfo(GlobalInfo:TStrings);

    function GetfAuto:Boolean;
    procedure SetfAuto(Auto:Boolean);

    procedure SetfLevel(Levelno:Integer);

  public
    fFeatured:Integer;
    function FeaturesAvailable(fs:TFeatureSet):String;
    function FeaturesNotAvailable(fs:TFeatureSet):String;
    function CacheInfoText(const datadword:Integer):String;
    function GlobalInfoText:String;
  published
    property Registers:Tregisters
      read GetRegisters write SetRegisters stored FALSE;
    property CPUInfo:tcpuinfo
      read Getfcpuinfo write Setfcpuinfo stored FALSE;
    property CPUSerialNo: TCPUSerialNo
      read fcpuserialno write fcpuserialno stored FALSE;
    property Auto:Boolean
      read GetfAuto write SetfAuto stored FALSE;
    property Vendor:TVendor
      read GetfVendor write SetfVendor stored FALSE;
    property VendorSign:String
      read fVendorSign write nullstr stored FALSE;
    property Levelno:Integer
      read fLevel write SetfLevel stored FALSE;
    property MaxLevel:Integer
      read fMaxLevel write nullint stored FALSE;
    property FeatureList:TStrings
      read GetfFeatureList write SetfFeatureList stored FALSE;
    property Features:TFeatureSet
      read GetfFeatureSet write SetfFeatureSet stored FALSE;
    property CacheInfo:TStrings
      read GetCacheInfo write SetCacheInfo stored FALSE;
    property GlobalInfo:TStrings
      read GetGlobalInfo write SetGlobalInfo stored FALSE;
    property OnLevelChange:TNotifyEvent
      read fLevelChanged write fLevelChanged;
    property OnAutoChange:TNotifyEvent
      read AutoChanged write AutoChanged;

    public
      BulkData:pointer;
      procedure About;
      //procedure ShowGlobalInfo;
      //procedure Validate;virtual;abstract;

  end;

procedure Register;

implementation
uses Windows, SysUtils, Dialogs, DsgnIntf;//, Dialogs;

const
  cpuID      = $A20F; //hard-coded cpuid instruction
                      //called with:
                      //  dw cpuID
                      //you may also call with (note the reversed order):
                      //  db $0F
                      //  db $0A
const
  TAB = #9;
  TAB2 = TAB+TAB;
  CRLF = #13#10;
  CRLF2 = CRLF+CRLF;

const
  FA_RESERVED = '---';
  FD_RESERVED = 'Reserved';
Type
  TFeatureStr = array [TFeature] of string;

var FeaturesAbbreviation: TFeatureStr = (
   'FPU', 'VME', 'DE', 'PSE', 'TSC', 'MSR', 'PAE', 'MCE',
   'CX8', 'APIC', FA_RESERVED, 'SEP', 'MTRR', 'PGE', 'MCA', 'CMOV',
   'PAT', 'PSE-36', 'PSN', 'CLFlush', FA_RESERVED, 'DTES', 'ACPI', 'MMX',
   'FXSR', 'XMM', 'ISSE2', 'SNOOP', FA_RESERVED, 'ACC', 'JMPE', FA_RESERVED
  );

var FeaturesDescription: TFeatureStr = (
    'Floating Point Unit',
    'Virtual Mode Extension',
    'Debugging Extension',
    'Page Size Extension',
    'Time Stamp Counter',
    'Model Specific Register',
    'Physical Address Extension',
    'Machine Check Exception',
    'Compare & Exchange 8 Byte Instruction',
    'On-Chip Advance Programmable Interrupt Controller Hardware',
    FD_RESERVED,
    'Fast System Call SYSENTER/SYSEXIT',
    'Memory Type Range Register',
    'Page Global Enable',
    'Memory Check Architecture',
    'Conditional MOVE Instruction',
    'Page Attribute Table',
    '36-bit Page Size Extension',
    'Processor Serial Number',
    'Cache Line Flush',
    FD_RESERVED,
    'Debug Trace and EMON Store',
    'Processor Duty Cycle Control',
    'MMX Technology',
    'Fast Floating Point Save and Restore',
    'Intel Streaming SIMD Extension',
    'Intel Streaming SIMD Extension 2',
    'Self Snoop',
    FD_RESERVED,
    'Automatic Clock Control',
    'JMPE 64-bit Architecture',
    FD_RESERVED
  );

const
  VendorName:array[TVendor] of pchar = (
  'Unknown', 'Intel', 'AMD', 'IBM/VIA Cyrix', 'Centaur/IDT', 'NexGen', 'UMC', 'Rise');
  VendorSigns:array[TVendor] of pchar =
    ('Unidentified', 'GenuineIntel', 'AuthenticAMD', 'CyrixInstead',
      'CentaurHauls', 'NexGenDriven', 'UMC UMC UMC ', 'RiseRiseRise');

{
//Bitwise routine
Type
  TBitNo = 0..DWORDBITNUM-1;
function SetBitNo(const Bits32:Integer; const no:TBitNo):Integer; begin
  Result:=Bits32 OR 1 shl no;
end;
function ClearBitNo(const Bits32:Integer; const no:TBitNo):Integer; begin
  Result:= Bits32 and not (1 shl no);
end;
function CheckBitNo(const Bits32:Integer; const no:TBitNo):Boolean; begin
  Result:= Bits32 and (1 shl no) > 0;
end;
function HiWord(const Bits32:Integer):WORD; begin
  Result:= trunc(Bits32 shr 16 and $FFFF) ;
end;
function LoWord(const Bits32:Integer):WORD; begin
  Result:= trunc(Bits32 and $FFFF);
end;
function HiWordHi(const Bits32:Integer):Byte; begin
  Result:=Bits32 shr 24 and $00FF;
end;
function HiWordLo(const Bits32:Integer):Byte; begin
  Result:=Bits32 shr 16 and $00FF;
end;
function LoWordHi(const Bits32:Integer):Byte; begin
  Result:=Bits32 shr 8 and $00FF;
end;
function LoWordLo(const Bits32:Integer):Byte; begin
  Result:=Bits32 and $00FF;
end;
}
var
  aBulkData:packed array[0..31] of byte;

Constructor TWinInfo.Create; begin  inherited;end;
Destructor TWinInfo.Destroy; begin  inherited;end;

Constructor TcpuInfo.Create; begin  inherited;end;
Destructor TcpuInfo.Destroy; begin  inherited;end;

Constructor TcpuSerialNo.Create; begin  inherited;end;
Destructor TcpuSerialNo.Destroy; begin  inherited;end;

function TCPUSerialNo.GetfPSN_String:String; begin
  Result:=inttohex(fPSN_ax, 4) + inttohex(fPSN_Hi,8) + IntToHex(fPSN_Lo,8)
end;

Constructor TRegisters.Create; begin  inherited;end;
Destructor TRegisters.Destroy; begin  inherited;end;

function tcpuinfo.GetcpuClass:TcpuClass;
begin
  if fAuto then
    fcpuClass:=TcpuClass(GetcpuClassInt);
  Result:=fcpuClass;
end;

function tcpuinfo.GetcpuClassInt:Integer;
// part of of them belongs to Intel
const
  AC_Mask=$40000;   //bit-18
  PSN_Mask=$200000; //bit-21
  cpu_ERR=0;        //Reserved
  cpu_386=3;        //Old cpu
  cpu_486=4;        //486 without Processor Serial Number Support
  cpu_586=5;        //486+ with PSN Support Disabled
  cpu_PSN=6;        //486+ with PSN Support Enabled
asm
  @@check_80386:
    pushfd                // push original EFLAGS
    pop EAX               // get original EFLAGS
    mov ECX, EAX          // save original EFLAGS
    xor EAX, AC_Mask      // flip AC bit in EFLAGS
    push EAX              // save new EFLAGS value on stack
    popfd                 // replace current EFLAGS value
    pushfd                // get new EFLAGS
    pop EAX               // store new EFLAGS in EAX
    xor EAX, ECX          // cant toggle AC bit, processor=80386
    mov EAX, cpu_386      // turn on 80386 processor flag
    jz @@end              // jump if 80386 processor
    push ECX
    popfd                 // restore AC bit in EFLAGS first
  @@check_80486:
                          // Checks ability to set/clear ID flag (Bit 21) in EFLAGS
                          // (indicating the presence of cpuID instruction)
    mov EAX, ECX          // get original EFLAGS
    xor EAX, PSN_Mask     // flip bit-21 (ID) in EFLAGS
    push EAX              // save new EFLAGS value
    popfd                 // replace current EFLAGS value
    pushfd                // get new EFLAGS
    pop EAX               // store new EFLAGS in EAX
    push ECX              // restore back
    popfd                 // original flags - intel's slipped here ;-(
    xor EAX, ECX          // compare ID bit,
    mov EAX, cpu_486      // 486 without PSN support
    je @@end              // cannot toggle ID bit
    mov EAX, cpu_586      // Processor Serial Number Supported
    and ECX, PSN_Mask     // PSN Enabled?
    jz @@end
    mov EAX, cpu_PSN       // Yes, PSN Enabled
  @@end:                   // done.
end;

const _cpuType :byte = 0; //it had to be here, do not move after funct declaration
function tacpuid.ExecuteCPUIDPtr; stdcall;
const
  _cpuTypeBit = 12;
  _PSNBitMask = $200000;
asm
  @@Begin:
    cmp nLevel, 3             // Cyrix workaround:
    jnz @@CyrixPass          // PSN-bit mus be enabled
    pushfd                   // no way to turn it back (off) ;)
    pop EAX                  // if you want to do so
    or EAX, _PSNBitMask      // pushfd at begin and popfd at end
    push EAX                 // beware of lost of flow-control
    popfd
  @@CyrixPass:
    cmp nLevel, 2
    jnz @@Synchronized
  @@MPCheck:                 // Multi Processor Check Synchronicity
                             // Differentiate only primary & non-primary
    mov EAX,1
    dw cpuID                 // execute service 1 call
    shr EAX, _cpuTypeBit     // extract cpuType
    and AL, 3                // validate bit-0 and bit-1
    cmp AL, _cpuType         // compare wih previous result
    mov _cpuType, AL         // save current value
    loopnz @@MPCheck
  @@Synchronized:
    mov EAX, nLevel
    dw cpuID
    push EAX
    mov EAX, [&ecx]          // var argument is REALLY a pointer-
    mov [EAX], ECX           // load it first to register & then
    mov EAX, [&edx]          // you can get the value it's refers to
    mov [EAX], EDX
    mov EAX, [&ebx]
    mov [EAX], EBX
    pop EAX
    push EBX
    mov EBX, [&eax]
    mov [EBX], EAX
    pop EBX
    cmp nLevel, 0             // is it a level 0 Query?
    jnz @@End
    push EAX                 // save eax result
    shr EAX, _cpuTypeBit     // extract cpuType
    and AL, 3                // validate bit-0 and bit-1
    mov _cpuType, AL
    pop EAX
  @@End:
    mov dword ptr aBulkData, EAX       // save all result
    mov dword ptr aBulkData[+4], EBX   // maybe used sometime
    mov dword ptr aBulkData[+8], ECX   //
    mov dword ptr aBulkData[+12], EDX  //
    mov @Result, EAX                   // done.
end;

function tacpuid.ExecuteCPUID; begin
  Result:=ExecuteCPUIDPtr(nLevel, pointer(eax), pointer(ebx), pointer(ecx), pointer(edx));
end;

procedure tacpuid.ExecuteLevel0;
const
  OFFSET = sizeof(Integer);
  C4 : string[OFFSET] = '****';
var
  v:TVendor;
begin
  with fcpuInfo do begin
    fcpuClass:=GetcpuClass;
    if fcpuClass < x586 then exit;
  end;
  fLevel:=0;
  with fRegisters do
    fMaxLevel:=ExecuteCPUID(fLevel, _eax,_ebx,_ecx,_edx);
  move(fRegisters._ebx, C4[1], OFFSET);
//  v:=Low(TVendor);
  for v:=Low(TVendor) to high(TVendor) do
    if pos(C4, VendorSigns[v]) = 1 then begin
      SetfVendor(v);
      break;
    end;
end;

function tacpuid.descriptbyte(const databyte:byte):String;
const  //cACHE DESCRIPTORS
  cn_NULL =0;             cs_NULL = 'null descriptor';

  cn_ctlb_4k =1;          cs_ctlb_4k = 'CODE TLB, 4K pages, 4 ways, 32 entries';
  cn_ctlb_4m =2;          cs_ctlb_4m = 'CODE TLB, 4M pages, fullway, 2 entries';
  cn_dtlb_4k =3;          cs_dtlb_4k = 'DATA TLB, 4K pages, 4 ways, 64 entries';
  cn_dtlb_4m =4;          cs_dtlb_4m = 'DATA TLB, 4M pages, 4 ways, 8 entries';

  cn_CL1_08k_4x32 =6;     cs_CL1_08k_4x32 = 'CODE L1 Cache 8K, 4 ways, 32b lines';
  cn_CL1_16k_4x32 =8;     cs_CL1_16k_4x32 = 'CODE L1 Cache 16K, 4 ways, 32b lines';
  cn_DL1_08k_4x32 =$A;    cs_DL1_08k_4x32 = 'DATA L1 Cache 8K, 2 ways, 32b lines';
  cn_DL1_16k_4x32 =$C;    cs_DL1_16k_4x32 = 'DATA L1 Cache 16K, 4 ways, 32b lines';

  cn_XL3_0512k =$22;      cs_XL3_0512k = 'CODE/DATA L3 Cache 512K, 4 ways, 64b lines';
  cn_XL3_1024k =$23;      cs_XL3_1024k = 'CODE/DATA L3 Cache 1024K, 8 ways, 64b lines';
  cn_XL3_2048k =$25;      cs_XL3_2048k = 'CODE/DATA L3 Cache 2048K, 8 ways, 64b lines';
  cn_XL3_4096k =$29;      cs_XL3_4096k = 'CODE/DATA L3 Cache 4096K, 8 ways, 64b lines';

  cn_non_L2_L3 =$40;      cs_non_L2_L3 = 'non integrated L2/L3 Cache';

  cn_XL2_0128k_4x32 =$41; cs_XL2_0128k_4x32 = 'CODE/DATA L2 Cache 128K, 4 ways, 32b lines';
  cn_XL2_0256k_4x32 =$42; cs_XL2_0256k_4x32 = 'CODE/DATA L2 Cache 256K, 4 ways, 32b lines';
  cn_XL2_0512k_4x32 =$43; cs_XL2_0512k_4x32 = 'CODE/DATA L2 Cache 512K, 4 ways, 32b lines';
  cn_XL2_1024k_4x32 =$44; cs_XL2_1024k_4x32 = 'CODE/DATA L2 Cache 1024K, 4 ways, 32b lines';
  cn_XL2_2048k_4x32 =$45; cs_XL2_2048k_4x32 = 'CODE/DATA L2 Cache 2048K, 4 ways, 32b lines';

  cn_ctlb_0064 =$50;      cs_ctlb_0064 = 'CODE TLB all pages, 64 entries';
  cn_ctlb_0128 =$51;      cs_ctlb_0128 = 'CODE TLB all pages, 128 entries';
  cn_ctlb_0256 =$52;      cs_ctlb_0256 = 'CODE TLB all pages, 512 entries';

  cn_dtlb_0064 =$5B;      cs_dtlb_0064 = 'DATA TLB all pages, 64 entries';
  cn_dtlb_0128 =$5C;      cs_dtlb_0128 = 'DATA TLB all pages, 128 entries';
  cn_dtlb_0256 =$5D;      cs_dtlb_0256 = 'DATA TLB all pages, 256 entries';

  cn_DL1_08k_4 =$66;      cs_DL1_08k_4 = 'DATA L1 Cache 8K, 4 ways';
  cn_DL1_16k_4 =$67;      cs_DL1_16k_4 = 'DATA L1 Cache 16K, 4 ways';
  cn_DL1_32k_4 =$68;      cs_DL1_32k_4 = 'DATA L1 Cache 32K, 4 ways';

  cn_TL1_12k =$70;        cs_TL1_12k = 'Trace L1 Cache 12K, 4 ways';
  cn_TL1_16k =$71;        cs_TL1_16k = 'Trace L1 Cache 16K, 4 ways';
  cn_TL1_32k =$72;        cs_TL1_32k = 'Trace L1 Cache 32K, 4 ways';

  cn_XL2_0128k_8x64 =$79; cs_XL2_0128k_8x64 = 'CODE/DATA L2 Cache 128K, 8 ways, 32b lines';
  cn_XL2_0512k_8x64 =$7A; cs_XL2_0512k_8x64 = 'CODE/DATA L2 Cache 512K, 8 ways, 32b lines';
  cn_XL2_1024k_8x64 =$7B; cs_XL2_1024k_8x64 = 'CODE/DATA L2 Cache 1024K, 8 ways, 32b lines';
  cn_XL2_2048k_8x64 =$7C; cs_XL2_2048k_8x64 = 'CODE/DATA L2 Cache 2048K, 8 ways, 32b lines';

  cn_XL2_0256k_8x32 =$82; cs_XL2_0256k_8x32 = 'CODE/DATA L2 Cache 256K, 8 ways, 32b lines';
  cn_XL2_0512k_8x32 =$83; cs_XL2_0512k_8x32 = 'CODE/DATA L2 Cache 512K, 8 ways, 32b lines';
  cn_XL2_1024k_8x32 =$84; cs_XL2_1024k_8x32 = 'CODE/DATA L2 Cache 1024K, 8 ways, 32b lines';
  cn_XL2_2048k_8x32 =$85; cs_XL2_2048k_8x32 = 'CODE/DATA L2 Cache 2048K, 8 ways, 32b lines';

  cn_xtlb_4k_cyrix =$70;  cs_xtlb_4k_cyrix = 'CODE/DATA TLB 4k pages, 4 ways, 32 entries';
  cn_xL1_16k_cyrix =$80;  cs_xL1_16k_cyrix = 'CODE/DATA TLB 16k pages, 4 ways, 32 entries';

  cs_unknown = 'Unresolved descriptor value';

begin
  case databyte of
    cn_ctlb_4k: Result:= cs_ctlb_4k;
    cn_ctlb_4m: Result:= cs_ctlb_4m;
    cn_dtlb_4k: Result:= cs_dtlb_4k;
    cn_dtlb_4m: Result:= cs_dtlb_4m;

    cn_CL1_08k_4x32: Result:= cs_CL1_08k_4x32;
    cn_CL1_16k_4x32: Result:= cs_CL1_16k_4x32;

    cn_DL1_08k_4x32: Result:= cs_DL1_08k_4x32;
    cn_DL1_16k_4x32: Result:= cs_DL1_16k_4x32;

    cn_XL3_0512k: Result:= cs_XL3_0512k;
    cn_XL3_1024k: Result:= cs_XL3_1024k;
    cn_XL3_2048k: Result:= cs_XL3_2048k;
    cn_XL3_4096k: Result:= cs_XL3_4096k;

    cn_non_L2_L3: Result:= cs_non_L2_L3;

    cn_XL2_0128k_4x32: Result:= cs_XL2_0128k_4x32;
    cn_XL2_0256k_4x32: Result:= cs_XL2_0256k_4x32;
    cn_XL2_0512k_4x32: Result:= cs_XL2_0512k_4x32;
    cn_XL2_1024k_4x32: Result:= cs_XL2_1024k_4x32;
    cn_XL2_2048k_4x32: Result:= cs_XL2_2048k_4x32;

    cn_ctlb_0064: Result:= cs_ctlb_0064;
    cn_ctlb_0128: Result:= cs_ctlb_0128;
    cn_ctlb_0256: Result:= cs_ctlb_0256;

    cn_dtlb_0064: Result:= cs_dtlb_0064;
    cn_dtlb_0128: Result:= cs_dtlb_0128;
    cn_dtlb_0256: Result:= cs_dtlb_0256;

    cn_DL1_08k_4: Result:= cs_DL1_08k_4;
    cn_DL1_16k_4: Result:= cs_DL1_16k_4;
    cn_DL1_32k_4: Result:= cs_DL1_32k_4;

    cn_TL1_12k: Result:= cs_TL1_12k;
//    cn_xtlb_4k_cyrix: Result:= cs_xtlb_4k_cyrix;
    cn_TL1_16k: Result:= cs_TL1_16k;
    cn_TL1_32k: Result:= cs_TL1_32k;

    cn_XL2_0128k_8x64: Result:= cs_XL2_0128k_8x64;
    cn_XL2_0512k_8x64: Result:= cs_XL2_0512k_8x64;
    cn_XL2_1024k_8x64: Result:= cs_XL2_1024k_8x64;
    cn_XL2_2048k_8x64: Result:= cs_XL2_2048k_8x64;
    cn_XL2_0256k_8x32: Result:= cs_XL2_0256k_8x32;
    cn_XL2_0512k_8x32: Result:= cs_XL2_0512k_8x32;
    cn_XL2_1024k_8x32: Result:= cs_XL2_1024k_8x32;
    cn_XL2_2048k_8x32: Result:= cs_XL2_2048k_8x32;
    cn_xL1_16k_cyrix: Result:= cs_xL1_16k_cyrix;
    else Result:=cs_unknown;
  end;
  Result:= inttohex(databyte,2)+#9+Result;
end;

function tacpuid.CacheInfoText(const datadword:Integer):String;
var i:Integer;
begin
  Result:='';
    for i:=1-1 to 4-1 do
      if (datadword shr (i*8) and $FF) > 0 then //begin
        Result:=Result+CRLF+descriptbyte(datadword shr (i*8) and $FF);
  Result:=trim(Result)
end;

procedure tcpuinfo.scancpumodel(const cpu:Integer); register;
// I know those are ugly, but they do fast :)
const
  i486=4; i586=5; i686=6; itanium=$F;
  NOFAMILY='(unknown cpu-family)';
  NOMODEL='(unknown cpu-model)';
var
  ctype, cfamily, cmodel, cstep:Integer;
  S:String;
begin
  ctype:=(cpu shr 12) and $03;
  cfamily:=(cpu shr 8) and $0F;
  cmodel:=(cpu shr 4) and $0F;
  cstep:=cpu and $0F;
  fcpuType:=TcpuType(ctype);
  fcpuStepping:=cstep;
  case fVendor of
    vnIntel: case cfamily of
      i486: case cmodel of
        0: S:='i486 DX 25/33';
        1: S:='i486 DX 50';
        2: S:='i486 SX';
        3: S:='i486 DX2 / i487';
        4: S:='i486 SL';
        5: S:='i486 SX2';
        7: S:='i486 DX2-WB';
        8: S:='i486 DX4';
        9: S:='i486 DX4-WB';
        else S:='i486 '+NOMODEL;
      end;
      i586: case cmodel of
        0: S:='P5 A-Step';
        1: S:='Pentium 60/66';
        2: S:='Pentium 75-200';
        3: S:='P24T OverDrive';
        4: S:='Pentium MMX 166/200';
        5: S:='P54C';
        7: S:='P55C 0.25 micron';
        else S:='Pentium '+NOMODEL;
      end;
      i686: case cmodel of
        0: S:='P6 A-Step';
        1: S:='Pentium Pro';
        3: S:='PII model 3 (0.28u)';
        5: S:='PII model 5 (0.25u),';
        6: S:='Celeron model 6';
        7: S:='PIII model 7 (0.25u)';
        8: S:='PIII/Xeon model 8 (0.18u) 256KB L2 Cache';
        $A: S:='PIII Xeon model A 1/2 MB L2 Cache';
        else S:='Pentium Pro'+NOMODEL;
      end;
      itanium: S:='Itanium IA-64';
      else S:=NOMODEL;
    end;
    vnAMD: case cfamily of
      i486: case cmodel of
        3: S:='80486 DX2';
        7: S:='80486 DX2 WB';
        8: S:='80486 DX4';
        9: S:='80486 DX4 WB';
        $E: S:='5x86';
        $F: S:='5x86 WB';
        else S:='4x86 '+NOMODEL;
      end;
      i586:case cmodel of
        0: S:='SSA5 PR-75/90/100)';
        1: S:='5K86 PR-120/133';
        2: S:='5K86 PR-166';
        3: S:='5K86 PR-200';
        6: S:='K6 0.3 micron';
        7: S:='K6 0.25 micron';
        8: S:='K6-II';
        9: S:='K6-III';
        else S:='K5/K6 '+NOMODEL;
      end;
      i686: case cmodel of
        1: S:='K7 0.25u';
        2: S:='K7 0.18u';
        3: S:='K7 0.18u 64KB L2 Cache';
        4: S:='K7 0.18u 256KB L2 Cache';
        else S:='K7 '+NOMODEL;
      end;
    else S:=NOMODEL;
    end;
    vnCyrix: case cfamily of
      i486: case cmodel of
        4: S:='Media GX/GXm';
        9: S:='5x86';
        else S:='5x86 '+NOMODEL;
      end;
      i586:case cmodel of
        2: S:='M1';
        4: S:='Media GX/GXm';
        else S:='M1 '+NOMODEL;
      end;
      i686: case cmodel of
        0: S:='M2';
        5: S:='III';
        else S:='M2 '+NOMODEL;
      end;
    else S:=NOMODEL;
    end;
    vnIDT: case cfamily of
      i486: case cmodel of
        0: S:='x486';
        else S:='x486 '+NOMODEL;
      end;
      i586:case cmodel of
        4: S:='C6';
        8: S:='C2';
        9: S:='C3';
        else S:='C6 '+NOMODEL;
      end;
      i686: case cmodel of
        0: S:='C7';
        else S:='C7 '+NOMODEL;
      end;
    else S:=NOMODEL;
    end;
    vnNexGen: case cfamily of
      i486: case cmodel of
        0: S:='nx486'
        else S:='nx486 '+NOMODEL;
      end;
      i586: case cmodel of
        0: S:='nx586 / nx586FPU';
        else S:='nx586 '+NOMODEL;
      end;
      i686: case cmodel of
        0: S:='nx686';
        else S:='nx686'+NOMODEL;
      end;
      else S:=NOMODEL;
    end;
    vnUMC: case cfamily of
      i486: case cmodel of
        1: S:='U5D';
        2: S:='U5S';
        else S:='U5-S/D '+NOMODEL;
      end;
      i586: case cmodel of
        0: S:='x586';
        else S:='x586 '+NOMODEL;
      end;
      i686: case cmodel of
        0: S:='x686';
        else S:='x686'+NOMODEL;
      end;
      else S:=NOMODEL;
    end;
    vnRise: case cfamily of
      i486: case cmodel of
        0: S:='mP5'
        else S:='mP5 '+NOMODEL;
      end;
      i586: case cmodel of
        0: S:='mP6 0.25u';
        2: S:='mP6 0.18u';
        else S:='mP6 '+NOMODEL;
      end;
      i686: case cmodel of
        0: S:='x686';
        else S:='x686'+NOMODEL;
      end;
       else S:=NOMODEL;
    end;
  end;
  case cfamily of
    4:fcpuFamily:='i486';
    5:fcpuFamily:='i586';
    6:fcpuFamily:='i686';
    $F:fcpuFamily:='itanium';
    else
      fcpuFamily:=NOFAMILY;
  end;
  fcpuModel:=VendorName[fVendor]+' '+S;
end;

function tacpuid.FeaturesAvailable(fs:TFeatureSet):String;
var fe:TFeature;
begin
  Result:='';
  if fs=[] then fs:=fFeatureSet;
  for fe:=low(TFeature) to high(TFeature) do
    if fe in fs then
      Result:=Result + CRLF + inttohex(ord(fe),2) + TAB+
        FeaturesAbbreviation[fe] + TAB2 + FeaturesDescription[fe];
  Result:=trim(Result);//remove extra CRLF
end;

function tacpuid.FeaturesNotAvailable(fs:TFeatureSet):String;
var fe:TFeature;
begin
  Result:='';
  if fs=[] then fs:=fFeatureSet;
  for fe:=low(TFeature) to high(TFeature) do
    if not (fe in fs) then
      Result:=Result + CRLF + inttohex(ord(fe),2) + TAB+
        FeaturesAbbreviation[fe] + TAB2 + FeaturesDescription[fe];
  Result:=trim(Result);//remove extra CRLF
end;

procedure tacpuid.ExecuteLevel1;
var
  fe:TFeature; S:String;
const
  YES='YES';
  NO='NO';
  NA='NA';
begin
  if fMaxLevel<1 then exit;
  fLevel:=1;
  with fRegisters do
    ExecuteCPUID(fLevel, _eax, _ebx, _ecx, _edx);

  fcpuSerialNo.fPSN_ax:=fRegisters._eax and $FFFF;
  
  with fRegisters, fcpuinfo do
    scancpumodel(_eax);

  fFeatureSet:=[];
  fFeatured:=fRegisters._edx;

  //check bit-n is set/on
  for fe:=low(TFeature) to high(TFeature) do
    if fFeatured and (1 shl ord(fe)) <> 0 then
      include(fFeatureSet, fe);

  fFeatureList.Clear;
  for fe:=low(TFeature) to high(TFeature) do begin
    S:=inttohex(ord(fe),2) + TAB + FeaturesAbbreviation[fe] + TAB;
    if fe in fFeatureSet then
      S:= S+TAB+YES
    else
      if fe in FeatureReserved then
        S:= S+TAB+NA
      else
        S:= S+TAB+NO;
    fFeatureList.Add(S+TAB+FeaturesDescription[fe]);
  end;
end;

procedure tacpuid.ExecuteLevel2;
var i,dw: Integer; first:Boolean;
begin
  if fMaxLevel < 2 then exit;
  fLevel:= 2;
  with fRegisters do begin
    First:=TRUE; i:=1;
    repeat
      ExecuteCPUID(fLevel, _eax, _ebx, _ecx, _edx);
      if First then begin
      //we dont know how intel will handle iteration > 1
      //in case of _eax always returning the largest value
      //then we should beware after endless loop
        First:=FALSE;
        i:=_eax and $F;
      end;
      dw:=_eax and not $FF; // blank-out AL
      with fCacheInfo do begin
        Clear;
        Text:=Text+ CacheInfoText(dw);
        Text:=Text+ CacheInfoText(_ebx);
        Text:=Text+ CacheInfoText(_ecx);
        Text:=Text+ CacheInfoText(_edx);
      end;
      dec(i);
    until i<1;
  end;
end;

procedure tacpuid.ExecuteLevel3;
begin
  fcpuSerialNo.fPSNEnabled:= fMaxLevel > 2;
  if not fcpuSerialNo.fPSNEnabled then exit;
  fLevel:= 3;
  with fRegisters do
    ExecuteCPUID(fLevel, _eax,_ebx, _ecx, _edx);

  fcpuSerialNo.fPSN_Lo:=fRegisters._ecx;
  fcpuSerialNo.fPSN_Hi:=fRegisters._edx;
end;

function tacpuid.ExecuteLevel(Const nLevel:Integer):Integer; begin
  if fMaxLevel< nLevel and $0FFFF then
    raise exception.Create('insufficient acces');
  fLevel:=nLevel;
  Result:=Execute;
end;

function tacpuid.Execute; begin
  with fRegisters do
    Result:=ExecuteCPUID(fLevel, _eax, _ebx, _ecx, _edx);
end;

Constructor tacpuid.Create;
begin
  inherited;
  fillchar(aBulkdata, sizeof(aBulkData),0);
  BulkData:=@aBulkData;
  fFeatureSet:=[];
  fCacheInfo:=TStringList.Create;
  fGlobalInfo:=TStringList.Create;
  fFeatureList:=TStringList.Create;
  fCPUSerialNo:=TCPUSerialNo.Create;
  fRegisters:=TRegisters.Create;
  fCPUInfo:=TCPUInfo.Create;
  fWinInfo:=TWinInfo.Create;
  Init;
end;

Destructor tacpuid.Destroy;
begin
  fWinInfo.Free;
  fCPUInfo.Free;
  fRegisters.Free;
  fCPUSerialNo.free;
  fFeatureList.Free;
  fGlobalInfo.Free;
  fCacheInfo.Free;
  BulkData:=nil;
  inherited;
end;

procedure tacpuid.donothing; asm
end;

procedure tacpuid.Init;
begin
  fVendorSign:=VendorSigns[fVendor]; //must be manually initialized
  RefereshInfo;
end;

function tacpuid.GetfAuto:Boolean; begin
  Result:=fcpuinfo.fAuto;
end;

procedure tacpuid.SetfAuto(Auto:Boolean); begin
  if fcpuinfo.fAuto=Auto then exit;
  if Assigned(AutoChanged) then
    AutoChanged(Self);
  fcpuinfo.fAuto:=Auto;
  if fcpuinfo.fAuto then
    RefereshInfo;
end;


procedure tacpuid.SetfLevel(Levelno:Integer); begin
  if fLevel=LevelNo then exit;
  if assigned(fLevelChanged) then
    fLevelChanged(Self);
  fLevel:=Levelno;
end;


procedure tacpuid.RefereshInfo;
var LastAuto:Boolean;
begin
  LastAuto:=fcpuinfo.fAuto;
  fcpuinfo.fAuto:=TRUE;
  ExecuteLevel0;
  ExecuteLevel1;
  ExecuteLevel2;
  ExecuteLevel3;
  fGlobalInfo.Text:=GlobalInfoText;
  with fcpuinfo do
    if fAuto <> LastAuto then
      fAuto:= LastAuto;
end;

function tacpuid.Getfcpuinfo:Tcpuinfo; begin
  Result:=fcpuinfo;
end;

procedure tacpuid.Setfcpuinfo; begin
  fcpuinfo.Assign(CPUInfo);
end;
function tacpuid.GetRegisters:TRegisters; begin
  Result:=fRegisters;
end;

procedure tacpuid.SetRegisters; begin
  fRegisters.Assign(Registers);
end;

function tacpuid.GetCacheInfo:TStrings; begin
  if fcpuinfo.fAuto then
    ExecuteLevel2;
  Result:=fCacheInfo;
end;

procedure tacpuid.SetCacheInfo; begin
  fCacheInfo.Assign(CacheInfo);
end;

function tacpuid.GetfFeatureSet:TFeatureSet; begin
  if fcpuinfo.fAuto then
    ExecuteLevel1;
  Result:=fFeatureSet;
end;

procedure tacpuid.SetfFeatureSet(FeatureSet:TFeatureSet); begin
  fFeatureSet:=FeatureSet;
end;

function tacpuid.GetfFeatureList:TStrings; begin
  if fcpuinfo.fAuto then
    ExecuteLevel1;
  Result:=fFeatureList;
end;

procedure tacpuid.SetfFeatureList(FeatureList:TStrings); begin
  fFeatureList.Assign(FeatureList);
end;

function tacpuid.GlobalInfoText:String; //(const fGlobalInfo:TStrings);
  var
    ms:TMemoryStatus;
    si:TSystemInfo;
    OS:TOSVersionInfo;
    hw:THWProfileInfo;
  {
    Value                         Platform
    VER_PLATFORM_WIN32s           Win32s on Windows 3.1.
    VER_PLATFORM_WIN32_WINDOWS    Win32 on Windows 95.
    VER_PLATFORM_WIN32_NT         Win32 on Windows NT.
  }
begin
  //WNetConnectionDialog
  OS.dwOSVersionInfoSize:=sizeof(OS);
  GetVersionEx(OS);
  GetSystemInfo(si);
  ms.dwLength:=sizeof(ms);
  GlobalMemoryStatus(ms);
  fillchar(hw,sizeof(hw),#0);
{$IFDEF NEW_ADVAPI32}
    GetCurrentHWProfile(hw);
{$ENDIF}
  Result:=format( 'Hardware Profile Info' + CRLF +
    //StringOfChar('=',16) + CRLF +
    'Docking Info: %d (%0.8XH)' + CRLF +
    'GUID: %s' + CRLF +
    'Name: %s' + CRLF2 +

    'System Info' + CRLF +
    //StringOfChar('=',16) + CRLF +
    'OEM ID: %d' + CRLF +
    'Page Size: %d (%0.8XH)' + CRLF +
    'Minimum Application Address: %0.8pH' + CRLF +
    'Maximum Application Address: %0.8pH' + CRLF +
    'Active Processor Mask: %d' + CRLF +
    'Number Of Processors: %d' + CRLF +
    'Processor Type: %d' + CRLF +
    'Allocation Granularity: %d (%0.8XH)' + CRLF +
    'Processor Level: %d' + CRLF +
    'Processor Revision: %d (%0.8XH)' + CRLF2 +

    'OS Version Info' + CRLF +
    //StringOfChar('=',16) + CRLF +
    'Major Version: %d (%0.8XH)' + CRLF +
    'Minor Version: %d (%0.8XH)' + CRLF +
    'Build Number: %d (%0.8XH)' + CRLF +
    'Platform ID: %d (%0.8XH)' + CRLF +
    'CSD Version: %s' + CRLF + CRLF +

    'Memory Status' + CRLF +
    //StringOfChar('=',16) + CRLF +
    'Length: %d (%0.8XH)' + CRLF +
    'Memory Load: %d (%0.8XH)' + CRLF +
    'Total Physical: %d (%0.8XH)' + CRLF +
    'Avail Physical: %d (%0.8XH)' + CRLF +
    'Total Page File: %d (%0.8XH)' + CRLF +
    'Avail Page File: %d (%0.8XH)' + CRLF +
    'Total Virtual: %d (%0.8XH)' + CRLF +
    'Avail Virtual: %d (%0.8XH)', [ hw.dwDockInfo, hw.dwDockInfo,
      hw.szHwProfileGuid,
      hw.szHwProfileName,
      si.dwOemID,
      si.dwPageSize, si.dwPageSize,
      si.lpMinimumApplicationAddress,
      si.lpMaximumApplicationAddress,
      si.dwActiveProcessorMask,
      si.dwNumberOfProcessors,
      si.dwProcessorType,
      si.dwAllocationGranularity, si.dwAllocationGranularity,
      si.wProcessorLevel,
      si.wProcessorRevision, si.wProcessorRevision,
      OS.dwMajorVersion, os.dwMajorVersion,
      OS.dwMinorVersion, os.dwMinorVersion,
      OS.dwBuildNumber, os.dwBuildNumber,
      OS.dwPlatformID, os.dwPlatformID,
      OS.szCSDVersion,
      ms.dwLength, ms.dwLength,
      ms.dwMemoryLoad, ms.dwMemoryLoad,
      ms.dwTotalPhys, ms.dwTotalPhys,
      ms.dwAvailPhys, ms.dwAvailPhys,
      ms.dwTotalPageFile, ms.dwTotalPageFile,
      ms.dwAvailPageFile, ms.dwAvailPageFile,
      ms.dwTotalVirtual, ms.dwTotalVirtual,
      ms.dwAvailVirtual, ms.dwAvailVirtual ]);
end;

function tacpuid.GetGlobalInfo:TStrings;
begin
  if fcpuinfo.fAuto then
    fGlobalInfo.Text := GlobalInfoText;
  Result:=fGlobalInfo;
end;

procedure tacpuid.SetGlobalInfo(GlobalInfo:TStrings); begin
   fGlobalInfo.Assign(GlobalInfo);
end;

//procedure tacpuid.ShowGlobalInfo; begin
//  showmessage(fGlobalInfo.Text);
//end;



function tacpuid.GetfVendor:TVendor; begin
  Result:=fVendor;
end;

procedure tacpuid.SetfVendor;
begin
  if fVendor=Vendor then exit;
  fVendor:=Vendor;
  fcpuinfo.fVendor:=Vendor;
  fVendorSign:=VendorSigns[fVendor];
end;


const
  MENU_ITEMS = 7;
  MENULIST : array[0..MENU_ITEMS-1] of String = (
    '&About aCPUID',
    '-',
    '&Cache Descriptor',
    'A&vailable Features',
    '&Unavailable Features',
    'Comprehensive &List of Features',
    '&Windows Global Info'
  );

resourcestring
  aDadang='dadang sofyan';
  aMail='mailto:aa@bitsmart.com';
  aCompany='Lembaga Manajemen Formasi - TAXES Division';
  aSnail='Jl. Sebret No.4B, Ps.Minggu, Jakarta';
  aZIP='12540';
  aPhone='+62-21-8111-71541';
  aCountry='INDONESIA';
  aDSPWarez='see also in DSP: abs_rw, AboutReg, CDCheck, aCPUID';
//  aCompound=aDadang+', 'aMail+CRLF+aCompany+CRLF+aSnail+CRLF+aPhone+CRLF+aCountry+CRLF+aDSPWarez;

Type
  tacpuidShow = Class(TComponentEditor)
  private
    procedure About(Sender:TObject);
    procedure CacheInfoClick(Sender:TObject);
    procedure AvailableClick(Sender:TObject);
    procedure UnavailableClick(Sender:TObject);
    procedure ComprehensiveClick(Sender:TObject);
    procedure GlobalInfoClick(Sender:TObject);
  public
    function GetVerbCount:Integer;override;
    function GetVerb(idx:Integer):String;override;
    procedure ExecuteVerb(idx:Integer);override;
    procedure Edit;override;
  end;

function tacpuidShow.GetVerbCount:Integer; begin Result:=MENU_ITEMS;
end;
function tacpuidShow.GetVerb(idx:Integer):String; begin
  Result:=MENULIST[idx]
end;

procedure tacpuidShow.ExecuteVerb(idx:Integer); begin
  case idx of
    0: About(Self);
    2: CacheInfoClick(Self);
    3: AvailableClick(Self);
    4: UnavailableClick(Self);
    5: ComprehensiveClick(Self);
    6: GlobalInfoClick(Self);
    else About(Self);
  end;
end;

procedure tacpuidShow.Edit; begin ExecuteVerb(0); end;

procedure tacpuid.About; begin
  Showmessage('aCPUID is an asm CPUID instruction wrapper component,'+CRLF+
    'including decriptor/dispatcher and additional system info getter.'+CRLF+
    'This component is free for educational purpose (commercial notes: '+CRLF+
    'I like money, maybe, but sure not yours, so if this stuff can help'+CRLF+
    'you make some, a greeting post-card will be enough for me ;-))'+
     CRLF2+
    'aa '+ aDadang +',  ' + aMail + CRLF + aCompany + CRLF + aSnail +
    ' - ' + aZIP + CRLF +  aPhone + CRLF + aCountry + CRLF2 + aDSPWarez);
end;

procedure tacpuidshow.About(Sender:TObject); begin
  (Component as tacpuid).About;
end;

procedure TacpuidShow.CacheInfoClick(Sender: TObject);
const
  HEADER = '##'+TAB+'Description'+CRLF;
  TITLE = ' Cache Info';
begin
  MessageDlg (HEADER+ (Component as tacpuid).CacheInfo.Text, mtInformation, [mbOK],0);
end;

procedure TacpuidShow.GlobalInfoClick(Sender: TObject);
const
  HEADER = '';
  TITLE = 'Global Info';
begin
  MessageDlg ((Component as tacpuid).GlobalInfoText, mtInformation, [mbOK],0);
end;

const HEADER = '##'+TAB+'Feature'+TAB2+'Description'+CRLF;

procedure TacpuidShow.AvailableClick(Sender: TObject);
const TITLE = ' Supported Features';
begin
  MessageDlg (HEADER + (Component as tacpuid).FeaturesAvailable([]), mtInformation, [mbOK],0);
end;

procedure TacpuidShow.UnavailableClick(Sender: TObject);
const TITLE = ' Unsupported Features';
begin
  MessageDlg (HEADER + (Component as tacpuid).FeaturesNotAvailable([]), mtInformation, [mbOK],0);
end;

procedure TacpuidShow.ComprehensiveClick(Sender: TObject);
const
  HEADER = '##'+TAB+'Feature'+TAB2+'Avail?'+TAB+'Description'+CRLF;
  TITLE = ' Comprehensive Feature List';
begin
  MessageDlg (HEADER+ (Component as tacpuid).FeatureList.Text, mtInformation, [mbOK],0);
end;



procedure Register; begin
  RegisterComponents('aa', [tacpuid]);
  RegisterComponentEditor(tacpuid, tacpuidShow);
end;

end.



