unit DynaLED;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TDynaLED = class(TGraphicControl)
  private
    { Private declarations }
    TextBitmap: TBitmap;
    LEDBitmap: TBitmap;
    FCapWidth: Integer;
    FCapHeight: Integer;
    FForeColor: TColor;
    FBackColor: TColor;
    FScale: Byte;

    Procedure SetForeColor(const value: TColor);
    Procedure SetBackColor(const value: TColor);
    Procedure SetScale(const value: Byte);
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
  protected
    { Protected declarations }
    Procedure ReDrawLED; virtual;
    procedure Paint; override;
    procedure AdjustSize; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property ForeColor: TColor read FForeColor write SetForeColor default clLime;
    property BackColor: TColor read FBackColor write SetBackColor default clGreen;
    property Scale: Byte read FScale write SetScale default 2;
    property AutoSize default true;
    property Caption;
    property Color;
    property Font;
    property ParentFont;
    property ParentColor;
    property Align;
    property Anchors;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

constructor TDynaLED.Create(AOwner: TComponent);
Begin
  Inherited;

  FForeColor:= clLime;
  FBackColor:= clGreen;
  FScale:= 2;

  LEDBitmap:= TBitmap.Create;
  LEDBitmap.HandleType:= bmDIB;

  TextBitmap:= TBitmap.Create;
  TextBitmap.PixelFormat:= pf1Bit;
  TextBitmap.Monochrome:= True;
  TextBitmap.HandleType:= bmDIB;

  AutoSize:= True;
End;

destructor TDynaLED.Destroy;
Begin
  TextBitmap.Free;
  LEDBitmap.Free;
  Inherited;
End;

Procedure TDynaLED.ReDrawLED;
Var
  x, y: Integer;
  R1, R: TRect;
  w, h: Integer;
Begin
  // Draw Text to TextBitmap
  TextBitmap.Assign(nil);
  TextBitmap.Canvas.Brush.Color:= clWhite;
  TextBitmap.Canvas.Font.Color:= clBlack;
  R1:= Rect(0,0,0,0);
  DrawTextEx(TextBitmap.Canvas.Handle, PChar(Caption), -1, R1, DT_NOCLIP or DT_NOPREFIX or DT_CALCRECT, nil);
  FCapWidth:= R1.Right-R1.Left;
  FCapHeight:= R1.Bottom-R1.Top;
  TextBitmap.Width:=FCapWidth;
  TextBitmap.Height:=FCapHeight;
  DrawTextEx(TextBitmap.Canvas.Handle, PChar(Caption), -1, R1, DT_NOCLIP or DT_NOPREFIX, nil);

  // Make LED from TextBitmap
  LEDBitmap.Assign(nil);
  LEDBitmap.Width:= TextBitmap.Width*(FScale+1);
  LEDBitmap.Height:= TextBitmap.Height*(FScale+1);
  AdjustSize;
  LEDBitmap.Canvas.Brush.Color:= Color;
  LEDBitmap.Canvas.FillRect(Rect(0,0,LEDBitmap.Width,LEDBitmap.Height));
  w:= TextBitmap.Width;
  If w>Width div (FScale+1) then
    w:=Width div (FScale+1);
  h:= TextBitmap.Height;
  If h>Height div (FScale+1) then
    h:=Height div (FScale+1);
  For x:=0 to w-1 do
    For y:=0 to h-1 do
    Begin
      If TextBitmap.Canvas.Pixels[x, y]=clWhite then
        LEDBitmap.Canvas.Brush.Color:= FBackColor
      Else
        LEDBitmap.Canvas.Brush.Color:= FForeColor;

      R:= Rect(x*(FScale+1),y*(FScale+1),x*(FScale+1)+FScale,y*(FScale+1)+FScale);
      LEDBitmap.Canvas.FillRect(R);
    End;
{  R:= Rect(0,0, 200, 50);
  LEDBitmap.Canvas.Font.Assign(Font);
  LEDBitmap.Canvas.Font.Color:= clWhite;
  LEDBitmap.Canvas.Brush.Color:= clRED;
//  LEDBitmap.Canvas.FillRect(R);             //ETO_CLIPPED
  //ExtTextOut(LEDBitmap.Canvas.Handle, 0, 0, ETO_CLIPPED, @R, PChar(Caption), Length(Caption), nil);
  DrawTextEx(LEDBitmap.Canvas.Handle, PChar(Caption), -1, R1, DT_LEFT, nil);
//  DrawTextEx(LEDBitmap.Canvas.Handle, PChar(S), Length(S), R, DT_LEFT, nil);
{}
End;

Procedure TDynaLED.SetForeColor(const value: TColor);
Begin
  If FForeColor<>Value then
  Begin
    FForeColor:= Value;
    ReDrawLED;
    Invalidate;
  End;
End;

Procedure TDynaLED.SetBackColor(const value: TColor);
Begin
  If FBackColor<>Value then
  Begin
    FBackColor:= Value;
    ReDrawLED;
    Invalidate;
  End;
End;

Procedure TDynaLED.SetScale(const value: Byte);
Begin
  If (FScale<>Value) and (Value>1) then
  Begin
    FScale:= Value;
    ReDrawLED;
    Invalidate;
  End;
End;

procedure TDynaLED.CMTextChanged(var Message: TMessage);
Begin
  ReDrawLED;
  Invalidate;
//  AdjustBounds;
End;

procedure TDynaLED.CMFontChanged(var Message: TMessage);
Begin
  TextBitmap.Canvas.Font.Assign(Font);
  ReDrawLED;
  Invalidate;
//  AdjustBounds;
End;

procedure TDynaLED.CMColorChanged(var Message: TMessage);
Begin
  ReDrawLED;
  Invalidate;
End;

procedure TDynaLED.AdjustSize;
Begin
  if not (csLoading in ComponentState) then
  Begin
    If AutoSize then
    Begin
      SetBounds(Left, Top, LEDBitmap.Width, LEDBitmap.Height);
    End Else
      SetBounds(Left, Top, Width, Height);
  End;
End;

procedure TDynaLED.Paint;
Begin
  Canvas.Draw(0,0,LEDBitmap);
  {Canvas.MoveTo(0,0);
  Canvas.LineTo(Width,Height);{}
End;

procedure Register;
begin
  RegisterComponents('Samson', [TDynaLED]);
end;

end.
