{***************************************************************
 *
 * Unit Name : EMHotLabel
 * Purpose   : Hotlink Label
 * Author    : Eugen Mihailescu
 * Company   : EM Quicksoft Romania SRL
 * Copyright : 1998,2002  All rights reserved.
 * Web Site  : http://www.em-quicksoft.com
 * Support   : support@em-quicksoft.com
 * History   : 03/19/2000
 *
 * Disclaimer: The objectives of this program is only educational.
 *             You MUST NOT use it without getting any material benefit on it
 *             You have the right to share this code with others without removing
 *             the copyright and/or disclaimer notice.
 *             You are allowed to include its code in your application
 *             only if it was designed for non commercial purpose,
 *             otherwise we claim $10 for each copy of your
 *             application which contains this code inside.
 ****************************************************************}

Unit EMHotLabel;

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Shellapi,
  MMSystem;

Type
  TEMHotLabel = Class(TLabel)
  private
    FLinkColor: TColor;
    FLinkColorMouseMove: TColor;
    FLinkMagnet: Boolean;
    FLinkSound: Boolean;
  protected
    Procedure Click; override;
    Procedure CMMouseEnter(Var Msg: TMessage); message CM_MOUSEENTER;
    Procedure CMMouseLeave(Var Msg: TMessage); message CM_MOUSELEAVE;
    Procedure CMFontChanged(Var Msg: TMessage); message CM_FONTCHANGED;
    Procedure SetLinkColor(Value: TColor);
  public
    Constructor Create(AOwner: TComponent); override;
  published
    Property LinkColor: TColor read FLinkColor write SetLinkColor;
    Property LinkColorMouseMove: TColor read FLinkColorMouseMove write FLinkColorMouseMove;
    Property LinkMagnet: Boolean read FLinkMagnet write FLinkMagnet;
    Property LinkSound: Boolean read FLinkSound write FLinkSound;
  End;

Procedure Register;

Implementation
//******************* TEMHotLabel.Create *************************

Constructor TEMHotLabel.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  Font.Color := clBlue;
  FLinkColor := Font.Color;
  FLinkColorMouseMove := clRed;
  LinkMagnet := True;
  LinkSound := True;
  Cursor := crHandPoint;
  Transparent := True;
End;
//******************* TEMHotLabel.Click *************************

Procedure TEMHotLabel.Click;
Begin
  If LinkSound Then
    sndPlaySound('MenuPopup', snd_Async + snd_NoDefault);
  Inherited Click;
End;
//******************* TEMHotLabel.CMMouseEnter *************************

Procedure TEMHotLabel.CMMouseEnter(Var Msg: TMessage);
Begin
  Font.Color := FLinkColorMouseMove;
  If LinkMagnet Then
    Begin
      Width := Width + 1;
      Height := Height + 1;
      Top := Top - 1;
      Left := Left - 1;
    End;
  If LinkSound Then
    sndPlaySound('MenuCommand', snd_Async + snd_NoDefault);
  Update;
End;
//******************* TEMHotLabel.CMMouseLeave *************************

Procedure TEMHotLabel.CMMouseLeave(Var Msg: TMessage);
Begin
  Font.Color := FLinkColor;
  If LinkMagnet Then
    Begin
      Width := Width - 1;
      Height := Height - 1;
      Top := Top + 1;
      Left := Left + 1;
    End;
  Update;
End;
//******************* TEMHotLabel.SetLinkColor *************************

Procedure TEMHotLabel.SetLinkColor(Value: TColor);
Begin
  If Value <> FLinkColor Then
    Begin
      FLinkColor := Value;
      Font.Color := FLinkColor;
      Update;
    End;
End;
//******************* TEMHotLabel.CMFontChanged *************************

Procedure TEMHotLabel.CMFontChanged(Var Msg: TMessage);
Begin
  If csDesigning In ComponentState Then
    Begin
      FLinkColor := Font.Color;
      Update;
    End;
End;

Procedure Register;
Begin
  RegisterComponents('EM-Quicksoft', [TEMHotLabel]);
End;

End.

