unit TestMultiTask;

interface

  { A few explanations are given in the code }

  { An interesting experience is to launch Task 1 in the synchronize demo
    once and then launch Task 2 twice ... and then look at the counters ... }

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MultiTasker, ExtCtrls, Buttons, FileCtrl;

type
  TWinTestMultiTasker = class(TForm)
    MultiTasker: TMultiTasker;
    Panel4: TPanel;
    Label6: TLabel;
    Memo1: TMemo;
    Panel5: TPanel;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button10: TButton;
    Button11: TButton;
    Button12: TButton;
    Button13: TButton;
    Panel3: TPanel;
    Label7: TLabel;
    Button7: TButton;
    CheckBox1: TCheckBox;
    Panel2: TPanel;
    Label5: TLabel;
    Button5: TButton;
    Button6: TButton;
    Panel6: TPanel;
    Panel7: TPanel;
    Button9: TButton;
    Button14: TButton;
    Label8: TLabel;
    Memo2: TMemo;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    FileListBox1: TFileListBox;
    SpeedButton1: TSpeedButton;
    procedure MultiTaskerTaskFinished;
    procedure MultiTaskerError;
    procedure MultiTaskerAllTaskFinished;
    procedure Button1Click(Sender: TObject);
    procedure Button10Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button13Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure Button14Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure FileListBox1DblClick(Sender: TObject);
    procedure MultiTaskerTaskLaunched;
    procedure SpeedButton1Click(Sender: TObject);
  private
    FNameCounter: Integer;
    FCounter: Integer;
  public
    procedure CreateAndFill;
  end;

var
  WinTestMultiTasker: TWinTestMultiTasker;

implementation

{$R *.DFM}

{ The TCaptionChanger class is created to show that an object can be dedicated
  to a particular task. It could also have been possible to create several
  procedures in the TForm but each one of them should have handled one TLabel :
     procedure ChangeL1; Label1.Caption:= ...
     procedure ChangeL2; Label2.Caption:= ...  and so on
}

type
  TCaptionChanger = class(TComponent)
  private
    FMultiTasker: TMultiTasker;
    FLabel: TLabel;
    procedure DoChange;
  public
    constructor CustomCreate(MultiTasker: TMultiTasker; Lbl: TLabel);
    {}
    procedure Execute;
  end;

constructor TCaptionChanger.CustomCreate(MultiTasker: TMultiTasker; Lbl: TLabel);
begin
  FMultiTasker:= MultiTasker;
  FLabel:= Lbl;
  inherited Create(MultiTasker);
end;

procedure TCaptionChanger.DoChange;
begin
  FLabel.Caption:= IntToStr(StrToInt(FLabel.Caption) + 1);
end;

procedure TCaptionChanger.Execute;
begin
  while FLabel.Tag = 0 do // We must use Synchronize because the TLabel uses a TCanvas
    FMultiTasker.Synchronize(DoChange);
  FLabel.Tag:= 0;
  Free; // Auto free on termination
end;

procedure TWinTestMultiTasker.CreateAndFill;
var i: Integer;
    F: TFileStream;
    S: string;
    B: Boolean;
    FLast: Integer;
    procedure CheckInterruption;
    begin
      if Abs(FLast - FCounter) > 1 then
      begin
        MultiTasker.LockExecution;
        S:= Format('%sInterrupted ! : %d => %d'#13#10, [S, FLast, FCounter]);
        FLast:= FCounter;
        MultiTasker.UnLockExecution;
      end;
    end;
begin
  { The file created has no signification. However, it can be interesting to
    look at because it shows "when" the thread is interrupted by one running
    the same procedure. By looking at all the files, you can trace the execution
    of all "CreateAndFill" threads }
  B:= CheckBox1.Checked;
  { Try to launch with and without LockExecution. Without, there may appear
    some errors because of concurrency access to FNameCounter }
  if B then
    MultiTasker.LockExecution;
  F:= TFileStream.Create('TestTxt' + IntToStr(FNameCounter) + '.txt', fmCreate or fmShareExclusive);
  try
    Sleep(10); // This will ensure that some errors will appear without LockExecution
    Inc(FNameCounter);
    if B then
      MultiTasker.UnLockExecution;
    for i:= 1 to 15000 do
    begin
      CheckInterruption;
      FLast:= FCounter;
      CheckInterruption;
      S:= FormatFloat(',0', FCounter) + #13#10;
      CheckInterruption;
      Inc(FCounter);
      CheckInterruption;
      F.Write(S[1], Length(S));
      CheckInterruption;
    end;
  finally
    F.Free;
  end;
  MultiTasker.Synchronize(FileListBox1.Update);
end;

procedure TWinTestMultiTasker.MultiTaskerTaskLaunched;
begin
  Memo1.Lines.Add(Format('Task "%s" launched', [MultiTasker.LastTaskId]));
end;

procedure TWinTestMultiTasker.MultiTaskerTaskFinished;
begin
  Memo1.Lines.Add(Format('Task "%s" finished', [MultiTasker.LastTaskId]));
end;

procedure TWinTestMultiTasker.MultiTaskerError;
begin
  Memo1.Lines.Add(Format('Task "%s" interrupted :', [MultiTasker.LastTaskId]));
  Memo1.Lines.Add(MultiTasker.LastError);
end;

procedure TWinTestMultiTasker.MultiTaskerAllTaskFinished;
begin
  Memo1.Lines.Add('All tasks finished !');
end;

procedure TWinTestMultiTasker.Button1Click(Sender: TObject);
var Lbl: TLabel;
    S: string;
    C: TCaptionChanger;
begin // "fun" code : uses the Tag value of the button to know which label is to
      // be used via its name ...
  S:= IntToStr(TComponent(Sender).Tag);
  Lbl:= FindComponent('Label' + S) as TLabel;
  C:= TCaptionChanger.CustomCreate(MultiTasker, Lbl);
  S:= MultiTasker.LaunchTask(C.Execute);
  Memo2.Lines.Add('Sync Task queued : ' + S);
end;

procedure TWinTestMultiTasker.Button4Click(Sender: TObject);
begin
  Button1.Click;
  Button2.Click;
  Button3.Click;
end;

procedure TWinTestMultiTasker.Button13Click(Sender: TObject);
begin
  Button10.Click;
  Button11.Click;
  Button12.Click;
end;

procedure TWinTestMultiTasker.Button7Click(Sender: TObject);
var S: string;
    i: Integer;
begin
  for i:= 1 to 9 do
  begin
    S:= MultiTasker.LaunchTask(CreateAndFill);
    Memo2.Lines.Add('File Task queued : ' + S);
  end;
end;

procedure TWinTestMultiTasker.Button10Click(Sender: TObject);
begin
  (FindComponent('Label' + IntToStr(TComponent(Sender).Tag)) as TLabel).Tag:= 1;
end;

procedure TWinTestMultiTasker.Button5Click(Sender: TObject);
begin
  MultiTasker.SuspendAll;
end;

procedure TWinTestMultiTasker.Button6Click(Sender: TObject);
begin
  MultiTasker.ResumeAll;
end;

procedure TWinTestMultiTasker.FormCreate(Sender: TObject);
begin
  FCounter:= 0;
  FNameCounter:= 0;
  FileListBox1.Directory:= ExtractFilePath(ParamStr(0));
end;

procedure TWinTestMultiTasker.Button9Click(Sender: TObject);
begin
  MultiTasker.GetActiveTasks(Memo2.Lines);
end;

procedure TWinTestMultiTasker.Button14Click(Sender: TObject);
begin
  MultiTasker.GetQueuedTasks(Memo2.Lines);
end;

procedure TWinTestMultiTasker.BitBtn1Click(Sender: TObject);
begin
  Memo1.Clear;
end;

procedure TWinTestMultiTasker.BitBtn2Click(Sender: TObject);
begin
  Memo2.Clear;
end;

procedure TWinTestMultiTasker.FileListBox1DblClick(Sender: TObject);
begin
  WinExec(PChar('Notepad ' + FileListBox1.FileName), SW_SHOW);
end;

procedure TWinTestMultiTasker.SpeedButton1Click(Sender: TObject);
var i: Integer;
begin
  for i:= FileListBox1.Items.Count - 1 downto 0 do
    DeleteFile(ExtractFilePath(ParamStr(0)) + '\' + FileListBox1.Items[i]);
  FileListBox1.Update;
end;

end.
