unit RptUn3;

interface

uses SysUtils, Objects, QStrings, ExcelRpt, DataUnit, WaitUn, MsgOkUn, Db,
  DBTables, Dialogs;

procedure MakeRpt3;

implementation

type
  PRemObj = ^TRemObj;
  TRemObj = record
    BaseID: Integer;
    ProdID: Integer;
    AM_AZS: Integer;
    AM_Dead: Integer;
    AM_Base: Integer;
    AM_Stor: Integer;
  end;

  PIncObj = ^TIncObj;
  TIncObj = record
    BaseID: Integer;
    ProdID: Integer;
    Amount: Integer;
  end;

  PBaseObj = ^TBaseObj;
  TBaseObj = record
    ID: Integer;
    Name: string;
    IsRentNull: Boolean;
  end;

  PProdObj = ^TProdObj;
  TProdObj = record
    ID: Integer;
    Name: string;
    SortOrd: Integer;
  end;

  PColObj = ^TColObj;
  TColObj = record
    IDO: Integer;
    SOrd: Integer;
    NmO: string;
    CIU: Integer;
    UseItg: Boolean;
    UseBase: Boolean;
    UseAZS: Boolean;
    UseStor: Boolean;
    UseDead: Boolean;
    UseOnWay: Boolean;
  end;

  PCD = ^TCD;
  TCD = record
    IsUsed: Boolean;
    ItgVal: Integer;
    Base: Integer;
    AZS: Integer;
    Stor: Integer;
    Dead: Integer;
    OnWay: Integer;
  end;

  PRow = ^TRow;
  TRow = record
    IDB: Integer;
    NmB: string;
    CDs: TVCollection;
    Itg: Boolean;
  end;

var
  Ns: string;
  RC1,RC2: Integer;
  CC: Integer;

  Rows1: TVCollection;
  Rows2: TVCollection;
  Cols: TVCollection;

  Rems: TVCollection;
  Incs: TVCollection;
  Bases: TVCollection;
  Prods: TVCollection;

  RemObj: PRemObj;
  IncObj: PIncObj;
  BaseObj: PBaseObj;
  ProdObj: PProdObj;

procedure LoadData;
begin
  with MyDM.Query1 do
  begin
    Active := False;

    SQL.Clear;
    SQL.Add('SELECT NAME');
    SQL.Add('FROM BAZE_ORG');
    Active := True;
    OrgName := Fields[0].AsString;
    Active := False;

    SQL.Clear;
    SQL.Add('SELECT OIL_BASES_ID, OIL_PRODUCTS_ID, AMOUNT_AZS, AMOUNT_DEAD, AMOUNT_BASE, AMOUNT_STORAGE');
    SQL.Add('FROM BASES_REMAINDER_FACT');
    SQL.Add('WHERE CURR_DATE = '+ FormSQLDate(RsDay,RsMonth,RsYear));
    Rems := TVCollection.CreateRefsList;
    Active := True;
    while not EOF do
    begin
      New(RemObj);
      with RemObj^ do
      begin
        BaseID := Fields[0].AsInteger;
        ProdID := Fields[1].AsInteger;
        AM_AZS := Fields[2].AsInteger;
        AM_Dead := Fields[3].AsInteger;
        AM_Base := Fields[4].AsInteger;
        AM_Stor := Fields[5].AsInteger;
      end;
      Rems.Add(RemObj);
      Next;
    end;
    Active := False;

    SQL.Clear;
    SQL.Add('SELECT OIL_BASES_ID, OIL_PRODUCTS_ID, AMOUNT');
    SQL.Add('FROM INCOME_OIL');
    SQL.Add('WHERE (INCOME_FLG=''N'') AND (DATE_INCOME <= '+
      FormSQLDate(RsDay,RsMonth,RsYear)+')');

    Incs := TVCollection.CreateRefsList;
    Active := True;
    while not EOF do
    begin
      New(IncObj);
      with IncObj^ do
      begin
        BaseID := Fields[0].AsInteger;
        ProdID := Fields[1].AsInteger;
        Amount := Fields[2].AsInteger;
      end;
      Incs.Add(IncObj);
      Next;
    end;
    Active := False;

    SQL.Clear;
    SQL.Add('SELECT ID, NAME, RENTERS_ID');
    SQL.Add('FROM OIL_BASES');
    Bases := TVCollection.CreateRefsList;
    Active := True;
    while not EOF do
    begin
      New(BaseObj);
      with BaseObj^ do
      begin
        ID := Fields[0].AsInteger;
        Name := Fields[1].AsString;
        IsRentNull := Fields[2].IsNull;
      end;
      Bases.Add(BaseObj);
      Next;
    end;
    Active := False;

    SQL.Clear;
    SQL.Add('SELECT ID, NAME, SORT_ORDER');
    SQL.Add('FROM OIL_PRODUCTS');
    Prods := TVCollection.CreateRefsList;
    Active := True;
    while not EOF do
    begin
      New(ProdObj);
      with ProdObj^ do
      begin
        ID := Fields[0].AsInteger;
        Name := Fields[1].AsString;
        SortOrd := Fields[2].AsInteger;
      end;
      Prods.Add(ProdObj);
      Next;
    end;
    Active := False;
  end;
end;

procedure FreeRows(var Rows: TVCollection);
var
  I,J: Integer;
  Rw: PRow;
begin
  for I := 0 to Rows.Count-1 do
  begin
    Rw := Rows.List^[I];
    with Rw^.CDs do
    begin
      for J := 0 to Count-1 do
        Dispose(PCD(List^[J]));
      Free;
    end;
    Finalize(Rw^);
    Dispose(Rw);
  end;
  FreeAndNil(Rows);
end;

procedure FreeData;
var
  I: Integer;
begin
  for I := 0 to Rems.Count-1 do
    Dispose(PRemObj(Rems.List^[I]));
  FreeAndNil(Rems);
  for I := 0 to Incs.Count-1 do
    Dispose(PIncObj(Incs.List^[I]));
  FreeAndNil(Incs);
  for I := 0 to Bases.Count-1 do
  begin
    Finalize(PBaseObj(Bases.List^[I])^);
    Dispose(PBaseObj(Bases.List^[I]));
  end;
  FreeAndNil(Bases);
  for I := 0 to Prods.Count-1 do
  begin
    Finalize(PProdObj(Prods.List^[I])^);
    Dispose(PProdObj(Prods.List^[I]));
  end;
  FreeAndNil(Prods);
  FreeRows(Rows1);
  FreeRows(Rows2);
  for I := 0 to Cols.Count-1 do
  begin
    Finalize(PColObj(Cols.List^[I])^);
    Dispose(PColObj(Cols.List^[I]));
  end;
  FreeAndNil(Cols);
end;

function GetBaseInfo(BaseID: Integer; var IsOilBase: Boolean): string;
var
  I: Integer;
  BO: PBaseObj;
begin
  for I := 0 to Bases.Count-1 do
  begin
    BO := Bases.List^[I];
    if BO^.ID = BaseID then
    begin
      Result := BO^.Name;
      IsOilBase := BO^.IsRentNull;
      Exit;
    end;
  end;
  Result := ' ';
end;

function GetProdInfo(ProdID: Integer; var SOrd: Integer): string;
var
  I: Integer;
  PO: PProdObj;
begin
  for I := 0 to Prods.Count-1 do
  begin
    PO := Prods.List^[I];
    if PO^.ID = ProdID then
    begin
      Result := PO^.Name;
      SOrd := PO^.SortOrd;
      Exit;
    end;
  end;
  Result := ' ';
  SOrd := 0;
end;

function RowCompFunc(Item1, Item2: Pointer): Integer;
begin
  Result := Q_CompText(PRow(Item1)^.NmB,PRow(Item2)^.NmB);
end;

procedure UpdateOstInfo(Rows: TVCollection);
var
  I,J,SO: Integer;
  Rw: PRow;
  MM: PCD;
  CO: PColObj;
  Pn: string;
  A: Boolean;
begin
  A := False;
  Rw := nil;
  for I := 0 to Rows.Count-1 do
  begin
    Rw := Rows.List^[I];
    if Rw^.IDB = RemObj^.BaseID then
    begin
      A := True;
      Break;
    end;
  end;
  if not A then
  begin
    New(Rw);
    with Rw^ do
    begin
      IDB := RemObj^.BaseID;
      NmB := Ns;
      CDs := TVCollection.CreateRefsList;
      Itg := False;
    end;
    for I := 0 to Cols.Count-1 do
    begin
      New(MM);
      with MM^ do
      begin
        IsUsed := False;
        ItgVal := 0;
        Base := 0;
        AZS := 0;
        Stor := 0;
        Dead := 0;
        OnWay := 0;
      end;
      Rw^.CDs.Add(MM);
    end;
    Rows.Add(Rw);
  end;
  Pn := GetProdInfo(RemObj^.ProdID,SO);
  I := 0;
  A := False;
  CO := nil;
  while I < Cols.Count do
  begin
    CO := Cols.List^[I];
    if CO^.IDO = RemObj^.ProdID then
    begin
      A := True;
      Break;
    end;
    if CO^.SOrd > SO then
      Break;
    Inc(I);
  end;
  if not A then
  begin
    New(CO);
    with CO^ do
    begin
      IDO := RemObj^.ProdID;
      SOrd := SO;
      NmO := Pn;
      CIU := 0;
      UseItg := False;
      UseBase := False;
      UseAZS := False;
      UseStor := False;
      UseDead := False;
      UseOnWay := False;
    end;
    Cols.Insert(I,CO);
    for J := 0 to Rows1.Count-1 do
    begin
      New(MM);
      with MM^ do
      begin
        IsUsed := False;
        ItgVal := 0;
        Base := 0;
        AZS := 0;
        Stor := 0;
        Dead := 0;
        OnWay := 0;
      end;
      PRow(Rows1.List^[J])^.CDs.Insert(I,MM);
    end;
    for J := 0 to Rows2.Count-1 do
    begin
      New(MM);
      with MM^ do
      begin
        IsUsed := False;
        ItgVal := 0;
        Base := 0;
        AZS := 0;
        Stor := 0;
        Dead := 0;
        OnWay := 0;
      end;
      PRow(Rows2.List^[J])^.CDs.Insert(I,MM);
    end;
  end;
  MM := Rw^.CDs.List^[I];
  with MM^, CO^ do
  begin
    IsUsed := True;
    Base := Base+Round(RemObj^.AM_Base*0.001);
    AZS := AZS+Round(RemObj^.AM_AZS*0.001);
    Stor := Stor+Round(RemObj^.AM_Stor*0.001);
    Dead := Dead+Round(RemObj^.AM_Dead*0.001);
    if (Base<>0) and not UseBase then
    begin
      UseBase := True;
      Inc(CIU);
    end;
    if (AZS<>0) and not UseAZS then
    begin
      UseAZS := True;
      Inc(CIU);
    end;
    if (Stor<>0) and not UseStor then
    begin
      UseStor := True;
      Inc(CIU);
    end;
    if (Dead<>0) and not UseDead then
    begin
      UseDead := True;
      Inc(CIU);
    end;
    Inc(ItgVal,Base+AZS+Stor+Dead);
    if (CIU>1) and not UseItg then
    begin
      UseItg := True;
      Inc(CIU);
    end;
  end;
end;

procedure UpdateIncInfo(Rows: TVCollection);
var
  I,J,SO: Integer;
  Rw: PRow;
  MM: PCD;
  CO: PColObj;
  Pn: string;
  A: Boolean;
begin
  A := False;
  Rw := nil;
  for I := 0 to Rows.Count-1 do
  begin
    Rw := Rows.List^[I];
    if Rw^.IDB = IncObj^.BaseID then
    begin
      A := True;
      Break;
    end;
  end;
  if not A then
  begin
    New(Rw);
    with Rw^ do
    begin
      IDB := IncObj^.BaseID;
      NmB := Ns;
      CDs := TVCollection.CreateRefsList;
      Itg := False;
    end;
    for I := 0 to Cols.Count-1 do
    begin
      New(MM);
      with MM^ do
      begin
        IsUsed := False;
        ItgVal := 0;
        Base := 0;
        AZS := 0;
        Stor := 0;
        Dead := 0;
        OnWay := 0;
      end;
      Rw^.CDs.Add(MM);
    end;
    Rows.Add(Rw);
  end;
  Pn := GetProdInfo(IncObj^.ProdID,SO);
  I := 0;
  A := False;
  CO := nil;
  while I < Cols.Count do
  begin
    CO := Cols.List^[I];
    if CO^.IDO = IncObj^.ProdID then
    begin
      A := True;
      Break;
    end;
    if CO^.SOrd > SO then
      Break;
    Inc(I);
  end;
  if not A then
  begin
    New(CO);
    with CO^ do
    begin
      IDO := IncObj^.ProdID;
      SOrd := SO;
      NmO := Pn;
      CIU := 0;
      UseItg := False;
      UseBase := False;
      UseAZS := False;
      UseStor := False;
      UseDead := False;
      UseOnWay := False;
    end;
    Cols.Insert(I,CO);
    for J := 0 to Rows1.Count-1 do
    begin
      New(MM);
      with MM^ do
      begin
        IsUsed := False;
        ItgVal := 0;
        Base := 0;
        AZS := 0;
        Stor := 0;
        Dead := 0;
        OnWay := 0;
      end;
      PRow(Rows1.List^[J])^.CDs.Insert(I,MM);
    end;
    for J := 0 to Rows2.Count-1 do
    begin
      New(MM);
      with MM^ do
      begin
        IsUsed := False;
        ItgVal := 0;
        Base := 0;
        AZS := 0;
        Stor := 0;
        Dead := 0;
        OnWay := 0;
      end;
      PRow(Rows2.List^[J])^.CDs.Insert(I,MM);
    end;
  end;
  MM := Rw^.CDs.List^[I];
  with MM^, CO^ do
  begin
    IsUsed := True;
    OnWay := OnWay+Round(IncObj^.Amount*0.001);
    if (OnWay<>0) and not UseOnWay then
      UseOnWay := True;
  end;
end;

procedure AddItgLine(Rows: TVCollection);
var
  NRw: PRow;
  NMM,MM: PCD;
  I,J: Integer;
begin
  New(NRw);
  with NRw^ do
  begin
    Pointer(NmB) := nil;
    CDs := TVCollection.CreateRefsList;
    Itg := True;
  end;
  for I := 0 to Cols.Count-1 do
  begin
    New(NMM);
    with NMM^ do
    begin
      IsUsed := False;
      ItgVal := 0;
      Base := 0;
      AZS := 0;
      Stor := 0;
      Dead := 0;
      OnWay := 0;
      for J := 0 to Rows.Count-1 do
      begin
        MM := PRow(Rows.List^[J])^.CDs.List^[I];
        if MM^.IsUsed then
        begin
          IsUsed := True;
          Inc(ItgVal,MM^.ItgVal);
          Inc(Base,MM^.Base);
          Inc(AZS,MM^.AZS);
          Inc(Stor,MM^.Stor);
          Inc(Dead,MM^.Dead);
          Inc(OnWay,MM^.OnWay);
        end;
      end;
    end;
    NRw^.CDs.Add(NMM);
  end;
  Rows.Add(NRw);
end;

procedure MakeRpt3;
var
  I,J,K,Q: Integer;
  V1,V2: Variant;
  IsOilBase: Boolean;
  ItgRw,R1,R2: PRow;
  CO: PColObj;
  MM,NMM: PCD;
begin
  with WaitForm do
  begin
    Show;
    Update;
  end;
  LoadData;

  Rows1 := TVCollection.CreateRefsList;
  Rows2 := TVCollection.CreateRefsList;
  Cols := TVCollection.CreateRefsList;
  if Rems.Count = 0 then
  begin
    WaitForm.Hide;
    Msg('      .');
    FreeData;
    Exit;
  end;
  for I := 0 to Rems.Count-1 do
  begin
    RemObj := Rems.List^[I];
    Ns := GetBaseInfo(RemObj^.BaseID,IsOilBase);
    if IsOilBase then
      UpdateOstInfo(Rows1)
    else
      UpdateOstInfo(Rows2);
  end;
  for I := 0 to Incs.Count-1 do
  begin
    IncObj := Incs.List^[I];
    Ns := GetBaseInfo(IncObj^.BaseID,IsOilBase);
    if IsOilBase then
      UpdateIncInfo(Rows1)
    else
      UpdateIncInfo(Rows2);
  end;
  Rows1.Sort(RowCompFunc);
  Rows2.Sort(RowCompFunc);
  CC := 0;
  for I := 0 to Cols.Count-1 do
    with PColObj(Cols.List^[I])^ do
    begin
      Inc(CC,CIU);
      if UseOnWay then
        Inc(CC);
    end;
  Inc(CC);
  AddItgLine(Rows1);
  AddItgLine(Rows2);

  New(ItgRw);
  with ItgRw^ do
    CDs := TVCollection.CreateRefsList;
  R1 := Rows1.List^[Rows1.Count-1];
  R2 := Rows2.List^[Rows2.Count-1];
  for I := 0 to Cols.Count-1 do
  begin
    New(NMM);
    with NMM^ do
    begin
      MM := R1^.CDs.List^[I];
      ItgVal := MM^.ItgVal;
      Base := MM^.Base;
      AZS := MM^.AZS;
      Stor := MM^.Stor;
      Dead := MM^.Dead;
      OnWay := MM^.OnWay;
      MM := R2^.CDs.List^[I];
      Inc(ItgVal,MM^.ItgVal);
      Inc(Base,MM^.Base);
      Inc(AZS,MM^.AZS);
      Inc(Stor,MM^.Stor);
      Inc(Dead,MM^.Dead);
      Inc(OnWay,MM^.OnWay);
    end;
    ItgRw^.CDs.Add(NMM);
  end;

  RC1 := Rows1.Count+1;
  RC2 := Rows2.Count+3;
  V1 := VarArrayCreate([1,RC1,1,CC], varVariant);
  V2 := VarArrayCreate([1,RC2,1,CC], varVariant);

  for I := 1 to CC do
  begin
    V1[1,I] := I;
    V2[1,I] := I;
  end;

  K := 2;
  for I := 0 to Rows1.Count-1 do
  begin
    R1 := Rows1.List^[I];
    V1[K,1] := R1^.NmB;
    Q := 2;
    for J := 0 to Cols.Count-1 do
    begin
      CO := Cols.List^[J];
      MM := R1^.CDs.List^[J];
      if not MM^.IsUsed then
      begin
        Inc(Q,CO^.CIU);
        if CO^.UseOnWay then
          Inc(Q);
        Continue;
      end else
      begin
        if CO^.UseItg then
        begin
          V1[K,Q] := MM^.ItgVal;
          Inc(Q);
        end;
        if CO^.UseBase then
        begin
          V1[K,Q] := MM^.Base;
          Inc(Q);
        end;
        if CO^.UseAZS then
        begin
          V1[K,Q] := MM^.AZS;
          Inc(Q);
        end;
        if CO^.UseStor then
        begin
          V1[K,Q] := MM^.Stor;
          Inc(Q);
        end;
        if CO^.UseDead then
        begin
          V1[K,Q] := MM^.Dead;
          Inc(Q);
        end;
        if CO^.UseOnWay then
        begin
          V1[K,Q] := MM^.OnWay;
          Inc(Q);
        end;
      end;
    end;
    Inc(K);
  end;

  K := 2;
  for I := 0 to Rows2.Count-1 do
  begin
    R2 := Rows2.List^[I];
    V2[K,1] := R2^.NmB;
    Q := 2;
    for J := 0 to Cols.Count-1 do
    begin
      CO := Cols.List^[J];
      MM := R2^.CDs.List^[J];
      if not MM^.IsUsed then
      begin
        Inc(Q,CO^.CIU);
        if CO^.UseOnWay then
          Inc(Q);
        Continue;
      end else
      begin
        if CO^.UseItg then
        begin
          V2[K,Q] := MM^.ItgVal;
          Inc(Q);
        end;
        if CO^.UseBase then
        begin
          V2[K,Q] := MM^.Base;
          Inc(Q);
        end;
        if CO^.UseAZS then
        begin
          V2[K,Q] := MM^.AZS;
          Inc(Q);
        end;
        if CO^.UseStor then
        begin
          V2[K,Q] := MM^.Stor;
          Inc(Q);
        end;
        if CO^.UseDead then
        begin
          V2[K,Q] := MM^.Dead;
          Inc(Q);
        end;
        if CO^.UseOnWay then
        begin
          V2[K,Q] := MM^.OnWay;
          Inc(Q);
        end;
      end;
    end;
    Inc(K);
  end;
  Inc(K);

  Q := 2;
  for J := 0 to Cols.Count-1 do
  begin
    CO := Cols.List^[J];
    MM := ItgRw^.CDs.List^[J];
    if CO^.UseItg then
    begin
      V2[K,Q] := MM^.ItgVal;
      Inc(Q);
    end;
    if CO^.UseBase then
    begin
      V2[K,Q] := MM^.Base;
      Inc(Q);
    end;
    if CO^.UseAZS then
    begin
      V2[K,Q] := MM^.AZS;
      Inc(Q);
    end;
    if CO^.UseStor then
    begin
      V2[K,Q] := MM^.Stor;
      Inc(Q);
    end;
    if CO^.UseDead then
    begin
      V2[K,Q] := MM^.Dead;
      Inc(Q);
    end;
    if CO^.UseOnWay then
    begin
      V2[K,Q] := MM^.OnWay;
      Inc(Q);
    end;
  end;

  if Rpt <> nil then
    Rpt.OpenNewFile(ReportsPath+Rpt03FileName)
  else
    Rpt := TExcelReport.OpenFile(ReportsPath+Rpt03FileName);
  if Rpt.ExcelWasFound then
    with Rpt do
    begin
      DisplayGridLines := False;
      DisplayPageBreaks := True;
      DisplayZeros := False;
      WinCaption1 := '   ';
      WinCaption2 := ' '+OrgName;

      FontSize := 8;
      V1[RC1,1] := Cells[10,2];
      V2[RC2-2,1] := Cells[17,2];
      V2[RC2,1] := Cells[19,2];

      Cells[1,2] := OrgName;
      Cells[2,2] := '     '+FormatDateTime('dd.mm.yyyy',RsDate)+' .';
      if CC > 23 then
        I := 23
      else
        I := CC;
      MergeRowCells(2,2,5,1+I);
      FontUnderline := xlUnderlineStyleSingle;
      FormatRange(2,2,2,1+I,frFontUnderline);
      FontUnderline := xlUnderlineStyleNone;
      AlignHorizontal(2,2,5,1+I,xlHAlignCenter);
      MergeRowCells(12,2,12,1+I);
      AlignHorizontal(12,2,12,1+I,xlHAlignCenter);

      Q := 3;
      for I := 0 to Cols.Count-1 do
      begin
        CO := Cols.List^[I];
        with CO^ do
        begin
          if UseOnWay then
            J := 1
          else
            J := 0;
          MergeRowCells(6,Q,6,Q+CIU+J-1);
          Cells[6,Q] := NmO;
          MergeRowCells(13,Q,13,Q+CIU+J-1);
          Cells[13,Q] := NmO;
          MergeAllCells(7,Q,8,Q);
          MergeAllCells(14,Q,15,Q);
          if UseItg then
          begin
            Cells[7,Q] := '';
            Cells[14,Q] := '';
            Inc(Q);
            MergeRowCells(7,Q,7,Q+CIU-2);
            Cells[7,Q] := '  ';
            MergeRowCells(14,Q,14,Q+CIU-2);
            Cells[14,Q] := '  ';
            if UseBase then
            begin
              Cells[8,Q] := '/';
              Cells[15,Q] := '/';
              Inc(Q);
            end;
            if UseAZS then
            begin
              Cells[8,Q] := '';
              Cells[15,Q] := '';
              Inc(Q);
            end;
            if UseStor then
            begin
              Cells[8,Q] := '.';
              Cells[15,Q] := '.';
              Inc(Q);
            end;
            if UseDead then
            begin
              Cells[8,Q] := '..';
              Cells[15,Q] := '..';
              Inc(Q);
            end;
          end else
          begin
            if UseBase then
            begin
              Cells[7,Q] := '/';
              Cells[14,Q] := '/';
            end
            else if UseAZS then
            begin
              Cells[7,Q] := '';
              Cells[14,Q] := '';
            end
            else if UseStor then
            begin
              Cells[7,Q] := '.';
              Cells[14,Q] := '.';
            end
            else if UseDead then
            begin
              Cells[7,Q] := '. ';
              Cells[14,Q] := '. ';
            end;
            Inc(Q);
          end;
          if UseOnWay then
          begin
            MergeAllCells(7,Q,8,Q);
            Cells[7,Q] := ' ';
            MergeAllCells(14,Q,15,Q);
            Cells[14,Q] := ' ';
            Inc(Q);
          end;
        end;
      end;

      if Rows2.Count > 1 then
      begin
        InsertRows(17,RC2-4);
        AlignHorizontal(17,2,12+RC2,2,xlHAlignLeft);
      end;
      CreateTable(V2,16,2,False);
      AlignHorizontal(13,2,16,CC+1,xlHAlignCenter);
      AlignVertical(13,2,16,CC+1,xlVAlignTop);
      DrawAllFrames(13,2,15+RC2,1+CC);

      InteriorColorIndex := 15;
      InteriorPattern := Integer($FFFFEFE3);
      InteriorPatternColorIndex := 2;
      FormatRange(13+RC2,2,13+RC2,CC+1,frInterior);
      FormatRange(15+RC2,2,15+RC2,CC+1,frInterior);

      if Rows1.Count > 1 then
      begin
        InsertRows(10,RC1-2);
        AlignHorizontal(10,2,7+RC1,2,xlHAlignLeft);
      end;
      CreateTable(V1,9,2,False);
      AlignHorizontal(6,2,9,CC+1,xlHAlignCenter);
      AlignVertical(6,2,9,CC+1,xlVAlignTop);

      DrawAllFrames(6,2,8+RC1,1+CC);
      FormatRange(8+RC1,2,8+RC1,CC+1,frInterior);

//      AutoFitColumns(2,CC+1);

      ProtectSheet('');
      ToggleScrollLockOn;
      Visible := True;
    end;

  FreeData;
  with ItgRw^.CDs do
  begin
    for I := 0 to Count-1 do
      Dispose(PCD(List^[I]));
    Free;
  end;
  Dispose(ItgRw);

  WaitForm.Hide;
end;

end.

