{
 Project UML Component Factory
 Unit PackageUtilities

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with the
 License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 the specific language governing rights and limitations under the License.

 The Original Code is PackageUtilities.pas

 The Initial Developer of the Original Code is Thierry.Coq
 Portions created by this individual are Copyright (C)
 2000 of this individual.
 Other pieces of this code is
 Copyright (C) 1995,99 Inprise Corporation where indicated

 Last modified: 25 oct 2000
 Version 1.0

}
unit PackageUtilities;

interface

uses SysUtils, Classes;

procedure GetListOfUnits( const packageName : String; emptyList : TStrings);

function CapitalizeFirstLetterOnly( aName : String) : String;

implementation

uses Windows;

type
  { Extracted from SysUtils.pas, used for understanding packages. Delphi 5 Only}

  PPkgName = ^TPkgName;
  TPkgName = packed record
    HashCode: Byte;
    Name: array[0..255] of Char;
  end;
  { Package flags:
    bit     meaning
    -----------------------------------------------------------------------------------------
    0     | 1: never-build                  0: always build
    1     | 1: design-time only             0: not design-time only      on => bit 2 = off
    2     | 1: run-time only                0: not run-time only         on => bit 1 = off
    3     | 1: do not check for dup units   0: perform normal dup unit check
    4..25 | reserved
    26..27| (producer) 0: pre-V4, 1: undefined, 2: c++, 3: Pascal
    28..29| reserved
    30..31| 0: EXE, 1: Package DLL, 2: Library DLL, 3: undefined
  }
  PPackageInfoHeader = ^TPackageInfoHeader;
  TPackageInfoHeader = packed record
    Flags: DWORD;
    RequiresCount: Integer;
    {Requires: array[0..9999] of TPkgName;
    ContainsCount: Integer;
    Contains: array[0..9999] of TUnitName;}
  end;

  PUnitName = ^TUnitName;
  TUnitName = packed record
    Flags : Byte;
    HashCode: Byte;
    Name: array[0..255] of Char;
  end;


var PackageHandle : Hinst;
    InfoTable: PPackageInfoHeader;
    ModuleFlags: DWORD;
    ModuleName: string;
    PkgName: PPkgName;

function GetModuleName(Module: HMODULE): string;
var
  ModName: array[0..MAX_PATH] of Char;
begin
  SetString(Result, ModName, Windows.GetModuleFileName(Module, ModName, SizeOf(ModName)));
end;


function PackageInfoTable(Module: HMODULE): PPackageInfoHeader;
var
  ResInfo: HRSRC;
  Data: THandle;
begin
  Result := nil;
  ResInfo := FindResource(Module, 'PACKAGEINFO', RT_RCDATA);
  if ResInfo <> 0 then
  begin
    Data := LoadResource(Module, ResInfo);
    if Data <> 0 then
    try
      Result := LockResource(Data);
      UnlockResource(Data);
    finally
      FreeResource(Data);
    end;
  end;
end;

procedure GetListOfUnits( const packageName : String; emptyList : TStrings);
var
  Count    : Integer;
  I        : Integer;
  UnitName : PUnitName;
  ShortPackageName : String;
begin
  if not assigned( emptyList) then Exit;
  emptyList.Clear;
  ShortPackageName := ChangeFileExt(ExtractFileName(packageName), '');
  PackageHandle := GetModuleHandle( PChar( packageName));
  InfoTable := PackageInfoTable( PackageHandle);
  if (InfoTable <> nil) and (InfoTable.Flags and pfModuleTypeMask = pfPackageModule) then
  begin
    if ModuleFlags = 0 then ModuleFlags := InfoTable.Flags;
    ModuleName := ChangeFileExt(ExtractFileName(GetModuleName(PackageHandle)), '');
    PkgName := PPkgName(Integer(InfoTable) + SizeOf(InfoTable^));

    // going through the 'requires' list.
    Count := InfoTable.RequiresCount;
    for I := 0 to Count - 1 do
    begin
      //emptyList.Add(  PkgName^.Name + ' (required)');
      Inc(Integer(PkgName), StrLen(PkgName.Name) + 2);
    end;

    // going through the 'contains' list
    Count    := Integer(Pointer(PkgName)^);
    UnitName := PUnitName(Integer(PkgName) + 4);
    for I := 0 to Count-1 do
    begin
      if lowerCase( UnitName^.Name) <> LowerCase( ShortPackageName) then
        emptyList.add( UnitName^.Name);
      Inc(Integer(UnitName), StrLen(UnitName.Name) + 3);
    end;
  end;
end;

function CapitalizeFirstLetterOnly( aName : String) : String;
begin
  result := UpperCase( aName[1]) + LowerCase( Copy(aName, 2, length( aName)-1));
end;

end.
