{ TPropsSaver and TLayoutManager Version 1.3
  Jos Sebastin Battig
  E-Mail: k2xt@iname.com

  History:

  Version 1.0: First relesed version
  Version 1.1: Added support for registering non-published properties using the
               procedure RegisterNonPublishedProperty. Also added the methods PropItemToString,
               and StringToPropItem, both methods support looking for non-published
               properties. Actually now TranslateToString and AssignFromString methods
               of the class TLoadSavePerformer use both methods. Encrypt and Decrypt
               delete from TLoadSavePerformer, you have to use them directly now from
               the unit that declares them. Added a sample of how to use the new feature
               of non-published properties in the unit PropsSaverNonPublishedProps. This
               unit declares the property ItemIndex for all descendant of TCustomComboBox,
               that's why now is suggested to don't use the the ComboBox Loader, make
               more sense to select the *ItemIndex property in the editor. I strongly suggest
               to use the non-published properties approach instead of the class loader. I let
               the TStringGrid sample as it's just for learning reasons, but also will be
               nice to declare both properties ColWidths and RowHeigths for TStringGrid
  Version 1.2: Now the component editor supports showing the subproperties of object
               properties. Added TDynPropertiesHolder component.
  Version 1.3: Added function RegisterSubproperties. This function allows the user to
               register a function to retrieve subproperties to be shown for a given
               class in the property item. This is very useful to selecting what properties
               the user wants to save for each item in a collection. In fact, the collection
               translator looks for defined subproperties, if no subproperties are found,
               it saves all properties for each item in the collection otherwise will save
               only the selected properties.
               Added the property ForceUseTranslator in the TPropInfoItem class. When this
               property is true, the related property is saved using the class translator
               instead of saving each subproperty separately. This is useful when saving
               collections, and the user only wants to save certain properties for each
               collection item.
               The function RegisterNonPublishedProperty now allows to receive two procedure
               pointers to functions intended to retrieve subproperties to be used at design time.
               This is useful in cases where the use registers a non published subproperties that
               is a collection, and in fact the user wants to choose what properties of each
               collection item he wants to save.
               Added functionality to paste a TIniPropSav component using a function
               enabled in the contextual menu of the TPropsSaver object.}

{$Define UseBasicLoadSavePerformers}
{$Define UseBasicClassTranslators}
{$Define UseGridSaverLoader}
{Define UseComboBoxLoader} // Don't recommended to use this if use the directive bellow
{$Define UseNonPublishedProps} // This declares the non-published property ItemIndex for TCustomComboBox
{$i UseDynProps.inc}

unit PropsSaver;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, TypInfo,
  TwoColsIntf, TwoColsComponent;

type
  TPropsSaver = class;
  TPropInfoItem = class;
  TCompInfoItem = class;
  TLoadSavePerformer = class;
  TCompInfoCollection = class;
  TCompInfoCollectionSubProp = class;

  TAutoLoad = (aldNone, aldBeforeOnCreateForm, aldBeforeOnShowForm, aldAfterOnCreateForm, aldAfterOnShowForm);
  TAutoSave = (asvNone, asvBeforeOnDestroyForm, asvBeforeOnHideForm, asvAfterOnDestroyForm, asvAfterOnHideForm);
  TPropertyType = (ptAny, ptProperties, ptMethods);
  TOpenMode = (omLoad, omSave);
  TLoadSaveMode = (lsmLoad, lsmSave);
  TLoadSaveModeSet = set of TLoadSaveMode;
  TPropsSaverErrors = (psePropertyNotFound, psePropertyAssignError, psePropertyReadError);

  TFromObjectToStringFunction = function (Obj : TObject; PropItem : TPropInfoItem = nil) : string;
  TFromStringToObjectProcedure = procedure (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil);
  TFromNonPublishedPropertyToString = function (Obj : TObject; const PropertyName : string; PropItem : TPropInfoItem = nil) : string;
  TFromStringToNonPublishedProperty = procedure (Obj : TObject; const PropertyName, Value : string; PropItem : TPropInfoItem = nil);
  TGetNonPublishedPropertyReference = function (Obj : TObject; const PropertyName : string) : TObject;
  TGetPropertySubProps = procedure (Obj : TObject; const PropertyName : string; List : TStrings);
  TGetObjectSubProps = procedure (Obj : TObject; List : TStrings);
  TComponentSaverProcedure = procedure (CompInfoItem : TCompInfoItem; Saver : TLoadSavePerformer);
  TComponentLoaderProcedure = procedure (CompInfoItem : TCompInfoItem; Loader : TLoadSavePerformer);

  TPropsSaverCollection = class (TCollection)
  public
    IsSubProp : Boolean;
  end;

  TPropsSaverCollectionItem = class (TTwoColsCollectionItem)
  private
    function GetPropsSaver : TPropsSaver;
  protected
    function AllowNotification : Boolean; override;
  public
    property PropsSaver : TPropsSaver read GetPropsSaver;
  end;

  TPropInfoCollection = class (TPropsSaverCollection)
  private
    FCompInfoItem : TCompInfoItem;
    function GetItems (Index : Integer) : TPropInfoItem;
    procedure SetItems (Index : Integer; Item : TPropInfoItem);
  protected
    function GetOwner : TPersistent; override;
  public
    function Add : TPropInfoItem;
    property CompInfoItem : TCompInfoItem read FCompInfoItem;
    property Items [Index : Integer] : TPropInfoItem read GetItems Write SetItems; default;
  end;

  TPropInfoItemClass = class of TPropInfoItem;
  TPropInfoItem = class (TPropsSaverCollectionItem)
  private
    FRootKey : HKEY;
    FBaseKey : string;
    FItemName : string;
    FOldBaseKey : string;
    FOldItemName : string;
    FPropertyName : string;
    FPropertyType : TPropertyType;
    FIfClassStoreAsLink : Boolean;
    FMode : TLoadSaveModeSet;
    FEncrypted : Boolean;
    FForceUseTranslator : Boolean;
    FCompInfoItemSubProps : TCompInfoItem;
    FCompInfoCollection : TCompInfoCollectionSubProp;
    FBeforeLoadingProperty,
    FAfterLoadingProperty,
    FBeforeSavingProperty,
    FAfterSavingProperty,
    FOnPropertyNoFoundError : TNotifyEvent;
    function GetComponent : TObject;
    procedure SetPropertyName (Value : string);
    function GetCompInfoItem : TCompInfoItem;
    function GetSubProperties : TPropInfoCollection;
    procedure SetSubProperties (Value : TPropInfoCollection);
    procedure NotifyComponentChanged;
    procedure SetBaseKey (const Value : string);
    procedure SetRootKey (Value : HKEY);
    procedure SetItemName (const Value : string);
    function GetCollection : TPropInfoCollection;
    procedure BindComponent;
  protected
    function GetTwoColsComponent : TTwoColsComponent; override;
    function GetDisplayName: string; override;
    procedure ReplaceWildcards (const wildcards, replacements : array of string); virtual;
    procedure RestoreWildcards; virtual;
  public
    constructor Create (ACollection : TCollection); override;
    destructor Destroy; override;
    procedure Save (Saver : TLoadSavePerformer);
    procedure Load (Loader : TLoadSavePerformer);
    procedure Assign (Source : TPersistent); override;
    property Component : TObject read GetComponent;
    property CompInfoItem : TCompInfoItem read GetCompInfoItem;
    property Collection : TPropInfoCollection read GetCollection;
    property CompInfoItemSubProps : TCompInfoItem read FCompInfoItemSubProps; // declared public for internal use
  published
    property RootKey : HKEY read FRootKey Write SetRootKey default HKEY_CURRENT_USER;
    property BaseKey : string read FBaseKey Write SetBaseKey;
    property ItemName : string read FItemName Write SetItemName;
    property PropertyType : TPropertyType read FPropertyType Write FPropertyType default ptProperties;
    property PropertyName : string read FPropertyName Write SetPropertyName;
    property IfClassStoreAsLink : Boolean read FIfClassStoreAsLink Write FIfClassStoreAsLink default True;
    property Mode : TLoadSaveModeSet read FMode Write FMode default [lsmLoad, lsmSave];
    property Encrypted : Boolean read FEncrypted Write FEncrypted default false;
    property ForceUseTranslator : Boolean read FForceUseTranslator Write FForceUseTranslator default False;
    property SubProperties : TPropInfoCollection read GetSubProperties Write SetSubProperties;
    property BeforeLoadingProperty : TNotifyEvent read FBeforeLoadingProperty write FBeforeLoadingProperty;
    property AfterLoadingProperty : TNotifyEvent read FAfterLoadingProperty write FAfterLoadingProperty;
    property BeforeSavingProperty : TNotifyEvent read FBeforeSavingProperty write FBeforeSavingProperty;
    property AfterSavingProperty : TNotifyEvent read FAfterSavingProperty write FAfterSavingProperty;
    property OnPropertyNoFoundError : TNotifyEvent read FOnPropertyNoFoundError Write FOnPropertyNoFoundError;
  end;

  TCompInfoCollection = class (TPropsSaverCollection)
  private
    FPropsSaver : TPropsSaver;
    function GetItems (Index : Integer) : TCompInfoItem;
    procedure SetItems (Index : Integer; Item : TCompInfoItem);
  protected
    function GetOwner : TPersistent; override;
  public
    function Add : TCompInfoItem;
    property PropsSaver : TPropsSaver read FPropsSaver;
    property Items [Index : Integer] : TCompInfoItem read GetItems Write SetItems; default;
  end;

  TCompInfoCollectionSubProp = class (TCompInfoCollection)
  private
    FPropInfoItem : TPropInfoItem;
  protected
    function GetOwner : TPersistent; override;
  public
    property PropInfoItem : TPropInfoItem read FPropInfoItem;
  end;

  TCompInfoItemClass = class of TCompInfoItem;
  TCompInfoItem = class (TPropsSaverCollectionItem)
  private
    FRootKey : HKEY;
    FBaseKey : string;
    FItemName : string;
    FOldBaseKey : string;
    FOldItemName : string;
    FProperties : TPropInfoCollection;
    FComponent : TObject;
    FBeforeLoadingProperties,
    FAfterLoadingProperties,
    FBeforeSavingProperties,
    FAfterSavingProperties : TNotifyEvent;
    procedure SetProperties (Value : TPropInfoCollection);
    procedure SetComponent (Value : TObject);
    procedure SetItemName (Value : string);
    function GetCollection : TCompInfoCollection;
  protected
    function GetTwoColsComponent : TTwoColsComponent; override;
    function GetDisplayName: string; override;
    procedure ReplaceWildcards (const wildcards, replacements : array of string); virtual;
    procedure RestoreWildcards; virtual;
  public
    constructor Create (Collection: TCollection); override;
    destructor Destroy; override;
    procedure SaveProperties (Saver : TLoadSavePerformer);
    procedure LoadProperties (Loader : TLoadSavePerformer);
    procedure Assign (Source : TPersistent); override;
    property Collection : TCompInfoCollection read GetCollection;
  published
    property RootKey : HKEY read FRootKey Write FRootKey default HKEY_CURRENT_USER;
    property BaseKey : string read FBaseKey Write FBaseKey;
    property ItemName : string read FItemName Write SetItemName;
    property Component : TObject read FComponent Write SetComponent;
    property Properties : TPropInfoCollection read FProperties Write SetProperties;
    property BeforeLoadingProperties : TNotifyEvent read FBeforeLoadingProperties write FBeforeLoadingProperties;
    property AfterLoadingProperties : TNotifyEvent read FAfterLoadingProperties write FAfterLoadingProperties;
    property BeforeSavingProperties : TNotifyEvent read FBeforeSavingProperties write FBeforeSavingProperties;
    property AfterSavingProperties : TNotifyEvent read FAfterSavingProperties write FAfterSavingProperties;
  end;

  TLoadSavePerformerClass = class of TLoadSavePerformer;
  TPropsSaver = class(TTwoColsComponent)
  private
    FAutoLoad : TAutoLoad;
    FAutoSave : TAutoSave;
    FComponents : TCompInfoCollection;
    FRootKey : HKEY;
    FBaseKey : string;
    FFileName : TFileName;
    FOldBaseKey : string;
    FOldFileName : TFileName;
    FIgnoreErrors : Boolean;
    FLayoutId : string;
    FBeforeLoadingProperties,
    FAfterLoadingProperties,
    FBeforeSavingProperties,
    FAfterSavingProperties : TNotifyEvent;
    FLoadSavePerformerClassName : string;
    FLoadSavePerformerclass : TLoadSavePerformerClass;
    FOldLoad, FOldSave : TNotifyEvent;
    procedure SetComponents (Value : TCompInfoCollection);
    procedure SetLoadSavePerformerClassName (Value : string);
    procedure AutoLoad (Sender : TObject);
    procedure AutoSave (Sender : TObject);
    procedure SetAutoLoadProperties (Value : TAutoLoad);
    procedure SetAutoSaveProperties (Value : TAutoSave);
  protected
    procedure Notification (AComponent: TComponent; Operation: TOperation); override;
    property LoadSavePerformerClass : TLoadSavePerformerClass read FLoadSavePerformerClass;
    procedure ReplaceWildcards (const wildcards, replacements : array of string); virtual;
    procedure RestoreWildcards; virtual;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
    procedure SaveProperties; virtual;
    procedure LoadProperties; virtual;
    class function LoadSavePerformers : TList;
    class function PropsSavers : TList;
  published
    property AutoLoadProperties : TAutoLoad read FAutoLoad Write SetAutoLoadProperties default aldNone;
    property AutoSaveProperties : TAutoSave read FAutoSave Write SetAutoSaveProperties default asvNone;
    property Components : TCompInfoCollection read FComponents Write SetComponents;
    property RootKey : HKEY read FRootKey Write FRootKey default HKEY_CURRENT_USER;
    property BaseKey : string read FBaseKey Write FBaseKey;
    property FileName : TFileName read FFileName Write FFileName;
    property IgnoreErrors : Boolean read FIgnoreErrors Write FIgnoreErrors default False;
    property LayoutId : string read FLayoutId Write FLayoutId;
    property BeforeLoadingProperties : TNotifyEvent read FBeforeLoadingProperties write FBeforeLoadingProperties;
    property AfterLoadingProperties : TNotifyEvent read FAfterLoadingProperties write FAfterLoadingProperties;
    property BeforeSavingProperties : TNotifyEvent read FBeforeSavingProperties write FBeforeSavingProperties;
    property AfterSavingProperties : TNotifyEvent read FAfterSavingProperties write FAfterSavingProperties;
    property LoadSavePerformerClassName : string read FLoadSavePerformerClassName Write SetLoadSavePerformerClassName;
  end;

  TLoadSavePerformer = class
  private
    FPropsSaver : TPropsSaver;
  protected
    function TranslateToString (PropItem : TPropInfoItem) : string; virtual;
    procedure AssignFromString (PropItem : TPropInfoItem; const s : string; NullValue : Boolean = False); virtual;
    procedure Open (OpenMode : TOpenMode); virtual; abstract;
    procedure Close; virtual; abstract;
    procedure SaveItem (PropItem : TPropInfoItem); virtual; abstract;
    procedure LoadItem (PropItem : TPropInfoItem); virtual; abstract;
  public
    constructor Create (APropsSaver : TPropsSaver); virtual;
    property PropsSaver : TPropsSaver read FPropsSaver;
  end;

  TClassTranslator = class
  private
    FForClass : TClass;
    FFromObjectToStringFunction : TFromObjectToStringFunction;
    FFromStringToObjectProcedure : TFromStringToObjectProcedure;
  public
    constructor Create (AForClass : TClass;
                        AFromObjectToStringFunction : TFromObjectToStringFunction;
                        AFromStringToObjectProcedure : TFromStringToObjectProcedure);
    property ForClass : TClass read FForClass;
    property FromObjectToStringFunction : TFromObjectToStringFunction read FFromObjectToStringFunction;
    property FromStringToObjectProcedure : TFromStringToObjectProcedure read FFromStringToObjectProcedure;
  end;

  TComponentSaverLoader = class
  private
    FForClass : TClass;
    FComponentSaverProcedure : TComponentSaverProcedure;
    FComponentLoaderProcedure : TComponentLoaderProcedure;
  public
    constructor Create (AForClass : TClass;
                        AComponentSaverProcedure : TComponentSaverProcedure;
                        AComponentLoaderProcedure : TComponentLoaderProcedure);
    property ForClass : TClass read FForClass;
    property ComponentSaverProcedure : TComponentSaverProcedure read FComponentSaverProcedure;
    property ComponentLoaderProcedure : TComponentLoaderProcedure read FComponentLoaderProcedure;
  end;

  TNonPublishedProperty = class
  private
    FForClass : TClass;
    FPropertyName : string;
    FFromPropToString : TFromNonPublishedPropertyToString;
    FFromStringToProp : TFromStringToNonPublishedProperty;
    FNonPublishedPropertyReference : TGetNonPublishedPropertyReference;
    FGetNonPublishedPropertySubProps : TGetPropertySubProps;
    RegCount : Integer;
  public
    constructor Create (AForClass : TClass;
                        APropertyName : string;
                        AFromPropToString : TFromNonPublishedPropertyToString;
                        AFromStringToProp : TFromStringToNonPublishedProperty;
                        ANonPublishedPropertyReference : TGetNonPublishedPropertyReference;
                        AGetNonPublishedPropertySubProps : TGetPropertySubProps);
    property ForClass : TClass read FForClass;
    property PropertyName : string read FPropertyName;
    property FromPropToString : TFromNonPublishedPropertyToString read FFromPropToString;
    property FromStringToProp : TFromStringToNonPublishedProperty read FFromStringToProp;
    property NonPublishedPropertyReference : TGetNonPublishedPropertyReference read FNonPublishedPropertyReference;
    property GetNonPublishedPropertySubProps : TGetPropertySubProps read FGetNonPublishedPropertySubProps;
  end;

  TSubProperties = class
  private
    FForClass : TClass;
    FGetPropertySubProps : TGetObjectSubProps;
  public
    constructor Create (AForClass : TClass;
                        AGetPropertySubProps : TGetObjectSubProps);
    property ForClass : TClass read FForClass;
    property GetPropertySubProps : TGetObjectSubProps read FGetPropertySubProps;
  end;

  EPropsSaver = class (Exception)
  private
    FErrorCode : TPropsSaverErrors;
  public
    constructor Create (AErrorCode : TPropsSaverErrors);
    property ErrorCode : TPropsSaverErrors read FErrorCode;
  end;

{ Some utility functions }
function GetComponentFullName (Comp : TComponent; ApplicationRoot : Boolean = false) : string;
function ResolveReference (const Ref : string; LocalRoot : TComponent) : TComponent;
function PropInfoToString (Obj : TObject; PropInfo : PPropInfo; IfClassStoreAsLink : Boolean; PropItem : TPropInfoItem = nil) : string;
procedure AssignStringToPropInfo (const s : string; Obj : TObject; PropInfo : PPropInfo; IfClassStoreAsLink : Boolean; LocalRoot : TComponent; NullValue : Boolean = False; PropItem : TPropInfoItem = nil);
function GetProperty (Obj : TObject; const PropertyName : string) : string;
procedure SetProperty (Obj : TObject; const PropertyName, value : string);
function PropItemToString (PropItem : TPropInfoItem) : string;
procedure StringToPropItem (PropItem : TPropInfoItem; const s : string; NullValue : Boolean = False);

{ Registration Functions }
procedure RegisterLoadSavePerformer (LoadSavePerformerClass : TLoadSavePerformerClass);
procedure RegisterCollectionItems (ACompInfoItemClass : TCompInfoItemClass; APropInfoItemClass : TPropInfoItemClass);
procedure RegisterTranslators (ForClass : TClass; FromObjectToStringFunction : TFromObjectToStringFunction; FromStringToObjectProcedure : TFromStringToObjectProcedure);
procedure UnRegisterTranslators (AForClass : TClass; AFromObjectToStringFunction : TFromObjectToStringFunction; AFromStringToObjectProcedure : TFromStringToObjectProcedure);
procedure RegisterComponentSaverLoader (ForClass : TClass; ComponentSaverProcedure : TComponentSaverProcedure; ComponentLoaderProcedure : TComponentSaverProcedure);
procedure UnRegisterComponentSaverLoader (AForClass : TClass; AComponentSaverProcedure : TComponentSaverProcedure; AComponentLoaderProcedure : TComponentSaverProcedure);
procedure RegisterNonPublishedProperty (ForClass : TClass; const PropertyName : string; FromPropToString : TFromNonPublishedPropertyToString; FromStringToProp : TFromStringToNonPublishedProperty;
                                        NonPublishedPropertyReference : TGetNonPublishedPropertyReference = nil;
                                        GetNonPublishedPropertySubProps : TGetPropertySubProps = nil);
procedure UnRegisterNonPublishedProperty (ForClass : TClass; const PropertyName : string);
procedure RegisterSubproperties (ForClass : TClass; GetPropertySubProps : TGetObjectSubProps);

{ Get Functions }
function GetTranslator (Obj : TObject) : TClassTranslator;
function GetComponentSaverLoader (Obj : TObject) : TComponentSaverLoader;
function GetNonPublishedProperty (Obj : TObject; const PropertyName : string) : TNonPublishedProperty; overload;
function GetNonPublishedProperty (ForClass : TClass; const PropertyName : string) : TNonPublishedProperty; overload;
procedure GetNonPublishedProperties (Obj : TObject; Props : TStringList);
function GetSubProperties (Obj : TObject; List : TStrings) : boolean;

{ Layout Functions }
procedure RestoreLayout (const ALayoutId : string; const wildcards, replacements : array of string);
procedure SaveLayout (const ALayoutId : string; const wildcards, replacements : array of string);

implementation

{$IfDef UseNonPublishedProps}
{$Define UseStdCtrls}
{$else}
{$IfDef UseComboBoxLoader}
{$Define UseStdCtrls}
{$EndIf}
{$EndIf}

uses
  PropsSaverCrypt
  {$IfDef UseBasicClassTranslators}
  , PropsSaverBasicClassTranslators
  {$EndIf}
  {$IfDef UseBasicLoadSavePerformers}
  , PropsSaverBasicLoadSavePerformers
  {$EndIf}
  {$IfDef UseGridSaverLoader}
  , PropsSaverGridSaverLoader, Grids
  {$EndIf}
  {$IfDef UseComboBoxLoader}
  , PropsSaverComboBoxLoader
  {$EndIf}
  {$IfDef UseNonPublishedProps}
  , PropsSaverNonPublishedProps
  {$EndIf}
  {$IfDef UseStdCtrls}
  , StdCtrls
  {$Endif}
  {$IfDef UseDynProps}
  , DynProps
  {$EndIf};

resourcestring
  ERR_LoadSavePerformerClassNotFound = 'Load/Save Performer Class Not Found';
  ERR_NoLoadSavePerformerAssigned = 'No Load/Save Performer Assigned';
  ERR_psePropertyNotFound = 'Property not found';
  ERR_psePropertyAssignError = 'Property assignation error';
  ERR_psePropertyReadError = 'Property reading value error';
  ERR_PropsMustBeNonNil = 'Props must be non-nil';

var
  LoadSavePerformers : TList;
  ClassTranslators : TList;
  ComponentLoaderSavers : TList;
  PropsSaversList : TList;
  LastLayouts : TStringList;
  NonPublishedProps : TList;
  SubProperties : TList;
  CompInfoItemClass : TCompInfoItemClass;
  PropInfoItemClass : TPropInfoItemClass;

{ TLastLayout }

type
  TLastLayout = class
    wildcards : array of string;
    replacements : array of string;
    { The method SetData is needed because Delphi 5 doesn't support direct assignment of the parameters
      AWildCards and AReplacements in the function SetLastLayout of the class members Wildcards and
      Replacements }
    procedure SetData (const awildcards, areplacements : array of string);
  end;

procedure TLastLayout.SetData;
var
  i, j : Integer;
begin
  SetLength (wildcards, High (AWildcards) - low (AWildcards) + 1);
  j := 0;
  for i := Low (AWildcards) to High (aWildcards) do
    begin
      wildcards [j] := awildcards [i];
      Inc (j);
    end;
  SetLength (replacements, High (areplacements) - low (areplacements) + 1);
  j := 0;
  for i := Low (areplacements) to High (areplacements) do
    begin
      replacements [j] := areplacements [i];
      Inc (j);
    end;
end;

{ Last Layout Functions }

function GetLastLayout (ALayoutId : string) : TLastLayout;
var
  i : Integer;
begin
  i := LastLayouts.IndexOf (ALayoutId);
  if i >= 0
    then
    begin
      Result := LastLayouts.Objects [i] as TLastLayout;
      Exit;
    end;
  Result := nil;
end;

procedure SetLastLayout (ALayoutId : string; const AWildcards, AReplacements : array of string);
var
  i : Integer;
  Layout : TLastLayout;
begin
  i := LastLayouts.IndexOf (ALayoutId);
  if i >= 0
    then with LastLayouts.Objects [i] as TLastLayout do
      begin
        SetData (AWildcards, AReplacements);
        Exit;
      end;
  Layout := TLastLayout.Create;
  try
    Layout.SetData (AWildcards, AReplacements);
    LastLayouts.AddObject (ALayoutId, Layout);
  except
    Layout.Free;
    raise;
  end;
end;

{ TPropsSaverCollectionItem }

function TPropsSaverCollectionItem.GetPropsSaver;
begin
  Result := TwoColsComponent as TPropsSaver;
end;

function TPropsSaverCollectionItem.AllowNotification;
begin
  Result := not (Collection as TPropsSaverCollection).IsSubProp;
end;

{ TPropInfoItem }

constructor TPropInfoItem.Create;
begin
  inherited;
  if Collection <> nil
    then
    begin
      FRootKey := Collection.CompInfoItem.RootKey;
      FBaseKey := Collection.CompInfoItem.BaseKey;
    end;
  FPropertyType := ptProperties;
  FIfClassStoreAsLink := True;
  FMode := [lsmSave, lsmLoad];
  FEncrypted := False;
  FCompInfoCollection := TCompInfoCollectionSubProp.Create (CompInfoItemClass);
  FCompInfoCollection.FPropsSaver := PropsSaver;
  FCompInfoCollection.IsSubProp := True;
  FCompInfoCollection.FPropInfoItem := Self;
  FCompInfoItemSubProps := TCompInfoItem.Create (FCompInfoCollection);
  //FCompInfoItemSubProps.Properties.IsSubProp := True;  Now the editor Supports showing subproperties
end;

destructor TPropInfoItem.Destroy;
begin
  FCompInfoItemSubProps.Free;
  FCompInfoCollection.Free;
  inherited;
end;

function TPropInfoItem.GetCollection;
begin
  Result := inherited Collection as TPropInfoCollection;
end;

procedure TPropInfoItem.SetBaseKey;
begin
  if Value <> FBaseKey
    then FBaseKey := Value;
  FCompInfoItemSubProps.BaseKey := Value;
end;

procedure TPropInfoItem.SetRootKey;
begin
  if Value <> FRootKey
    then FRootKey := Value;
  FCompInfoItemSubProps.RootKey := Value;
end;

procedure TPropInfoItem.SetItemName;
begin
  if Value <> FItemName
    then FItemName := Value;
  FCompInfoItemSubProps.ItemName := Value;
end;

function TPropInfoItem.GetComponent;
begin
  if Collection <> nil
    then Result := Collection.CompInfoItem.Component
    else Result := nil;
end;

procedure TPropInfoItem.BindComponent;
var
  PropInfo : PPropInfo;
  Obj : TObject;
  NonPubProp : TNonPublishedProperty;
  PropName : string;
begin
  if FPropertyName <> ''
    then
    begin
      if FPropertyName [1] in ['*', '+']
        then PropName := system.Copy (FPropertyName, 2, Length (FPropertyName) - 1)
        else PropName := FPropertyName;
    end
    else
    begin
      FCompInfoItemSubProps.Component := nil;
      Exit;
    end;
  if Component <> nil
    then
    begin
      PropInfo := GetPropInfo (Component, PropName, [tkClass]);
      if PropInfo <> nil
        then
        begin
          Obj := GetObjectProp (Component, PropInfo, TObject);
          if Obj <> nil
            then FCompInfoItemSubProps.Component := Obj;
        end
        else
        begin
          NonPubProp := GetNonPublishedProperty (component, PropName);
          if (NonPubProp <> nil) and Assigned (NonPubProp.NonPublishedPropertyReference)
            then FCompInfoItemSubProps.Component := NonPubProp.NonPublishedPropertyReference (Component, PropName);
        end
    end
    else FCompInfoItemSubProps.Component := nil;
end;

procedure TPropInfoItem.NotifyComponentChanged;
begin
  if (Component <> nil) and (FPropertyName <> '')
    then BindComponent;
end;

procedure TPropInfoItem.SetPropertyName;
begin
  FPropertyName := Value;
  BindComponent;
  if csDesigning in PropsSaver.ComponentState
    then
    begin
      if FItemName = ''
        then if (Collection <> nil) and (Collection.CompInfoItem.Collection.IsSubProp)
          then ItemName := Collection.CompInfoItem.ItemName + '.' + Value
          else ItemName := Value;
      DisplayName := Value;
      if (PropsSaver.EditorForm <> nil) and (not Collection.IsSubProp)
        then PropsSaver.EditorForm.ItemModified (Self);
    end;
end;

function TPropInfoItem.GetTwoColsComponent;
begin
  if Collection <> nil
    then Result := Collection.CompInfoItem.Collection.PropsSaver
    else Result := nil;
end;

procedure TPropInfoItem.Save;
begin
  if not (lsmSave in FMode)
    then Exit;
  if Assigned (FBeforeSavingProperty)
    then FBeforeSavingProperty (Self);
  try
    if (SubProperties.Count > 0) and (not FForceUseTranslator)
      then FCompInfoItemSubProps.SaveProperties (Saver)
      else Saver.SaveItem (Self);
  except
    on E : EPropsSaver do
      if E.ErrorCode = psePropertyNotFound
        then if Assigned (FOnPropertyNoFoundError)
          then FOnPropertyNoFoundError (Self)
          else if not PropsSaver.IgnoreErrors
            then raise;
    on Exception do
      if not PropsSaver.IgnoreErrors
        then raise EPropsSaver.Create (psePropertyReadError);
  end;
  if Assigned (FAfterSavingProperty)
    then FAfterSavingProperty (Self);
end;

procedure TPropInfoItem.Load;
begin
  if not (lsmLoad in FMode)
    then Exit;
  if Assigned (FBeforeLoadingProperty)
    then FBeforeLoadingProperty (Self);
  try
    if (SubProperties.Count > 0) and (not FForceUseTranslator)
      then FCompInfoItemSubProps.LoadProperties (Loader)
      else Loader.LoadItem (Self);
  except
    on E : EPropsSaver do
      if E.ErrorCode = psePropertyNotFound
        then if Assigned (FOnPropertyNoFoundError)
          then FOnPropertyNoFoundError (Self)
          else if not PropsSaver.IgnoreErrors
            then raise;
    on Exception do
      if not PropsSaver.IgnoreErrors
        then raise EPropsSaver.Create (psePropertyAssignError);
  end;
  if Assigned (FAfterLoadingProperty)
    then FAfterLoadingProperty (Self);
end;

function TPropInfoItem.GetDisplayName;
begin
  if (Component <> nil) and (Component is TComponent)
    then Result := (Component as TComponent).Name + '.' + PropertyName
    else Result := PropertyName;
end;

function TPropInfoItem.GetCompInfoItem;
begin
  if Collection <> nil
    then Result := Collection.CompInfoItem
    else Result := nil;
end;

procedure TPropInfoItem.Assign;
begin
  RootKey := (Source as TPropInfoItem).RootKey;
  BaseKey := (Source as TPropInfoItem).BaseKey;
  ItemName := (Source as TPropInfoItem).ItemName;
  PropertyType := (Source as TPropInfoItem).PropertyType;
  PropertyName := (Source as TPropInfoItem).PropertyName;
  IfClassStoreAsLink := (Source as TPropInfoItem).IfClassStoreAsLink;
  Mode := (Source as TPropInfoItem).Mode;
  FCompInfoItemSubProps.Assign ((Source as TPropInfoItem).FCompInfoItemSubProps);
  BeforeLoadingProperty := (Source as TPropInfoItem).BeforeLoadingProperty;
  AfterLoadingProperty := (Source as TPropInfoItem).AfterLoadingProperty;
  BeforeSavingProperty := (Source as TPropInfoItem).BeforeSavingProperty;
  AfterSavingProperty := (Source as TPropInfoItem).AfterSavingProperty;
  OnPropertyNoFoundError := (Source as TPropInfoItem).OnPropertyNoFoundError;
end;

function TPropInfoItem.getSubProperties;
begin
  Result := FCompInfoItemSubProps.Properties;
end;

procedure TPropInfoItem.SetSubProperties;
begin
  FCompInfoItemSubProps.Properties.Assign (Value);
end;

procedure TPropInfoItem.ReplaceWildcards;
var
  i : Integer;
begin
  FOldBaseKey := FBaseKey;
  FOldItemName := FItemName;
  for i := Low (wildcards) to High (wildcards) do
    begin
      FBaseKey := StringReplace (FBaseKey, wildcards [i], replacements [i], [rfReplaceAll, rfIgnoreCase]);
      FItemName := StringReplace (FItemName, wildcards [i], replacements [i], [rfReplaceAll, rfIgnoreCase]);
    end;
  FCompInfoItemSubProps.ReplaceWildcards (wildcards, replacements);
end;

procedure TPropInfoItem.RestoreWildcards;
begin
  FBaseKey := FOldBaseKey;
  FItemName := FOldItemName;
  FCompInfoItemSubProps.RestoreWildcards;
end;

{ TPropInfoCollection }

function TPropInfoCollection.GetOwner;
begin
  Result := FCompInfoItem;
end;

function TPropInfoCollection.GetItems;
begin
  Result := inherited Items [Index] as TPropInfoItem;
end;

procedure TPropInfoCollection.SetItems;
begin
  inherited Items [Index] := Item;
end;

function TPropInfoCollection.Add;
begin
  Result := inherited Add as TPropInfoItem;
end;

{ TCompInfoItem }

constructor TCompInfoItem.Create;
begin
  inherited;
  FProperties := TPropInfoCollection.Create (PropInfoItemClass);
  FProperties.FCompInfoItem := Self;
  FRootKey := PropsSaver.RootKey;
  FBaseKey := PropsSaver.BaseKey;
end;

destructor TCompInfoItem.Destroy;
begin
  FProperties.Free;
  inherited;
end;

function TCompInfoItem.GetCollection;
begin
  Result := inherited Collection as TCompInfoCollection;
end;

procedure TCompInfoItem.SetProperties;
begin
  FProperties.Assign (Value);
end;

procedure TCompInfoItem.SetComponent;
var
  i : Integer;
begin
  FComponent := Value;
  if (FComponent <> nil) and (csDesigning in PropsSaver.ComponentState) and (Component is TComponent) and
     (FItemName = '')
    then FItemName := (FComponent as TComponent).Name;
  for i := 0 to FProperties.Count - 1 do
    FProperties [i].NotifyComponentChanged;
  if csDesigning in PropsSaver.ComponentState
    then
    begin
      DisplayName := FItemName;
      if (PropsSaver.EditorForm <> nil) and (not Collection.IsSubProp)
        then PropsSaver.EditorForm.ItemModified (Self);
    end;
end;

procedure TCompInfoItem.SetItemName;
begin
  if Value <> FItemName
    then
    begin
      FItemName := Value;
      if csDesigning in PropsSaver.ComponentState
        then if (PropsSaver.EditorForm <> nil) and (not Collection.IsSubProp)
          then PropsSaver.EditorForm.ItemModified (Self);
    end;
end;

procedure TCompInfoItem.SaveProperties;
var
  i : Integer;
begin
  if Assigned (FBeforeSavingProperties)
    then FBeforeSavingProperties (Self);
  for i := 0 to Properties.Count - 1 do
    Properties [i].Save (Saver);
  if Assigned (FAfterSavingProperties)
    then FAfterSavingProperties (Self);
end;

procedure TCompInfoItem.LoadProperties;
var
  i : Integer;
begin
  if Assigned (FBeforeLoadingProperties)
    then FBeforeLoadingProperties (Self);
  for i := 0 to Properties.Count - 1 do
    Properties [i].Load (Loader);
  if Assigned (FAfterLoadingProperties)
    then FAfterLoadingProperties (Self);
end;

function TCompInfoItem.GetDisplayName;
begin
  if (Component <> nil) and (Component is TComponent)
    then Result := (Component as TComponent).Name
    else Result := FItemName;
end;

function TCompInfoItem.GetTwoColsComponent;
begin
  Result := Collection.PropsSaver;
end;

procedure TCompInfoItem.Assign;
var
  i : Integer;
begin
  RootKey := (Source as TCompInfoItem).RootKey;
  BaseKey := (Source as TCompInfoItem).BaseKey;
  ItemName := (Source as TCompInfoItem).ItemName;
  Component := (Source as TCompInfoItem).Component;
  BeforeLoadingProperties := (Source as TCompInfoItem).BeforeLoadingProperties;
  AfterLoadingProperties := (Source as TCompInfoItem).AfterLoadingProperties;
  BeforeSavingProperties := (Source as TCompInfoItem).BeforeSavingProperties;
  AfterSavingProperties := (Source as TCompInfoItem).AfterSavingProperties;
  for i := 0 to (Source as TCompInfoItem).Properties.Count - 1 do
    with Properties.Add do
      Assign ((Source as TCompInfoItem).Properties.Items [i]);
end;

procedure TCompInfoItem.ReplaceWildcards;
var
  i : Integer;
begin
  FOldBaseKey := FBaseKey;
  FOldItemName := FItemName;
  for i := Low (wildcards) to High (wildcards) do
    begin
      FBaseKey := StringReplace (FBaseKey, wildcards [i], replacements [i], [rfReplaceAll, rfIgnoreCase]);
      FItemName := StringReplace (FItemName, wildcards [i], replacements [i], [rfReplaceAll, rfIgnoreCase]);
    end;
  for i := 0 to Properties.Count - 1 do
    Properties [i].ReplaceWildcards (wildcards, replacements);
end;

procedure TCompInfoItem.RestoreWildcards;
var
  i : Integer;
begin
  FBaseKey := FOldBaseKey;
  FItemName := FOldItemName;
  for i := 0 to Properties.Count - 1 do
    Properties [i].RestoreWildcards;
end;

{ TCompInfoCollection }

function TCompInfoCollection.GetOwner;
begin
  Result := PropsSaver;
end;

function TCompInfoCollection.GetItems;
begin
  Result := inherited Items [Index] as TCompInfoItem;
end;

procedure TCompInfoCollection.SetItems;
begin
  inherited Items [Index] := Item;
end;

function TCompInfoCollection.Add;
begin
  Result := inherited Add as TCompInfoItem;
end;

{ TCompInfoCollectionSubProp }

function TCompInfoCollectionSubProp.GetOwner;
begin
  Result := FPropInfoItem;
end;

{ TPropsSaver }

constructor TPropsSaver.Create;
begin
  inherited;
  FComponents := TCompInfoCollection.Create (CompInfoItemClass);
  FComponents.FPropsSaver := Self;
  FRootKey := HKEY_CURRENT_USER;
  FBaseKey := '\Software\Company Name\Product Name';
  FAutoLoad := aldNone;
  FAutoSave := asvNone;
  PropsSaversList.Add (Self);
end;

destructor TPropsSaver.Destroy;
begin
  PropsSaversList.Remove (Self);
  FComponents.Free;
  inherited;
end;

function GetNotifyEvent (Obj : TObject; const MethodName : string) : TNotifyEvent;
var
  Method : TMethod;
  PropInfo : PPropInfo;
begin
  Result := nil;
  PropInfo := GetPropInfo (Obj, MethodName, [tkMethod]);
  if PropInfo <> nil
    then
    begin
      Method := GetMethodProp (Obj, PropInfo);
      if Method.Code <> nil
        then Result := TNotifyEvent (Method);
    end;
end;

procedure SetNotifyEvent (Obj : TObject; Evt : TNotifyEvent; const MethodName : string);
var
  Method : TMethod;
  PropInfo : PPropInfo;
begin
  PropInfo := GetPropInfo (Obj, MethodName, [tkMethod]);
  if PropInfo <> nil
    then
    begin
      Method := TMethod (evt);
      SetMethodProp (Obj, PropInfo, Method);
    end;
end;

procedure TPropsSaver.SetAutoLoadProperties;
begin
  if Value <> FAutoLoad
    then
    begin
      if not (csDesigning in ComponentState)
        then
        begin
          case FAutoLoad of
            aldAfterOnCreateForm, aldBeforeOnCreateForm : SetNotifyEvent (Owner, FOldLoad, 'OnCreate');
            aldAfterOnShowForm, aldBeforeOnShowForm : SetNotifyEvent (Owner, FOldLoad, 'OnShow');
          end;
          case value of
            aldAfterOnCreateForm, aldBeforeOnCreateForm :
              begin
                FOldLoad := GetNotifyEvent (Owner, 'OnCreate');
                SetNotifyEvent (Owner, AutoLoad, 'OnCreate');
              end;
            aldAfterOnShowForm, aldBeforeOnShowForm :
              begin
                FOldLoad := GetNotifyEvent (Owner, 'OnShow');
                SetNotifyEvent (Owner, AutoLoad, 'OnShow');
              end;
          end;
        end;
      FAutoLoad := Value;
    end;
end;

procedure TPropsSaver.SetAutoSaveProperties;
begin
  if FAutoSave <> value
    then
    begin
      if not (csDesigning in ComponentState)
        then
        begin
          case FAutoSave of
            asvAfterOnDestroyForm, asvBeforeOnDestroyForm : SetNotifyEvent (Owner, FOldSave, 'OnDestroy');
            asvAfterOnHideForm, asvBeforeOnHideForm : SetNotifyEvent (Owner, FOldSave, 'OnDestroy');
          end;
          case value of
            asvAfterOnDestroyForm, asvBeforeOnDestroyForm :
              begin
                FOldSave := GetNotifyEvent (Owner, 'OnDestroy');
                SetNotifyEvent (Owner, AutoSave, 'OnDestroy');
              end;
            asvAfterOnHideForm, asvBeforeOnHideForm :
              begin
                FOldSave := GetNotifyEvent (Owner, 'OnHide');
                SetNotifyEvent (Owner, AutoSave, 'OnHide');
              end;
          end;
        end;
      FAutoSave := Value;
    end;
end;

procedure TPropsSaver.AutoLoad;
begin
  if AutoLoadProperties in [aldAfterOnCreateForm, aldAfterOnShowForm]
    then if Assigned (FOldLoad)
      then FOldLoad (Owner);
  LoadProperties;
  if AutoLoadProperties in [aldBeforeOnCreateForm, aldBeforeOnShowForm]
    then if Assigned (FOldLoad)
      then FOldLoad (Owner);
end;

procedure TPropsSaver.AutoSave;
begin
  if AutoSaveProperties in [asvAfterOnDestroyForm, asvAfterOnHideForm]
    then if Assigned (FOldSave)
      then FOldSave (Owner);
  SaveProperties;
  if AutoSaveProperties in [asvBeforeOnDestroyForm, asvBeforeOnHideForm]
    then if Assigned (FOldSave)
      then FOldSave (Owner);
end;

procedure TPropsSaver.SetComponents;
begin
  FComponents.Assign (Value);
end;

procedure TPropsSaver.Notification;
var
  i : Integer;
begin
  if AComponent = self
    then Exit;
  if Operation = opRemove
    then for i := 0 to FComponents.Count - 1 do
      if FComponents [i].Component = AComponent
        then FComponents [i].Component := nil;
  inherited;
end;

class function TPropsSaver.LoadSavePerformers;
begin
  Result := PropsSaver.LoadSavePerformers;
end;

class function TPropsSaver.PropsSavers;
begin
  Result := PropsSaversList;
end;

procedure TPropsSaver.SetLoadSavePerformerClassName;
var
  i : Integer;
begin
  if Value <> ''
    then for i := 0 to TPropsSaver.LoadSavePerformers.Count - 1 do
      if Value = TLoadSavePerformerClass (TPropsSaver.LoadSavePerformers [i]).ClassName
        then
        begin
          FLoadSavePerformerClassName := Value;
          FLoadSavePerformerClass := TLoadSavePerformerClass (TPropsSaver.LoadSavePerformers [i]);
          Exit;
        end
      else {}
    else
    begin
      FLoadSavePerformerClassName := '';
      FLoadSavePerformerClass := nil;
      Exit;
    end;
  raise Exception.Create (ERR_LoadSavePerformerClassNotFound);
end;

procedure DefaultComponentSaver (CompInfoItem : TCompInfoItem; Saver : TLoadSavePerformer);
begin
  CompInfoItem.SaveProperties (Saver);
end;

procedure TPropsSaver.SaveProperties;
var
  i : Integer;
  Saver : TLoadSavePerformer;
  ComponentSaverLoader : TComponentSaverLoader;
  LastLayout : TLastLayout;
begin
  if FLoadSavePerformerclass = nil
    then raise Exception.Create (ERR_NoLoadSavePerformerAssigned);
  if Assigned (FBeforeSavingProperties)
    then FBeforeSavingProperties (Self);
  Saver := FLoadSavePerformerclass.Create (Self);
  try
    Saver.Open (omSave);
    LastLayout := GetLastLayout (LayoutId);
    if LastLayout <> nil
      then ReplaceWildcards (LastLayout.wildcards, LastLayout.replacements);
    try
      for i := 0 to Components.Count - 1 do
        begin
          ComponentSaverLoader := GetComponentSaverLoader (Components [i].Component);
          if ComponentSaverLoader <> nil
            then ComponentSaverLoader.ComponentSaverProcedure (Components [i], Saver);
        end;
    finally
      if LastLayout <> nil
        then RestoreWildcards;
    end;
    Saver.Close;
  finally
    Saver.Free;
  end;
  if Assigned (FAfterSavingProperties)
    then FAfterSavingProperties (Self);
end;

procedure DefaultComponentLoader (CompInfoItem : TCompInfoItem; Loader : TLoadSavePerformer);
begin
  CompInfoItem.LoadProperties (Loader);
end;

procedure TPropsSaver.LoadProperties;
var
  i : Integer;
  Loader : TLoadSavePerformer;
  ComponentSaverLoader : TComponentSaverLoader;
  LastLayout : TLastLayout;
begin
  if FLoadSavePerformerclass = nil
    then raise Exception.Create (ERR_NoLoadSavePerformerAssigned);
  if Assigned (FBeforeLoadingProperties)
    then FBeforeLoadingProperties (Self);
  Loader := FLoadSavePerformerclass.Create (Self);
  try
    Loader.Open (omLoad);
    LastLayout := GetLastLayout (LayoutId);
    if LastLayout <> nil
      then ReplaceWildcards (LastLayout.wildcards, LastLayout.replacements);
    try
      for i := 0 to Components.Count - 1 do
        begin
          ComponentSaverLoader := GetComponentSaverLoader (Components [i].Component);
          if ComponentSaverLoader <> nil
            then ComponentSaverLoader.ComponentLoaderProcedure (Components [i], Loader);
        end;
    finally
      if LastLayout <> nil
        then RestoreWildcards;
    end;
    Loader.Close;
  finally
    Loader.Free;
  end;
  if Assigned (FAfterLoadingProperties)
    then FAfterLoadingProperties (Self);
end;

procedure TPropsSaver.ReplaceWildcards;
var
  i : integer;
begin
  FOldBaseKey := FBaseKey;
  FOldFileName := FFileName;
  for i := Low (wildcards) to High (wildcards) do
    begin
      FBaseKey := StringReplace (FBaseKey, wildcards [i], replacements [i], [rfReplaceAll, rfIgnoreCase]);
      FFileName := StringReplace (FFileName, wildcards [i], replacements [i], [rfReplaceAll, rfIgnoreCase]);
    end;
  for i := 0 to Components.Count - 1 do
    Components [i].ReplaceWildcards (wildcards, replacements);
end;

procedure TPropsSaver.RestoreWildcards;
var
  i : Integer;
begin
  FBaseKey := FOldBaseKey;
  FFileName := FOldFileName;
  for i := 0 to Components.Count - 1 do
    Components [i].RestoreWildcards;
end;

{ TLoadSavePerformer }

constructor TLoadSavePerformer.Create;
begin
  inherited Create;
  FPropsSaver := APropsSaver;
end;

function TLoadSavePerformer.TranslateToString;
begin
  Result := PropItemToString (PropItem);
end;

procedure TLoadSavePerformer.AssignFromString;
begin
  StringToPropItem (PropItem, s);
end;

{ TClassTranslator }

constructor TClassTranslator.Create;
begin
  inherited Create;
  FForClass := AForClass;
  FFromObjectToStringFunction := AFromObjectToStringFunction;
  FFromStringToObjectProcedure := AFromStringToObjectProcedure;
end;

{ TComponentSaverLoader }

constructor TComponentSaverLoader.Create;
begin
  inherited Create;
  FForClass := AForClass;
  FComponentSaverProcedure := AComponentSaverProcedure;
  FComponentLoaderProcedure := AComponentLoaderProcedure;
end;

{ TNonPublishedProperty }

constructor TNonPublishedProperty.Create;
begin
  inherited Create;
  FForClass := AForClass;
  FPropertyName := APropertyName;
  FFromPropToString := AFromPropToString;
  FFromStringToProp := AFromStringToProp;
  FNonPublishedPropertyReference := ANonPublishedPropertyReference;
  FGetNonPublishedPropertySubProps := AGetNonPublishedPropertySubProps;
end;

{ TSubProperties }

constructor TSubProperties.Create;
begin
  inherited Create;
  FForClass := AForClass;
  FGetPropertySubProps := AGetPropertySubProps;
end;

{ EPropsSaver }

constructor EPropsSaver.Create;
begin
  case AErrorCode of
    psePropertyNotFound : inherited Create (ERR_psePropertyNotFound);
    psePropertyAssignError : inherited Create (ERR_psePropertyAssignError);
    psePropertyReadError : inherited Create (ERR_psePropertyReadError);
    else inherited Create ('Unknown Error Code');
  end;
  FErrorCode := AErrorCode;
end;

{ GetComponentFullName }

function GetComponentFullName;
  function GetCompName (AComp : TComponent) : string;
  begin
    if AComp.Owner <> nil
      then if AComp.Owner is TApplication
        then Result := 'Application.' + AComp.Name
        else if AComp.Owner is TComponent
          then Result := GetCompName (AComp.Owner) + '.' + AComp.Name
          else Result := AComp.Name
      else if AComp is TApplication
        then Result := AComp.Name
        else if ApplicationRoot
          then Result := 'Application.' + AComp.Name
          else Result := AComp.Name;
  end;
begin
  if Comp <> nil
    then Result := GetCompName (Comp)
    else Result := '';
end;

{ ResolveReference }

function ResolveReference;
var
  p : Integer;
  CompStr : string;
  function ResolveFrom (FromComp : TComponent; ARef : string) : TComponent;
  var
    p : Integer;
    Root : string;
  begin
    p := Pos ('.', ARef);
    if p > 0
      then
      begin
        Root := system.Copy (ARef, 1, p - 1);
        FromComp := FromComp.FindComponent (Root);
        if FromComp <> nil
          then Result := ResolveFrom (FromComp, system.Copy (ARef, p + 1, Length (ARef) - p))
          else Result := nil;
      end
      else Result := FromComp.FindComponent (ARef);
  end;
begin
  p := Pos ('.', Ref);
  if p > 0
    then
    begin
      CompStr := system.Copy (Ref, 1, p - 1);
      if UpperCase (CompStr) = 'APPLICATION'
        then Result := ResolveFrom (Application, system.Copy (Ref, p + 1, Length (Ref) - p))
        else Result := ResolveFrom (LocalRoot, Ref);
    end
    else Result := ResolveFrom (LocalRoot, Ref);
  if (Result = nil) and (LocalRoot <> Application)
    then Result := ResolveReference (Ref, Application);
end;

{ PropItemToString }

function PropItemToString (PropItem : TPropInfoItem) : string;
var
  PropInfo : PPropInfo;
  function TranslateNonPublishedProp (const PropName : string; var StrResult : string) : Boolean;
  var
    NonPublishedProp : TNonPublishedProperty;
  begin
    NonPublishedProp := GetNonPublishedProperty (PropItem.Component, PropName);
    if NonPublishedProp <> nil
      then
      begin
        StrResult := NonPublishedProp.FromPropToString (PropItem.Component, PropName, PropItem);
        Result := True;
      end
      else Result := False;
  end;
begin
  Result := '';
  with PropItem do
    begin
      if (Component = nil) or (PropertyName = '')
        then Exit;
      if not (PropertyName [1] in ['*', '+'])
        then
        begin
          PropInfo := GetPropInfo (Component.ClassInfo, PropertyName);
          if PropInfo <> nil
            then Result := PropInfoToString (Component, PropInfo, IfClassStoreAsLink, PropItem)
            else if not TranslateNonPublishedProp (PropertyName, Result)
              then raise EPropsSaver.Create (psePropertyNotFound)
        end
        else if not TranslateNonPublishedProp (system.Copy (PropertyName, 2, Length (PropertyName) - 1), Result)
          then raise EPropsSaver.Create (psePropertyNotFound);
      if Encrypted and (Length (Result) > 0)
        then Encrypt (Result [1], Length (Result), true);
    end;
end;

{ StringToPropItem }

procedure StringToPropItem (PropItem : TPropInfoItem; const s : string; NullValue : Boolean = False);
var
  PropInfo : PPropInfo;
  s2 : string;
  function TranslateToNonPublishedProp (const PropName : string; s : string) : boolean;
  var
    NonPublishedProp : TNonPublishedProperty;
  begin
    NonPublishedProp := GetNonPublishedProperty (PropItem.Component, PropName);
    if NonPublishedProp <> nil
      then
      begin
        NonPublishedProp.FromStringToProp (PropItem.Component, PropName, s, PropItem);
        Result := True;
      end
      else Result := False;
  end;
begin
  with PropItem do
    begin
      if (Component = nil) or (PropertyName = '')
        then Exit;
      s2 := s;
      if Encrypted and (Length (s2) > 0)
        then Decrypt (s2 [1], Length (s), true);
      if not (PropertyName [1] in ['*', '+'])
        then
        begin
          PropInfo := GetPropInfo (Component.ClassInfo, PropertyName);
          if PropInfo <> nil
            then AssignStringToPropInfo (s2, Component, PropInfo, IfClassStoreAsLink, PropItem.PropsSaver.Owner, NullValue, PropItem)
            else if not TranslateToNonPublishedProp (PropertyName, s2)
              then raise EPropsSaver.Create (psePropertyNotFound);
        end
        else if not TranslateToNonPublishedProp (system.Copy (PropertyName, 2, Length (PropertyName) - 1), s2)
          then raise EPropsSaver.Create (psePropertyNotFound);
    end;
end;

{ PropInfoToString }

function PropInfoToString;
var
  oObj : TObject;
  Translator : TClassTranslator;
  Method : TMethod;
  IntToId : TIntToIdent;
  s : string;
  i : Integer;
begin
  Result := '';
  case PropInfo^.PropType^.Kind of
    tkInteger :
      begin
        IntToId := FindIntToIdent (PropInfo^.PropType^);
        if Assigned (IntToId)
          then
          begin
            i := GetOrdProp (Obj, PropInfo);
            if IntToId (i, s)
              then Result := s
              else Result := IntToStr (i);
          end
          else Result := IntToStr (GetOrdProp (Obj, PropInfo));
      end;
    tkEnumeration : Result := GetEnumProp (Obj, PropInfo);
    tkFloat : Result := FloatToStr (GetFloatProp (Obj, PropInfo));
    tkString,
    tkWChar,
    tkLString,
    tkWString,
    tkChar: Result := GetStrProp (Obj, PropInfo);
    tkSet : Result := '[' + GetSetProp (Obj, PropInfo) + ']';
    tkClass :
      begin
        if IfClassStoreAsLink
          then
          begin
            oObj := GetObjectProp (Obj, PropInfo, TComponent) as TComponent;
            if Obj <> nil
              then Result := GetComponentFullName (oObj as TComponent)
              else Result := ''
          end
          else
          begin
            oObj := GetObjectProp (Obj, PropInfo, TObject);
            Translator := GetTranslator (oObj);
            if Translator <> nil
              then Result := Translator.FromObjectToStringFunction (oObj, PropItem);
          end;
      end;
    tkMethod :
      begin
        Method := GetMethodProp (Obj, PropInfo);
        if (Method.Data <> nil) and (TObject (Method.Data) is TComponent)
          then Result := GetComponentFullName (TComponent (Method.Data)) + '.' + TObject (Method.Data).MethodName (Method.Code);
      end;
    tkVariant : Result := GetVariantProp (Obj, PropInfo);
    tkInt64 : Result := IntToStr (GetInt64Prop (Obj, PropInfo));
    tkArray,
    tkRecord,
    tkDynArray : Result := ''; { No support for this type kinds }
  end;
end;

{ AssignStringToPropInfo }

procedure AssignStringToPropInfo;
var
  oObj : TObject;
  Translator : TClassTranslator;
  Method : TMethod;
  MetName, ObjName : string;
  i : Integer;
  IdToInt : TIdentToInt;
begin
  case PropInfo^.PropType^.Kind of
    tkInteger : if (s <> '') and (not NullValue)
      then
      begin
        try
          SetOrdProp (Obj, PropInfo, StrToInt (s));
        except
          on EConvertError do
            begin
              IdToInt := FindIdentToInt (PropInfo^.PropType^);
              if Assigned (IdtoInt)
                then
                begin
                  if IdToInt (s, i)
                    then SetOrdProp (Obj, PropInfo, i)
                    else raise;
                end
                else raise;
            end;
        end;
      end
      else SetOrdProp (Obj, PropInfo, 0);
    tkEnumeration : SetEnumProp (Obj, PropInfo, s);
    tkFloat : if (s <> '') and (not NullValue)
      then SetFloatProp (Obj, PropInfo, StrToFloat (s))
      else SetFloatProp (Obj, PropInfo, 0);
    tkString,
    tkWChar,
    tkLString,
    tkWString,
    tkChar: SetStrProp (Obj, PropInfo, s);
    tkSet : SetSetProp (Obj, PropInfo, s);
    tkClass :
      begin
        if IfClassStoreAsLink
          then
          begin
            oObj := ResolveReference (s, LocalRoot);
            SetObjectProp (Obj, PropInfo, oObj);
          end
          else
          begin
            oObj := GetObjectProp (Obj, PropInfo, TObject);
            Translator := GetTranslator (oObj);
            if Translator <> nil
              then Translator.FromStringToObjectProcedure (s, oObj, PropItem);
          end;
      end;
    tkMethod :
      begin
        if s = ''
          then Exit;
        MetName := '';
        for i := Length (s) downto 1 do
          if s [i] = '.'
            then
            begin
              MetName := system.Copy (s, i + 1, Length (s) - i);
              ObjName := system.Copy (s, 1, i - 1);
              Break;
            end;
        if MetName <> ''
          then
          begin
            Method.Data := ResolveReference (ObjName, LocalRoot);
            if Method.Data <> nil
              then
              begin
                Method.Code := TObject (Method.Data).MethodAddress (MetName);
                if Method.Code <> nil
                  then SetMethodProp (Obj, PropInfo, Method);
              end;
          end;
      end;
    tkVariant : SetVariantProp (Obj, PropInfo, s);
    tkInt64 : if (s <> '') and (not NullValue)
      then SetInt64Prop (Obj, PropInfo, StrToInt (s))
      else SetInt64Prop (Obj, PropInfo, 0);
    tkArray,
    tkRecord,
    tkDynArray : { No support for this type kinds };
  end;
end;

{ GetProperty }

function GetProperty (Obj : TObject; const PropertyName : string) : string;
var
  PropInfo : PPropInfo;
begin
  PropInfo := GetPropInfo (Obj.ClassInfo, PropertyName);
  if PropInfo <> nil
    then Result := PropInfoToString (Obj, PropInfo, True)
    else raise Exception.Create (ERR_psePropertyNotFound);
end;

{ SetProperty }

procedure SetProperty (Obj : TObject; const PropertyName, value : string);
var
  PropInfo : PPropInfo;
begin
  PropInfo := GetPropInfo (Obj.ClassInfo, PropertyName);
  if PropInfo <> nil
    then AssignStringToPropInfo (Value, Obj, PropInfo, True, Application)
    else raise Exception.Create (ERR_psePropertyNotFound);
end;

{ RegisterLoadSavePerformers }

procedure RegisterLoadSavePerformer;
begin
  LoadSavePerformers.Add (LoadSavePerformerClass);
end;

{ RegisterCollectionItems }

procedure RegisterCollectionItems;
begin
  CompInfoItemClass := ACompInfoItemClass;
  PropInfoItemClass := APropInfoItemClass;
end;

{ RegisterTranslators }

procedure RegisterTranslators;
var
  ClassTranslator : TClassTranslator;
begin
  ClassTranslator := TClassTranslator.Create (ForClass, FromObjectToStringFunction, FromStringToObjectProcedure);
  try
    ClassTranslators.Insert (0, ClassTranslator);
  except
    ClassTranslator.Free;
    raise;
  end;
end;

{ UnRegisterTranslators }

procedure UnRegisterTranslators;
var
  i : Integer;
begin
  for i := 0 to ClassTranslators.Count - 1 do
    with TClassTranslator (ClassTranslators [i]) do
      if (ForClass = AForClass) and (@FromObjectToStringFunction = @AFromObjectToStringFunction) and
         (@FromStringToObjectProcedure = @AFromStringToObjectProcedure)
        then
        begin
          Free;
          ClassTranslators.Delete (i);
          Exit;
        end;
end;

{ RegisterComponentLoaderSaver }

procedure RegisterComponentSaverLoader;
var
  ComponentSaverLoader : TComponentSaverLoader;
begin
  ComponentSaverLoader := TComponentSaverLoader.Create (ForClass, ComponentSaverProcedure, ComponentLoaderProcedure);
  try
    ComponentLoaderSavers.Insert (0, ComponentSaverLoader);
  except
    ComponentSaverLoader.Free;
    raise;
  end;
end;

{ UnRegisterComponentLoaderSaver }

procedure UnRegisterComponentSaverLoader;
var
  i : Integer;
begin
  for i := 0 to ComponentLoaderSavers.Count - 1 do
    with TComponentSaverLoader (ComponentLoaderSavers [i]) do
      if (AForClass = ForClass) and (@ComponentSaverProcedure = @AComponentSaverProcedure) and
         (@ComponentLoaderProcedure = @AComponentLoaderProcedure)
        then
        begin
          Free;
          ComponentLoaderSavers.Delete (i);
          Exit;
        end;
end;

{ RegisterNonPublishedProperty }

procedure RegisterNonPublishedProperty;
var
  Prop : TNonPublishedProperty;
begin
  Prop := GetNonPublishedProperty (ForClass, PropertyName);
  if Prop <> nil
    then
    begin
      Inc (Prop.RegCount);
      Exit;
    end;
  Prop := TNonPublishedProperty.Create (ForClass, PropertyName, FromPropToString, FromStringToProp, NonPublishedPropertyReference, GetNonPublishedPropertySubProps);
  try
    NonPublishedProps.Add (Prop);
  except
    Prop.Free;
    raise;
  end;
end;

{ UnRegisterNonPublishedProperty }

procedure UnRegisterNonPublishedProperty;
var
  Prop : TNonPublishedProperty;
begin
  Prop := GetNonPublishedProperty (ForClass, PropertyName);
  if Prop <> nil
    then if Prop.RegCount = 0
      then
      begin
        NonPublishedProps.Remove (Prop);
        Prop.Free;
      end
      else Dec (Prop.RegCount);
end;

{ RegisterSubproperties }

procedure RegisterSubproperties (ForClass : TClass; GetPropertySubProps : TGetObjectSubProps);
var
  SubProp : TSubProperties;
begin
  SubProp := TSubProperties.Create (ForClass, GetPropertySubProps);
  try
    SubProperties.Add (SubProp);
  except
    SubProp.Free;
    raise;
  end;
end;

{ GetTranslator }

function GetTranslator;
var
  i : Integer;
begin
  Result := nil;
  for i := 0 to ClassTranslators.Count - 1 do
    if Obj is TClassTranslator (ClassTranslators [i]).ForClass
      then
      begin
        Result := TClassTranslator (ClassTranslators [i]);
        break;
      end;
end;

{ GetComponentSaverLoader }

function GetComponentSaverLoader;
var
  i : Integer;
begin
  Result := nil;
  for i := 0 to ComponentLoaderSavers.Count - 1 do
    if Obj is TComponentSaverLoader (ComponentLoaderSavers [i]).ForClass
      then
      begin
        Result := TComponentSaverLoader (ComponentLoaderSavers [i]);
        break;
      end;
end;

{ GetNonPublishedProperty version 1}

function GetNonPublishedProperty (Obj : TObject; const PropertyName : string) : TNonPublishedProperty;
var
  i : integer;
  PropName : string;
begin
  if (PropertyName <> '') and (PropertyName [1] in ['*', '+'])
    then PropName := system.Copy (PropertyName, 2, Length (PropertyName) - 1)
    else PropName := PropertyName;
  for i := 0 to NonPublishedProps.Count - 1 do
    if (Obj is TNonPublishedProperty (NonPublishedProps [i]).ForClass) and
       (TNonPublishedProperty (NonPublishedProps [i]).PropertyName = PropName)
      then
      begin
        Result := TNonPublishedProperty (NonPublishedProps [i]);
        Exit;
      end;
  Result := nil;
end;

{ GetNonPublishedProperty version 2}

function GetNonPublishedProperty (ForClass : TClass; const PropertyName : string) : TNonPublishedProperty;
var
  i : integer;
  PropName : string;
begin
  if (PropertyName <> '') and (PropertyName [1] in ['*', '+'])
    then PropName := system.Copy (PropertyName, 2, Length (PropertyName) - 1)
    else PropName := PropertyName;
  for i := 0 to NonPublishedProps.Count - 1 do
    if (ForClass.InheritsFrom (TNonPublishedProperty (NonPublishedProps [i]).ForClass)) and
       (TNonPublishedProperty (NonPublishedProps [i]).PropertyName = PropName)
      then
      begin
        Result := TNonPublishedProperty (NonPublishedProps [i]);
        Exit;
      end;
  Result := nil;
end;

{ GetNonPublishedProperties }

procedure GetNonPublishedProperties;
var
  i : integer;
begin
  if Props = nil
    then raise Exception.Create (ERR_PropsMustBeNonNil);
  for i := 0 to NonPublishedProps.Count - 1 do
    if Obj is TNonPublishedProperty (NonPublishedProps [i]).ForClass
      then Props.Add (TNonPublishedProperty (NonPublishedProps [i]).PropertyName);
end;

{ GetSubProperties }

function GetSubProperties;
var
  i : integer;
begin
  Result := False;
  for i := 0 to SubProperties.Count - 1 do
    begin
      if Obj is TSubProperties (SubProperties [i]).ForClass
        then
        begin
          TSubProperties (SubProperties [i]).GetPropertySubProps (Obj, List);
          Result := True;
          Exit;
        end;
    end;
end;

{ Layout Functions }

procedure RestoreLayout (const ALayoutId : string; const wildcards, replacements : array of string);
var
  i : Integer;
begin
  SetLastLayout (ALayoutId, wildcards, replacements);
  for i := 0 to PropsSaversList.Count - 1 do
    with TPropsSaver (PropsSaversList [i]) do
      if ALayoutId = LayoutId
        then LoadProperties;
end;

procedure SaveLayout (const ALayoutId : string; const wildcards, replacements : array of string);
var
  i : Integer;
begin
  SetLastLayout (ALayoutId, wildcards, replacements);
  for i := 0 to PropsSaversList.Count - 1 do
    with TPropsSaver (PropsSaversList [i]) do
      if ALayoutId = LayoutId
        then SaveProperties;
end;

initialization
  RegisterCollectionItems (TCompInfoItem, TPropInfoItem);
  LoadSavePerformers := TList.Create;
  ClassTranslators := TList.Create;
  ComponentLoaderSavers := TList.Create;
  PropsSaversList := TList.Create;
  NonPublishedProps := TList.Create;
  LastLayouts := TStringList.Create;
  SubProperties := TList.Create;
  LastLayouts.Sorted := True;
  {$IfDef UseBasicClassTranslators}
  RegisterTranslators (TObject, ObjectToString, StringToObject);
  RegisterTranslators (TStrings, StringsToString, StringToStrings);
  RegisterTranslators (TComponent, ComponentToString, StringToComponent);
  RegisterTranslators (TCollection, CollectionToString, StringToCollection);
  RegisterSubProperties (TCollection, GetCollectionItemSubProps);
  {$EndIf}
  {$IfDef UseBasicLoadSavePerformers}
  RegisterLoadSavePerformer (TRegistryLoadSavePerformer);
  RegisterLoadSavePerformer (TIniFileLoadSavePerformer);
  {$EndIf}
  RegisterComponentSaverLoader (TObject, DefaultComponentSaver, DefaultComponentLoader);
  {$IfDef UseGridSaverLoader}
  RegisterComponentSaverLoader (TStringGrid, StringGridSaver, StringGridLoader);
  {$EndIf}
  {$IfDef UseComboBoxLoader}
  RegisterComponentSaverLoader (TCustomComboBox, ComboSaver, ComboLoader);
  {$EndIf}
  {$IfDef UseNonPublishedProps}
  RegisterNonPublishedProperty (TCustomComboBox, 'ItemIndex', ItemIndexToString, StringToItemIndex);
  {$EndIf}
finalization
  while SubProperties.Count > 0 do
    begin
      TSubProperties (SubProperties [0]).Free;
      SubProperties.Delete (0);
    end;
  SubProperties.Free;
  while LastLayouts.Count > 0 do
    begin
      (LastLayouts.Objects [0] as TLastLayout).Free;
      LastLayouts.Delete (0);
    end;
  LastLayouts.Free;
  while NonPublishedProps.Count > 0 do
    begin
      TNonPublishedProperty (NonPublishedProps [0]).Free;
      NonPublishedProps.Delete (0);
    end;
  NonPublishedProps.Free;
  PropsSaversList.Free;
  LoadSavePerformers.Free;
  while ClassTranslators.Count > 0 do
    begin
      TClassTranslator (ClassTranslators [0]).Free;
      ClassTranslators.Delete (0);
    end;
  while ComponentLoaderSavers.Count > 0 do
    begin
      TComponentSaverLoader (ComponentLoaderSavers [0]).Free;
      ComponentLoaderSavers.Delete (0);
    end;
  ComponentLoaderSavers.Free;
  ClassTranslators.Free;
end.




