unit PropsSaverBasicClassTranslators;

interface

uses
  classes, PropsSaver;

function ObjectToString (Obj : TObject; PropItem : TPropInfoItem = nil) : string;
procedure StringToObject (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil);
function StringsToString (Obj : TObject; PropItem : TPropInfoItem = nil) : string;
procedure StringToStrings (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil);
function CollectionToString (Obj : TObject; PropItem : TPropInfoItem = nil) : string;
procedure StringToCollection (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil);
function ComponentToString (Obj : TObject; PropItem : TPropInfoItem = nil): string;
procedure StringToComponent (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil);
procedure GetCollectionItemSubProps (Obj : TObject; List : TStrings);
function GetCollectionReference (Obj : TObject; const PropertyName : string) : TObject;

implementation

uses
  TypInfo, SysUtils, Forms;

type
  TStringArray = array of string;

function CheckInArray (const s : string; Strings : TStringArray) : Boolean;
var
  i : Integer;
begin
  if Strings = nil
    then
    begin
      Result := True;
      Exit;
    end;
  for i := Low (Strings) to High (Strings) do
    if s = Strings [i]
      then
      begin
        Result := True;
        Exit;
      end;
  Result := False;
end;

function ObjectToStringEx (Obj : TObject; PropItem : TPropInfoItem = nil; PropsToUse : TStringArray = nil) : string;
var
  PropList : PPropList;
  PropCount : Integer;
  i, n : Integer;
  ValuesList : TStringList;
  PropInfo : PPropInfo;
  procedure AddValue (const s : string);
  begin
    ValuesList.Add (PropList^ [i].Name + '=' + s);
  end;
begin
  if Obj <> nil
    then
    begin
      PropCount := GetTypeData (Obj.ClassInfo)^.PropCount;
      GetMem (PropList, PropCount * SizeOf (PPropInfo));
      try
        n := GetPropList (Obj.ClassInfo, tkProperties, PropList);
        ValuesList := TStringList.Create;
        try
          for i := 0 to n - 1 do
            begin
              PropInfo := GetPropInfo (Obj.ClassInfo, PropList^ [i].Name);
              if (PropInfo <> nil) and (CheckInArray (PropList^ [i].Name, PropsToUse))
                then AddValue (PropInfoToString (Obj, PropInfo, False, PropItem));
            end;
          Result := ValuesList.CommaText;
        finally
          ValuesList.Free;
        end;
      finally
        FreeMem (PropList, PropCount * SizeOf (PPropInfo));
      end;
    end
    else Result := '';
end;

procedure StringToObjectEx (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil; PropsToUse : TStringArray = nil);
var
  i : Integer;
  ValuesList : TStringList;
  PropInfo : PPropInfo;
begin
  if Obj <> nil
    then
    begin
      ValuesList := TStringList.Create;
      try
        ValuesList.CommaText := s;
        for i := 0 to ValuesList.Count - 1 do
          begin
            PropInfo := GetPropInfo (Obj.ClassInfo, ValuesList.Names [i]);
            if (PropInfo <> nil) and (CheckInArray (ValuesList.Names [i], PropsToUse))
              then AssignStringToPropInfo (ValuesList.Values [ValuesList.Names [i]], Obj, PropInfo, False, Application, False, PropItem);
          end;
      finally
        ValuesList.Free;
      end;
    end;
end;

function ObjectToString (Obj : TObject; PropItem : TPropInfoItem = nil) : string;
begin
  Result := ObjectToStringEx (Obj, PropItem);
end;

procedure StringToObject (const s : string; Obj : TObject; PropItem : TPropInfoItem = nil);
begin
  StringToObjectEx (s, Obj, PropItem);
end;

function StringsToString;
begin
  if Obj <> nil
    then Result := (Obj as TStrings).CommaText
    else Result := '';
end;

procedure StringToStrings;
begin
  if Obj <> nil
    then (Obj as TStrings).commaText := s;
end;

function ComponentToString;
var
  BinStream:TMemoryStream;
  StrStream: TStringStream;
  s: string;
  Component : TComponent;
begin
  Component := Obj as TComponent;
  BinStream := TMemoryStream.Create;
  try
    StrStream := TStringStream.Create(s);
    try
      BinStream.WriteComponent(Component);
      BinStream.Seek(0, soFromBeginning);
      ObjectBinaryToText(BinStream, StrStream);
      StrStream.Seek(0, soFromBeginning);
      Result:= StrStream.DataString;
    finally
      StrStream.Free;
    end;
  finally
    BinStream.Free
  end;
end;

procedure StringToComponent;
var
  StrStream:TStringStream;
  BinStream: TMemoryStream;
begin
  StrStream := TStringStream.Create (s);
  try
    BinStream := TMemoryStream.Create;
    try
      ObjectTextToBinary(StrStream, BinStream);
      BinStream.Seek(0, soFromBeginning);
      BinStream.ReadComponent(Obj as TComponent);
    finally
      BinStream.Free;
    end;
  finally
    StrStream.Free;
  end;
end;

procedure CreatePropsArray (PropItem : TPropInfoItem; var Result : TStringArray);
var
  i, j : Integer;
  PropName : string;
begin
  if PropItem = nil
    then
    begin
      SetLength (Result, 0);
      Exit;
    end;
  SetLength (Result, PropItem.SubProperties.Count);
  j := 0;
  for i := 0 to PropItem.SubProperties.Count - 1 do
    with PropItem.SubProperties [i] do
      begin
        if PropertyName <> ''
          then
          begin
            if PropertyName [1] in ['+', '*']
              then PropName := system.Copy (PropertyName, 2, Length (PropertyName) - 1)
              else PropName := PropertyName;
            Result [j] := PropName;
            Inc (j);
          end;
      end;
  SetLength (Result, j);
end;

function CollectionToString;
var
  i : Integer;
  ColItems : TStringList;
  SubProps : TStringArray;
begin
  if Obj <> nil
    then
    begin
      colItems := TStringList.Create;
      try
        with Obj as TCollection do
          begin
            CreatePropsArray (PropItem, SubProps);
            for i := 0 to Count - 1 do
              colItems.Add ('<' + ObjectToStringEx (Items [i], PropItem, subProps) + '>');
          end;
        Result := ColItems.CommaText;
      finally
        colItems.Free;
      end;
    end;
end;

procedure StringToCollection;
var
  i : Integer;
  ColItems : TStringList;
  SubProps : TStringArray;
begin
  if Obj <> nil
    then
    begin
      colItems := TStringList.Create;
      try
        ColItems.CommaText := s;
        with Obj as TCollection do
          begin
            Clear;
            CreatePropsArray (PropItem, SubProps);
            for i := 0 to ColItems.Count - 1 do
              StringToObjectEx (system.Copy (ColItems [i], 2, Length (ColItems [i]) - 2), Add, PropItem, subProps);
          end;
      finally
        colItems.Free;
      end;
    end;
end;

procedure GetCollectionItemSubProps (Obj : TObject; List : TStrings);
var
  i, n, PropCount : Integer;
  PropList : PPropList;
begin
  with Obj as TCollection do
    begin
      PropCount  := GetTypeData (ItemClass.ClassInfo)^.PropCount;
      GetMem (PropList, PropCount * SizeOf (PPropInfo));
      try
        n := GetPropList (ItemClass.ClassInfo, tkAny, PropList);
        for i := 0 to n - 1 do
          List.Add (PropList^ [i].Name);
      finally
        FreeMem (PropList, PropCount * SizeOf (PPropInfo));
      end;
    end;
end;

function GetCollectionReference (Obj : TObject; const PropertyName : string) : TObject;
var
  PropInfo : PPropInfo;
begin
  PropInfo := GetPropInfo (Obj.ClassInfo, PropertyName);
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass)
    then Result := GetObjectProp (Obj, PropInfo, TCollection)
    else Result := nil;
end;

end.
