unit PropsSaverBasicLoadSavePerformers;

interface

uses
  PropsSaver, Registry, IniFiles;

type
  TRegistryLoadSavePerformer = class (TLoadSavePerformer)
  private
    Reg : TRegistry;
  protected
    procedure Open (OpenMode : TOpenMode); override;
    procedure Close; override;
    procedure SaveItem (PropItem : TPropInfoItem); override;
    procedure LoadItem (PropItem : TPropInfoItem); override;
  public
    destructor Destroy; override;
  end;

  TIniFileLoadSavePerformer = class (TLoadSavePerformer)
  private
    Ini: TIniFile;
  protected
    procedure Open (OpenMode : TOpenMode); override;
    procedure Close; override;
    procedure SaveItem (PropItem : TPropInfoItem); override;
    procedure LoadItem (PropItem : TPropInfoItem); override;
  public
    destructor Destroy; override;
  end;

implementation

{ TRegistryLoadSavePerformer }

destructor TRegistryLoadSavePerformer.Destroy;
begin
  Reg.Free;
  inherited;
end;

procedure TRegistryLoadSavePerformer.Open;
begin
  Close;
  Reg := TRegistry.Create;
end;

procedure TRegistryLoadSavePerformer.Close;
begin
  Reg.Free;
  Reg := nil;
end;

procedure TRegistryLoadSavePerformer.SaveItem;
begin
  with PropItem do
    begin
      Reg.RootKey := RootKey;
      Reg.OpenKey (BaseKey, True);
      Reg.WriteString (ItemName, TranslateToString (PropItem));
    end;
end;

procedure TRegistryLoadSavePerformer.LoadItem;
var
  s : string;
begin
  with PropItem do
    begin
      Reg.RootKey := RootKey;
      Reg.OpenKey (BaseKey, True);
      if Reg.ValueExists (ItemName)
        then
        begin
          s := Reg.ReadString (ItemName);
          AssignFromString (PropItem, s);
        end;
    end;
end;

{ TIniFileLoadSavePerformer }

destructor TIniFileLoadSavePerformer.Destroy;
begin
  Ini.Free;
  inherited;
end;

procedure TIniFileLoadSavePerformer.Open;
begin
  Close;
  Ini := TIniFile.Create (PropsSaver.FileName);
end;

procedure TIniFileLoadSavePerformer.Close;
begin
  Ini.Free;
  Ini := nil;
end;

procedure TIniFileLoadSavePerformer.SaveItem;
begin
  with PropItem do
    Ini.WriteString (BaseKey, ItemName, TranslateToString (PropItem));
end;

procedure TIniFileLoadSavePerformer.LoadItem;
var
  s : string;
begin
  with PropItem do
    begin
      if Ini.ValueExists (BaseKey, ItemName)
        then
        begin
          s := Ini.ReadString (BaseKey, ItemName, TranslateToString (PropItem));
          AssignFromString (PropItem, s);
        end;
    end;
end;

end.
