unit PropsSaverComboBoxLoader;

interface

uses
  PropsSaver;

procedure ComboSaver (CompInfoItem : TCompInfoItem; Saver : TLoadSavePerformer);
procedure ComboLoader (CompInfoItem : TCompInfoItem; Loader : TLoadSavePerformer);

implementation

uses
  SysUtils, Classes, stdCtrls;

{$M+} // This is to include TypeInfo for the class TComboBoxComponent
type
  TCustomComboBoxComponent = class
  private
    ComboBox : TCustomComboBox;
    function GetItemIndex : Integer;
    procedure SetItemIndex (Value : Integer);
  published
    property ItemIndex : Integer read GetItemIndex Write SetItemIndex;
  end;
{$M-}

function TCustomComboBoxComponent.GetItemIndex;
begin
  Result := ComboBox.ItemIndex;
end;

procedure TCustomComboBoxComponent.SetItemIndex;
begin
  ComboBox.ItemIndex := Value;
end;

procedure ComboSaver (CompInfoItem : TCompInfoItem; Saver : TLoadSavePerformer);
var
  ComboComponent : TCustomComboBoxComponent;
  ComboComp : TCompInfoItem;
  ItemIndexProp : TPropInfoItem;
  function CreateProp : TPropInfoItem;
  begin
    Result := TPropInfoItem.Create (ComboComp.Properties);
    try
      Result.RootKey := CompInfoItem.RootKey;
      Result.BaseKey := CompInfoItem.BaseKey;
      Result.IfClassStoreAsLink := False;
    except
      Result.Free;
      raise;
    end;
  end;
begin
  CompInfoItem.SaveProperties (Saver);
  ComboComponent := TCustomComboBoxComponent.Create;
  try
    ComboComponent.ComboBox := CompInfoItem.Component as TCustomComboBox;
    ComboComp := TCompInfoItem.Create (CompInfoItem.Collection);
    try
      ComboComp.Component := ComboComponent;
      ItemIndexProp := CreateProp;
      try
        ItemIndexProp.ItemName := 'ItemIndex';
        ItemIndexProp.PropertyName := 'ItemIndex';
        ComboComp.SaveProperties (Saver);
      finally
        ItemIndexProp.Free;
      end;
    finally
      ComboComp.Free;
    end;
  finally
    ComboComponent.Free;
  end;
end;

procedure ComboLoader (CompInfoItem : TCompInfoItem; Loader : TLoadSavePerformer);
var
  ComboComponent : TCustomComboBoxComponent;
  ComboComp : TCompInfoItem;
  ItemIndexProp : TPropInfoItem;
  function CreateProp : TPropInfoItem;
  begin
    Result := TPropInfoItem.Create (ComboComp.Properties);
    try
      Result.RootKey := CompInfoItem.RootKey;
      Result.BaseKey := CompInfoItem.BaseKey;
      Result.IfClassStoreAsLink := False;
    except
      Result.Free;
      raise;
    end;
  end;
begin
  CompInfoItem.LoadProperties (Loader);
  ComboComponent := TCustomComboBoxComponent.Create;
  try
    ComboComponent.ComboBox := CompInfoItem.Component as TCustomComboBox;
    ComboComp := TCompInfoItem.Create (CompInfoItem.Collection);
    try
      ComboComp.Component := ComboComponent;
      ItemIndexProp := CreateProp;
      try
        ItemIndexProp.ItemName := 'ItemIndex';
        ItemIndexProp.PropertyName := 'ItemIndex';
        ComboComp.LoadProperties (Loader);
      finally
        ItemIndexProp.Free;
      end;
    finally
      ComboComp.Free;
    end;
  finally
    ComboComponent.Free;
  end;
end;

end.
