unit PropsSaverEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TwoColsEditor, Menus, ImgList, StdCtrls, Buttons, ExtCtrls,
  TwoColsComponent, PropsSaver;

type
  TProcedure = procedure of object;
  TFormPropsSaverEditor = class(TFormTwoColsEditor)
    Bevel1: TBevel;
    SpeedButtonLoadWizzard: TSpeedButton;
    SpeedButtonSaveWizzard: TSpeedButton;
    PanelSubProps: TPanel;
    BevelProperties: TBevel;
    LabelCurComp: TLabel;
    SpeedButtonBack: TSpeedButton;
    Bevel2: TBevel;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButtonSaveWizzardClick(Sender: TObject);
    procedure SpeedButtonLoadWizzardClick(Sender: TObject);
    procedure ListBoxRigthDblClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PanelButtonsPropertiesResize(Sender: TObject);
    procedure ListBoxLeftClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpeedButtonBackClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    LeftStack : TList;
    procedure ExecutePropsSaverProc (Proc : TProcedure; const WizName, AClass : string);
    procedure UpdateCurComp;
  protected
    function CurComp : TCompInfoItem;
    function GetItemCaption (Item : TCollectionItem) : string; override;
    function GetOwnerOfRigthItem (RigthItem : TcollectionItem) : TCollectionItem; override;
    function CollectionLeft : TCollection; override;
    function CollectionRigth : TCollection; override;
    procedure UpdateEnabled; override;
  public
    { Overriden IPropsSaverEditor Methods }
    procedure ItemDeleted (Item : TCollectionItem); override;
    procedure ItemAdded (Item : TCollectionItem); override;
    procedure ItemModified (Item : TCollectionItem); override;
    procedure CollectionModified (Collection : TCollection); override;
    { End of Overriden IPropsSaverEditor Methods }
  end;

var
  FormPropsSaverEditor: TFormPropsSaverEditor;

implementation

uses
  TypInfo, PropsSaverEditorWizzardName, PropsSaverWizSetsList, Registry,
  PropsSaverDsgnColTranslator, PropsSaverManageWizzards;

resourcestring
  ERR_SelectAComponent = 'You must select a component first';
  MSG_NoWizzards = 'There are no defined wizzards for this component class';

{$R *.DFM}

function TFormPropsSaverEditor.GetOwnerOfRigthItem;
begin
  Result := (RigthItem as TPropInfoItem).Collection.CompInfoItem;
end;

function TFormPropsSaverEditor.CollectionLeft : TCollection;
begin
  Result := (component as TPropsSaver).Components
end;

function TFormPropsSaverEditor.CollectionRigth : TCollection;
begin
  if LeftStack.Count <= 0
    then if (ListBoxLeft.ItemIndex >= 0) and (ListBoxLeft.Items.Count > 0)
      then Result := (ListBoxLeft.Items.Objects [ListBoxLeft.ItemIndex] as TCompInfoItem).Properties
      else Result := nil
    else Result := TcompInfoItem (LeftStack [LeftStack.Count - 1]).Properties;
end;

function TFormPropsSaverEditor.GetItemCaption;
begin
  if Item is TCompInfoItem
    then Result := inherited GetItemCaption (Item)
    else with Item as TPropInfoItem do
      Result := Format ('%d - %s', [Index, PropertyName]);
end;

procedure TFormPropsSaverEditor.FormCreate(Sender: TObject);
begin
  LeftStack := TList.Create;
  LeftItemClass := TCompInfoItem;
  RigthItemclass := TPropInfoItem;
  PanelButtonsProperties.Height := 25;
  inherited;
end;

procedure TFormPropsSaverEditor.ExecutePropsSaverProc;
var
  PropsSaver : TPropsSaver;
begin
  PropsSaver := TPropsSaver.Create (Self);
  try
    PropsSaver.LoadSavePerformerClassName := 'TRegistryLoadSavePerformer';
    PropsSaver.BaseKey := GetRegKey + '\Wizzards\' + Aclass + '\' + WizName;
    with PropsSaver.Components.Add do
      begin
        Component := CurComp;
        with Properties.Add do
          begin
            PropertyName := 'Properties';
            ItemName := 'Properties';
            IfClassStoreAsLink := False;
          end;
      end;
    TMethod (Proc).Data := PropsSaver;
    RegisterTranslators (TPropInfoCollection, PropCollectionToString, StringToPropCollection);
    try
      Proc;
    finally
      UnRegisterTranslators (TPropInfoCollection, PropCollectionToString, StringToPropCollection);
    end;
  finally
    PropsSaver.Free;
  end;
end;

procedure TFormPropsSaverEditor.SpeedButtonSaveWizzardClick(
  Sender: TObject);
var
  Method : TMethod;
begin
  inherited;
  if (CurComp = nil) or (CurComp.Component = nil)
    then raise Exception.Create (ERR_SelectAComponent);
  FormPropertiesWizzardName := TFormPropertiesWizzardName.Create (Self);
  try
    FormPropertiesWizzardName.Obj := CurComp.Component;
    if FormPropertiesWizzardName.ShowModal = mrOk
      then
      begin
        Method.Code := @TPropsSaver.SaveProperties;
        ExecutePropsSaverProc (TProcedure (Method), FormPropertiesWizzardName.EditSetName.Text,
                               FormPropertiesWizzardName.ListBoxBaseClasses.Items [FormPropertiesWizzardName.ListBoxBaseClasses.ItemIndex]);
        if FormManageWizzards <> nil
          then FormManageWizzards.RefreshWizzards;                       
      end;
  finally
    FormPropertiesWizzardName.Free;
  end;
end;

procedure TFormPropsSaverEditor.SpeedButtonLoadWizzardClick(
  Sender: TObject);
var
  Method : TMethod;
  WizName, BaseClassName : string;
begin
  inherited;
  if (CurComp = nil) or (CurComp.Component = nil)
    then raise Exception.Create (ERR_SelectAComponent);
  FormWizSetList := TFormWizSetList.Create (Self);
  try
    FormWizSetList.BaseKey := GetRegKey;
    FormWizSetList.Obj := CurComp.Component;
    if FormWizSetList.FramePropertiesWizzards.ListViewBaseClass.Items.Count <= 0
      then
      begin
        MessageDlg (MSG_NoWizzards, mtInformation, [mbOk], 0);
        Exit;
      end;
    if FormWizSetList.ShowModal = mrOk
      then
      begin
        with FormWizSetList.FramePropertiesWizzards do
          begin
            WizName := ListViewWizzards.Selected.Caption;
            BaseClassName := ListViewBaseClass.Selected.Caption;
          end;
      end
      else Exit;
  finally
    FormWizSetList.Free;
  end;
  Method.Code := @TPropsSaver.LoadProperties;
  ExecutePropsSaverProc (TProcedure (Method), WizName, BaseclassName);
end;

procedure TFormPropsSaverEditor.ListBoxRigthDblClick(Sender: TObject);
begin
  inherited;
  if ListBoxRigth.Items.Count > 0
    then
    begin
      LeftStack.Add ((SelectedRigth as TPropInfoItem).CompInfoItemSubProps);
      UpdateItemsList (ListBoxRigth, CollectionRigth);
      UpdateCurComp;
      UpdateEnabled;
    end;
end;

procedure TFormPropsSaverEditor.FormDestroy(Sender: TObject);
begin
  inherited;
  LeftStack.Free;
end;

procedure TFormPropsSaverEditor.PanelButtonsPropertiesResize(
  Sender: TObject);
begin
  inherited;
  BevelProperties.Width := PanelButtonsProperties.Width - 6;
  SpeedButtonBack.Left := PanelButtonsProperties.Width - 26;
end;

procedure TFormPropsSaverEditor.ListBoxLeftClick(Sender: TObject);
begin
  LeftStack.Clear;
  inherited;
  UpdateCurComp;
end;

procedure TFormPropsSaverEditor.FormShow(Sender: TObject);
begin
  inherited;
  UpdateCurComp;
end;

procedure TFormPropsSavereditor.UpdatecurComp;
  function BuildLabel (Comp : TCompInfoItem) : string;
  begin
    if Comp.Collection is TCompInfoCollectionSubProp
      then Result := BuildLabel ((Comp.Collection as TCompInfoCollectionSubProp).PropInfoItem.CompInfoItem) + '.' +
                     (Comp.Collection as TCompInfoCollectionSubProp).PropInfoItem.PropertyName
      else Result := Comp.DisplayName;
  end;
begin
  if curComp <> nil
    then LabelCurComp.Caption := BuildLabel (CurComp)
    else LabelCurComp.Caption := '<No item selected>';
end;

function TFormPropsSaverEditor.CurComp;
begin
  if LeftStack.Count <= 0
    then Result := SelectedLeft as TCompInfoItem
    else Result := TCompInfoItem (LeftStack [LeftStack.Count - 1]);
end;

procedure TFormPropsSaverEditor.UpdateEnabled;
begin
  inherited;
  SpeedButtonBack.Enabled := LeftStack.Count > 0;
  PanelSubProps.Visible := LeftStack.Count > 0;
  SpeedButtonSaveWizzard.Enabled := ListBoxRigth.Items.Count > 0;
  if LeftStack.Count > 0
    then PanelButtonsProperties.Height := 60
    else PanelButtonsProperties.Height := 25;
end;

procedure TFormPropsSaverEditor.SpeedButtonBackClick(Sender: TObject);
begin
  inherited;
  LeftStack.Delete (LeftStack.Count - 1);
  UpdateItemsList (ListBoxRigth, CollectionRigth);
  UpdateCurComp;
  UpdateEnabled;
end;

procedure TFormPropsSaverEditor.ItemDeleted (Item : TCollectionItem);
begin
  if (Item is TPropInfoItem) and ((Item as TPropInfoItem).CompInfoItem <> CurComp)
    then exit
    else inherited;
end;

procedure TFormPropsSaverEditor.ItemAdded (Item : TCollectionItem);
begin
  if Item is TPropInfoItem
    then if (Item as TPropInfoItem).CompInfoItem = CurComp
      then
      begin
        ListBoxRigth.Items.AddObject (GetItemCaption (Item as RigthItemClass), Item);
        UpdateEnabled;
      end  
      else exit
    else inherited;
end;

procedure TFormPropsSaverEditor.ItemModified (Item : TCollectionItem);
begin
  if (Item is TPropInfoItem) and ((Item as TPropInfoItem).CompInfoItem <> CurComp)
    then exit
    else inherited;
end;

procedure TFormPropsSaverEditor.CollectionModified (Collection : TCollection);
begin
  if (Collection is TCompInfoCollectionSubProp) and ((Collection as TCompInfoCollectionSubProp).PropInfoItem.CompInfoItem <> CurComp)
    then exit
    else inherited;
end;

procedure TFormPropsSaverEditor.SpeedButton1Click(Sender: TObject);
begin
  inherited;
  if FormManageWizzards = nil
    then
    begin
      FormManageWizzards := TFormManageWizzards.Create (Application);
      FormManageWizzards.FramePropertiesWizzards.BaseKey := GetRegKey;
      FormManageWizzards.FramePropertiesWizzards.Obj := nil;
    end;
  FormManageWizzards.Show;
end;

end.
