unit PropsSaverGridSaverLoader;

interface

uses
  PropsSaver;

procedure StringGridSaver (CompInfoItem : TCompInfoItem; Saver : TLoadSavePerformer);
procedure StringGridLoader (CompInfoItem : TCompInfoItem; Loader : TLoadSavePerformer);

implementation

uses
  SysUtils, Grids, Classes;

{$M+} // This is to include TypeInfo for the class TColumnComponent
type
  TColumnComponent = class
  private
    StringGrid : TStringGrid;
    Cols : TStringList;
    Rows : TStringList;
    function GetColWidths : TStrings;
    procedure SetColWidths (Value : TStrings);
    function GetRowHeigths : TStrings;
    procedure SetRowHeights (Value : TStrings);
  public
    destructor Destroy; override;
  published
    property ColWidths : TStrings read GetColWidths Write SetColWidths;
    property RowHeights : TStrings read GetRowHeigths Write SetRowHeights;
  end;
{$M-}

destructor TColumnComponent.Destroy;
begin
  Cols.Free;
  Rows.Free;
  inherited;
end;

function TColumnComponent.GetColWidths;
var
  i : Integer;
begin
  if Cols = nil
    then
    begin
      Cols := TStringList.Create;
      for i := 0 to StringGrid.ColCount - 1 do
        Cols.Add (IntToStr (StringGrid.ColWidths [i]));
    end;
  Result := Cols;
end;

procedure TColumnComponent.SetColWidths;
var
  i : Integer;
begin
  for i := 0 to Value.Count - 1 do
    StringGrid.ColWidths [i] := StrToInt (Value [i]);
end;

function TColumnComponent.GetRowHeigths;
var
  i : Integer;
begin
  if Rows = nil
    then
    begin
      Rows := TStringList.Create;
      for i := 0 to StringGrid.ColCount - 1 do
        Rows.Add (IntToStr (StringGrid.RowHeights [i]));
    end;
  Result := Rows;
end;

procedure TColumnComponent.SetRowHeights;
var
  i : Integer;
begin
  for i := 0 to Value.Count - 1 do
    StringGrid.RowHeights [i] := StrToInt (Value [i]);
end;

procedure StringGridSaver (CompInfoItem : TCompInfoItem; Saver : TLoadSavePerformer);
var
  ColumnComponent : TColumnComponent;
  GridComp : TCompInfoItem;
  ColProp, RowProp : TPropInfoItem;
  function CreateProp : TPropInfoItem;
  begin
    Result := TPropInfoItem.Create (GridComp.Properties);
    try
      Result.RootKey := CompInfoItem.RootKey;
      Result.BaseKey := CompInfoItem.BaseKey;
      Result.IfClassStoreAsLink := False;
    except
      Result.Free;
      raise;
    end;
  end;
begin
  CompInfoItem.SaveProperties (Saver);
  ColumnComponent := TColumnComponent.Create;
  try
    ColumnComponent.StringGrid := CompInfoItem.Component as TStringGrid;
    GridComp := TCompInfoItem.Create (CompInfoItem.Collection);
    try
      GridComp.Component := ColumnComponent;
      ColProp := CreateProp;
      try
        ColProp.ItemName := CompInfoItem.ItemName + '.ColWidths';
        ColProp.PropertyName := 'ColWidths';
        RowProp := CreateProp;
        try
          RowProp.ItemName := CompInfoItem.ItemName + '.RowHeights';
          RowProp.PropertyName := 'RowHeights';
          GridComp.SaveProperties (Saver);
        finally
          RowProp.Free;
        end;
      finally
        ColProp.Free;
      end;
    finally
      GridComp.Free;
    end;
  finally
    ColumnComponent.Free;
  end;
end;

procedure StringGridLoader (CompInfoItem : TCompInfoItem; Loader : TLoadSavePerformer);
var
  ColumnComponent : TColumnComponent;
  GridComp : TCompInfoItem;
  ColProp, RowProp : TPropInfoItem;
  function CreateProp : TPropInfoItem;
  begin
    Result := TPropInfoItem.Create (GridComp.Properties);
    try
      Result.RootKey := CompInfoItem.RootKey;
      Result.BaseKey := CompInfoItem.BaseKey;
      Result.IfClassStoreAsLink := False;
    except
      Result.Free;
      raise;
    end;
  end;
begin
  CompInfoItem.LoadProperties (Loader);
  ColumnComponent := TColumnComponent.Create;
  try
    ColumnComponent.StringGrid := CompInfoItem.Component as TStringGrid;
    GridComp := TCompInfoItem.Create (CompInfoItem.Collection);
    try
      GridComp.Component := ColumnComponent;
      ColProp := CreateProp;
      try
        ColProp.ItemName := CompInfoItem.ItemName + '.ColWidths';
        ColProp.PropertyName := 'ColWidths';
        RowProp := CreateProp;
        try
          RowProp.ItemName := CompInfoItem.ItemName + '.RowHeights';
          RowProp.PropertyName := 'RowHeights';
          GridComp.LoadProperties (Loader);
          ColumnComponent.ColWidths := ColumnComponent.ColWidths;
          ColumnComponent.Rowheights := ColumnComponent.Rowheights;
        finally
          RowProp.Free;
        end;
      finally
        ColProp.Free;
      end;
    finally
      GridComp.Free;
    end;
  finally
    ColumnComponent.Free;
  end;
end;

end.
