unit PropsSaverManageWizzards;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PropsSaverFrameWizzards, Buttons, ExtCtrls, StdCtrls, DsgnWnds, DsgnIntf,
  Menus, comCtrls;

type
  TFormManageWizzards = class(TDesignWindow)
    FramePropertiesWizzards: TFramePropertiesWizzards;
    PanelButtons: TPanel;
    ButtonRemoveclass: TSpeedButton;
    ButtonRemoveWizzards: TSpeedButton;
    PopupMenuClasses: TPopupMenu;
    Removeallwizzardsforselectedclass1: TMenuItem;
    PopupMenuWizzards: TPopupMenu;
    Removeselectedwizzard1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FramePropertiesWizzardsListBoxBaseClassClick(
      Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonRemoveclassClick(Sender: TObject);
    procedure ButtonRemoveWizzardsClick(Sender: TObject);
    procedure FramePropertiesWizzardsListViewBaseClassEdited(
      Sender: TObject; Item: TListItem; var S: String);
    procedure FramePropertiesWizzardsListViewWizzardsEdited(
      Sender: TObject; Item: TListItem; var S: String);
  private
    procedure UpdateEnabled;
    procedure ChangeKey (const Key, NewKey : string);
  protected
    function UniqueName(Component: TComponent): string; override;
  public
    procedure RefreshWizzards;
  end;

var
  FormManageWizzards: TFormManageWizzards;

implementation

uses
  Registry, TwoColsEditor;

{$R *.DFM}

procedure TFormManageWizzards.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
  with TRegIniFile.Create(TFormTwoColsEditor.GetRegKey) do
    try
      WriteInteger ('Wizzards Manager', 'Width', Width);
      WriteInteger ('Wizzards Manager', 'Height', Height);
      WriteInteger ('Wizzards Manager', 'Top', top);
      WriteInteger ('Wizzards Manager', 'Left', Left);
      WriteInteger ('Wizzards Manager', 'PanelTop', FramePropertiesWizzards.Panel1.Height);
    finally
      Free;
    end;
end;

procedure TFormManageWizzards.FormDestroy(Sender: TObject);
begin
  FormManageWizzards := nil;
end;

procedure TFormManageWizzards.FormShow(Sender: TObject);
begin
  if FramePropertiesWizzards.ListViewWizzards.Items.Count > 0
    then FramePropertiesWizzards.ListViewWizzards.Selected := FramePropertiesWizzards.ListViewWizzards.Items [0];
  UpdateEnabled;
end;

procedure TFormManageWizzards.UpdateEnabled;
begin
  with FramePropertiesWizzards do
    begin
      ButtonRemoveclass.Enabled := (ListViewBaseClass.Items.Count > 0) and (ListViewBaseClass.Selected <> nil);
      Removeallwizzardsforselectedclass1.Enabled := ButtonRemoveclass.Enabled;
      ButtonRemoveWizzards.Enabled := (ListViewWizzards.Items.Count > 0) and (ListViewWizzards.Selected <> nil);
      Removeselectedwizzard1.Enabled := ButtonRemoveWizzards.Enabled;
    end;
end;

procedure TFormManageWizzards.FramePropertiesWizzardsListBoxBaseClassClick(
  Sender: TObject);
begin
  FramePropertiesWizzards.ListViewBaseClassClick(Sender);
  UpdateEnabled;
end;

procedure TFormManageWizzards.FormCreate(Sender: TObject);
begin
  with TRegIniFile.Create(TFormTwoColsEditor.GetRegKey) do
    try
      Width := ReadInteger ('Wizzards Manager', 'Width', Width);
      Height := ReadInteger ('Wizzards Manager', 'Height', Height);
      Top := ReadInteger ('Wizzards Manager', 'Top', top);
      Left := ReadInteger ('Wizzards Manager', 'Left', Left);
      FramePropertiesWizzards.Panel1.Height := ReadInteger ('Wizzards Manager', 'PanelTop', FramePropertiesWizzards.Panel1.Height);
    finally
      Free;
    end;
end;

procedure TFormManageWizzards.ButtonRemoveclassClick(Sender: TObject);
var
  Reg : TRegistry;
begin
  Reg := TRegistry.Create;
  try
    with FramePropertiesWizzards do
      with ListViewBaseClass do
        Reg.DeleteKey (BaseKey + '\Wizzards\' + Selected.Caption);
  finally
    Reg.Free;
  end;
  RefreshWizzards;
end;

procedure TFormManageWizzards.ButtonRemoveWizzardsClick(Sender: TObject);
var
  Reg : TRegistry;
begin
  Reg := TRegistry.Create;
  try
    with FramePropertiesWizzards do
      with ListViewBaseClass do
        Reg.DeleteKey (BaseKey + '\Wizzards\' + Selected.Caption + '\' + ListViewWizzards.Selected.Caption);
  finally
    Reg.Free;
  end;
  RefreshWizzards;
end;

function TFormManageWizzards.UniqueName;
begin
  Result := '';
end;

procedure TFormManageWizzards.RefreshWizzards;
begin
  FramePropertiesWizzards.Obj := nil;
end;

procedure TFormManageWizzards.ChangeKey;
var
  Reg : TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.MoveKey (Key, NewKey, True);
  finally
    Reg.Free;
  end;
end;

procedure TFormManageWizzards.FramePropertiesWizzardsListViewBaseClassEdited(
  Sender: TObject; Item: TListItem; var S: String);
begin
  with FramePropertiesWizzards do
    ChangeKey (BaseKey + '\Wizzards\' + Item.Caption, BaseKey + '\Wizzards\' + s);
end;

procedure TFormManageWizzards.FramePropertiesWizzardsListViewWizzardsEdited(
  Sender: TObject; Item: TListItem; var S: String);
begin
  with FramePropertiesWizzards do
    ChangeKey (BaseKey + '\Wizzards\' + ListViewBaseClass.Selected.Caption  + '\' + Item.Caption,
               BaseKey + '\Wizzards\' + ListViewBaseClass.Selected.Caption  + '\' + s);
end;

end.
