{
 Project UML Component Factory
 Unit FMainTestElementDynamiLoading.pas

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with the
 License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 the specific language governing rights and limitations under the License.

 The Original Code is FMainTestElementDynamiLoading.pas

 The Initial Developer of the Original Code is Thierry.Coq
 Portions created by this individual are Copyright (C)
 2000 of this individual.

 Last modified: 25 oct 2000
 Version 1.0

 purpose :
 this form allows to test the Dynamic Element Library. It shows :
 - how to load dynamically a package,
 - get the element classes contained in it,
 - get the units,
 - create instances of elements and use them,
 - keep the element count,
 - unload the package after use.
}
unit FMainTestElementDynamiLoading;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, UCFNote, ExtCtrls, Menus;

type
  TFormTestElementDynamicLoading = class(TForm)
    EditPackageName: TEdit;
    Label2: TLabel;
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    Button2: TButton;
    LBElements: TListBox;
    Label3: TLabel;
    Label4: TLabel;
    LblPackageName: TLabel;
    LBUnits: TListBox;
    Label5: TLabel;
    UCFNote1: TUCFNote;
    Button3: TButton;
    MemoDescription: TMemo;
    Label1: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    LBElementCount: TListBox;
    PopupMenu1: TPopupMenu;
    CreateAnInstance1: TMenuItem;
    FreeAllInstances1: TMenuItem;
    Label10: TLabel;
    Button4: TButton;
    BtnLoadingRegisteredPackages: TButton;
    BtnUnloadRegisteredPackages: TButton;
    Bevel1: TBevel;
    BtnRegister: TButton;
    EdtEltName: TEdit;
    Label11: TLabel;
    Label12: TLabel;
    BtnUnregister: TButton;
    Bevel2: TBevel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure LBElementsClick(Sender: TObject);
    procedure CreateAnInstance1Click(Sender: TObject);
    procedure FreeAllInstances1Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure BtnLoadingRegisteredPackagesClick(Sender: TObject);
    procedure BtnRegisterClick(Sender: TObject);
    procedure BtnUnloadRegisteredPackagesClick(Sender: TObject);
  private
    { Dclarations prives }
    procedure ListElementsAvailable;
    procedure LoadPackage;
    procedure RefreshInstanceCount;
  public
    { Dclarations publiques }
  end;

var
  FormTestElementDynamicLoading: TFormTestElementDynamicLoading;

implementation

uses SysUtils, DynamicLibraryElement, PackageUtilities;

{$R *.DFM}

procedure TFormTestElementDynamicLoading.Button1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    EditPackageName.Text := OpenDialog1.FileName;
    LblPackageName.Caption := ExtractFileName( OpenDialog1.FileName);
  end;
end;

procedure TFormTestElementDynamicLoading.Button2Click(Sender: TObject);
begin
  LoadPackage;
end;

procedure TFormTestElementDynamicLoading.ListElementsAvailable;
var iElt  : integer;
    anElt : TDynamicLibraryElementClass;
    pName : String;
    nbr   : integer;
begin
  LBElements.Clear;
  nbr := DLEManager.Count;
  for iElt := 0 to nbr-1 do
  begin
    anElt := DLEManager.Elements[ iElt];
    pName := DLEManager.PackageNames[ iElt];
    LBElements.Items.add( anElt.ClassName + ' ('+ pName+ ')');
  end;
  RefreshInstanceCount;
end;

procedure TFormTestElementDynamicLoading.LoadPackage;
begin
  DLEManager.LoadPackage( EditPackageName.Text, LBUnits.Items);
  ListElementsAvailable;
end;

procedure TFormTestElementDynamicLoading.Button3Click(Sender: TObject);
var
  iElt      : integer;
  anElt     : TDynamicLibraryElement;
  aDLEClass : TDynamicLibraryElementClass;
begin
  if LBElements.ItemIndex < 0 then Exit;
  iElt := LBElements.ItemIndex;
  aDLEClass := DLEManager.Elements[ iElt];
  anElt     := aDLEClass.Create( self);
  try
    anElt.Execute;
  finally
    FreeAndNil( anElt);
  end;
end;

procedure TFormTestElementDynamicLoading.LBElementsClick(Sender: TObject);
var
  iElt      : integer;
  anElt     : TDynamicLibraryElement;
  aDLEClass : TDynamicLibraryElementClass;
begin
  if LBElements.ItemIndex < 0 then Exit;
  iElt := LBElements.ItemIndex;
  aDLEClass := DLEManager.Elements[ iElt];
  MemoDescription.Lines.CommaText := aDLEClass.GetDescription;
end;

procedure TFormTestElementDynamicLoading.CreateAnInstance1Click(Sender: TObject);
var
  iElt      : integer;
  anElt     : TDynamicLibraryElement;
  aDLEClass : TDynamicLibraryElementClass;
begin
  if LBElements.ItemIndex < 0 then Exit;
  iElt := LBElements.ItemIndex;
  aDLEClass := DLEManager.Elements[ iElt];
  anElt     := aDLEClass.Create( self);
  RefreshInstanceCount;
end;

procedure TFormTestElementDynamicLoading.RefreshInstanceCount;
var
  iElt      : integer;
  anElt     : TDynamicLibraryElement;
  aDLEClass : TDynamicLibraryElementClass;
begin
  LBElementCount.Clear;
  for iElt := 0 to LBElements.Items. Count-1 do
  begin
    aDLEClass := DLEManager.Elements[ iElt];
    LBElementCount.Items[ iElt] := IntToStr( aDLEClass.InstanceCount);
  end;
end;

procedure TFormTestElementDynamicLoading.FreeAllInstances1Click(Sender: TObject);
var
  iElt      : integer;
  anElt     : TDynamicLibraryElement;
  aDLEClass : TDynamicLibraryElementClass;
begin
  if LBElements.ItemIndex < 0 then Exit;
  iElt := LBElements.ItemIndex;
  aDLEClass := DLEManager.Elements[ iElt];
  aDLEClass.FreeAllInstances( aDLEClass);
  RefreshInstanceCount;
end;

procedure TFormTestElementDynamicLoading.Button4Click(Sender: TObject);
begin
  DLEManager.UnloadPackage( EditPackageName.Text);
  ListElementsAvailable;
end;

procedure TFormTestElementDynamicLoading.BtnLoadingRegisteredPackagesClick(
  Sender: TObject);
begin
  DLEManager.LoadRegisteredPackages;
  ListElementsAvailable;

end;

procedure TFormTestElementDynamicLoading.BtnRegisterClick(Sender: TObject);
begin
  DLEManager.RegisterDLEIntoRegistry( EdtEltName.Text,
                                                 EditPackageName.Text);
end;

procedure TFormTestElementDynamicLoading.BtnUnloadRegisteredPackagesClick(
  Sender: TObject);
begin
  DLEManager.UnregisterDLE( EdtEltName.Text);

end;

end.
