(*+++++ Transparent Windows +++++++ V 1.0 ++++++++++++++++++++++++++++++++++++++

For learnig, testing and commercial use

?***********************************************?
? by Michael Glitzner (michael.glitzner@aon.at) ?
?***********************************************?

* ABOUT THIS COMPONENT *
This component should capsule the new Windows 2000 API calls for Alpha Blending
(transparency). I think you can use this component in nearly any Delphi
(tested in BD5) version with Win2000 or higher (e.g. Windows Whistler).
Install this component in Delphi 5 with |COMPONENT|INSTALL COMPONENT|. Then you
have to choose the "transparentWnd.pas" file. After this the component is
installed in |WIN32|.
Only drop the component on the Form and use the prefered method to set
transparency.


Derived from:
TComponent

Properties:
Percent
= Percentage of transparency ( only in use with SetTransparent() )

Inherited properties:
Properties from TComponent

Methods:
SetTransparent()
= Set the transparency of the parent window to the property percent.

SetTransparent(percent)
= Set the transparency of the parent window to percent(variable).

SetTransparentHWND(hwnd, percent)
= Set the transparency of the window with the hwnd to percent(variable).

Events:
Events from TComponent
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)

unit TransparentWnd;

interface

uses
  Windows, Messages, Classes, Controls, Forms;

type
  TTransparentWnd = class(TComponent)
  private
    { Private declarations }
  protected
    _percent : shortint;
    { Protected declarations }
  public
    procedure SetTransparent(percent : shortint); overload;
    procedure SetTransparentHWND(hwnd: longint; percent : shortint);
    procedure SetTransparent; overload;
    { Public declarations }
  published
    property Percent: shortint read _percent write _percent default 0;
    { Published declarations }
  end;

  function SetLayeredWindowAttributes(hwnd: longint; crey: byte; bAlpha: byte; dwFlags: longint): longint; stdcall; external 'USER32.DLL';
  function SetWindowLongA(hwnd: longint; nIndex: longint; dwNewLong: longint): longint; stdcall; external 'USER32.DLL';
  function GetWindowLongA(hwnd: longint; nIndex: longint): longint; stdcall; external 'USER32.DLL';

  procedure Register;

implementation

const LWA_ALPHA = $2;
const GWL_EXSTYLE = (-20);
const WS_EX_LAYERED = $80000;
const WS_EX_TRANSPARENT = $20;

procedure TTransparentWnd.SetTransparent;
begin
  Self.SetTransparentHWND((Self.Owner as TForm).Handle,100-Self._percent);
end;


procedure TTransparentWnd.SetTransparentHWND(hwnd: longint; percent : shortint);
var old: longint;
begin
  old:=GetWindowLongA(hwnd,GWL_EXSTYLE);
  SetWindowLongA(hwnd,GWL_EXSTYLE,old or WS_EX_LAYERED);
  SetLayeredWindowAttributes(hwnd, 0, (255 * percent) DIV 100, LWA_ALPHA);
end;

procedure TTransparentWnd.SetTransparent(percent: shortint);
begin
  Self.SetTransparentHWND((Self.Owner as TForm).Handle,100-percent);
end;

procedure Register;
begin
  RegisterComponents('Win32', [TTransparentWnd]);
end;

end.
