{ Pattern for component that has a collection and that each item of that collection
  has also a collection }
  
unit TwoColsComponent;

interface

uses
  Classes, TwoColsIntf;

type
  TTwoColsComponent = class;
  TTwoColsCollectionItem = class (TCollectionItem)
  protected
    function AllowNotification : Boolean; virtual;
    function GetTwoColsComponent : TTwoColsComponent; virtual; abstract;
    procedure SetIndex (Value: Integer); override;
  public
    constructor Create (ACollection : TCollection); override;
    destructor Destroy; override;
    property TwoColsComponent : TTwoColsComponent read GetTwoColsComponent;
  end;

  TTwoColsComponent = class (TComponent)
  private
    FEditorForm : ITwoColsEditor;
  public
    property EditorForm : ITwoColsEditor read FEditorForm Write FEditorForm;
  end;

implementation

{ TLayoutCollectionItem }

constructor TTwoColsCollectionItem.Create;
begin
  inherited;
  if (Collection <> nil) and (TwoColsComponent.EditorForm <> nil) and (AllowNotification)
    then TwoColsComponent.EditorForm.ItemAdded (Self);
end;

destructor TTwoColsCollectionItem.Destroy;
begin
  if (Collection <> nil) and (TwoColsComponent.EditorForm <> nil) and AllowNotification
    then TwoColsComponent.EditorForm.ItemDeleted (Self);
  inherited;
end;

function TTwoColsCollectionItem.AllowNotification;
begin
  Result := True;
end;

procedure TTwoColsCollectionItem.SetIndex;
begin
  inherited;
  if (Collection <> nil) and (TwoColsComponent.EditorForm <> nil) and AllowNotification
    then TwoColsComponent.EditorForm.CollectionModified (Collection);
end;

end.
