unit TwoColsEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, LibIntf, DsgnWnds, Buttons, StdCtrls, ExtCtrls,
  Menus, ImgList, TwoColsIntf;

const
  WM_UpdateListBox = WM_USER + 100;
  WM_SelectItem = WM_USER + 101;

type
  TCollectionItemClass = class of TCollectionItem;
  TFormTwoColsEditorClass = class of TFormTwoColsEditor;

  TFormTwoColsEditor = class(TDesignWindow, ITwoColsEditor)
    PanelComponents: TPanel;
    Splitter1: TSplitter;
    PanelProperties: TPanel;
    PanelButtonsComponents: TPanel;
    PanelButtonsProperties: TPanel;
    ListBoxLeft: TListBox;
    ListBoxRigth: TListBox;
    ButtonAddComponent: TSpeedButton;
    ButtonRemoveComponent: TSpeedButton;
    ButtonMoveUpComponent: TSpeedButton;
    ButtonMoveDownComponent: TSpeedButton;
    ButtonAddProperty: TSpeedButton;
    ButtonRemoveProperty: TSpeedButton;
    ButtonMoveUpProperty: TSpeedButton;
    ButtonMoveDownProperty: TSpeedButton;
    ImageList: TImageList;
    PopupMenuComponents: TPopupMenu;
    PopupMenuProperties: TPopupMenu;
    Addcomponent1: TMenuItem;
    Removeselectedcomponents1: TMenuItem;
    Moveup1: TMenuItem;
    Movedown1: TMenuItem;
    N1: TMenuItem;
    Selectall1: TMenuItem;
    Addproperty1: TMenuItem;
    Removeselectedproperties1: TMenuItem;
    Moveup2: TMenuItem;
    Movedown2: TMenuItem;
    N2: TMenuItem;
    Selectall2: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListBoxLeftClick(Sender: TObject);
    procedure ListBoxRigthClick(Sender: TObject);
    procedure ButtonAddComponentClick(Sender: TObject);
    procedure ButtonRemoveComponentClick(Sender: TObject);
    procedure ButtonRemovePropertyClick(Sender: TObject);
    procedure ButtonAddPropertyClick(Sender: TObject);
    procedure Selectall1Click(Sender: TObject);
    procedure PopupMenuComponentsPopup(Sender: TObject);
    procedure PopupMenuPropertiesPopup(Sender: TObject);
    procedure ButtonMoveUpComponentClick(Sender: TObject);
    procedure ButtonMoveDownComponentClick(Sender: TObject);
    procedure ButtonMoveUpPropertyClick(Sender: TObject);
    procedure ButtonMoveDownPropertyClick(Sender: TObject);
    procedure ListBoxDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ListBoxEndDrag(Sender, Target: TObject; X,
      Y: Integer);
    procedure ListBoxMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ListBoxDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure ListBoxLeftKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBoxRigthKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    FComponent : TComponent;
    Selecting : Boolean;
    Pasting : Boolean;
    ItemSelected : integer;
    OriginalItem : integer;
    FLeftItemClass, FRigthItemClass : TCollectionItemClass;
    procedure UpdateSelectionItems (lb : TListbox);
    procedure WMUpdateListBox (var Msg : TMessage); message WM_UpdateListBox;
    procedure Cut;
    procedure Copy;
    procedure Paste;
    procedure SelectAll;
    function GetActiveListBox : TListBox;
    procedure RemoveItems (ListBox : TListBox);
    procedure UpdateSelection;
    procedure DeselectAll (ListBox : TListbox);
    procedure MoveItem (Item : TcollectionItem; where : integer);
    procedure WMSelectItem (var Msg : TMessage); message WM_SelectItem;
  protected
    function GetItemCaption (Item : TCollectionItem) : string; virtual;
    function GetOwnerOfRigthItem (rigthItem : TcollectionItem) : TCollectionItem; virtual; abstract;
    function CollectionLeft : TCollection; virtual; abstract;
    function CollectionRigth : TCollection; virtual; abstract;
    function SelectedLeft : TCollectionItem; virtual;
    function SelectedRigth : TCollectionItem; virtual;
    procedure UpdateItemsList (lb : TListBox; Col : Tcollection);
    procedure UpdateEnabled; virtual;
  public
    class function GetRegKey: string;
    procedure ComponentDeleted(AComponent: IPersistent); override;
    procedure SelectionChanged(ASelection: TDesignerSelectionList); override;
    procedure FormModified; override;
    function GetEditState: TEditState; override;
    function UniqueName(Component: TComponent): string; override;
    procedure EditAction (Action : TEditAction); override;
    { IPropsSaverEditor Methods }
    procedure ItemDeleted (Item : TCollectionItem); virtual;
    procedure ItemAdded (Item : TCollectionItem); virtual;
    procedure ItemModified (Item : TCollectionItem); virtual;
    procedure CollectionModified (Collection : TCollection); virtual;
    { End of IPropsSaverEditor Methods }
    property Component : TComponent read FComponent Write FComponent;
    property LeftItemClass : TCollectionItemClass read FLeftItemClass write FLeftItemClass;
    property RigthItemClass : TCollectionItemClass read FRigthItemClass write FRigthItemClass;
  end;

procedure ShowTwoColsEditor (ADesigner: IFormDesigner; AComp : TComponent; EditorClass : TFormTwoColsEditorClass);

implementation

uses
  TypInfo, Registry, TwoColsComponent;

var
  TwoColsEditors : TList;
  TwoColsClipboard : TList;

{$R *.DFM}

procedure ShowTwoColsEditor;
var
  i : integer;
begin
  for i := 0 to TwoColsEditors.Count - 1 do
    if TFormTwoColsEditor (TwoColsEditors [i]).Component = AComp
      then
      begin
        TFormTwoColsEditor (TwoColsEditors [i]).Show;
        TFormTwoColsEditor (TwoColsEditors [i]).BringToFront;
        Exit;
      end;
  with EditorClass.Create (Application) do
    begin
      try
        Designer := ADesigner;
        Component := AComp;
        Show;
      except
        Free;
        raise;
      end;
    end;
end;

{ TFormPropsSaverEditor }

procedure TFormTwoColsEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TFormTwoColsEditor.SelectionChanged (ASelection: TDesignerSelectionList);
var
  i, j : integer;
begin
  if Selecting
    then Exit;
  for i := 0 to ListBoxLeft.Items.Count - 1 do
    ListBoxLeft.Selected [i] := False;
  for i := 0 to ListBoxRigth.Items.Count - 1 do
    ListBoxRigth.Selected [i] := False;
  j := -1;
  for i := 0 to ASelection.Count - 1 do
    begin
      j := ListBoxLeft.Items.IndexOfObject (ASelection [i]);
      if j >= 0
        then ListBoxLeft.selected [j] := True;
    end;
  if j = -1
    then
    begin
      for i := 0 to ASelection.Count - 1 do
        begin
          j := ListBoxRigth.Items.IndexOfObject (ASelection [i]);
          if j >= 0
            then ListBoxRigth.selected [j] := True;
        end;
    end
    else if ListBoxLeft.SelCount = 1
      then
      begin
        ListBoxLeft.ItemIndex := j;
        ListBoxLeftClick (ListBoxLeft);
      end;
end;

procedure TFormTwoColsEditor.ComponentDeleted(AComponent: IPersistent);
begin
  if ExtractPersistent (AComponent) = Component
    then
    begin
      FComponent := nil;
      Close;
    end;
end;

procedure TFormTwoColsEditor.FormCreate(Sender: TObject);
begin
  TwoColsEditors.Add (Self);
  ItemSelected := -1;
end;

procedure TFormTwoColsEditor.FormDestroy(Sender: TObject);
begin
  TwoColsEditors.Remove (Self);
  TwoColsClipboard.Free;
  TwoColsClipboard := nil;
  with TRegIniFile.Create(GetRegKey) do
    try
      WriteInteger(Component.Name, 'Width', Width);
      WriteInteger(Component.Name, 'Height', Height);
      WriteInteger(Component.Name, 'Top', top);
      WriteInteger(Component.Name, 'Left', Left);
      WriteInteger (Component.Name, 'PanelComponentsWidth', PanelComponents.Width);
    finally
      Free;
    end;
  (Component as TTwoColsComponent).EditorForm := nil;
end;

procedure TFormTwoColsEditor.FormModified;
begin
  if Component <> nil
    then Caption := Component.Name;
end;

procedure TFormTwoColsEditor.FormShow(Sender: TObject);
begin
  (Component as TTwoColsComponent).EditorForm := Self;
  with TRegIniFile.Create(GetRegKey) do
    try
      Width := ReadInteger(Component.Name, 'Width', Width);
      Height := ReadInteger(Component.Name, 'Height', Height);
      top := ReadInteger(Component.Name, 'Top', top);
      Left := ReadInteger(Component.Name, 'Left', Left);
      PanelComponents.Width := ReadInteger (Component.Name, 'PanelComponentsWidth', PanelComponents.Width);
    finally
      Free;
    end;
  FormModified;
  UpdateItemsList (ListBoxLeft, CollectionLeft);
  UpdateItemsList (ListBoxRigth, CollectionRigth);
end;

procedure TFormTwoColsEditor.UpdateSelectionItems;
var
  i : Integer;
  ComponentList: TDesignerSelectionList;
begin
  ComponentList := TDesignerSelectionList.Create;
  try
    for i := 0 to lb.Items.Count - 1 do
      if lb.selected [i]
        then ComponentList.Add (lb.Items.Objects [i] as TCollectionItem);
    if ComponentList.Count = 0
      then ComponentList.Add(Component);
  except
    ComponentList.Free;
    raise;
  end;
  Selecting := True;
  try
    SetSelection(ComponentList);
  finally
    Selecting := False;
  end;
end;

procedure TFormTwoColsEditor.UpdateItemsList;
var
  i : integer;
begin
  if Col = nil
    then Exit;
  lb.Items.BeginUpdate;
  try
    lb.Items.Clear;
    if Component <> nil
      then
      begin
        for i := 0 to Col.Count - 1 do
          lb.Items.AddObject (GetItemCaption (TCollectionItem (col.Items [i])), col.Items [i]);
      end;
  finally
    lb.Items.EndUpdate;
  end;
end;

procedure TFormTwoColsEditor.DeselectAll;
var
  i, j : integer;
begin
  j := ListBox.ItemIndex;
  for i := 0 to ListBox.Items.Count - 1 do
    ListBox.Selected [i] := False;
  if (ListBox.ItemIndex >= 0) and (ListBox.Items.Count > 0)
    then ListBox.Selected [ListBox.ItemIndex] := True;
  ListBox.ItemIndex := j;
end;

procedure TFormTwoColsEditor.ListBoxLeftClick(Sender: TObject);
begin
  DeselectAll (ListBoxRigth);
  UpdateSelectionItems (ListBoxLeft);
  UpdateItemsList (ListBoxRigth, CollectionRigth);
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.ListBoxRigthClick(Sender: TObject);
begin
  Deselectall (ListBoxLeft);
  UpdateSelectionItems (ListBoxrigth);
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.ItemDeleted;
var
  i : integer;
begin
  TwoColsClipboard.Free;
  TwoColsClipboard := nil;
  if Item is LeftItemClass
    then for i := 0 to ListBoxLeft.Items.Count - 1 do
      if ListBoxLeft.Items.Objects [i] = Item
        then
        begin
          ListBoxLeft.Items.Delete (i);
          Exit;
        end
        else {}
    else for i := 0 to ListBoxRigth.Items.Count - 1 do
      if ListBoxRigth.Items.Objects [i] = Item
        then
        begin
          ListBoxRigth.Items.Delete (i);
          Exit;
        end;
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.ItemAdded (Item : TCollectionItem);
begin
  if not Pasting
    then
    begin
      TwoColsClipboard.Free;
      TwoColsClipboard := nil;
    end;
  if Item is LeftItemClass
    then ListBoxLeft.Items.AddObject (GetItemCaption (Item as LeftItemClass), Item)
    else if (ListBoxLeft.ItemIndex >= 0) and
            (GetOwnerOfRigthItem (Item as RigthItemClass) = ListBoxLeft.Items.Objects [ListBoxLeft.ItemIndex])
      then ListBoxRigth.Items.AddObject (GetItemCaption (Item as RigthItemClass), Item);
  UpdateEnabled;
end;

function TFormTwoColsEditor.GetItemCaption;
begin
  with Item do
    Result := Format ('%d - %s', [Index, DisplayName]);
end;

procedure TFormTwoColsEditor.ItemModified (Item : TCollectionItem);
var
  i : integer;
begin
  TwoColsClipboard.Free;
  TwoColsClipboard := nil;
  if Item is LeftItemClass
    then for i := 0 to ListBoxLeft.Items.Count - 1 do
      if ListBoxLeft.Items.Objects [i] = Item
        then
        begin
          ListBoxLeft.Items [i] := GetItemCaption (Item as LeftItemClass);
          Exit;
        end
        else {}
    else for i := 0 to ListBoxRigth.Items.Count - 1 do
      if ListBoxRigth.Items.Objects [i] = Item
        then
        begin
          ListBoxRigth.Items [i] := GetItemCaption (Item as RigthItemClass);
          Exit;
        end;
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.CollectionModified (Collection : TCollection);
begin
  TwoColsClipboard.Free;
  TwoColsClipboard := nil;
  if Collection.ItemClass = LeftItemClass
    then PostMessage (Handle, WM_UpdateListBox, 0, 0)
    else PostMessage (Handle, WM_UpdateListBox, 0, 1);
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.WMUpdateListBox;
begin
  case Msg.lParam of
    0 : UpdateItemsList (ListBoxLeft, CollectionLeft);
    1 : UpdateItemsList (ListBoxRigth, CollectionRigth);
  end;
  UpdateEnabled;
end;

function TFormTwoColsEditor.GetEditState;
  function ListBoxSelected (ListBox : TListBox) : Boolean;
  var
    i : integer;
  begin
    Result := False;
    for i := 0 to Listbox.Items.Count - 1 do
      if ListBox.Selected [i]
        then
        begin
          Result := True;
          Exit;
        end;
  end;
begin
  if GetActiveListBox.Items.Count > 0
    then Result := [esCanSelectAll]
    else Result := [];
  if TwoColsClipboard <> nil
    then Result := [esCanPaste];
  if ListBoxSelected (ListBoxLeft) or ListBoxSelected(ListBoxRigth)
    then Result := Result + [esCanCopy, esCanCut, esCanDelete];
end;

function TFormTwoColsEditor.UniqueName;
begin
  Result := '';
end;

function TFormTwoColsEditor.GetActiveListbox;
begin
  if ActiveControl is TListBox
    then Result := ActiveControl as TListbox
    else Result := ListBoxLeft
end;

procedure TFormTwoColsEditor.EditAction;
begin
  case Action of
    eaCut: Cut;
    eaCopy: Copy;
    eaPaste: Paste;
    eaDelete: RemoveItems (GetActiveListbox);
    eaSelectAll:
      begin
        SelectAll;
        UpdateSelection;
      end;
  end;
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.Cut;
begin
  Copy;
  RemoveItems (GetActiveListbox);
end;

procedure TFormTwoColsEditor.Copy;
var
  I : Integer;
  ComponentList : TList;
begin
  TwoColsClipboard.Free;
  TwoColsClipboard := nil;
  ComponentList := TList.Create;
  try
    with GetActiveListBox do
      for I := 0 to Items.Count - 1 do
        if Selected[I]
          then ComponentList.Add (TCollectionItem (Items.Objects[I]));
    TwoColsClipboard := ComponentList;
  except
    ComponentList.Free;
    raise;
  end;
end;

procedure TFormTwoColsEditor.Paste;
var
  i : Integer;
begin
  Pasting := True;
  try
    if TwoColsClipboard = nil
      then Exit;
    if GetActivelistBox = ListBoxLeft
      then
      begin
        for i := 0 to TwoColsClipboard.Count - 1 do
          with CollectionLeft.Add do
            Assign (TCollectionItem (TwoColsClipboard [i]));
      end
      else
      begin
        if SelectedLeft <> nil
          then for i := 0 to TwoColsClipboard.Count - 1 do
            with CollectionRigth.Add do
              Assign (TCollectionItem (TwoColsClipboard [i]));
      end;
  finally
    Pasting := False;
  end;
end;

procedure TFormTwoColsEditor.SelectAll;
var
  i : integer;
begin
  for i := 0 to GetActiveListBox.Items.Count - 1 do
    GetActiveListBox.selected [i] := True;
  UpdateSelection;
end;

procedure TFormTwoColsEditor.RemoveItems;
var
  i : integer;
begin
  for i := GetActiveListBox.Items.Count - 1 downto 0 do
    if GetActiveListBox.Selected [i]
      then GetActiveListBox.Items.Objects [i].Free;
  UpdateSelection;
end;

procedure TFormTwoColsEditor.UpdateSelection;
begin
  if GetActiveListBox = ListBoxLeft
    then UpdateSelectionItems (ListBoxLeft)
    else UpdateSelectionItems (ListBoxRigth);
end;

procedure TFormTwoColsEditor.UpdateEnabled;
var
  i : integer;
  procedure SetEnabled (comp : TComponent; Value: Boolean);
  var
    PropInfo : PPropInfo;
  begin
    PropInfo := GetPropInfo (comp.classinfo, 'Enabled');
    if PropInfo <> nil
      then SetOrdProp (comp, PropInfo, integer (Value));
  end;
begin
  for i := 0 to ComponentCount - 1 do
    case components [i].Tag of
      1 : SetEnabled (components [i], (ListBoxLeft.ItemIndex >= 0) and (ListBoxLeft.Items.Count > 0));
      2 : SetEnabled (components [i], (ListBoxLeft.ItemIndex > 0) and (ListBoxLeft.Items.Count > 0) and (ListBoxLeft.SelCount > 0));
      3 : SetEnabled (components [i], (ListBoxLeft.ItemIndex >= 0) and (ListBoxLeft.Items.Count > 0) and (ListBoxLeft.SelCount > 0) and
                                      (ListBoxLeft.ItemIndex < ListBoxLeft.Items.Count - 1));
      4 : SetEnabled (components [i], (ListBoxRigth.ItemIndex >= 0) and (ListBoxRigth.Items.Count > 0) and (ListBoxRigth.SelCount > 0));
      5 : SetEnabled (components [i], (ListBoxRigth.ItemIndex > 0) and (ListBoxRigth.Items.Count > 0) and (ListBoxRigth.SelCount > 0));
      6 : SetEnabled (components [i], (ListBoxRigth.ItemIndex >= 0) and (ListBoxRigth.Items.Count > 0) and (ListBoxRigth.SelCount > 0) and
                                      (ListBoxRigth.ItemIndex < ListBoxRigth.Items.Count - 1));
      7 : SetEnabled (components [i], (ListBoxLeft.ItemIndex >= 0) and (ListBoxLeft.Items.Count > 0) and (ListBoxLeft.SelCount > 0));                                      
    end;
end;

procedure TFormTwoColsEditor.ButtonAddComponentClick(Sender: TObject);
begin
  CollectionLeft.Add;
  ListBoxLeft.Setfocus;
  ListBoxLeft.ItemIndex := ListBoxLeft.Items.Count - 1;
  DeselectAll (ListBoxLeft);
  ListBoxLeft.selected [ListBoxLeft.ItemIndex] := True;
  UpdateSelection;
  Designer.Modified;
end;

procedure TFormTwoColsEditor.ButtonRemoveComponentClick(
  Sender: TObject);
begin
  ListBoxLeft.SetFocus;
  RemoveItems (ListBoxLeft);
  Designer.Modified;
end;

procedure TFormTwoColsEditor.ButtonRemovePropertyClick(Sender: TObject);
begin
  ListBoxRigth.SetFocus;
  RemoveItems (ListBoxRigth);
  Designer.Modified;
end;

procedure TFormTwoColsEditor.ButtonAddPropertyClick(Sender: TObject);
begin
  if (ListBoxLeft.Items.Count > 0) and (ListBoxLeft.ItemIndex >= 0)
    then
    begin
      CollectionRigth.Add;
      ListBoxRigth.Setfocus;
      ListBoxRigth.ItemIndex := ListBoxRigth.Items.Count - 1;
      DeselectAll (ListBoxRigth);
      ListBoxRigth.selected [ListBoxRigth.ItemIndex] := True;
      UpdateSelection;
      Designer.Modified;
    end;
end;

procedure TFormTwoColsEditor.MoveItem;
begin
  Item.Index := Item.Index + where;
  PostMessage (Handle, WM_SelectItem, 0, Longint (Item));
  Designer.Modified;
end;

procedure TFormTwoColsEditor.WMSelectItem;
var
  lb : TListBox;
  i : integer;
  Item : TcollectionItem;
begin
  Item := TCollectionItem (Msg.lParam);
  if Item is RigthItemClass
    then lb := ListBoxRigth
    else lb := ListBoxLeft;
  i := lb.Items.IndexOfObject (Item);
  if i >= 0
    then
    begin
      lb.Selected [i] := True;
      lb.ItemIndex := i;
      OriginalItem := i;
    end;
  UpdateEnabled;
end;

procedure TFormTwoColsEditor.Selectall1Click(Sender: TObject);
begin
  Selectall;
end;

procedure TFormTwoColsEditor.PopupMenuComponentsPopup(Sender: TObject);
begin
  ListBoxLeft.SetFocus;
end;

procedure TFormTwoColsEditor.PopupMenuPropertiesPopup(Sender: TObject);
begin
  ListBoxRigth.SetFocus;
end;

procedure TFormTwoColsEditor.ButtonMoveUpComponentClick(
  Sender: TObject);
begin
  MoveItem (SelectedLeft, -1);
end;

procedure TFormTwoColsEditor.ButtonMoveDownComponentClick(
  Sender: TObject);
begin
  MoveItem (SelectedLeft, 1);
end;

procedure TFormTwoColsEditor.ButtonMoveUpPropertyClick(Sender: TObject);
begin
  MoveItem (SelectedRigth, -1);
end;

procedure TFormTwoColsEditor.ButtonMoveDownPropertyClick(
  Sender: TObject);
begin
  MoveItem (SelectedRigth, 1);
end;

class function TFormTwoColsEditor.GetRegKey;
begin
  Result := DelphiIDE.GetBaseRegKey + '\jsbEditors\Collection Editor';
end;

procedure TFormTwoColsEditor.ListBoxDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
var
  ItemPos : integer;
begin
  with Sender as TListBox do
    begin
      if (ItemSelected >= 0) and (Source = Sender)
        then Selected [ItemSelected] := False;
      ItemPos := ItemAtPos (point (x, y), True);
      Accept := (Source = Sender) and (ItemPos >= 0) and (Items.Objects [ItemPos] <> Items.Objects [OriginalItem]);
      if Accept
        then
        begin
          Selected [ItemPos] := True;
          ItemSelected := ItemPos;
        end;
    end;
end;

procedure TFormTwoColsEditor.ListBoxEndDrag(Sender,
  Target: TObject; X, Y: Integer);
begin
  with Sender as tListbox do
    if ItemSelected >= 0
      then Selected [ItemSelected] := False;
  ItemSelected := -1;    
end;

procedure TFormTwoColsEditor.ListBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  OriginalItem := (Sender as tListbox).ItemIndex;
end;

procedure TFormTwoColsEditor.ListBoxDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  oriItem, Item : tCollectionItem;
begin
  with Sender as TListBox do
    begin
      Item := Items.Objects [ItemSelected] as TcollectionItem;
      oriItem := Items.Objects [OriginalItem] as TcollectionItem;
      oriItem.Index := Item.Index;
    end;
  PostMessage (Handle, WM_SelectItem, 0, Longint (oriItem));
  OriginalItem := -1;
  Designer.Modified;
end;

procedure TFormTwoColsEditor.ListBoxLeftKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    vk_Insert : ButtonAddComponent.Click;
    vk_Delete : if ButtonRemoveComponent.Enabled
      then ButtonRemoveComponent.Click;
  end;
end;

procedure TFormTwoColsEditor.ListBoxRigthKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    vk_Insert : if ButtonAddProperty.enabled
      then ButtonAddProperty.Click;
    vk_Delete : if ButtonRemoveProperty.Enabled
      then ButtonRemoveProperty.Click;
  end;
end;

function TFormTwoColsEditor.SelectedLeft;
begin
  if (ListBoxLeft.ItemIndex >= 0) and (ListBoxLeft.Items.Count > 0)
    then Result := ListBoxLeft.Items.Objects [ListBoxLeft.ItemIndex] as LeftItemClass
    else Result := nil;
end;

function TFormTwoColsEditor.SelectedRigth;
begin
  if (ListBoxRigth.ItemIndex >= 0) and (ListBoxRigth.Items.Count > 0)
    then Result := ListBoxRigth.Items.Objects [ListBoxRigth.ItemIndex] as RigthItemClass
    else Result := nil;
end;

initialization
  TwoColsEditors := TList.Create;
finalization
  TwoColsEditors.Free;
  TwoColsClipboard.Free;
end.
