{***************************************************************
 *
 * Unit Name : WebNavigator
 * Purpose   : Web Navigator for Delphi 3,4,5
 * Author    : Eugen Mihailescu
 * Company   : EM Quicksoft Romania SRL
 * Copyright : 1998,2002  All rights reserved.
 * Web Site  : http://www.em-quicksoft.com
 * Support   : support@em-quicksoft.com
 * History   : 03/20/2000
 *
 * Disclaimer: The objectives of this program is only educational.
 *             You MUST NOT use it without getting any material benefit on it
 *             You have the right to share this code with others without removing
 *             the copyright and/or disclaimer notice.
 *             You are allowed to include its code in your application
 *             only if it was designed for non commercial purpose,
 *             otherwise we claim $10 for each copy of your
 *             application which contains this code inside.
 ****************************************************************}

Unit WebNavigator;

Interface

Uses
  Windows,
  Classes,
  SysUtils,
  Forms,
  Dialogs;

Type
  TWebExecute = Procedure(URLAddress, LangFile, SaveLangFile: PChar); stdcall;

  TWebNavigator = Class(TComponent)
  private
    FURLAddress: String;
    FLanguageFileName: TFileName;
    FSaveDefaultLanguage: TFileName;
  protected
    { Protected declarations }
  public
    Procedure Execute;
  published
    Property URLAddress: String read FURLAddress write FURLAddress;
    Property LanguageFileName: TFileName read FLanguageFileName write FLanguageFileName;
    Property SaveDefaultLanguage: TFileName read FSaveDefaultLanguage write FSaveDefaultLanguage;
  End;

Procedure Register;
Implementation
//******************* TWebNavigator.Execute *************************

Procedure TWebNavigator.Execute;
Var
  H                 : THandle;
  FWebExec          : TWebExecute;
Begin
  H := LoadLibrary('Webbrows.dll');
  If Not H < HINSTANCE_ERROR Then
    Begin
      MessageDlg(Format('File %s not found. Cannot load browser interface.', [ExtractFilePath(Application.ExeName) + 'Webbrows.dll']), mtInformation, [mbOK], 0);
      Exit;
    End;
  Try
    @FWebExec := getprocaddress(H, PChar('WebExecute'));
    If @FWebExec <> Nil Then FWebExec(PChar(FURLAddress), PChar(LanguageFileName), PChar(FSaveDefaultLanguage));
  Finally
    FreeLibrary(H);
  End;
End;
//******************* Not available *************************

Procedure Register;
Begin
  RegisterComponents('EM-Quicksoft', [TWebNavigator]);
End;

End.

