
{*******************************************************}
{                                                       }
{       Animated Menus 2.3                              }
{       T_AM2000_ButtonArrayOptions                     }
{                                                       }
{       Copyright (c) 1997-2000 AnimatedMenus.com       }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000buttonarray;

{$I am2000.inc}

interface

uses
  Windows, Classes, Graphics, Menus, Controls,
  {$IFDEF Delphi4OrHigher} ImgList, {$ENDIF}
  am2000options;

type

  // TButtonArrayOptions
  T_AM2000_ButtonArrayOptions = class(T_AM2000_ControlOptions)
  private
    LastLeft, LastRight, LastTop, LastBottom: Integer;
    FRows         : Integer;
    FColumns      : Integer;
    FHints        : TStrings;
    FItemIndex    : Integer;
    FAllowAllUp   : Boolean;
    FBitmap       : TBitmap;
    FCount        : Integer;
    TempBmp       : TBitmap;

    procedure SetHints(Value: TStrings);
    procedure SetItemIndex(Value: Integer);
    procedure SetBitmap(const Value: TBitmap);

  public
    LastItemIndex : Integer;

    constructor Create(AParent: TMenuItem); override;
    destructor Destroy; override;

    procedure Paint(Params: P_AM2000_PaintMenuItemParams); override;
    procedure Assign(Source: TPersistent); override;

    function GetWidth(Canvas: TCanvas; Options: T_AM2000_BaseOptions): Integer; override;
    function GetHeight(Canvas: TCanvas; Options: T_AM2000_BaseOptions): Integer; override;

    function GetIndexAt(X, Y: Integer): Integer;

  published
    property Rows         : Integer
      read FRows write FRows default 1;
    property Columns         : Integer
      read FColumns write FColumns default 1;
    property Hints        : TStrings
      read FHints write SetHints;
    property ItemIndex    : Integer
      read FItemIndex write SetItemIndex default -1;
    property AllowAllUp   : Boolean
      read FAllowAllUp write FAllowAllUp default True;
    property Bitmap       : TBitmap
      read FBitmap write SetBitmap;
    property Count        : Integer
      read FCount write FCount default 0;

  end;



implementation

uses
  CommCtrl, Dialogs, SysUtils,
  am2000menuitem, am2000utils;


{ T_AM2000_ButtonArrayOptions }

constructor T_AM2000_ButtonArrayOptions.Create;
begin
  inherited;

  FAllowAllUp:= True;
  FItemIndex:= -1;
  LastItemIndex:= -1;
  FRows:= 1;
  FColumns:= 1;
  FBitmap:= TBitmap.Create;
  TempBmp:= TBitmap.Create;
  FHints:= TStringList.Create;
end;

destructor T_AM2000_ButtonArrayOptions.Destroy;
begin
  FHints.Free;
  FBitmap.Free;
  TempBmp.Free;
  inherited;
end;

function T_AM2000_ButtonArrayOptions.GetHeight(Canvas: TCanvas; Options: T_AM2000_BaseOptions): Integer;
begin
  Result:= Bitmap.Height +4;
end;

function T_AM2000_ButtonArrayOptions.GetWidth(Canvas: TCanvas; Options: T_AM2000_BaseOptions): Integer;
begin
  Result:= Bitmap.Width +4;
  ClearBitmap(TempBmp);
end;

procedure T_AM2000_ButtonArrayOptions.SetBitmap(const Value: TBitmap);
begin
  FBitmap.Assign(Value);
  ClearBitmap(TempBmp);
end;

procedure T_AM2000_ButtonArrayOptions.SetHints(Value: TStrings);
begin
  FHints.Assign(Value);
end;

procedure T_AM2000_ButtonArrayOptions.SetItemIndex(Value: Integer);
begin
  if Value <> -1
  then TMenuItem2000(Parent).TurnSiblingsOff;

  LastItemIndex:= FItemIndex;
  FItemIndex:= Value;
  RepaintFloatingMenus;
end;

procedure T_AM2000_ButtonArrayOptions.Paint(Params: P_AM2000_PaintMenuItemParams);
// draws TMenuItem2000 with ControlStyle = ctlButtonArray
var
  R: TRect;
  I, J, NextLII: Integer;
  il: HImageList;
  Bmp: TBitmap;
  Size: TPoint;
begin
  if (FBitmap = nil)
  or (FBitmap.Empty)
  then Exit;

  with Params^
  do begin
    // draw background
    if Canvas.Brush.Color <> Options.Colors.Menu
    then Canvas.Brush.Color:= Options.Colors.Menu;
    if Canvas.Brush.Style <> bsSolid
    then Canvas.Brush.Style:= bsSolid;

    R:= mir.LineRect;
    Canvas.FrameRect(R);
    InflateRect(R, -1, -1);
    Canvas.FrameRect(R);

    // create temporary bitmap (because this code is very slow)
    if TempBmp.Empty
    then begin
      with TempBmp, Canvas
      do begin
        Width:= Bitmap.Width;
        Height:= Bitmap.Height;
        Brush.Style:= bsSolid;
        Brush.Color:= Options.Colors.Menu;
        FillRect(ClipRect);
      end;

      Bmp:= TBitmap.Create;
      Bmp.Assign(Bitmap);

      // draw bitmap
      il:= ImageList_Create(Bitmap.Width, Bitmap.Height, ilc_Color24 + ilc_Mask, 1, 1);
      ImageList_AddMasked(il, Bmp.Handle, Bmp.Canvas.Pixels[0, Bitmap.Height -1]);
      ImageList_Draw(il, 0, TempBmp.Canvas.Handle, 0, 0, ild_Normal);
      ImageList_Destroy(il);

      Bmp.Free;
    end;

    // fast draw the bitmap
    BitBlt(Canvas.Handle, mir.LineLeft +2, mir.Top +2, Bitmap.Width, Bitmap.Height,
      TempBmp.Canvas.Handle, 0, 0, SrcCopy);

    // count
    if Count = 0 then Count:= Rows * Columns;

    // draw rectangle
    // get size
    Size:= Point(Bitmap.Width div Columns, Bitmap.Height div Rows);

    // draw item index
    if (ItemIndex >=0)
    then begin
      I:= ItemIndex mod Columns;
      J:= ItemIndex div Columns;

      R:= Rect(I * Size.X + mir.LineLeft +2, J * Size.Y + mir.Top +2, 0, 0);
      R.Right:= R.Left + Size.X;
      R.Bottom:= R.Top + Size.Y;

      // draw edge
      DrawEdge(Canvas.Handle, R, bdr_SunkenOuter, bf_Rect)
    end;


    // save coordinates
    LastLeft:= mir.LineLeft;
    LastRight:= mir.LineRight;
    LastTop:= mir.Top;
    LastBottom:= mir.Top + mir.Height;

    // draw current item
    NextLii:= -1;

    if (isSelected in State)
    and (MousePos.X > mir.LineLeft -2)
    and (MousePos.X < mir.LineRight)
    and (MousePos.Y > mir.Top -1)
    and (MousePos.Y < mir.Top + mir.Height)
    then begin
      I:= (MousePos.X - mir.LineLeft -2) div Size.X;
      J:= (MousePos.Y - mir.Top -2) div Size.Y;

      NextLII:= I + J * Columns;
      if (I >= 0) and (I < Columns)
      and (J >= 0) and (J < Rows)
      and (NextLII < Count)
      then begin
        // get current item's size
        R:= Rect(I * Size.X + mir.LineLeft +2, J * Size.Y + mir.Top +2, 0, 0);
        R.Right:= R.Left + Size.X;
        R.Bottom:= R.Top + Size.Y;

        // draw edge
        if (msLeftButton in MouseState)
        then DrawEdge(Canvas.Handle, R, bdr_SunkenOuter, bf_Rect)
        else DrawEdge(Canvas.Handle, R, bdr_RaisedInner, bf_Rect);

      end { acceptable I and J } ;
    end;

    LastItemIndex:= NextLII;
  end;
end;

function T_AM2000_ButtonArrayOptions.GetIndexAt(X, Y: Integer): Integer;
var
  I, J: Integer;
  Size: TPoint;
begin
  Result:= -1;
  if (X > LastLeft)
  and (X < LastRight)
  and (Y > LastTop -1)
  and (Y < LastBottom)
  then begin
    Size:= Point(Bitmap.Width div Columns, Bitmap.Height div Rows);
    I:= (X - LastLeft -2) div Size.X;
    J:= (Y - LastTop -2) div Size.Y;
    Result:= I + J * Columns;
  end;
end;

procedure T_AM2000_ButtonArrayOptions.Assign(Source: TPersistent);
var
  S: T_AM2000_ButtonArrayOptions;
begin
  inherited;

  if Source is T_AM2000_ButtonArrayOptions
  then begin
    S:= T_AM2000_ButtonArrayOptions(Source);

    Rows:= S.Rows;
    Columns:= S.Columns;
    ItemIndex:= S.ItemIndex;
    AllowAllUp:= S.AllowAllUp;
    Count:= S.Count;

    Hints.Assign(S.Hints);
    Bitmap.Assign(S.Bitmap);
  end;
end;

end.
