
{*******************************************************}
{                                                       }
{       Animated Menus 2.3                              }
{       T_AM2000_MenuItemCache Unit                     }
{       English Version                                 }
{                                                       }
{       Copyright (c) 1997-2000 AnimatedMenus.com       }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000cache;

interface

uses
  Windows, CommCtrl, Classes, Graphics;

const
  // am2000designer
  SBrowseForMenuTemplateFolder =
    'Please select a folder for Animated Menus templates.';
  SInsertMenuStart =
    'Insert ';
  SInsertMenuFinish =
    ' Menu';
  SBitmapDialogFilter =
    'Bitmaps (*.bmp)|*.bmp|All Files (*.*)|*.*';
  SSaveBitmapDialogTitle =
    'Save Bitmap As';
  SOpenBitmapDialogTitle =
    'Open Bitmap';
  SDefaultMenuItemCaption =
    'Menu Item';
  SDefaultMenuItemName =
    'MenuItem';
  SHiddenDoesntAffectToplevel =
    'Hidden property doesn''t affect top-level menu items.';

  // am2000cache
  SShift = 'Shift';
  SCtrl = 'Ctrl';
  SAlt = 'Alt';
  SIns = 'Ins';
  SDel = 'Del';
  SWin = 'Win';
  SBackspace = 'Back';

  // am2000mainmenu
  SMenuIndexOutOfBounds =
    'Menu index is out of bounds';
  SOnlyMainMenuCanBeMerged =
    'Only main menus could be merged. Assign a TMainMenu2000 component to the menu bar';

  // am2000menuitem
  SInvalidShortCut =
    'Invalid shortcut: ''';
  SThisIsNotAButtonArray =
    ': This is not a Button Array control';
  SThisIsNotAButton =
    ': This is not a Button control';
  SThisIsNotABitmap =
    ': This is not a Bitmap control';
  SThisisNotAGif =
    ': This is not a Gif image';
  SThisIsNotAEdit =
    ': This is not an Editbox control';
  SThisIsNotACombo =
    ': This is not an Combobox control';
  SThisIsNotAList =
    ': This is not a Listbox control';
  SThisIsNotAText =
    ': This is not a Text control';
  SMenuNotFound =
    'Menu not found';
  SNoNewMenu =
    'You cannot use NewMenu function with Animated Menus.';
  SNoNewPopupMenu =
    'You cannot use NewPopupMenu function with Animated Menus.';
  SOmissionPoints =
    '...';
  SShortcutDivider =
    ';';
  SSystemClose =
    'SYSTEMCLOSE';
  SSystemMaximize =
    'SYSTEMMAXIMIZE';
  SSystemMinimize =
    'SYSTEMMINIMIZE';
  SSystemMove =
    'SYSTEMMOVE';
  SSystemRestore =
    'SYSTEMRESTORE';
  SSystemSize =
    'SYSTEMSIZE';

  // am2000popup
  SDraggableMenuInfo =
    'Drag to make this menu float';
  SExpandCaption =
    'Expand (Ctrl+Down)';
  SEmptyCaption =
    '(Empty)';
  SMenuItemNotFound =
    'Menu item not found.';
  SNewLine =
    '\n';
  SAlignRight =
    '\a';
  SSeparator =
    '-';
  SCannotMoveItem =
    'Cannot move menu item into it''s own submenu';

  // am2000
  SWelcomeMessage = 'Hi!'#13#13+
    'Thank you for using Animated Menus Evaluation '+
    'Edition. We hope you''ll enjoy this software. Visit our website at'+
    'http://www.animatedmenus.com/ for the documentation, samples, '+
    'knwoledge base, latest software updates, and gifts for users.'#13#13+
    'Thank you!'#13#13+
    'Sincerely, Andrew Cher,'#13+
    'AnimatedMenus.com, Inc.';

  // am2000bitmap
  SValueMustBeBetween1And4 =
    'Value must be between 1 and 4.';

  // am2000editbox
  SOnlyTextMenuItemCanBeEdited =
    'Only text menu item can be edited.';

type
  T_AM2000_MenuItemCacheItem = class
  public
    ShortCuts   : String;
    Hint        : String;
    HasBitmap   : Boolean;
    DefaultIndex: Integer;
  end;

  T_AM2000_MenuItemCache = class(TStringList)
  private
    FBitmapCount: Integer;

    function GetItem(const Caption: String): T_AM2000_MenuItemCacheItem;

  public
    Images: HImageList;
    property BitmapCount: Integer read FBitmapCount;
    property Items    [const Caption: String]: T_AM2000_MenuItemCacheItem read GetItem; default;

    constructor Create;
    destructor Destroy; override;
    function AddItem(ACaption, AShortCuts, AHint: String; ABitmap: Boolean): T_AM2000_MenuItemCacheItem;

    function IsDefault(Caption: String): Boolean;
    function HasBitmap(Caption: String): Boolean;
    procedure Draw(Canvas: TCanvas; X, Y: Integer; Caption: String; Color1, Color2: TColor; Enabled: Boolean);
    procedure DrawIndex(Canvas: TCanvas; X, Y, ImageIndex: Integer; Color1, Color2: TColor; Enabled: Boolean);

    function GetBitmapIndex(Caption: String): Integer;

  end;


var
  MenuItemCache: T_AM2000_MenuItemCache;

implementation

{$R autobitmaps.res}

uses
  SysUtils,
  am2000utils;

{ Utilities }

function GetStandardCaption(S: String): String;
var
  J: Integer;
begin
  Result:= '';
  S:= AnsiUpperCase(S);
  for J:= 1 to Length(S) do begin
    if S[J] = #9 then Exit;
    if IsCharAlpha(S[J]) then AppendStr(Result, S[J]);
  end;
end;


{ T_AM2000_MenuItemCache }

constructor T_AM2000_MenuItemCache.Create;
var
  hBmp: HBitmap;
begin
  inherited;
  FBitmapCount:= 0;

  // initialize default menu items
  { About }
  AddItem('ABOUT', '', 'Displays the version number of this program and copyright, legal, and licensing notices', False);
  { Address Book }
  AddItem('ADDRESSBOOK', '', 'Displays the list of contacts', True);
  { Align Bottom }
  AddItem('ALIGNBOTTOM', '', 'Horizontally aligns the bottom edges of selected objects', True);
  { Align Center }
  AddItem('ALIGNCENTER', SCtrl+'+E', 'Centers the paragraph between the margins', True);
  { Align Justify }
  AddItem('ALIGNJUSTIFY', SCtrl+'+J', 'Justifies the paragraph between the margins', True);
  { Align Left }
  AddItem('ALIGNLEFT', SCtrl+'+L', 'Aligns the paragraph at the left margin', True);
  { Align Middle }
  AddItem('ALIGNMIDDLE', '', 'Horizontally aligns the middles of selected objects', True);
  { Align Right }
  AddItem('ALIGNRIGHT', SCtrl+'+R', 'Aligns the paragraph at the right margin', True);
  { Align Top }
  AddItem('ALIGNTOP', '', 'Horizontally aligns the top edges of selected objects', True);
  { Arrange Icons }
  AddItem('ARRANGEICONS', '', 'Arrange icons at the bottom of the window', False);
  { AutoFit to Contents }
  AddItem('AUTOFITTOCONTENTS', '', '', True);
  { AutoFit to Window }
  AddItem('AUTOFITTOWINDOW', '', '', True);
  { Auto Format }
  AddItem('AUTOFORMAT', '', '', True);
  { Auto Preview }
  AddItem('AUTOPREVIEW', '', 'Shows or hides the autopreview pane', True);
  { Auto Shapes }
  AddItem('AUTOSHAPES', '', '', True);
  { Auto Summarize }
  AddItem('AUTOSUMMARIZE', '', '', True);
  { Auto Text }
  AddItem('AUTOTEXT', '', '', True);
  { Back }
  AddItem('BACK', '', 'Return to the previous page in the hyperlink history list', True);
  { Background }
  AddItem('BACKGROUND', '', '', True);
  { Backward }
  AddItem('BACKWARD', '', 'Return to the previous page in the hyperlink history list', True);
  { Best of the Web }
  AddItem('BESTOFTHEWEB', '', '', True);
  { Bold }
  AddItem('BOLD', SCtrl+'+B', 'Bold Text', True);
  { Bookmarks }
  AddItem('BOOKMARKS', '', 'Insert a bookmark at the cursor', True);
  { Bullets }
  AddItem('BULLETS', '', 'Inserts a bullet on this line', True);
  { Bullets And Numbering }
  AddItem('BULLETSANDNUMBERING', '', '', True);
  { Cascade }
  AddItem('CASCADE', '', 'Arranges the windows as overlapping tiles', True);
  { Center }
  AddItem('CENTER', SCtrl+'+E', 'Centers selected objects vertically', True);
  { Chart }
  AddItem('CHART', '', '', True);
  { Check Spelling }
  AddItem('CHECKSPELLING', 'F7', 'Checks the spelling in the active document', True);
  { Clear }
  AddItem('CLEAR', SDel+';'+SShift+'+'+SDel, 'Erases the selection', False);
  { Clip Art }
  AddItem('CLIPART', '', '', True);
  { Columns }
  AddItem('COLUMNS', '', '', True);
  { Columns to the Left }
  AddItem('COLUMNSTOTHELEFT', '', '', True);
  { Columns to the Right }
  AddItem('COLUMNSTOTHERIGHT', '', '', True);
  { Comment }
  AddItem('COMMENT', '', '', True);
  { Comments }
  AddItem('COMMENTS', '', '', True);
  { Contents }
  AddItem('CONTENTS', '', 'Displays help topics', True);
  { Contents And Index }
  AddItem('CONTENTSANDINDEX', 'F1', 'Displays help topics', True);
  { Copy }
  AddItem('COPY', SCtrl+'+C;'+SCtrl+'+'+SIns, 'Copies the selection to the clipboard', True);
  { Create }
  AddItem('CREATE', '', 'Create a new document', True);
  { Create 2 }
  AddItem('CREATE_B', '', 'Create a new document (second choice)', True);
  { Create New Query }
  AddItem('CREATENEWQUERY', '', '', True);
  { Customize }
  AddItem('CUSTOMIZE', '', 'Customizes toolbar buttons, menu commands, and shortcut key assignments', False);
  { Cut }
  AddItem('CUT', SCtrl+'+X;'+SShift+'+'+SDel, 'Cuts the selection and moves it to the clipboard', True);
  { Data Range Properties }
  AddItem('DATARANGEPROPERTIES', '', '', True);
  { Date }
  AddItem('DATE', '', 'Inserts today''s date', True);
  { Date And Time }
  AddItem('DATEANDTIME', '', 'Inserts today''s date and/or time', True);
  { Date Time }
  AddItem('DATETIME', '', 'Inserts today''s date and/or time', True);
  { Decrease Indent }
  AddItem('DECREASEINDENT', '', 'Decrease indent', True);
  { Delete }
  AddItem('DELETE', ''+SDel+';'+SCtrl+'+'+SDel, 'Erases the selection', True);
  { Delete 2 }
  AddItem('DELETE_B', '', 'Erases the selection (second choice)', True);
  { Distribute Columns Evenly }
  AddItem('DISTRIBUTECOLUMNSEVENLY', '', '', True);
  { Distribute Horizontally }
  AddItem('DISTRIBUTEHORIZONTALLY', '', '', True);
  { Distribute Rows Evenly }
  AddItem('DISTRIBUTEROWSEVENLY', '', '', True);
  { Distribute Vertically }
  AddItem('DISTRIBUTEVERTICALLY', '', '', True);
  { Document Map }
  AddItem('DOCUMENTMAP', '', '', True);
  { Down }
  AddItem('DOWN', '', '', True);
  { Draw Table }
  AddItem('DRAWTABLE', '', '', True);
  { Edit }
  AddItem('EDIT', '', 'Contains commands for the clipboard, finding text, and editing links', False);
  { Edit Query }
  AddItem('EDITQUERY', '', '', True);
  { Envelopes And Labels }
  AddItem('ENVELOPESANDLABELS', '', '', True);
  { Exchange Folder }
  AddItem('EXCHANGEFOLDER', '', '', True);
  { Exit }
  AddItem('EXIT', '', 'Closes this program after prompting you to save all unsaved files', False);
  { Fax Recipient }
  AddItem('FAXRECIPIENT', '', '', True);
  { File }
  AddItem('FILE', '', 'Contains commands for saving documents and opening saved documents', False);
  { Find }
  AddItem('FIND', SCtrl+'+F;F3', 'Finds the specified text in the active document', True);
  { Find Next }
  AddItem('FINDNEXT', SShift+'+F3', 'Repeats the last find', True);
  { Fixed Column Width }
  AddItem('FIXEDCOLUMNWIDTH', '', '', True);
  { Font }
  AddItem('FONT', '', 'View or edit the character attributes of the selected text', True);
  { Font Size }
  AddItem('FONTSIZE', '', 'View or edit the font size of the selected text', True);
  { Font Style }
  AddItem('FONTSTYLE', '', 'View or edit the font style of the selected text', True);
  { Forward }
  AddItem('FORWARD', '', 'Go to the next page in the hyperlink history list', True);
  { Free Stuff }
  AddItem('FREESTUFF', '', '', True);
  { Frequently Asked Questions }
  AddItem('FREQUENTLYASKEDQUESTIONS', '', '', True);
  { From File }
  AddItem('FROMFILE', '', '', True);
  { From Scanner or Camera }
  AddItem('FROMSCANNERORCAMERA', '', '', True);
  { Full Screen }
  AddItem('FULLSCREEN', 'F11', 'Zooms the active document to the full screen', True);
  { Function }
  AddItem('FUNCTION', '', '', True);
  { Go To }
  AddItem('GOTO', SCtrl+'+G', 'Moves the cursor to the specified line', False);
  { Go To Bookmark }
  AddItem('GOTOBOOKMARK', '', 'Moves the cursor to the specified bookmark', True);
  { Group }
  AddItem('GROUP', '', 'Assembles to or more selecte objects into a single object', True);
  { Height }
  AddItem('HEIGHT', '', '', True);
  { Help }
  AddItem('HELP', '', 'Contains commands for solving problem situations', False);
  { Help On Help }
  AddItem('HELPONHELP', 'F1', 'Display instructions about how to use help', True);
  { Help Topics }
  AddItem('HELPTOPICS', '', 'Displays help topics', True);
  { Hide Detail }
  AddItem('HIDEDETAIL', '', '', True);
  { Hide Gridlines }
  AddItem('HIDEGRIDLINES', '', '', True);
  { Highlight Changes }
  AddItem('HIGHLIGHTCHANGES', '', '', True);
  { Home }
  AddItem('HOME', '', 'Browses the web', True);
  { Hyperlink }
  AddItem('HYPERLINK', '', 'Insert a new hyperlink', True);
  { Increase Indent }
  AddItem('INCREASEINDENT', '', 'Increase indent', True);
  { Indent }
  AddItem('INDENT', '', 'Increase Indent', True);
  { Insert Object }
  AddItem('INSERTOBJECT', '', '', True);
  { Italic }
  AddItem('ITALIC', SCtrl+'+I', 'Italic Text', True);
  { Justify }
  AddItem('JUSTIFY', SCtrl+'+J', 'Justifies the paragraph between the margins', True);
  { Left }
  AddItem('LEFT', SCtrl+'+L', '', True);
  { Macros }
  AddItem('MACROS', '', '', True);
  { Mail }
  AddItem('MAIL', '', '', True);
  { Mail Recipient }
  AddItem('MAILRECIPIENT', '', '', True);
  { Mail Recipient (as Attachement) }
  AddItem('MAILRECIPIENTASATTACHMENT', '', '', True);
  { Map }
  AddItem('MAP', '', '', True);
  { Mark As Read }
  AddItem('MARKASREAD', '', 'Marks the selected messages as read', True);
  { Mark As Unread }
  AddItem('MARKASUNREAD', '', 'Marks the selected messages as unread', True);
  { Master Document }
  AddItem('MASTERDOCUMENT', '', '', True);
  { Meet Now }
  AddItem('MEETNOW', '', '', True);
  { Merge Cells }
  AddItem('MERGECELLS', '', '', True);
  { Microsoft Excel Help }
  AddItem('MICROSOFTEXCELHELP', '', '', True);
  { Microsoft Home Page }
  AddItem('MICROSOFTHOMEPAGE', '', '', True);
  { Microsoft Office Home Page }
  AddItem('MICROSOFTOFFICEHOMEPAGE', '', '', True);
  { Microsoft Script Debugger }
  AddItem('MICROSOFTSCRIPTDEBUGGER', '', '', True);
  { Microsoft Word Help }
  AddItem('MICROSOFTWORDHELP', '', '', True);
  { Move Down }
  AddItem('MOVEDOWN', '', 'Moves the selection down', True);
  { Move Left }
  AddItem('MOVELEFT', '', 'Moves the selection left', True);
  { Move Right }
  AddItem('MOVERIGHT', '', 'Moves the selection right', True);
  { Move To Folder }
  AddItem('MOVETOFOLDER', '', 'Moves selected items to another folder', True);
  { Move Up }
  AddItem('MOVEUP', '', 'Moves the selection up', True);
  { New }
  AddItem('NEW', SCtrl+'+N', 'Create a new document', True);
  { New Folder }
  AddItem('NEWFOLDER', '', 'Create a new folder', True);
  { New Folder 2 }
  AddItem('NEWFOLDER_B', '', 'Create a new folder (second choice)', True);
  { New Mail }
  AddItem('NEWMAIL', '', 'Create a new email message', True);
  { New Mail Message }
  AddItem('NEWMAILMESSAGE', '', 'Create a new email message', True);
  { New Note }
  AddItem('NEWNOTE', '', 'Create a new note', True);
  { New Task }
  AddItem('NEWTASK', '', 'Create a new task', True);
  { New Window }
  AddItem('NEWWINDOW', '', 'Open another window for the active document', False);
  { Normal }
  AddItem('NORMAL', '', '', True);
  { Note }
  AddItem('NOTE', '', 'Create a new note', True);
  { Numbering }
  AddItem('NUMBERING', '', 'Inserts a numbering on this line', True);
  { Object }
  AddItem('OBJECT', '', '', True);
  { Online Layout }
  AddItem('ONLINELAYOUT', '', '', True);
  { Online Support }
  AddItem('ONLINESUPPORT', '', '', True);
  { Open }
  AddItem('OPEN', SCtrl+'+O', 'Opens an existing document from a file', True);
  { Options }
  AddItem('OPTIONS', '', 'Modifies settings for this program such as screen appearance, printing, editing, and other options', False);
  { Organization Chart }
  AddItem('ORGANIZATIONCHART', '', '', True);
  { Outline }
  AddItem('OUTLINE', '', '', True);
  { Page Break Preview }
  AddItem('PAGEBREAKPREVIEW', '', '', True);
  { Page Layout }
  AddItem('PAGELAYOUT', '', '', True);
  { Page Setup }
  AddItem('PAGESETUP', '', 'Change the printing options', False);
  { Paragraph }
  AddItem('PARAGRAPH', '', 'View or edit the paragraph attributes of the selected text', True);
  { Parameters }
  AddItem('PARAMETERS', '', '', True);
  { Paste }
  AddItem('PASTE', SCtrl+'+V;'+SShift+'+'+SIns, 'Inserts the clipboard contents at the insertion point', True);
  { Paste Link }
  AddItem('PASTELINK', '', 'Insert Clipboard contents and a link to its source', False);
  { Paste Special }
  AddItem('PASTESPECIAL', '', 'Insert Clipboard contents with options', False);
  { Pivot Table Report }
  AddItem('PIVOTTABLEREPORT', '', '', True);
  { Print }
  AddItem('PRINT', SCtrl+'+P', 'Print the active document', True);
  { Print Layout }
  AddItem('PRINTLAYOUT', '', '', True);
  { Print Preview }
  AddItem('PRINTPREVIEW', '', 'Display full pages', True);
  { Printer Setup }
  AddItem('PRINTERSETUP', '', 'Change the printer and printing options', False);
  { Product News }
  AddItem('PRODUCTNEWS', '', '', True);
  { Properties }
  AddItem('PROPERTIES', '', 'Display properties for the active document', True);
  { Record New Macro }
  AddItem('RECORDNEWMACRO', '', '', True);
  { Redo }
  AddItem('REDO', SCtrl+'+'+SShift+'+Z;'+SAlt+'+'+SShift+'+'+SBackspace, 'Redoes the last action that was undone', True);
  { Refresh }
  AddItem('REFRESH', '', 'Updates the display to reflect any changes', True);
  { Refresh Data }
  AddItem('REFRESHDATA', '', '', True);
  { Regroup }
  AddItem('REGROUP', '', 'Reassembled a group of objects that have been disassembled using the Ungroup command', True);
  { Reload }
  AddItem('RELOAD', '', 'Reloads the content of the current page', True);
  { Remove All Arrows }
  AddItem('REMOVEALLARROWS', '', '', True);
  { Remove Dependent Arrows }
  AddItem('REMOVEDEPENDENTARROWS', '', '', True);
  { Remove Precedent Arrows }
  AddItem('REMOVEPRECEDENTARROWS', '', '', True);
  { Repeat }
  AddItem('REPEAT', '', 'Repeats the last action', True);
  { Replace }
  AddItem('REPLACE', SCtrl+'+H', 'Replaces specific text with different text', True);
  { Right }
  AddItem('RIGHT', SCtrl+'+R', '', True);
  { Routing Recipient }
  AddItem('ROUTINGRECIPIENT', '', '', True);
  { Rows Above }
  AddItem('ROWSABOVE', '', '', True);
  { Rows Below }
  AddItem('ROWSBELOW', '', '', True);
  { Ruler }
  AddItem('RULER', '', 'Shows or hides the ruler', True);
  { Run Database Query }
  AddItem('RUNDATABASEQUERY', '', '', True);
  { Run Web Query }
  AddItem('RUNWEBQUERY', '', '', True);
  { Save }
  AddItem('SAVE', SCtrl+'+S;'+SShift+'+F12;'+SAlt+'+'+SShift+'+F2', 'Saves the active document', True);
  { Save All }
  AddItem('SAVEALL', '', 'Saves all opened documents', True);
  { Save As }
  AddItem('SAVEAS', 'F12;'+SCtrl+'+'+SShift+'+S', 'Saves a copy of the document in a separate file', False);
  { Save As Web Page }
  AddItem('SAVEASWEBPAGE', '', '', True);
  { Search }
  AddItem('SEARCH', '', 'Contains commands for searching through your document', False);
  { Search the Web }
  AddItem('SEARCHTHEWEB', '', 'Searches the Word Wide Web for the specified resource', True);
  { Select All }
  AddItem('SELECTALL', SCtrl+'+A', 'Select the entire document', False);
  { Send Feedback }
  AddItem('SENDFEEDBACK', '', '', True);
  { Send Mail }
  AddItem('SENDMAIL', '', 'Compose and send new mail message', True);
  { Service }
  AddItem('SERVICE', '', '', False);
  { Set Bookmark }
  AddItem('SETBOOKMARK', '', 'Insert a bookmark at the cursor', True);
  { Set Date }
  AddItem('SETDATE', '', 'Set today''s date', True);
  { Set Font }
  AddItem('SETFONT', '', 'View or edit the character attributes of the selected text', True);
  { Set Language }
  AddItem('SETLANGUAGE', '', '', True);
  { Set Time }
  AddItem('SETTIME', '', 'Set current time', True);
  { Show Detail }
  AddItem('SHOWDETAIL', '', '', True);
  { Sort }
  AddItem('SORT', '', 'Arranges items in the window', True);
  { Spell Check }
  AddItem('SPELLCHECK', 'F7', 'Checks the spelling in the active document', True);
  { Split Cells }
  AddItem('SPLITCELLS', '', '', True);
  { Split Window }
  AddItem('SPLITWINDOW', '', 'Splits the active document window horizontally', True);
  { System Close }
  AddItem(SSystemClose, '', 'Close the active window and prompts to save the pages', True);
  { System Maximize }
  AddItem(SSystemMaximize, '', 'Enlarge the window to full size', True);
  { System Minimize }
  AddItem(SSystemMinimize, '', 'Reduce the window to an icon', True);
  { System Move }
  AddItem(SSystemMove, '', 'Change the window position', True);
  { System Restore }
  AddItem(SSystemRestore, '', 'Restore the window to normal size', True);
  { System Size }
  AddItem(SSystemSize, '', 'Change the window size', True);
  { Table }
  AddItem('TABLE', '', '', False);
  { Table of Contents in Frame }
  AddItem('TABLEOFCONTENTSINFRAME', '', '', True);
  { Task }
  AddItem('TASK', '', 'Create a new task', True);
  { Text Box }
  AddItem('TEXTBOX', '', '', True);
  { Text Directions }
  AddItem('TEXTDIRECTIONS', '', '', True);
  { Theme }
  AddItem('THEME', '', '', True);
  { Tile }
  AddItem('TILE', '', 'Arranges the windows as nonoverlapping tiles', True);
  { Time }
  AddItem('TIME', '', 'Inserts current time', True);
  { Toolbox }
  AddItem('TOOLBOX', '', '', True);
  { Tools }
  AddItem('TOOLS', '', 'Contains additional commands for working with external tools', False);
  { Trace Dependents }
  AddItem('TRACEDEPENDENTS', '', '', True);
  { Trace Error }
  AddItem('TRACEERROR', '', '', True);
  { Trace Precedents }
  AddItem('TRACEPRECEDENTS', '', '', True);
  { Underline }
  AddItem('UNDERLINE', SCtrl+'+U', 'Underlined text', True);
  { Undo }
  AddItem('UNDO', SCtrl+'+Z;'+SAlt+'+'+SBackspace, 'Reverses the last action', True);
  { Ungroup }
  AddItem('UNGROUP', '', 'Disassembles a grouped object into individual objects', True);
  { Unindent }
  AddItem('UNINDENT', '', 'Decrease indent', True);
  { Up }
  AddItem('UP', '', '', True);
  { Up One Level }
  AddItem('UPONELEVEL', '', 'Moves the cursor up one level', True);
  { View }
  AddItem('VIEW', '', 'Contains commands for controlling the display of your document', False);
  { Visual Basic Editor }
  AddItem('VISUALBASICEDITOR', '', '', True);
  { Web Discussions }
  AddItem('WEBDISCUSSIONS', '', '', True);
  { Web Layout }
  AddItem('WEBLAYOUT', '', '', True);
  { Web Tutorial }
  AddItem('WEBTUTORIAL', '', '', True);
  { What's This? }
  AddItem('WHATSTHIS', SShift+'+F1', 'Shows help about the selected element', True);
  { Width }
  AddItem('WIDTH', '', '', True);
  { Window }
  AddItem('WINDOW', '', 'Contains commands for controlling position of windows of your application', False);
  { Word Art }
  AddItem('WORDART', '', '', True);

  // create image list
  Images:= ImageList_LoadBitmap(HInstance, 'BMP_AM2000_MENUITEMCACHE0', 16, 200, ColorToRGB(clFuchsia));
  hBmp:= LoadBitmap(HInstance, 'BMP_AM2000_MENUITEMCACHE1');
  ImageList_AddMasked(Images, hBmp, ColorToRGB(clFuchsia));
  DeleteObject(hBmp);
end;

destructor T_AM2000_MenuItemCache.Destroy;
var
  I: Integer;
begin
  for I:= 0 to Count -1 do
    T_AM2000_MenuItemCacheItem(Objects[I]).Free;

  // destroy image list
  ImageList_Destroy(Images);

  inherited;
end;

function T_AM2000_MenuItemCache.AddItem(ACaption, AShortCuts, AHint: String;
  ABitmap: Boolean): T_AM2000_MenuItemCacheItem;
var
  Index: Integer;
begin
  ACaption:= GetStandardCaption(ACaption);
  if not Find(ACaption, Index) then begin
    Result:= T_AM2000_MenuItemCacheItem.Create;
    AddObject(ACaption, Result);
  end
  else
    Result:= T_AM2000_MenuItemCacheItem(Objects[Index]);

  Result.ShortCuts:= AShortCuts;
  Result.Hint:=      AHint;
  Result.HasBitmap:= ABitmap;
  if ABitmap then begin
    Result.DefaultIndex:= FBitmapCount;
    Inc(FBitmapCount);
  end
  else
    Result.DefaultIndex:= -1;
end;

function T_AM2000_MenuItemCache.GetItem(const Caption: String): T_AM2000_MenuItemCacheItem;
var
  Index: Integer;
  S: String;
begin
  S:= GetStandardCaption(Caption);
  if Find(S, Index)
  then Result:= T_AM2000_MenuItemCacheItem(Objects[Index])
  else Result:= nil;
end;

procedure T_AM2000_MenuItemCache.Draw(Canvas: TCanvas; X, Y: Integer; Caption: String;
  Color1, Color2: TColor; Enabled: Boolean);
var
  Index: Integer;
begin
  if not Find(GetStandardCaption(Caption), Index)
  then Exit;

  if Enabled
  then ImageList_Draw(Images, Index, Canvas.Handle, X, Y, ild_Transparent)
  else ImgDisabledBlt(Canvas, X, Y, Images, Index, Color1, Color2);
end;

procedure T_AM2000_MenuItemCache.DrawIndex(Canvas: TCanvas; X, Y,
  ImageIndex: Integer; Color1, Color2: TColor; Enabled: Boolean);
var
  I: Integer;
begin
  if (ImageIndex < 0) or (ImageIndex >= FBitmapCount)
  then Exit;

  // get the number of ImageIndex
  for I:= 0 to Count -1 do
    if T_AM2000_MenuItemCacheItem(Objects[I]).DefaultIndex = ImageIndex
    then begin
      if Enabled
      then ImageList_Draw(Images, I, Canvas.Handle, X, Y, ild_Transparent)
      else ImgDisabledBlt(Canvas, X, Y, Images, I, Color1, Color2);

      Exit;
    end;
end;

function T_AM2000_MenuItemCache.IsDefault(Caption: String): Boolean;
var
  Index: Integer;
begin
  Result:= Find(GetStandardCaption(Caption), Index);
end;

function T_AM2000_MenuItemCache.HasBitmap(Caption: String): Boolean;
var
  Index: Integer;
begin
  Result:= Find(GetStandardCaption(Caption), Index)
    and T_AM2000_MenuItemCacheItem(Objects[Index]).HasBitmap;
end;

function T_AM2000_MenuItemCache.GetBitmapIndex(Caption: String): Integer;
var
  Index: Integer;
begin
  if Find(GetStandardCaption(Caption), Index)
  then Result:= T_AM2000_MenuItemCacheItem(Objects[Index]).DefaultIndex
  else Result:= -1;
end;


initialization
  MenuItemCache:= T_AM2000_MenuItemCache.Create;

finalization
  MenuItemCache.Free;
  MenuItemCache:= nil;

end.
