
{*******************************************************}
{                                                       }
{       Animated Menus 2.3                              }
{       TMainMenu2000 Component Unit                    }
{                                                       }
{       Copyright (c) 1997-2000 AnimatedMenus.com       }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000mainmenu;

{$I am2000.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, Dialogs, Menus, Buttons, ComCtrls,
  am2000options, am2000menuitem, am2000utils;

type

  // mainmenu manager
  TCustomMainMenu2000 = class(TMainMenu)
  private
    FEMI2000           : TEditableMenuItem2000; // layer for comptibility with Delphi and your apps
    FMergedMenus       : TList;
    FMergedMenuItems   : TList;
    ParentMenu         : TCustomMainMenu2000;
    PopulatedMenus     : TList;
    FVersion           : T_AM2000_Version;

{$IFNDEF Delphi4OrHigher}
    FImages            : TImageList;
{$ENDIF}

    function GetMenuItem2000: TMenuItem2000;
    function GetMergedMenuItem(const Index: Integer): TMenuItem;
    function GetMergedMenuItemsCount: Integer;
    procedure SetVersion(Value: T_AM2000_Version);

  protected

    property Version: T_AM2000_Version
      read FVersion write SetVersion stored False;

    procedure Loaded; override;

{$IFNDEF Delphi4OrHigher}
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
{$ENDIF}

  public
    Ole2ObjectWindow    : HWnd;
    Ole2MenuDescriptor  : HMenu;

    property Items2000 : TMenuItem2000
      read GetMenuItem2000;

    property MergedMenus : TList
      read FMergedMenus;

{$IFNDEF Delphi4OrHigher}
    property Images : TImageList read FImages write FImages;
{$ENDIF}

    property MergedMenuItems [const Index: Integer] : TMenuItem read GetMergedMenuItem;
    property MergedMenuItemsCount : Integer read GetMergedMenuItemsCount;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Merge2000(Menu: TMenu);
    procedure Unmerge2000(Menu: TMenu);
    procedure Merge2(Menu: TMenu);
    procedure Unmerge2(Menu: TMenu); // backward-compatibility function
    procedure RebuildMergedMenuItems; // backward-compatibility function

    function IsShortCut(var Msg: TWMKey): Boolean;
{$IFDEF Delphi4OrHigher}
    override;
{$ENDIF}

//    function FindItem2000(Value: Word; Kind: TFindItemKind): TMenuItem;

    // Ole2 menu support
    procedure PopulateOle2Menu2000(hSharedMenu: HMenu; Groups: array of Integer;
      var Widths: array of Longint);
    procedure SetOle2MenuHandle2000(hMenuShared, hOleMenu: HMenu; hwndActiveObject: HWND);
//    function GetHandle: HMENU; override;
    function IsMenuPopulated(hMenu: HMenu): Boolean;

  published
    property Items       : TEditableMenuItem2000  read FEMI2000;

  end;


{$IFDEF Delphi3OrHigher }
  function GetMainMenu2000(Form: TCustomForm): TCustomMainMenu2000;
{$ELSE}
  function GetMainMenu2000(Form: TForm): TCustomMainMenu2000;
{$ENDIF}

implementation

uses
  Consts,
  am2000menubar, am2000cache;

type

  // forgot about this
  TFalseMenu = class(TComponent)
  private
{$IFDEF Delphi4OrHigher}
    FBiDiMode: TBiDiMode;
{$ENDIF}
    FItems: TMenuItem;
  end;


{ GetMainMenu2000 }

{$IFDEF Delphi3OrHigher }
  function GetMainMenu2000(Form: TCustomForm): TCustomMainMenu2000;
{$ELSE}
  function GetMainMenu2000(Form: TForm): TCustomMainMenu2000;
{$ENDIF}
var
  MenuBar: TCustomMenuBar2000;
begin
  MenuBar:= GetActiveMenuBar;
  if (MenuBar <> nil)
  then
    if MenuBar.Menu is TCustomMainMenu2000
    then Result:= TCustomMainMenu2000(MenuBar.Menu)
    else raise EMenuError.Create(SOnlyMainMenuCanBeMerged)
  else
    Result:= nil;
end;

function GetMenuBar(AMenu: TMenu): TCustomMenuBar2000;
var
  I: Integer;
begin
  Result:= nil;
  for I:= 0 to ActiveMenuBarList.Count -1 do
    with TCustomMenuBar2000(ActiveMenuBarList[I]) do
      if Menu = AMenu
      then begin
        Result:= TCustomMenuBar2000(ActiveMenuBarList[I]);
        Exit;
      end;
end;


{ TCustomMainMenu2000 }

constructor TCustomMainMenu2000.Create(AOwner: TComponent);
begin
  inherited;

  // we don't need TMenuItem so we just destoy it
  // and create TMenuItem2000 instead of.
  TFalseMenu(Self).FItems.Free;
  TFalseMenu(Self).FItems:= TMenuItem2000.Create(Self);
{$IFDEF Delphi4OrHigher}
  TFalseMenu(Self).FBiDiMode:= BiDiMode;
{$ENDIF}
  FEMI2000:= TEditableMenuItem2000.Create(Self);

  FMergedMenus:=  TList.Create;
  FMergedMenus.Add(Self);

  FMergedMenuItems:= TList.Create;

  FVersion:= SVersion;
end;

destructor TCustomMainMenu2000.Destroy;
begin
  if (ParentMenu <> nil)
  then ParentMenu.Unmerge2(Self);

  PopulatedMenus.Free;
  FMergedMenus.Free;
  FMergedMenuItems.Free;

  if (csDesigning in ComponentState)
  then SendMessage(GetMnuDsgnHandle, wm_Close, 0, 0);

  inherited;
end;

procedure TCustomMainMenu2000.Loaded;
var
  F: TForm;
  I: Integer;
  M: TCustomMenuBar2000;
begin
  inherited;

  if (not (csDesigning in ComponentState))
  and (Owner is TForm)
  then begin
    F:= TForm(Owner);

    // if no menu bars specified, create one for me
    if (F.Menu = Self)
    and (F.FormStyle <> fsMdiChild)
    and (GetMenuBar(Self) = nil)
    then begin
      F.Menu:= nil;
      M:= TCustomMenuBar2000.Create(Owner);
      M.Align:= alTop;
      M.Parent:= F;
      M.Menu:= Self;
    end;

    // if this menu is the main menu for a mdi child form
    // then scan the parent mdi form and merge with
    // it's main menu
    if (F.Menu = Self)
    and (F.FormStyle = fsMdiChild)
    and (ActiveMenuBarList.Count > 0)
    then
      for I:= 0 to ActiveMenuBarList.Count -1 do begin
        M:= TCustomMenuBar2000(ActiveMenuBarList[I]);
        if (M.OwnerForm <> nil)
        and (M.OwnerForm.FormStyle = fsMdiForm)
        and (M.Menu is TCustomMainMenu2000)
        then begin
          ParentMenu:= TCustomMainMenu2000(M.Menu);
          ParentMenu.Merge2000(Self);
          Exit;
        end;
      end;
  end;
end;

function TCustomMainMenu2000.GetMenuItem2000: TMenuItem2000;
begin
  Result:= TMenuItem2000(inherited Items);
end;

procedure TCustomMainMenu2000.Merge2(Menu: TMenu);
begin
  Merge2000(Menu);
end;

procedure TCustomMainMenu2000.Merge2000(Menu: TMenu);
begin
end;

procedure TCustomMainMenu2000.Unmerge2(Menu: TMenu);
begin
  Unmerge2000(Menu);
end;

procedure TCustomMainMenu2000.Unmerge2000(Menu: TMenu);
begin
end;

procedure TCustomMainMenu2000.RebuildMergedMenuItems;
var
  GroupIndex, MenuIndex, ItemIndex: Integer;
  BreakGroupIndex: Boolean;
begin
  FMergedMenuItems.Clear;

  for GroupIndex:= 0 to 255
  do begin
    MenuIndex:= FMergedMenus.Count -1;
    BreakGroupIndex:= False;
    while (MenuIndex >= 0) and (not BreakGroupIndex) do
      with TMenu(FMergedMenus[MenuIndex]) do begin
        for ItemIndex:= 0 to Items.Count -1 do
          if Items[ItemIndex].GroupIndex = GroupIndex then begin
            FMergedMenuItems.Add(Items[ItemIndex]);
            BreakGroupIndex:= True;
          end;

        Dec(MenuIndex);
      end;
  end;
end;

function TCustomMainMenu2000.GetMergedMenuItem(const Index: Integer): TMenuItem;
begin
  Result:= nil;

  if (FMergedMenuItems = nil) or (Index < 0)
  then Exit;

  if FMergedMenuItems.Count = 0
  then RebuildMergedMenuItems;

  if (Index < FMergedMenuItems.Count)
  then Result:= TMenuItem(FMergedMenuItems[Index]);
end;

function TCustomMainMenu2000.GetMergedMenuItemsCount: Integer;
begin
  if FMergedMenuItems = nil
  then
    Result:= 0

  else begin
    if (FMergedMenuItems.Count = 0)
    then RebuildMergedMenuItems;

    Result:= FMergedMenuItems.Count;
  end;
end;

{$IFNDEF Delphi4OrHigher}
procedure TCustomMainMenu2000.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  if (Operation = opRemove)
  and (Acomponent = FImages)
  then FImages:= nil;
end;
{$ENDIF}

function TCustomMainMenu2000.IsShortCut(var Msg: TWMKey): Boolean;
begin
  Result:= IsShortCutEx(Msg, Items2000, csDesigning in ComponentState);
end;

//function TCustomMainMenu2000.FindItem2000(Value: Word; Kind: TFindItemKind): TMenuItem;
//begin
//  Result:= FindItem(Value, Kind);
//end;

{ Ole2 support }

procedure TCustomMainMenu2000.PopulateOle2Menu2000(hSharedMenu: HMenu; Groups: array of Integer;
      var Widths: array of Longint);
begin
end;

procedure TCustomMainMenu2000.SetOle2MenuHandle2000(hMenuShared, hOleMenu: HMenu; hwndActiveObject: HWND);
begin
end;

function TCustomMainMenu2000.IsMenuPopulated(hMenu: HMenu): Boolean;
begin
  Result:= False;
end;

procedure TCustomMainMenu2000.SetVersion(Value: T_AM2000_Version);
begin
end;

end.
