unit ScriptDemoWin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, dwsComp, dwsTokenizer, Menus, dwsUtilsLib, dwsDemoLib,
  dwsErrors, ComCtrls, ExtCtrls, mwPasToRtf;

type
  TfrmScriptDemo = class(TForm)
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    NewItem: TMenuItem;
    OpenItem: TMenuItem;
    SaveItem: TMenuItem;
    ExitItem: TMenuItem;
    N1: TMenuItem;
    ScriptMenu: TMenuItem;
    CompileItem: TMenuItem;
    ExecuteItem: TMenuItem;
    N2: TMenuItem;
    ShowTreeItem: TMenuItem;
    DemosMenu: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    N3: TMenuItem;
    VariablesItem: TMenuItem;
    HelpMenu: TMenuItem;
    StopItem: TMenuItem;
    BlockItem: TMenuItem;
    SaveAsItem: TMenuItem;
    dwsUtilsLib1: TdwsUtilsLib;
    fncPrint: TdwsFunc;
    fncGetCapitals: TdwsFunc;
    Script: TDelphiWebScript;
    CommentModeItem: TMenuItem;
    dwsDemoLib1: TdwsDemoLib;
    N4: TMenuItem;
    NormalModeItem: TMenuItem;
    AspModeItem: TMenuItem;
    Panel2: TPanel;
    DebugItem: TMenuItem;
    VarChangeItem: TMenuItem;
    DocuItem: TMenuItem;
    N5: TMenuItem;
    AboutItem: TMenuItem;
    HomepageItem: TMenuItem;
    Panel3: TPanel;
    memoSource: TRichEdit;
    ShowUsedPlugins: TMenuItem;
    lbLog: TListBox;
    lbMsgs: TListBox;
    memoResult: TRichEdit;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    procedure NewItemClick(Sender: TObject);
    procedure CompileItemClick(Sender: TObject);
    procedure ExecuteItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure ShowTreeItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure VariablesItemClick(Sender: TObject);
    procedure StopItemClick(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure OpenItemClick(Sender: TObject);
    procedure BlockItemClick(Sender: TObject);
    procedure SaveAsItemClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure fncGetCapitalsEval(Sender: TObject; scr: TDelphiWebScript; pat: Integer; args: TArgList; var Result: Variant);
    procedure fncPrintEval(Sender: TObject; scr: TDelphiWebScript; pat: Integer; args: TArgList; var Result: Variant);
    procedure ModeItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure memoSource1Change(Sender: TObject);
    procedure DemosMenuClick(Sender: TObject);
    procedure scriptVarChange(scr: TDelphiWebScript; item: TDataItem;
      oldval, newval: Variant);
    procedure DebugItemClick(Sender: TObject);
    procedure scriptDebug(scr: TDelphiWebScript; expr: TExpr);
    procedure VarChangeItemClick(Sender: TObject);
    procedure DocuItemClick(Sender: TObject);
    procedure HomepageItemClick(Sender: TObject);
    procedure lbMsgsDblClick(Sender: TObject);
    procedure memoSourceChange(Sender: TObject);
    procedure lbMsgsClick(Sender: TObject);
    procedure ShowUsedPluginsClick(Sender: TObject);

  private
    ScriptChanged: Boolean;
    ActiveMsg: TMsg;
    scrfile: string;
    DebugLine: Integer;
    DebugExpr: TExpr;
    procedure OpenScript(sname: string);
    procedure UpdateSyntax;
    procedure CreateDemosMenu;
    procedure ShowMsg(Msg: TMsg);
    procedure ShowFirstMsg;
  public
    progpath: string;
    scriptpath: string;
    docupath: string
  end;

var
  frmScriptDemo: TfrmScriptDemo;

implementation

uses ScriptDemoTree, ScriptDemoVar, ShellApi, ScriptDemoDebug;

{$R *.DFM}

procedure TfrmScriptDemo.FormCreate(Sender: TObject);
begin
  progpath := ExtractFilePath(Application.Exename);
  scriptpath := progpath + '..\scripts\';
  docupath := progpath + '..\help\';

  ScriptChanged := True;
  DebugLine := 0;

  BlockItem.Checked := Script.Blocking;
  NormalModeItem.Click;
  DemosMenu.OnClick := nil;

  OpenScript ('Startup.dws');

  CreateDemosMenu;
end;

procedure TfrmScriptDemo.CreateDemosMenu;
var
  mi: TMenuItem;
  sr: TSearchRec;
  res: Integer;
begin
  res := FindFirst(scriptpath + '*.dws', faAnyFile, sr);

  while res = 0 do
  begin
    mi := TMenuItem.Create(MainMenu1);
    mi.Caption := sr.Name;
    mi.OnClick := DemosMenuClick;
    DemosMenu.Add(mi);
    res := FindNext(sr);
  end;

  FindClose(sr);
end;

procedure TfrmScriptDemo.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  NewItemClick(Sender);
  CanClose := not memoSource.Modified;
end;

procedure TfrmScriptDemo.NewItemClick(Sender: TObject);
var
  res: Integer;
begin
  if memoSource.Modified then
  begin
    res := MessageDlg('There are unsaved changes! Do you want to save now?',
      mtInformation, [mbYes, mbNo, mbCancel], 0);
    if res = mrCancel then exit;
    if res = mrYes then SaveItemClick(self);
  end;
  memoSource.Clear;
  memoSource.Modified := false;
  ScriptChanged := True;
  Caption := 'DWS Demo';

  frmScriptDemo.ActiveControl := memoSource;
end;

procedure TfrmScriptDemo.OpenItemClick(Sender: TObject);
begin
  OpenDialog1.InitialDir := scriptpath;
  if OpenDialog1.Execute then
  begin
    NewItemClick(sender);
    if memoSource.Modified then exit;
    memoSource.Lines.Clear;
    scrfile := OpenDialog1.FileName;
    Caption := 'DWS Demo - ' + scrfile;
    memoSource.Lines.LoadFromFile(OpenDialog1.FileName);
    UpdateSyntax;
    ScriptChanged := true;
    memoSource.Modified := false;
  end;
end;

procedure TfrmScriptDemo.SaveItemClick(Sender: TObject);
begin
  if scrfile = '' then
    SaveAsItemClick(Sender)
  else
  begin
    memoSource.Lines.SaveToFile(scrfile);
    memoSource.Modified := false;
  end;
end;

procedure TfrmScriptDemo.SaveAsItemClick(Sender: TObject);
begin
  SaveDialog1.InitialDir := scriptpath;
  if SaveDialog1.Execute then
  begin
    scrfile := SaveDialog1.FileName;
    Caption := 'DWS Demo - ' + scrfile;
    memoSource.Lines.SaveToFile(scrfile);
  end;
end;

procedure TfrmScriptDemo.ExitItemClick(Sender: TObject);
begin
  Close
end;

procedure TfrmScriptDemo.CompileItemClick(Sender: TObject);
var
  x: integer;
begin
  // Assigns the script code in the memo to the script component
  Script.Text := memoSource.Lines.Text;
  ScriptChanged := False;
  lbLog.Clear;

  // Compiles the script
  Script.Compile;
  lbMsgs.Clear;
  for x := 0 to Script.Msgs.Count - 1 do
    lbMsgs.Items.AddObject(script.Msgs[x].AsInfo, Script.Msgs[x]);
  lbMsgs.Visible := lbMsgs.Items.Count > 0;
  Splitter1.Visible := lbMsgs.Items.Count > 0;
  ShowFirstMsg;
end;

procedure TfrmScriptDemo.ExecuteItemClick(Sender: TObject);
var
  x: Integer;
  t: LongInt;
begin
  if ScriptChanged then Script.Text := memoSource.Lines.Text;
  if not Script.Compiled then CompileItemClick(Sender);
  if not Script.Compiled then exit;

  lbLog.Items.Add('*** Executing...  press [Ctrl+Del] to stop');
  t := GetTickCount;

  if Script.Execute then
  begin
    // Execution successful!
    // Shows the value of the script variable "result"
    lbLog.Items.Add(Format('*** Executed. [%.2f s]', [(LongInt(GetTickCount) - t) / 1000]));
    memoResult.Text := Script.Result;
  end;

  // Display messages
  lbMsgs.Items.Clear;
  for x := 0 to Script.Msgs.Count - 1 do
    lbMsgs.Items.AddObject(Script.Msgs[x].AsInfo, Script.Msgs[x]);
  lbMsgs.Visible := lbMsgs.Items.Count > 0;
  Splitter1.Visible := lbMsgs.Items.Count > 0;
end;

procedure TfrmScriptDemo.StopItemClick(Sender: TObject);
begin
  // If a script is executing it will be stopped now.
  Script.Stop;
end;

procedure TfrmScriptDemo.BlockItemClick(Sender: TObject);
begin
  // Switch blocking mode
  Script.Blocking := not Script.Blocking;
  BlockItem.Checked := Script.Blocking;
end;

procedure TfrmScriptDemo.DebugItemClick(Sender: TObject);
begin
  DebugItem.Checked := not DebugItem.Checked;
  if DebugItem.Checked then
    Script.OnDebug := scriptDebug
  else
    Script.OnDebug := nil;
  ScriptChanged := true;
end;

procedure TfrmScriptDemo.VarChangeItemClick(Sender: TObject);
begin
  VarChangeItem.Checked := not VarChangeItem.Checked;
  if VarChangeItem.Checked then
    Script.OnVarChange := scriptVarChange
  else
    Script.OnVarChange := nil;
  ScriptChanged := True;
end;

procedure TfrmScriptDemo.ModeItemClick(Sender: TObject);
begin
  // Switch html mode
  TMenuItem(sender).Checked := True;
  if NormalModeItem.Checked then
    Script.Mode := tmNormal
  else if AspModeItem.Checked then
    Script.Mode := tmHtmlAsp
  else if CommentModeItem.Checked then
    Script.Mode := tmHtmlComment;
end;

procedure TfrmScriptDemo.VariablesItemClick(Sender: TObject);
begin
  frmScriptDemoVar.ShowModal;
end;

procedure OpenFile(const FileName: string);
begin
  if ShellExecute(Application.MainForm.Handle, 'open', PChar(FileName), nil, nil, SW_SHOW) <= 32 then
    ShowMessage('"' + Filename + '" not found');
end;

procedure TfrmScriptDemo.DocuItemClick(Sender: TObject);
begin
  OpenFile(docupath + 'scrip_en.html');
end;

procedure TfrmScriptDemo.HomepageItemClick(Sender: TObject);
begin
  OpenFile('http://www.dwscript.com');
end;

procedure TfrmScriptDemo.ShowTreeItemClick(Sender: TObject);
begin
  // Shows the code tree, generated by the script parser in a TTreeView component
  Script.ShowTree(frmScriptTree.TV);
  frmScriptTree.ShowModal;
end;

procedure TfrmScriptDemo.OpenScript;
begin
  NewItemClick(nil);
  memoSource.Lines.LoadFromFile(scriptpath + sname); // Since we don't want RTF..
  ScriptChanged := false;
  UpdateSyntax;
  memoSource.Modified := false;
  scrfile := scriptpath + sname;
  Caption := 'DWS Demo - ' + scrfile;
end;

procedure TfrmScriptDemo.DemosMenuClick(Sender: TObject);
begin
  Script.SetGlobal('opentime', now);
  Script.SetGlobal('exectime', now);
  OpenScript(StringReplace(TMenuItem(Sender).Caption,'&','',[rfReplaceAll]));
end;

procedure TfrmScriptDemo.AboutItemClick(Sender: TObject);
begin
  ShowMessage('Demonstration program for DelphiWebScript ' + Script.Version);
end;

procedure TfrmScriptDemo.memoSource1Change(Sender: TObject);
begin
  ScriptChanged := True;
end;

procedure TfrmScriptDemo.scriptVarChange(scr: TDelphiWebScript; item: TDataItem; oldval, newval: Variant);
var
  n, o: string;
begin
  try
    n := VarToStr(newval);
  except
    n := '[...]';
  end;
  try
    o := VarToStr(oldval);
  except
    o := '[...]';
  end;
  lbLog.Items.Add(format('Variable %s changes from %s to %s', [item.Name, o, n]));
end;

procedure TfrmScriptDemo.scriptDebug(scr: TDelphiWebScript; expr: TExpr);
var
  res: Integer;
begin
  if (expr.FPos.Line = DebugLine) and (expr <> DebugExpr) then exit;

  DebugLine := expr.FPos.Line;
  DebugExpr := expr;
  memoSource.SelStart := scr.Msgs.GetErrorLineStart(expr.FPos);
  memoSource.SelLength := scr.Msgs.GetErrorLineEnd(expr.FPos) - memoSource.SelStart;

  res := frmScriptDemoDebug.ShowModal;

  memoSource.SelLength := 0;

  if res = mrAbort then
  begin
    Script.Stop;
  end
  else if res = mrAll then
  begin
    Script.OnVarChange := nil;
    Script.OnDebug := nil;
  end;

end;


procedure TfrmScriptDemo.fncPrintEval;
var
  x: Integer;
  s: string;
begin
  {
  fncPrint.Declaration: Print (all open)
  -> The number and type of the arguments varies
  }
  s := '';
  for x := 0 to args.count - 1 do
    s := s + VarToStr(args[x].Eval);
  lbLog.Items.Add(s);
end;

procedure TfrmScriptDemo.fncGetCapitalsEval;
var
  v: Variant;
begin
  v := VarArrayCreate([0, 4, 0, 1], varOleStr);

  v[0, 0] := 'SWITZERLAND';
  v[1, 0] := 'GERMANY';
  v[2, 0] := 'AUSTRIA';
  v[3, 0] := 'ITALIA';
  v[4, 0] := 'FRANCE';

  v[0, 1] := 'BERN';
  v[1, 1] := 'BERLIN';
  v[2, 1] := 'VIENNA';
  v[3, 1] := 'ROME';
  v[4, 1] := 'PARIS';

  result := v;
end;

procedure TfrmScriptDemo.lbMsgsDblClick(Sender: TObject);
begin
  if lbMsgs.ItemIndex > -1 then
    ShowMsg(TMsg(lbMsgs.Items.Objects[lbMsgs.ItemIndex]));
end;

procedure TfrmScriptDemo.ShowMsg;
var
  b: Boolean;
begin
  memoSource.OnChange := nil;
  b := memoSource.Modified;

  if (msg <> ActiveMsg) and (ActiveMsg is TScriptMsg) then
    UpdateSyntax;
    
  if Assigned(msg) and (msg is TScriptMsg) then
    with msg as TScriptMsg do 
    begin
      memoSource.SelStart := Script.Msgs.GetErrorLineStart(TScriptMsg(msg).Pos) - 1;
      memoSource.SelLength := Script.Msgs.GetErrorLineEnd(TScriptMsg(msg).Pos) - memoSource.SelStart;
      memoSource.SelAttributes.Style := [fsBold];
      memoSource.SelAttributes.Color := clRed;
      memoSource.SelStart := TScriptMsg(msg).Pos.Pos - 1;
      memoSource.SelLength := 1;
    end;

  ActiveMsg := msg;
  memoSource.Modified := b;
  memoSource.OnChange := memoSourceChange;
end;

procedure TfrmScriptDemo.ShowFirstMsg;
var
  x: Integer;
begin
  for x := 0 to lbMsgs.Items.Count - 1 do
    if Assigned(lbMsgs.Items.Objects[x]) then
    begin
      ShowMsg(TMsg(lbMsgs.Items.Objects[x]));
      lbLog.ItemIndex := x;
      break;
    end;
end;

procedure TfrmScriptDemo.memoSourceChange(Sender: TObject);
begin
  memoSource.OnChange := nil;
  ScriptChanged := True;
  ShowMsg(nil);
  UpdateSyntax;
  memoSource.OnChange := memoSourceChange;
end;

procedure TfrmScriptDemo.lbMsgsClick(Sender: TObject);
begin
  lbMsgs.OnDblClick(self);
end;

procedure TfrmScriptDemo.ShowUsedPluginsClick(Sender: TObject);
var 
  x: Integer;
  s: string;
begin
  x := 0;
  while Script.LoadedPlugins[x] <> '' do
  begin
    s := Script.LoadedPlugins[x];
    lbLog.Items.Add(s);
    lblog.Items.Add('Plugin-Title: ' + Script.GetPluginInfo(s, 'Title'));
    lblog.Items.Add('Plugin-Version: ' + Script.GetPluginInfo(s, 'Version'));
    lblog.Items.Add('Plugin-Author: ' + Script.GetPluginInfo(s, 'Author'));
    Inc(x);
  end;
end;

procedure TfrmScriptDemo.UpdateSyntax;
var                     
  TempMS: TMemoryStream;
  CurrEdit: TRichEdit;
  PasCon: TPasConversion;
  pos, top: Integer;
  OnChange: TNotifyEvent;
begin
  CurrEdit := memoSource;
  if (Script.Mode <> tmNormal) or (Length(CurrEdit.Text) <= 0) then
    exit;
  
  pos := memoSource.selstart;
  top := SendMessage(CurrEdit.Handle, EM_GETFIRSTVISIBLELINE, 0, 0);
  OnChange := CurrEdit.OnChange;
  TempMS := TMemoryStream.Create;
  try
    CurrEdit.Lines.SaveToStream(TempMS);
    CurrEdit.OnChange := nil;
    PasCon:=TPasConversion.Create;
    try
      try
        PasCon.UseDelphiHighlighting(progpath + 'ScriptDemo.ini');
        PasCon.LoadFromStream(TempMS);
        PasCon.ConvertReadStream;
        CurrEdit.PlainText := False;
        CurrEdit.Lines.BeginUpdate;
        CurrEdit.Lines.LoadFromStream(PasCon);
        SendMessage(CurrEdit.Handle, EM_LINESCROLL, 0, top);
        CurrEdit.Lines.EndUpdate;
      finally
        CurrEdit.PlainText := True;
        PasCon.Free;
      end;
    except
      // Remove highlighting
      TempMS.Position := 0;
      CurrEdit.SelAttributes := CurrEdit.DefAttributes;
      CurrEdit.Lines.LoadFromStream(TempMS);
    end;
  finally
    memoSource.SelStart := Pos;
    TempMS.Free;
    CurrEdit.OnChange := OnChange;
  end;
end;

end.

