unit oaStage;

{ Office 97 Assistant Package v2.23 }
{ Copyright  1999, 2000 Kambiz R. Khojasteh }
{ email: khojasteh@mail.com }

{$IFDEF VER100}
  {$DEFINE OA_D3}
{$ENDIF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Buttons;

type

  THintBulb = class(TSpeedButton)
  private
    Rgn: HRgn;
    Painting: Boolean;
    Bitmap: TBitmap;
    GlyphIndex: Integer;
    procedure RebuildRegion;
  protected
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure PaintTo(ACanvas: TCanvas; X, Y: Integer);
    function MouseOver: Boolean;
  end;

  {$IFDEF OA_D3}
  TCanResizeEvent = procedure(Sender: TObject; var NewWidth, NewHeight: Integer;
    var Resize: Boolean) of object;
  {$ENDIF}

  TGetPaletteEvent = procedure(Sender: TObject; var Palette: HPALETTE) of object;

  TAssistantStage = class(TForm)
    procedure FormResize(Sender: TObject);
  private
    FOnHintClick: TNotifyEvent;
    FOnMoving: TNotifyEvent;
    FOnGetPalette: TGetPaletteEvent;
    {$IFDEF OA_D3}
    FOnCanResize: TCanResizeEvent;
    function CanResize(var NewWidth, NewHeight: Integer): Boolean;
    procedure WMWindowPosChanging(var Message: TWMWindowPosChanging); message WM_WINDOWPOSCHANGING;
    {$ENDIF}
    procedure WMMoving(var Message: TMessage); message WM_MOVING;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMNCLButtonDblClk(var Message: TWMMouse); message WM_NCLBUTTONDBLCLK;
    procedure WMNCLButtonDown(var Message: TWMMouse); message WM_NCLBUTTONDOWN;
    procedure WMNCLButtonUp(var Message: TWMMouse); message WM_NCLBUTTONUP;
    procedure WMNCRButtonUp(var Message: TWMMouse); message WM_NCRBUTTONUP;
    {$IFNDEF OA_D3}
    procedure CMBiDiModeChanged(var Message: TMessage); message CM_BIDIMODECHANGED;
    {$ENDIF}
    procedure DoHintClick(Sender: TObject);
  protected
    function GetPalette: HPALETTE; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Deactivate; override;
  public
    HintBulb: THintBulb;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure MakeTopMost;
    property OnHintClick: TNotifyEvent read FOnHintClick write FOnHintClick;
    property OnMoving: TNotifyEvent read FOnMoving write FOnMoving;
    property OnGetPalette: TGetPaletteEvent read FOnGetPalette write FOnGetPalette;
    {$IFDEF OA_D3}
    property OnCanResize: TCanResizeEvent read FOnCanResize write FOnCanResize;
    {$ENDIF}
  end;

implementation

{$R *.DFM}
{$R *.RES}

uses
  oaBitmap;

{ THintBulb }

constructor THintBulb.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csOpaque];
  Rgn := NULLREGION;
  Flat := True;
  Visible := False;
  Glyph.LoadFromResourceName(hInstance, 'BULBGLYPHS256');
  NumGlyphs := 4;
  Width := Glyph.Width div NumGlyphs;
  Height := Glyph.Height;
  Bitmap := TBitmap.Create;
  Bitmap.Width := Width;
  Bitmap.Height := Height;
  GlyphIndex := -1;
  RebuildRegion;
end;

destructor THintBulb.Destroy;
begin
  if Rgn <> NULLREGION then
    DeleteObject(Rgn);
  Bitmap.Free;
  inherited Destroy;
end;

procedure THintBulb.RebuildRegion;
begin
  if Rgn <> NULLREGION then DeleteObject(Rgn);
  Bitmap.Canvas.CopyRect(Bitmap.Canvas.ClipRect, Glyph.Canvas, Bitmap.Canvas.ClipRect);
  Rgn := CreateBitmapRgn(Bitmap, Bitmap.Canvas.Pixels[0,0])
end;

procedure THintBulb.Paint;
begin
  if not Painting and Assigned(Parent) then
  begin
    Painting := True;
    try
      Parent.Refresh;
    finally
      Painting := False;
    end;
  end;
end;

procedure THintBulb.PaintTo(ACanvas: TCanvas; X, Y: Integer);
var
  NewIndex: Integer;
begin
  if not Enabled then
    NewIndex := 1
  else if FState = bsDown then
    NewIndex := 2
  else if MouseOver then
    NewIndex := 3
  else
    NewIndex := 0;
  if GlyphIndex <> NewIndex then
  begin
    GlyphIndex := NewIndex;
    Bitmap.Canvas.CopyRect(Rect(0, 0, Width, Height), Glyph.Canvas,
      Rect(GlyphIndex * Width, 0, (GlyphIndex+1) * Width, Height));
    Bitmap.Transparent := True;
  end;
  ACanvas.Draw(X, Y, Bitmap);
end;

procedure THintBulb.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(Parent) and Parent.Visible then
    Parent.Refresh;
end;

function THintBulb.MouseOver: Boolean;
var
  MousePos: TPoint;
begin
  Result := False;
  if Enabled and Visible and (Rgn <> NULLREGION) then
  begin
    GetCursorPos(MousePos);
    MousePos := ScreenToClient(MousePos);
    Result := PtInRegion(Rgn, MousePos.X, MousePos.Y);
  end;
  if not Result and (GlyphIndex = 3) then
    Invalidate;
end;

{ TAssistantStage }

constructor TAssistantStage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  HintBulb := THintBulb.Create(Self);
  HintBulb.Parent := Self;
  HintBulb.OnClick := DoHintClick;
end;

destructor TAssistantStage.Destroy;
begin
  HintBulb.Free;
  inherited Destroy;
end;

procedure TAssistantStage.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or WS_POPUP;
    WindowClass.style := WindowClass.style or CS_SAVEBITS or CS_BYTEALIGNCLIENT;
  end;
end;

procedure TAssistantStage.Deactivate;
begin
  inherited Deactivate;
  SetWindowPos(Handle, HWND_TOP, 0, 0, 0, 0, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
end;

function TAssistantStage.GetPalette: HPALETTE;
begin
  Result := inherited GetPalette;
  if Assigned(FOnGetPalette) then
    FOnGetPalette(Self, Result);
end;

{$IFDEF OA_D3}
function TAssistantStage.CanResize(var NewWidth, NewHeight: Integer): Boolean;
begin
  Result := True;
  if Assigned(FOnCanResize) then
    FOnCanResize(Self, NewWidth, NewHeight, Result);
end;

procedure TAssistantStage.WMWindowPosChanging(var Message: TWMWindowPosChanging);
begin
  if (ComponentState * [csReading, csLoading]) = [] then
    with Message.WindowPos^ do
      if (flags and SWP_NOSIZE = 0) and not CanResize(cx, cy) then
        flags := flags or SWP_NOSIZE;
  inherited;
end;
{$ENDIF}

procedure TAssistantStage.WMMoving(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMoving) then
    FOnMoving(Self);
end;

procedure TAssistantStage.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TAssistantStage.WMNCHitTest(var Message: TWMNCHitTest);
begin
  inherited;
  if not Enabled then
    Message.Result := htNoWhere
  else if (Message.Result = htClient) and not HintBulb.MouseOver then
    Message.Result := htCaption;
end;

procedure TAssistantStage.WMNCLButtonDblClk(var Message: TWMMouse);
var
  MousePos: TPoint;
begin
  inherited;
  MousePos := ScreenToClient(SmallPointToPoint(Message.Pos));
  if PtInRect(ClientRect, MousePos) then DblClick;
end;

procedure TAssistantStage.WMNCLButtonDown(var Message: TWMMouse);
var
  MousePos: TPoint;
begin
  inherited;
  MousePos := ScreenToClient(SmallPointToPoint(Message.Pos));
  if Assigned(OnMouseUp) then
    MouseUp(mbLeft, [ssLeft], Message.Pos.X, Message.Pos.Y);
  if PtInRect(ClientRect, MousePos) then Click;
end;

procedure TAssistantStage.WMNCLButtonUp(var Message: TWMMouse);
begin
  inherited;
  if Assigned(OnMouseUp) then
    MouseUp(mbLeft, [ssLeft], Message.Pos.X, Message.Pos.Y);
end;

procedure TAssistantStage.WMNCRButtonUp(var Message: TWMMouse);
begin
  inherited;
  if Assigned(OnMouseUp) then
    MouseUp(mbRight, [ssRight], Message.Pos.X, Message.Pos.Y);
  if Assigned(PopupMenu) then
    PopupMenu.Popup(Message.XPos, Message.YPos);
end;

{$IFNDEF OA_D3}
procedure TAssistantStage.CMBiDiModeChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(PopupMenu) then PopupMenu.BiDiMode := BiDiMode;
end;
{$ENDIF}

procedure TAssistantStage.FormResize(Sender: TObject);
begin
  HintBulb.SetBounds(ClientWidth - HintBulb.Width - 5, 5,
    HintBulb.Width, HintBulb.Height);
  if Visible then Invalidate;
end;

procedure TAssistantStage.DoHintClick(Sender: TObject);
begin
  if Assigned(OnHintClick) then
    OnHIntClick(Self);
end;

procedure TAssistantStage.MakeTopMost;
begin
  RecreateWnd;
  Perform(WM_QUERYNEWPALETTE, 0, 0);
  Update;
end;

end.
