{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000 
  http://www.zeitungsjunge.de/delphi/ }

unit rjStreams;

{$I rj.inc}

interface

uses
 Windows,
 Classes;

type
 { TrjStream }

 TrjStream = class
 protected
  function GetPosition: Integer; virtual;
  function GetSize: Integer; virtual;
  procedure SetPosition (const Value: Integer); virtual;
  procedure SetSize (const NewSize: Integer); virtual;
 public
  {$IFDEF CheckFree}
  constructor Create;
  destructor Destroy; override;
  {$ENDIF}
  function Read (out Buffer; const Count: Integer): Integer; virtual;
  function ReadAnsiChar (out c: AnsiChar): Boolean; virtual;
  function ReadBinaryAnsiString (out s: AnsiString): Boolean;
  function ReadTextAnsiString (out s: AnsiString; const Count: Integer): Integer;
  property Position: Integer read GetPosition write SetPosition;
  procedure Reset; virtual;
  property Size: Integer read GetSize write SetSize;
  function Write (const Buffer; const Count: Integer): Integer; virtual;
  function WriteAnsiChar (const c: AnsiChar): Boolean;
  function WriteBinaryAnsiString (const s: AnsiString): Boolean;
  function WriteTextAnsiString (const s: AnsiString): Integer;
 end;

 { TrjCustomMemoryStream }
 
 TrjCustomMemoryStream = class (TrjStream)
 private
  FCapacity: Integer;
  FPosition: Integer;
  FSize: Integer;
 protected
  function GetPosition: Integer; override;
  function GetSize: Integer; override;
  procedure SetPosition (const NewPosition: Integer); override;
  procedure SetCapacity (const NewCapacity: Integer); virtual; abstract;
 public
  property Capacity: Integer read FCapacity write SetCapacity;
  procedure Reset; override;
 end;
 
 { TrjMemoryStream }
 
 TrjMemoryStream = class (TrjCustomMemoryStream)
 private
  FMemory: Pointer;
  function GetAsString: AnsiString;
  procedure SetAsString (const s: AnsiString);
 protected
  procedure SetCapacity (const NewCapacity: Integer); override;
  procedure SetSize (const NewSize: Integer); override;
 public
  property AsString: AnsiString read GetAsString write SetAsString;
  property Memory: Pointer read FMemory;
  function Read (out Buffer; const Count: Integer): Integer; override;
  function Write (const Buffer; const Count: Integer): Integer; override;
 end;
 
 { TrjAnsiStringStream }

 TrjAnsiStringStream = class (TrjCustomMemoryStream)
 private
  FDataString: AnsiString;
  function GetDataString: AnsiString;
  procedure SetDataString (const s: AnsiString);
 protected
  procedure SetCapacity (const NewCapacity: Integer); override;
  procedure SetSize (const NewSize: Integer); override;
 public
  property DataString: AnsiString read GetDataString write SetDataString;
  function Read (out Buffer; const Count: Integer): Integer; override;
  function ReadAnsiChar (out c: AnsiChar): Boolean; override;
  function Write (const Buffer; const Count: Integer): Integer; override;
 end;
 
 { TrjFileStream }
 
 TrjFileStream = class (TrjStream)
 private
  FFileHandle: THandle;
  FFileName: AnsiString;                // Store FileName for error-reporting only.
 published
  constructor Create (const AFileName: AnsiString; const DesiredAccess, ShareMode, CreationDistribution, FlagsAndAttributes: Cardinal);
  destructor Destroy; override;
  function Read (out Buffer; const Count: Integer): Integer; override;
  function ReadAnsiChar (out c: AnsiChar): Boolean; override;
  procedure Reset; override;
  function Write (const Buffer; const Count: Integer): Integer; override;
 end;
 
 { TrjFiler }

 TrjFiler = class
 private
  FBuffer: Pointer;
  FBufferSize: Integer;
  FBufferPosition: Integer;
  FBufferEnd: Integer;
  FStream: TStream;
 protected
 public
  constructor Create (const Stream: TStream); overload;
  constructor Create (const Stream: TStream; const BufferSize: Integer); overload;
  destructor Destroy; override;
 end;
 
 { TrjReader }
 
 TrjReader = class (TrjFiler)
 private
  function GetPosition: Integer;
  procedure SetPosition (const Value: Integer);
  procedure ReadBuffer;
 protected
 public
  procedure FlushBuffer;
  property Position: Integer read GetPosition write SetPosition;
  procedure Read (var Buffer; const Count: Integer);
  function ReadByte: Byte;
  function ReadWord: Word;
  function ReadCardinal: Cardinal;
  function ReadInteger: Integer;
  function ReadBinaryAnsiString: AnsiString;
 end;

 { TrjWriter }

 TrjWriter = class (TrjFiler)
 private
  function GetPosition: Longint;
  procedure SetPosition (Value: Integer);
  procedure WriteBuffer;
 protected
 public
  destructor Destroy; override;
  procedure FlushBuffer;
  property Position: Integer read GetPosition write SetPosition;
  procedure Write (const Buffer; const Count: Integer);
  procedure WriteByte (const Value: Byte);
  procedure WriteCardinal (const Value: Cardinal);
  procedure WriteInteger (const Value: Integer);
  procedure WriteBinaryAnsiString (const Value: AnsiString);
  procedure WriteWord (const Value: Word);
 end;

implementation

uses
 Consts,
 
 rjConsts,
 rjExceptions,
 rjResource,
 rjStrings,
 rjTypes
 {$IFDEF CheckFree}, rjCheckFree{$ENDIF};

{ **************************************************************************** }
{ TrjStream
{ **************************************************************************** }

{$IFDEF CheckFree}

constructor TrjStream.Create;
begin
 inherited Create;
 RegisterObjectCreate (Self);
end;

{$ENDIF}

{$IFDEF CheckFree}

destructor TrjStream.Destroy;
begin
 RegisterObjectFree (Self);
 inherited Destroy;
end;

{$ENDIF}

function TrjStream.GetPosition: Integer;
begin
 Result := 0;
end;

function TrjStream.GetSize: Integer;
begin
 Result := 0;                           // Nothing at this level.
end;

function TrjStream.Read (out Buffer; const Count: Integer): Integer;
begin
 Result := 0;
end;

function TrjStream.ReadAnsiChar (out c: AnsiChar): Boolean;
begin
 Result := Read (c, SizeOf (c)) = SizeOf (c);
end;

function TrjStream.ReadBinaryAnsiString (out s: AnsiString): Boolean;
var
 l                  : Integer;
begin
 Result := Read (l, SizeOf (l)) = SizeOf (l);
 if Result then
  begin
   SetLength (s, l);
   Result := Read (Pointer (s)^, l) = l;
  end
 else
  Result := False;
end;

function TrjStream.ReadTextAnsiString (out s: AnsiString; const Count: Integer): Integer;
begin
 if Count > 0 then
  begin
   SetLength (s, Count);
   Result := Read (Pointer (s)^, Count);
   if Result < Count then SetLength (s, Result);
  end
 else
  Result := 0;
end;

procedure TrjStream.Reset;
begin
end;

procedure TrjStream.SetPosition (const Value: Integer);
begin
end;

procedure TrjStream.SetSize (const NewSize: Integer);
begin
end;

function TrjStream.Write (const Buffer; const Count: Integer): Integer;
begin
 Result := 0;                           // Nothing at this level.
end;

function TrjStream.WriteAnsiChar (const c: AnsiChar): Boolean;
begin
 Result := Write (c, SizeOf (c)) = SizeOf (c);
end;

function TrjStream.WriteBinaryAnsiString (const s: AnsiString): Boolean;
var
 l                  : Integer;
begin
 l := Length (s);
 Result := Write (l, SizeOf (l)) = SizeOf (l);
 if Result then Result := Write (Pointer (s)^, l) = l;
end;

function TrjStream.WriteTextAnsiString (const s: AnsiString): Integer;
begin
 if Pointer (s) <> nil then
  begin
   Result := Integer (Pointer (Cardinal (s) - 4)^);
   Result := Write (Pointer (s)^, Result);
  end
 else
  Result := 0;
end;

{ TrjCustomMemoryStream }

procedure TrjCustomMemoryStream.Reset;
begin
 FPosition := 0;
end;

function TrjCustomMemoryStream.GetSize: Integer;
begin
 Result := FSize;
end;

function TrjCustomMemoryStream.GetPosition: Integer;
begin
 Result := FPosition;
end;

procedure TrjCustomMemoryStream.SetPosition (const NewPosition: Integer);
begin
 if NewPosition > FSize then
  FPosition := FSize
 else
  if NewPosition < 0 then
   FPosition := 0
  else
   FPosition := NewPosition;
end;

{ TrjMemoryStream }

function TrjMemoryStream.GetAsString: AnsiString;
begin
 SetString (Result, PAnsiChar (FMemory), FSize);
end;

function TrjMemoryStream.Read (out Buffer; const Count: Integer): Integer;
begin
 Result := FSize - FPosition;
 if Result > Count then Result := Count;
 Move (Pointer (Integer (FMemory) + FPosition)^, Buffer, Result);
 Inc (FPosition, Result);
end;

procedure TrjMemoryStream.SetAsString (const s: AnsiString);
var
 l                  : Integer;
begin
 if Pointer (s) = nil then
  begin
   FreeMem (FMemory, FCapacity);
   FMemory := nil;
   l := 0;
  end
 else
  begin
   l := Integer (Pointer (Cardinal (s) - 4)^);
   ReallocMem (FMemory, l);
   Move (Pointer (s)^, FMemory^, l);
  end;
 FCapacity := l;
 FSize := l;
 FPosition := 0;
end;

procedure TrjMemoryStream.SetCapacity (const NewCapacity: Integer);
begin
 ReallocMem (FMemory, NewCapacity);
 FCapacity := NewCapacity;
 if FSize > NewCapacity then FSize := NewCapacity;
 if FPosition > NewCapacity then FPosition := NewCapacity;
end;

procedure TrjMemoryStream.SetSize (const NewSize: Integer);
begin
 if NewSize > FCapacity then ReallocMem (FMemory, NewSize);
 FSize := NewSize;
 if FPosition > NewSize then FPosition := NewSize;
end;

function TrjMemoryStream.Write (const Buffer; const Count: Integer): Integer;
var
 NewPosition        : Integer;
begin
 if Count > 0 then
  begin
   NewPosition := FPosition + Count;
   if NewPosition > FSize then
    begin
     if NewPosition > FCapacity then
      begin
       FCapacity := ((NewPosition + (NewPosition div 4) + 4) div 4) * 4;
       ReallocMem (FMemory, FCapacity);
      end;
     FSize := NewPosition;
    end;
   Move (Buffer, Pointer (Integer (FMemory) + FPosition)^, Count);
   Inc (FPosition, Count);
  end;
 Result := Count;
end;

{ TrjAnsiStringStream }

function TrjAnsiStringStream.GetDataString: AnsiString;
begin
 SetLength (FDataString, FSize);
 Result := FDataString;
end;

function TrjAnsiStringStream.Read (out Buffer; const Count: Integer): Integer;
begin
 Result := FSize - FPosition;
 if Result > Count then Result := Count;
 Move (Pointer (Integer (FDataString) + FPosition)^, Buffer, Result);
 Inc (FPosition, Result);
end;

function TrjAnsiStringStream.ReadAnsiChar (out c: AnsiChar): Boolean;
begin
 Result := FPosition < FSize;
 if Result then
  begin
   c := PAnsiCharArray (FDataString)[FPosition];
   Inc (FPosition, SizeOf (c));
  end;
end;

procedure TrjAnsiStringStream.SetCapacity (const NewCapacity: Integer);
begin
 SetLength (FDataString, NewCapacity);
 FCapacity := NewCapacity;
 if FSize > NewCapacity then FSize := NewCapacity;
 if FPosition > NewCapacity then FPosition := NewCapacity;
end;

procedure TrjAnsiStringStream.SetDataString (const s: AnsiString);
begin
 if Pointer (s) = nil then
  begin
   FDataString := '';
   FCapacity := 0;
   FSize := 0;
   FPosition := 0;
  end
 else
  begin
   FDataString := s;
   FCapacity := Integer (Pointer (Cardinal (s) - 4)^);
   FSize := FCapacity;
   FPosition := 0;
  end;
end;

procedure TrjAnsiStringStream.SetSize (const NewSize: Integer);
begin
 if NewSize > FCapacity then SetLength (FDataString, NewSize);
 FSize := NewSize;
 if FPosition > NewSize then FPosition := NewSize;
end;

function TrjAnsiStringStream.Write (const Buffer; const Count: Integer): Integer;
var
 NewPosition        : Integer;
begin
 NewPosition := FPosition + Count;
 if NewPosition > FSize then
  begin
   if NewPosition > FCapacity then
    begin
     FCapacity := ((NewPosition + (NewPosition div 4) + 4) div 4) * 4;
     SetLength (FDataString, FCapacity);
    end;
   FSize := NewPosition;
  end;
 Move (Buffer, Pointer (Integer (FDataString) + FPosition)^, Count);
 Inc (FPosition, Count);
 
 Result := Count;
end;

{ TrjFileStream }

constructor TrjFileStream.Create (const AFileName: AnsiString; const DesiredAccess, ShareMode, CreationDistribution, FlagsAndAttributes: Cardinal);
begin
 inherited Create;
 FFileName := AFileName;
 FFileHandle := CreateFile (PChar (AFileName), DesiredAccess, ShareMode, nil, CreationDistribution, FlagsAndAttributes, 0);
 if FFileHandle = INVALID_HANDLE_VALUE then raise EFileOpen.CreateFmt (sErrorOpeningFile, [AFileName, GetLastErrorString (GetLastError)]);
end;

destructor TrjFileStream.Destroy;
begin
 if FFileHandle <> INVALID_HANDLE_VALUE then CloseHandle (FFileHandle);
 inherited Destroy;
end;

function TrjFileStream.Read (out Buffer; const Count: Integer): Integer;
begin
 if not ReadFile (FFileHandle, Buffer, Count, Cardinal (Result), nil) then
  raise EFileRead.CreateFmt (sErrorReadingFile, [FFileName, GetLastErrorString (GetLastError)]);
end;

function TrjFileStream.ReadAnsiChar (out c: AnsiChar): Boolean;
var
 BytesRead          : Cardinal;
begin
 ReadFile (FFileHandle, c, SizeOf (c), BytesRead, nil);
 Result := BytesRead = SizeOf (c);
end;

function TrjFileStream.Write (const Buffer; const Count: Integer): Integer;
begin
 if not WriteFile (FFileHandle, Buffer, Count, Cardinal (Result), nil) then
  raise EFileRead.CreateFmt (sErrorReadingFile, [FFileName, GetLastErrorString (GetLastError)]);
end;

procedure TrjFileStream.Reset;
begin
 SetFilePointer (FFileHandle, 0, nil, FILE_BEGIN);
end;

{ TrjFiler }

constructor TrjFiler.Create (const Stream: TStream);
begin
 Create (Stream, BUFFER_SIZE);
end;

constructor TrjFiler.Create (const Stream: TStream; const BufferSize: Integer);
begin
 inherited Create;
 FStream := Stream;
 GetMem (FBuffer, BufferSize);
 FBufferSize := BufferSize;
end;

destructor TrjFiler.Destroy;
begin
 if FBuffer <> nil then FreeMem (FBuffer, FBufferSize);
 inherited Destroy;
end;

{ TrjReader }

procedure TrjReader.FlushBuffer;
begin
 FStream.Position := Position;
 FBufferPosition := 0;
 FBufferEnd := 0;
end;

function TrjReader.GetPosition: Integer;
begin
 Result := FStream.Position - (FBufferEnd - FBufferPosition);
end;

procedure TrjReader.Read (var Buffer; const Count: Integer);
asm
        PUSH    ESI
        PUSH    EDI
        PUSH    EBX
        MOV     EDI,EDX
        MOV     EBX,ECX
        MOV     ESI,EAX
        JMP     @@6
@@1:    MOV     ECX,[ESI].TrjReader.FBufferEnd
        SUB     ECX,[ESI].TrjReader.FBufferPosition
        JA      @@2
        MOV     EAX,ESI
        CALL    TrjReader.ReadBuffer
        MOV     ECX,[ESI].TrjReader.FBufferEnd
@@2:    CMP     ECX,EBX
        JB      @@3
        MOV     ECX,EBX
@@3:    PUSH    ESI
        SUB     EBX,ECX
        MOV     EAX,[ESI].TrjReader.FBuffer
        ADD     EAX,[ESI].TrjReader.FBufferPosition
        ADD     [ESI].TrjReader.FBufferPosition,ECX
        MOV     ESI,EAX
        MOV     EDX,ECX
        SHR     ECX,2
        CLD
        REP     MOVSD
        MOV     ECX,EDX
        AND     ECX,3
        REP     MOVSB
        POP     ESI
@@6:    OR      EBX,EBX
        JNE     @@1
        POP     EBX
        POP     EDI
        POP     ESI
end;

procedure TrjReader.ReadBuffer;
begin
 FBufferEnd := FStream.Read (FBuffer^, FBufferSize);
 if FBufferEnd = 0 then raise EReadError.Create (SReadError);
 FBufferPosition := 0;
end;

function TrjReader.ReadByte: Byte;
begin
 Read (Result, SizeOf (Result));
end;

function TrjReader.ReadCardinal: Cardinal;
begin
 Read (Result, SizeOf (Result));
end;

function TrjReader.ReadInteger: Integer;
begin
 Read (Result, SizeOf (Result));
end;

function TrjReader.ReadBinaryAnsiString: AnsiString;
var
 l                  : Integer;
begin
 Read (l, SizeOf (l));
 SetLength (Result, l);
 Read (Pointer (Result)^, l);
end;

function TrjReader.ReadWord: Word;
begin
 Read (Result, SizeOf (Result));
end;

procedure TrjReader.SetPosition (const Value: Integer);
begin
 FStream.Position := Value;
 FBufferPosition := 0;
 FBufferEnd := 0;
end;

{ TrjWriter }

destructor TrjWriter.Destroy;
begin
 WriteBuffer;
 inherited Destroy;
end;

procedure TrjWriter.FlushBuffer;
begin
 WriteBuffer;
end;

function TrjWriter.GetPosition: Longint;
begin
 Result := FStream.Position + FBufferPosition;
end;

procedure TrjWriter.SetPosition (Value: Integer);
var
 StreamPosition     : Integer;
begin
 StreamPosition := FStream.Position;
 { Only flush the buffer if the repostion is outside the buffer range }
 if (Value < StreamPosition) or (Value > StreamPosition + FBufferPosition) then
  begin
   WriteBuffer;
   FStream.Position := Value;
  end
 else
  FBufferPosition := Value - StreamPosition;
end;

procedure TrjWriter.Write (const Buffer; const Count: Integer); assembler;
asm
        PUSH    ESI
        PUSH    EDI
        PUSH    EBX
        MOV     ESI,EDX
        MOV     EBX,ECX
        MOV     EDI,EAX
        JMP     @@6
@@1:    MOV     ECX,[EDI].TrjWriter.FBufferSize
        SUB     ECX,[EDI].TrjWriter.FBufferPosition
        JA      @@2
        MOV     EAX,EDI
        CALL    TrjWriter.WriteBuffer
        MOV     ECX,[EDI].TrjWriter.FBufferSize
@@2:    CMP     ECX,EBX
        JB      @@3
        MOV     ECX,EBX
@@3:    SUB     EBX,ECX
        PUSH    EDI
        MOV     EAX,[EDI].TrjWriter.FBuffer
        ADD     EAX,[EDI].TrjWriter.FBufferPosition
        ADD     [EDI].TrjWriter.FBufferPosition,ECX
@@5:    MOV     EDI,EAX
        MOV     EDX,ECX
        SHR     ECX,2
        CLD
        REP     MOVSD
        MOV     ECX,EDX
        AND     ECX,3
        REP     MOVSB
        POP     EDI
@@6:    OR      EBX,EBX
        JNE     @@1
        POP     EBX
        POP     EDI
        POP     ESI
end;

procedure TrjWriter.WriteBuffer;
begin
 if FBufferPosition > 0 then
  begin
   if FStream.Write (FBuffer^, FBufferPosition) <> FBufferPosition then
    raise EWriteError.Create (SWriteError);

   FBufferPosition := 0;
  end;
end;

procedure TrjWriter.WriteByte (const Value: Byte);
begin
 Write (Value, SizeOf (Value));
end;

procedure TrjWriter.WriteCardinal (const Value: Cardinal);
begin
 Write (Value, SizeOf (Value));
end;

procedure TrjWriter.WriteInteger (const Value: Integer);
begin
 Write (Value, SizeOf (Value));
end;

procedure TrjWriter.WriteBinaryAnsiString (const Value: AnsiString);
var
 l                  : Integer;
begin
 l := Length (Value);
 Write (l, SizeOf (l));
 Write (Pointer (Value)^, l);
end;

procedure TrjWriter.WriteWord (const Value: Word);
begin
 Write (Value, SizeOf (Value));
end;

end.

