{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000 
  http://www.zeitungsjunge.de/delphi/ }

unit rjStrings;

{$I rj.inc}

interface

uses
 {$IFDEF RangeChecking}SysUtils, {$ENDIF}
 Windows,
 
 rjTypes;

function PadLeft (const Source: AnsiString; const c: AnsiChar; const Count: Integer): AnsiString;
function PadRight (const Source: AnsiString; const c: AnsiChar; const Count: Integer): AnsiString;

function TrimChar (const Source: AnsiString; const c: AnsiChar): AnsiString;
function TrimCharLeft (const Source: AnsiString; const c: AnsiChar): AnsiString;
function TrimSet (const Source: AnsiString; const s: TAnsiCharSet): AnsiString;

procedure AddSlash (var Path: AnsiString);
procedure DelSlash (var Path: AnsiString);

function StrLen (const p: PAnsiChar): Integer;

procedure AssignIfEmpty (var Dest: AnsiString; const Source: AnsiString);

function PCompStr (p1, p2: PAnsiChar): Integer;
function PCompStrL (p1, p2: PAnsiChar; MaxLen: Integer): Integer;
function PSameStrL (p1, p2: PAnsiChar; MaxLen: Integer): Boolean;

function PCompText (p1, p2: PAnsiChar): Integer;
function PCompTextL (p1, p2: PAnsiChar; MaxLen: Integer): Integer;
function PSameTextL (p1, p2: PAnsiChar; MaxLen: Cardinal): Boolean;

function CompStr (const s1, s2: AnsiString): Integer;
function SameStr (const s1, s2: AnsiString): Boolean;

function CompText (const s1, s2: AnsiString): Integer;
function SameText (const s1, s2: AnsiString): Boolean;
function SameTextL (const s1, s2: AnsiString; const MaxLen: Integer): Boolean;

function SameBufferText (const Buffer: Pointer; const BufferLength: Integer; const s2: AnsiString): Boolean;
function SameBufferStr (const Buffer: Pointer; const BufferLength: Integer; const s2: AnsiString): Boolean;

function GetLastErrorString (const MessageID: DWORD): AnsiString;

function LoadStringFromResource (const Instance: THandle; const ResourceName: AnsiString): AnsiString;

function CharExists (const s: AnsiString; const c: AnsiChar): Boolean;
{ Returns True if AnsiChar c exists in AnsiString s. }

procedure AppendChar (var s: AnsiString; const c: AnsiChar);
{ Adds a character to a string. }

function FirstCharsFromWords (const s: AnsiString; const MaxCharCount: Integer; const WordSeparators: TAnsiCharSet): AnsiString;

function ProperCase (const s: AnsiString): AnsiString;

function IsEmptyString (const s: AnsiString): Boolean;

function PScanSetL (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start, BufferLength: Integer): Integer;
function PScanNotSetL (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start, BufferLength: Integer): Integer;

function PScanSetBack (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start: Integer): Integer;
function PScanNotSetBack (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start: Integer): Integer;

procedure SkipChar (var Source: PAnsiChar; const Search: Char);
procedure SkipCharBack (var Source: PAnsiChar; const Search: Char);

procedure SkipNotChar (var Source: PAnsiChar; const Search: Char);
procedure SkipNotCharBack (var Source: PAnsiChar; const Search: Char);

procedure SkipSet (var p: PAnsiChar; const Search: TAnsiCharSet);
procedure SkipSetBack (var p: PAnsiChar; const Search: TAnsiCharSet);

procedure SkipNotSet (var p: PAnsiChar; const Search: TAnsiCharSet);
procedure SkipNotSetBack (var p: PAnsiChar; const Search: TAnsiCharSet);

procedure PLowerCase (Run: PAnsiChar);
procedure LowerCase (var s: AnsiString); overload;
function LowerCaseFunc (const s: AnsiString): AnsiString;

procedure PUpperCase (Run: PAnsiChar);
procedure UpperCase (var s: AnsiString);
function UpperCaseFunc (const s: AnsiString): AnsiString;

procedure ReplaceStr (var Str: string; const sub1, sub2: string);

function ScanChar (const Source: AnsiString; const c: AnsiChar; const Start: Integer): Integer;
{ Forward scan from Start looking for next matching char. (X).
  Returns: Position where/if found; otherwise, 0. }

function ScanCharBack (const Source: AnsiString; const c: AnsiChar; const Start: Integer): Integer;
{ Backward/reverse scan from Start location (1 = First Char., 0 = String End) looking for single character, X.
  Returns: Position where/if found; otherwise, 0. }

function PScanChar (const Source: PAnsiChar; const c: Char): Integer;

function ScanSet (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ ScanS sucht das erste Zeichen, das im Set ist.
  Rckgabewerte: Position bei Erfolg, 0 wenn nicht gefunden oder Start zu gro / klein.}

function ScanSetBack (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ Backward/reverse scan from Start location (1 = First Char., 0 = String End) looking for first char in set Search.
  Returns: Position where/if found; otherwise, 0. }

function ScanNotSet (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ Scans Source for the first Character in Set Search.
  Returns: Position where/if found; otherwise, 0. }

function ScanNotSetBack (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ ScanSetBack: Wie ScanNS, sucht jedoch rckwrts}

function PScanNotSet (const Source: PAnsiChar; const Search: TAnsiCharSet): Integer;

procedure ReplaceChar (var Source: AnsiString; const x, y: AnsiChar);
{ Search and replace all instances of char. X with char Y. }

procedure TomCat (const s: AnsiString; var d: AnsiString; var InUse: Integer); overload;
{ String concatenation with smart memory allocation.
  Offers a speed advantage when building a long resultant string (D)
  from many small string fragments.

  InUse is a user-supplied variable which is updated by the procedure
  to track the portion of D actually in use" at any time
  (typically less than the allocated length).
  Initialize to zero or Length(D) as appropriate at the outset
  but do not manually alter otherwise.

  Once concatenation is finished, use SetLength (D, InUse)
  to trim any unused excess from the resultant. }

procedure TomCat (const c: AnsiChar; var d: AnsiString; var InUse: Integer); overload;
procedure TomCat (const Source: Pointer; const SourceLength: Integer; var Dest: AnsiString; var InUse: Integer); overload;

function LongMonthName (const i: Integer): AnsiString;
{ Funktioniert auch mit LongMonthNames [i], in SysUtils definiert.
  Diese Funktionen nutzen also nur dann, wenn SysUtils nicht gebraucht wird. }
function ShortMonthName (const i: Integer): AnsiString;
function LongDayName (const i: Integer): AnsiString;
function ShortDayName (const i: Integer): AnsiString;

{ Hash Functions }
function HashRJ (const s: AnsiString): Cardinal;
function HashPJW (const s: AnsiString): Cardinal;
function HashElf (const s: AnsiString): Cardinal;
function HashBKDR (const s: AnsiString): Cardinal;

const
 
 UpperCharArray     : array[#0..#255] of Cardinal = (
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
  20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
  30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
  40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
  50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
  60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
  70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
  80, 81, 82, 83, 84, 85, 86, 87, 88, 89,
  90, 91, 92, 93, 94, 95, 96, 65, 66, 67,
  68, 69, 70, 71, 72, 73, 74, 75, 76, 77,
  78, 79, 80, 81, 82, 83, 84, 85, 86, 87,
  88, 89, 90, 123, 124, 125, 126, 127, 128, 129,
  130, 131, 132, 133, 134, 135, 136, 137, 138, 139,
  140, 141, 142, 143, 144, 145, 146, 147, 148, 149,
  150, 151, 152, 153, 138, 155, 140, 157, 142, 159,
  160, 161, 162, 163, 164, 165, 166, 167, 168, 169,
  170, 171, 172, 173, 174, 175, 176, 177, 178, 179,
  180, 181, 182, 183, 184, 185, 186, 187, 188, 189,
  190, 191, 192, 193, 194, 195, 196, 197, 198, 199,
  200, 201, 202, 203, 204, 205, 206, 207, 208, 209,
  210, 211, 212, 213, 214, 215, 216, 217, 218, 219,
  220, 221, 222, 223, 192, 193, 194, 195, 196, 197,
  198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
  208, 209, 210, 211, 212, 213, 214, 247, 216, 217,
  218, 219, 220, 221, 254, 159);
 {$Z4}
 
 AsciiLowerChars    : array[#0..#255] of Char = (
  #000, #001, #002, #003, #004, #005, #006, #007, #008, #009, #010, #011, #012, #013, #014, #015,
  #016, #017, #018, #019, #020, #021, #022, #023, #024, #025, #026, #027, #028, #029, #030, #031,
  #032, #033, #034, #035, #036, #037, #038, #039, #040, #041, #042, #043, #044, #045, #046, #047,
  #048, #049, #050, #051, #052, #053, #054, #055, #056, #057, #058, #059, #060, #061, #062, #063,
  #064, #097, #098, #099, #100, #101, #102, #103, #104, #105, #106, #107, #108, #109, #110, #111,
  #112, #113, #114, #115, #116, #117, #118, #119, #120, #121, #122, #091, #092, #093, #094, #095,
  #096, #097, #098, #099, #100, #101, #102, #103, #104, #105, #106, #107, #108, #109, #110, #111,
  #112, #113, #114, #115, #116, #117, #118, #119, #120, #121, #122, #123, #124, #125, #126, #127,
  #128, #129, #130, #131, #132, #133, #134, #135, #136, #137, #154, #139, #156, #141, #158, #143,
  #144, #145, #146, #147, #148, #149, #150, #151, #152, #153, #154, #155, #156, #157, #158, #255,
  #160, #161, #162, #163, #164, #165, #166, #167, #168, #169, #170, #171, #172, #173, #174, #175,
  #176, #177, #178, #179, #180, #181, #182, #183, #184, #185, #186, #187, #188, #189, #190, #191,
  #224, #225, #226, #227, #228, #229, #230, #231, #232, #233, #234, #235, #236, #237, #238, #239,
  #240, #241, #242, #243, #244, #245, #246, #215, #248, #249, #250, #251, #252, #253, #254, #223,
  #224, #225, #226, #227, #228, #229, #230, #231, #232, #233, #234, #235, #236, #237, #238, #239,
  #240, #241, #242, #243, #244, #245, #246, #247, #248, #249, #250, #251, #252, #253, #254, #255);
 
 AsciiUpperChars    : array[#0..#255] of Char = (
  #000, #001, #002, #003, #004, #005, #006, #007, #008, #009, #010, #011, #012, #013, #014, #015,
  #016, #017, #018, #019, #020, #021, #022, #023, #024, #025, #026, #027, #028, #029, #030, #031,
  #032, #033, #034, #035, #036, #037, #038, #039, #040, #041, #042, #043, #044, #045, #046, #047,
  #048, #049, #050, #051, #052, #053, #054, #055, #056, #057, #058, #059, #060, #061, #062, #063,
  #064, #065, #066, #067, #068, #069, #070, #071, #072, #073, #074, #075, #076, #077, #078, #079,
  #080, #081, #082, #083, #084, #085, #086, #087, #088, #089, #090, #091, #092, #093, #094, #095,
  #096, #065, #066, #067, #068, #069, #070, #071, #072, #073, #074, #075, #076, #077, #078, #079,
  #080, #081, #082, #083, #084, #085, #086, #087, #088, #089, #090, #123, #124, #125, #126, #127,
  #128, #129, #130, #131, #132, #133, #134, #135, #136, #137, #138, #139, #140, #141, #142, #143,
  #144, #145, #146, #147, #148, #149, #150, #151, #152, #153, #138, #155, #140, #157, #142, #159,
  #160, #161, #162, #163, #164, #165, #166, #167, #168, #169, #170, #171, #172, #173, #174, #175,
  #176, #177, #178, #179, #180, #181, #182, #183, #184, #185, #186, #187, #188, #189, #190, #191,
  #192, #193, #194, #195, #196, #197, #198, #199, #200, #201, #202, #203, #204, #205, #206, #207,
  #208, #209, #210, #211, #212, #213, #214, #215, #216, #217, #218, #219, #220, #221, #222, #223,
  #192, #193, #194, #195, #196, #197, #198, #199, #200, #201, #202, #203, #204, #205, #206, #207,
  #208, #209, #210, #211, #212, #213, #214, #247, #216, #217, #218, #219, #220, #221, #222, #159);
 
 AsciiReverseChars  : array[#0..#255] of Char = (
  #000, #001, #002, #003, #004, #005, #006, #007, #008, #009, #010, #011, #012, #013, #014, #015,
  #016, #017, #018, #019, #020, #021, #022, #023, #024, #025, #026, #027, #028, #029, #030, #031,
  #032, #033, #034, #035, #036, #037, #038, #039, #040, #041, #042, #043, #044, #045, #046, #047,
  #048, #049, #050, #051, #052, #053, #054, #055, #056, #057, #058, #059, #060, #061, #062, #063,
  #064, #097, #098, #099, #100, #101, #102, #103, #104, #105, #106, #107, #108, #109, #110, #111,
  #112, #113, #114, #115, #116, #117, #118, #119, #120, #121, #122, #091, #092, #093, #094, #095,
  #096, #065, #066, #067, #068, #069, #070, #071, #072, #073, #074, #075, #076, #077, #078, #079,
  #080, #081, #082, #083, #084, #085, #086, #087, #088, #089, #090, #123, #124, #125, #126, #127,
  #128, #129, #130, #131, #132, #133, #134, #135, #136, #137, #154, #139, #156, #141, #158, #143,
  #144, #145, #146, #147, #148, #149, #150, #151, #152, #153, #138, #155, #140, #157, #142, #255,
  #160, #161, #162, #163, #164, #165, #166, #167, #168, #169, #170, #171, #172, #173, #174, #175,
  #176, #177, #178, #179, #180, #181, #182, #183, #184, #185, #186, #187, #188, #189, #190, #191,
  #224, #225, #226, #227, #228, #229, #230, #231, #232, #233, #234, #235, #236, #237, #238, #239,
  #240, #241, #242, #243, #244, #245, #246, #215, #248, #249, #250, #251, #252, #253, #254, #223,
  #192, #193, #194, #195, #196, #197, #198, #199, #200, #201, #202, #203, #204, #205, #206, #207,
  #208, #209, #210, #211, #212, #213, #214, #247, #216, #217, #218, #219, #220, #221, #222, #159);
 
implementation

uses
 rjConsts;

{ **************************************************************************** }

procedure AddSlash (var Path: AnsiString);
var
 Length, NewLength  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Path);
 { If string is nil, return PATH_SEPARATOR. }
 if p = nil then
  begin
   Path := PATH_SEPARATOR;
   Exit;
  end;
 { If last character of string is PATH_SEPARATOR, we have nothing to do. }
 Length := Integer (Pointer (Cardinal (Path) - 4)^) - 1;
 if p[Length] = PATH_SEPARATOR then Exit;
 { Backtrack spaces at the end of string. }
 NewLength := PScanNotSetBack (p, ANSI_WHITE_SPACE, Length);
 { String consists of spaces only (is empty): return PATH_SEPARATOR. }
 if NewLength = -1 then
  begin
   Path := PATH_SEPARATOR;
   Exit;
  end;
 { There are spaces at the end of string,
   and the last of the real characters is a PATH_SEPARATOR:
   Shorten string and exit. }
 if p[NewLength] = PATH_SEPARATOR then
  begin
   SetLength (Path, NewLength + 1);
   Exit;
  end;
 { The last possibility: We found some character in path which is
   not whitespace or PATH_SEPARATOR. Now adjust path length
   and set last character to PATH_SEPARATOR. }
 SetLength (Path, NewLength + 2);
 { We must not use p here instead of Pointer (Path),
   because Path might have been reallocated by SetLength. }
 Char (Pointer (Cardinal (Path) + Cardinal (NewLength) + 1)^) := PATH_SEPARATOR;
end;

{ ********** }

procedure DelSlash (var Path: AnsiString);
var
 Length, NewLength  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Path);
 { If string is nil, simply exit. }
 if p = nil then Exit;
 { Backtrack spaces and PATH_SEPARATORs at the end of string. }
 Length := Integer (Pointer (Cardinal (p) - 4)^) - 1;
 NewLength := PScanNotSetBack (p, (ANSI_WHITE_SPACE + [PATH_SEPARATOR]), Length);
 { If NewLength = Length: No space or PATH_SEPARATOR to delete. Simply exit. }
 if NewLength = Length then Exit;
 { If string consists of spaces or PATH_SEPARATORs only:
   String is virtually empty, return empty Path. }
 if NewLength = -1 then
  begin
   Path := '';
   Exit;
  end;
 { Set new Length of string. }
 SetLength (Path, NewLength + 1);
end;

{ **************************************************************************** }

function StrLen (const p: PAnsiChar): Integer;
{ StrLen replacement. 1.5 times faster than the Delphi original. }
begin
 Result := 0;
 if p <> nil then
  while p[Result] <> #0 do
   if p[Result + 1] <> #0 then
    if p[Result + 2] <> #0 then
     if p[Result + 3] <> #0 then
      Inc (Result, 4)
     else
      begin
       Inc (Result, 3);
       Exit;
      end
    else
     begin
      Inc (Result, 2);
      Exit;
     end
   else
    begin
     Inc (Result);
     Exit;
    end;
end;

{******************************************************************************}
{ Padding Strings }
{******************************************************************************}

function PadLeft (const Source: AnsiString; const c: AnsiChar; const Count: Integer): AnsiString;
var
 i, l               : Integer;
 p1, p2             : PAnsiChar;
begin
 { Figures out the lenght of Source. }
 p1 := Pointer (Source);
 if p1 <> nil then
  l := Integer (Pointer (Cardinal (p1) - 4)^)
 else
  l := 0;
 
 { Do we really need to pad Source or is s already longer then length? }
 if Count > l then
  begin
   { Fill the leading characters with Char X.
     We start with the last character, so we can compare against zero. }
   SetLength (Result, Count);
   p2 := Pointer (Result);
   i := Count - l;
   repeat
    Dec (i);
    p2[i] := c;
   until i = 0;
   { If string s has any contents (Length (Source) > 0) ... }
   if l > 0 then
    begin
     { ... copy all characters (one by one) from Source to the result string.
       We start with the last character, so we can compare against zero. }
     Inc (p2, Count - l);
     repeat
      Dec (l);
      p2[l] := p1[l];
     until l = 0;
    end;
  end
 else
  { If we don't need to pad, simply return Source. }
  Result := Source;
end;

{ ********** }

function PadRight (const Source: AnsiString; const c: AnsiChar; const Count: Integer): AnsiString;
var
 i, l               : Integer;
 p1, p2             : PAnsiChar;
begin
 { Figures out the lenght of Source. }
 p1 := Pointer (Source);
 if p1 <> nil then
  l := Integer (Pointer (Cardinal (p1) - 4)^)
 else
  l := 0;
 { Do we really need to pad Source or is Source already longer then length? }
 if Count > l then
  begin
   SetLength (Result, Count);
   p2 := Pointer (Result);
   { Copy Source to begining of Result. }
   i := l - 1;
   while i >= 0 do
    begin
     p2[i] := p1[i];
     Dec (i);
    end;
   { Fill trailing chars of Result with char. }
   Inc (p2, l);
   i := Count - l;
   repeat
    Dec (i);
    p2[i] := c;
   until i = 0;
  end
 else
  { If we don't need to add anything, simply return Source. }
  Result := Source;
end;

{ **************************************************************************** }

function TrimChar (const Source: AnsiString; const c: AnsiChar): AnsiString;
label
 BeginZero, BeginOne, BeginTwo, BeginThree,
  EndZero, EndOne, EndTwo, EndThree,
  ReturnEmptyString;
var
 l, Length          : Integer;
 p, e               : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto ReturnEmptyString;
 
 Length := Integer (Pointer (Cardinal (p) - 4)^);
 if Length = 0 then goto ReturnEmptyString;
 
 l := Length;
 e := p + l - 1;
 
 while l >= 4 do
  begin
   if p^ <> c then goto BeginZero;
   if p[1] <> c then goto BeginOne;
   if p[2] <> c then goto BeginTwo;
   if p[3] <> c then goto BeginThree;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if p^ <> c then goto BeginZero;
    if p[1] <> c then goto BeginOne;
    if p[2] <> c then goto BeginTwo;
    Inc (p, 3);
   end;
  2:
   begin
    if p^ <> c then goto BeginZero;
    if p[1] <> c then goto BeginOne;
    Inc (p, 2);
   end;
  1:
   begin
    if p^ <> c then goto BeginZero;
    Inc (p);
   end;
 end;
 
 goto BeginZero;
 
 BeginThree:
 Inc (p, 3);
 goto BeginZero;
 
 BeginTwo:
 Inc (p, 2);
 goto BeginZero;
 
 BeginOne:
 Inc (p);
 
 BeginZero:
 
 l := e - p + 1;
 if l = 0 then goto ReturnEmptyString;
 
 while l >= 4 do
  begin
   if e^ <> c then goto EndZero;
   if e[-1] <> c then goto EndOne;
   if e[-2] <> c then goto EndTwo;
   if e[-3] <> c then goto EndThree;
   Dec (e, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if e^ <> c then goto EndZero;
    if e[-1] <> c then goto EndOne;
    goto EndTwo;
   end;
  2:
   begin
    if e^ <> c then goto EndZero;
    goto EndOne;
   end;
 end;
 
 goto EndZero;
 
 EndThree:
 Dec (l, 3);
 goto EndZero;
 
 EndTwo:
 Dec (l, 2);
 goto EndZero;
 
 EndOne:
 Dec (l);
 
 EndZero:
 
 if l = Length then
  Result := Source
 else
  begin
   SetLength (Result, l);
   e := Pointer (Result);

   while l >= 4 do
    begin
     Cardinal (Pointer (e)^) := Cardinal (Pointer (p)^);
     Inc (e, 4);
     Inc (p, 4);
     Dec (l, 4);
    end;

   case l of
    3:
     begin
      e^ := p^;
      e[1] := p[1];
      e[2] := p[2];
     end;
    2:
     begin
      e^ := p^;
      e[1] := p[1];
     end;
    1:
     begin
      e^ := p^;
     end;
   end;
  end;
 Exit;
 
 ReturnEmptyString:
 Result := '';
end;

{ ********** }

function TrimCharLeft (const Source: AnsiString; const c: AnsiChar): AnsiString;
label
 BeginZero, BeginOne, BeginTwo, BeginThree,
  ReturnEmptyString;
var
 l                  : Integer;
 p, e               : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto ReturnEmptyString;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if l = 0 then goto ReturnEmptyString;
 
 while l >= 4 do
  begin
   if p^ <> c then goto BeginZero;
   if p[1] <> c then goto BeginOne;
   if p[2] <> c then goto BeginTwo;
   if p[3] <> c then goto BeginThree;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if p^ <> c then goto BeginZero;
    if p[1] <> c then goto BeginOne;
    if p[2] <> c then goto BeginTwo;
    Inc (p, 3);
    Dec (l, 3);
   end;
  2:
   begin
    if p^ <> c then goto BeginZero;
    if p[1] <> c then goto BeginOne;
    Inc (p, 2);
    Dec (l, 2);
   end;
  1:
   begin
    if p^ <> c then goto BeginZero;
    Inc (p);
    Dec (l);
   end;
 end;
 
 goto BeginZero;
 
 BeginThree:
 Inc (p, 3);
 Dec (l, 3);
 goto BeginZero;
 
 BeginTwo:
 Inc (p, 2);
 Dec (l, 2);
 goto BeginZero;
 
 BeginOne:
 Inc (p);
 Dec (l);
 
 BeginZero:
 
 if p = Pointer (Source) then
  Result := Source
 else
  begin
   SetLength (Result, l);
   e := Pointer (Result);

   while l >= 4 do
    begin
     Cardinal (Pointer (e)^) := Cardinal (Pointer (p)^);
     Inc (e, 4);
     Inc (p, 4);
     Dec (l, 4);
    end;

   case l of
    3:
     begin
      e^ := p^;
      e[1] := p[1];
      e[2] := p[2];
     end;
    2:
     begin
      e^ := p^;
      e[1] := p[1];
     end;
    1:
     begin
      e^ := p^;
     end;
   end;
  end;
 Exit;
 
 ReturnEmptyString:
 Result := '';
end;

{ ********** }

function TrimSet (const Source: AnsiString; const s: TAnsiCharSet): AnsiString;
label
 BeginZero, BeginOne, BeginTwo, BeginThree,
  EndZero, EndOne, EndTwo, EndThree,
  ReturnEmptyString;
var
 l, Length          : Integer;
 p, e               : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto ReturnEmptyString;
 
 Length := Integer (Pointer (Cardinal (p) - 4)^);
 if Length = 0 then goto ReturnEmptyString;
 
 l := Length;
 e := p + l - 1;
 
 while l >= 4 do
  begin
   if not (p^ in s) then goto BeginZero;
   if not (p[1] in s) then goto BeginOne;
   if not (p[2] in s) then goto BeginTwo;
   if not (p[3] in s) then goto BeginThree;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if not (p^ in s) then goto BeginZero;
    if not (p[1] in s) then goto BeginOne;
    if not (p[2] in s) then goto BeginTwo;
    Inc (p, 3);
   end;
  2:
   begin
    if not (p^ in s) then goto BeginZero;
    if not (p[1] in s) then goto BeginOne;
    Inc (p, 2);
   end;
  1:
   begin
    if not (p^ in s) then goto BeginZero;
    Inc (p);
   end;
 end;
 
 goto BeginZero;
 
 BeginThree:
 Inc (p, 3);
 goto BeginZero;
 
 BeginTwo:
 Inc (p, 2);
 goto BeginZero;
 
 BeginOne:
 Inc (p);
 
 BeginZero:
 
 l := e - p + 1;
 if l = 0 then goto ReturnEmptyString;
 
 while l >= 4 do
  begin
   if not (e^ in s) then goto EndZero;
   if not (e[-1] in s) then goto EndOne;
   if not (e[-2] in s) then goto EndTwo;
   if not (e[-3] in s) then goto EndThree;
   Dec (e, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if not (e^ in s) then goto EndZero;
    if not (e[-1] in s) then goto EndOne;
    goto EndTwo;
   end;
  2:
   begin
    if not (e^ in s) then goto EndZero;
    goto EndOne;
   end;
 end;
 
 goto EndZero;
 
 EndThree:
 Dec (l, 3);
 goto EndZero;
 
 EndTwo:
 Dec (l, 2);
 goto EndZero;
 
 EndOne:
 Dec (l);
 
 EndZero:
 
 if l = Length then
  Result := Source
 else
  begin
   SetLength (Result, l);
   e := Pointer (Result);

   while l >= 4 do
    begin
     Cardinal (Pointer (e)^) := Cardinal (Pointer (p)^);
     Inc (e, 4);
     Inc (p, 4);
     Dec (l, 4);
    end;

   case l of
    3:
     begin
      e^ := p^;
      e[1] := p[1];
      e[2] := p[2];
     end;
    2:
     begin
      e^ := p^;
      e[1] := p[1];
     end;
    1:
     begin
      e^ := p^;
     end;
   end;
  end;
 Exit;
 
 ReturnEmptyString:
 Result := '';
end;

{ **************************************************************************** }

procedure ReplaceChar (var Source: AnsiString; const x, y: AnsiChar);
var
 p                  : PAnsiChar;
 l                  : Integer;
begin
 p := Pointer (Source);
 if p = nil then Exit;
 UniqueString (Source);
 
 p := Pointer (Source);
 if p = nil then Exit;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   if p^ = x then p^ := y;
   if p[1] = x then p[1] := y;
   if p[2] = x then p[2] := y;
   if p[3] = x then p[3] := y;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if p^ = x then p^ := y;
    if p[1] = x then p[1] := y;
    if p[2] = x then p[2] := y;
   end;
  2:
   begin
    if p^ = x then p^ := y;
    if p[1] = x then p[1] := y;
   end;
  1:
   begin
    if p^ = x then p^ := y;
   end;
 end;
end;

{ **************************************************************************** }

procedure TomCat (const s: AnsiString; var d: AnsiString; var InUse: Integer);
var
 PCharS, PCharD     : PAnsiChar;
 lS, lD, NewInUse   : Integer;
begin
 PCharS := Pointer (s);
 if PCharS = nil then Exit;
 
 lS := Integer (Pointer (Cardinal (PCharS) - 4)^);
 if lS <= 0 then Exit;
 
 PCharD := Pointer (d);
 if PCharD = nil then
  lD := 0
 else
  lD := Integer (Pointer (Cardinal (PCharD) - 4)^);
 
 NewInUse := InUse + lS;
 
 if NewInUse > lD then
  begin
   SetLength (d, (NewInUse + (NewInUse div 2) + 3) and $FFFFFFFC);
   PCharD := Pointer (d);
  end;
 
 Inc (PCharD, InUse);
 while lS >= 4 do
  begin
   Cardinal (Pointer (PCharD)^) := Cardinal (Pointer (PCharS)^);
   Inc (PCharD, 4);
   Inc (PCharS, 4);
   Dec (lS, 4);
  end;
 
 case lS of
  3:
   begin
    PCharD^ := PCharS^;
    PCharD[1] := PCharS[1];
    PCharD[2] := PCharS[2];
   end;
  2:
   begin
    PCharD^ := PCharS^;
    PCharD[1] := PCharS[1];
   end;
  1:
   begin
    PCharD^ := PCharS^;
   end;
 end;
 
 InUse := NewInUse;
end;

{ ********** }

procedure TomCat (const c: AnsiChar; var d: AnsiString; var InUse: Integer);
var
 PCharD             : PAnsiChar;
 lD, NewInUse       : Integer;
begin
 PCharD := Pointer (d);
 if PCharD = nil then
  lD := 0
 else
  lD := Integer (Pointer (Cardinal (PCharD) - 4)^);
 
 NewInUse := InUse + 1;                 // Add the length of one AnsiChar
 
 if NewInUse > lD then
  begin
   SetLength (d, (NewInUse + (NewInUse div 2) + 3) and $FFFFFFFC);
   PCharD := Pointer (d);
  end;
 
 PCharD[InUse] := c;
 
 InUse := NewInUse;
end;

{ ********** }

procedure TomCat (const Source: Pointer; const SourceLength: Integer; var Dest: AnsiString; var InUse: Integer);
var
 PCharS, PCharD     : PAnsiChar;
 lS, lD, NewInUse   : Integer;
begin
 if Source = nil then Exit;
 if SourceLength <= 0 then Exit;
 
 PCharS := Source;
 lS := SourceLength;
 
 PCharD := Pointer (Dest);
 if PCharD = nil then
  lD := 0
 else
  lD := Integer (Pointer (Cardinal (PCharD) - 4)^);
 
 NewInUse := InUse + lS;
 
 if NewInUse > lD then
  begin
   SetLength (Dest, (NewInUse + (NewInUse div 2) + 3) and $FFFFFFFC);
   PCharD := Pointer (Dest);
  end;
 
 Inc (PCharD, InUse);
 while lS >= 4 do
  begin
   Cardinal (Pointer (PCharD)^) := Cardinal (Pointer (PCharS)^);
   Inc (PCharD, SizeOf (Cardinal));
   Inc (PCharS, SizeOf (Cardinal));
   Dec (lS, SizeOf (Cardinal));
  end;
 
 case lS of
  3:
   begin
    PCharD^ := PCharS^;
    PCharD[1] := PCharS[1];
    PCharD[2] := PCharS[2];
   end;
  2:
   begin
    PCharD^ := PCharS^;
    PCharD[1] := PCharS[1];
   end;
  1:
   begin
    PCharD^ := PCharS^;
   end;
 end;
 
 InUse := NewInUse;
end;

{******************************************************************************}
{ Comparing Strings }
{******************************************************************************}

function PCompStr (p1, p2: PAnsiChar): Integer;
begin
 if p1 = p2 then
  begin
   Result := 0;
   Exit;
  end;
 
 if p1 = nil then
  begin
   Result := -Byte (p2^);
   Exit;
  end;
 
 if p2 = nil then
  begin
   Result := Byte (p1^);
   Exit;
  end;
 
 if (p1^ = #0) or (p1^ <> p2^) then
  begin
   Result := Byte (p1^) - Byte (p2^);
   Exit;
  end;
 
 while (p1^ <> #0) and (p1^ = p2^) do
  if (p1[1] <> #0) and (p1[1] = p2[1]) then
   if (p1[2] <> #0) and (p1[2] = p2[2]) then
    if (p1[3] <> #0) and (p1[3] = p2[3]) then
     begin
      Inc (p1, 4);
      Inc (p2, 4);
     end
    else
     begin
      Inc (p1, 3);
      Inc (p2, 3);
      Break;
     end
   else
    begin
     Inc (p1, 2);
     Inc (p2, 2);
     Break;
    end
  else
   begin
    Inc (p1);
    Inc (p2);
    Break;
   end;
 
 Result := Byte (p1^) - Byte (p2^);
end;

{ ********** }

function PCompStrL (p1, p2: PAnsiChar; MaxLen: Integer): Integer;
label
 Zero, One, Two, Three, Match;
begin
 if p1 = p2 then
  begin
   Result := 0;
   Exit;
  end;
 
 if p1 = nil then
  begin
   Result := -Byte (p2^);
   Exit;
  end;
 
 if p2 = nil then
  begin
   Result := Byte (p1^);
   Exit;
  end;
 
 while MaxLen >= 4 do
  begin
   if (p1^ = #0) or (p1^ <> p2^) then goto Zero;
   if (p1[1] = #0) or (p1[1] <> p2[1]) then goto One;
   if (p1[2] = #0) or (p1[2] <> p2[2]) then goto Two;
   if (p1[3] = #0) or (p1[3] <> p2[3]) then goto Three;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (MaxLen, 4);
  end;
 
 case MaxLen of
  3:
   begin
    if (p1^ = #0) or (p1^ <> p2^) then goto Zero;
    if (p1[1] = #0) or (p1[1] <> p2[1]) then goto One;
    if (p1[2] = #0) or (p1[2] <> p2[2]) then goto Two;
   end;
  2:
   begin
    if (p1^ = #0) or (p1^ <> p2^) then goto Zero;
    if (p1[1] = #0) or (p1[1] <> p2[1]) then goto One;
   end;
  1:
   begin
    if (p1^ = #0) or (p1^ <> p2^) then goto Zero;
   end;
 end;
 
 Match:
 Result := 0;
 Exit;
 
 Zero:
 Result := Byte (p1^) - Byte (p2^);
 Exit;
 
 One:
 Result := Byte (p1[1]) - Byte (p2[1]);
 Exit;
 
 Two:
 Result := Byte (p1[2]) - Byte (p2[2]);
 Exit;
 
 Three:
 Result := Byte (p1[3]) - Byte (p2[3]);
end;

{ ********** }

function PSameStrL (p1, p2: PAnsiChar; MaxLen: Integer): Boolean;
label
 Fail, Match;
begin
 if p1 = p2 then goto Match;
 
 if (p1 = nil) or (p2 = nil) then goto Fail;
 
 while MaxLen >= 4 do
  begin
   if (p1^ = #0) or (p1^ <> p2^) then goto Fail;
   if (p1[1] = #0) or (p1[1] <> p2[1]) then goto Fail;
   if (p1[2] = #0) or (p1[2] <> p2[2]) then goto Fail;
   if (p1[3] = #0) or (p1[3] <> p2[3]) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (MaxLen, 4);
  end;
 
 case MaxLen of
  3:
   begin
    if (p1^ = #0) or (p1^ <> p2^) then goto Fail;
    if (p1[1] = #0) or (p1[1] <> p2[1]) then goto Fail;
    if (p1[2] = #0) or (p1[2] <> p2[2]) then goto Fail;
   end;
  2:
   begin
    if (p1^ = #0) or (p1^ <> p2^) then goto Fail;
    if (p1[1] = #0) or (p1[1] <> p2[1]) then goto Fail;
   end;
  1:
   begin
    if (p1^ = #0) or (p1^ <> p2^) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{ ********** }

function PCompText (p1, p2: PAnsiChar): Integer;
begin
 if p1 = p2 then
  begin
   Result := 0;
   Exit;
  end;
 
 if p1 = nil then
  begin
   Result := -Byte (p2^);
   Exit;
  end;
 
 if p2 = nil then
  begin
   Result := Byte (p1^);
   Exit;
  end;
 
 while (p1^ <> #0) and ((p1^ = p2^) or (p1^ = AsciiReverseChars[p2^])) do
  if (p1[1] <> #0) and ((p1[1] = p2[1]) or (p1[1] = AsciiReverseChars[p2[1]])) then
   if (p1[2] <> #0) and ((p1[2] = p2[2]) or (p1[2] = AsciiReverseChars[p2[2]])) then
    if (p1[3] <> #0) and ((p1[3] = p2[3]) or (p1[3] = AsciiReverseChars[p2[3]])) then
     begin
      Inc (p1, 4);
      Inc (p2, 4);
     end
    else
     begin
      Inc (p1, 3);
      Inc (p2, 3);
      Break;
     end
   else
    begin
     Inc (p1, 2);
     Inc (p2, 2);
     Break;
    end
  else
   begin
    Inc (p1);
    Inc (p2);
    Break;
   end;
 
 Result := Byte (AsciiUpperChars[p1^]) - Byte (AsciiUpperChars[p2^]);
end;

{ ********** }

function PCompTextL (p1, p2: PAnsiChar; MaxLen: Integer): Integer;
label
 Zero, One, Two, Three, Match;
begin
 if p1 = p2 then goto Match;
 
 if p1 = nil then
  begin
   Result := -Byte (p2^);
   Exit;
  end;
 
 if p2 = nil then
  begin
   Result := Byte (p1^);
   Exit;
  end;
 
 while MaxLen >= 4 do
  begin
   if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Zero;
   if (p1[1] = #0) or ((p1[1] <> p2[1]) and (AsciiReverseChars[p1[1]] <> p2[1])) then goto One;
   if (p1[2] = #0) or ((p1[2] <> p2[2]) and (AsciiReverseChars[p1[2]] <> p2[2])) then goto Two;
   if (p1[3] = #0) or ((p1[3] <> p2[3]) and (AsciiReverseChars[p1[3]] <> p2[3])) then goto Three;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (MaxLen, 4);
  end;
 
 case MaxLen of
  3:
   begin
    if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Zero;
    if (p1[1] = #0) or ((p1[1] <> p2[1]) and (AsciiReverseChars[p1[1]] <> p2[1])) then goto One;
    if (p1[2] = #0) or ((p1[2] <> p2[2]) and (AsciiReverseChars[p1[2]] <> p2[2])) then goto Two;
   end;
  2:
   begin
    if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Zero;
    if (p1[1] = #0) or ((p1[1] <> p2[1]) and (AsciiReverseChars[p1[1]] <> p2[1])) then goto One;
   end;
  1:
   begin
    if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Zero;
   end;
 end;
 
 Match:
 Result := 0;
 Exit;
 
 Zero:
 Result := Byte (AsciiUpperChars[p1^]) - Byte (AsciiUpperChars[p2^]);
 Exit;
 
 One:
 Result := Byte (AsciiUpperChars[p1[1]]) - Byte (AsciiUpperChars[p2[1]]);
 Exit;
 
 Two:
 Result := Byte (AsciiUpperChars[p1[2]]) - Byte (AsciiUpperChars[p2[2]]);
 Exit;
 
 Three:
 Result := Byte (AsciiUpperChars[p1[3]]) - Byte (AsciiUpperChars[p2[3]]);
end;

{ ********** }

function PSameTextL (p1, p2: PAnsiChar; MaxLen: Cardinal): Boolean;
label
 Fail, Match;
begin
 if p1 = p2 then goto Match;
 
 if (p1 = nil) or (p2 = nil) then goto Fail;
 
 while MaxLen >= 4 do
  begin
   if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Fail;
   if (p1[1] = #0) or ((p1[1] <> p2[1]) and (AsciiReverseChars[p1[1]] <> p2[1])) then goto Fail;
   if (p1[2] = #0) or ((p1[2] <> p2[2]) and (AsciiReverseChars[p1[2]] <> p2[2])) then goto Fail;
   if (p1[3] = #0) or ((p1[3] <> p2[3]) and (AsciiReverseChars[p1[3]] <> p2[3])) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (MaxLen, 4);
  end;
 
 case MaxLen of
  3:
   begin
    if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Fail;
    if (p1[1] = #0) or ((p1[1] <> p2[1]) and (AsciiReverseChars[p1[1]] <> p2[1])) then goto Fail;
    if (p1[2] = #0) or ((p1[2] <> p2[2]) and (AsciiReverseChars[p1[2]] <> p2[2])) then goto Fail;
   end;
  2:
   begin
    if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Fail;
    if (p1[1] = #0) or ((p1[1] <> p2[1]) and (AsciiReverseChars[p1[1]] <> p2[1])) then goto Fail;
   end;
  1:
   begin
    if (p1^ = #0) or ((p1^ <> p2^) and (AsciiReverseChars[p1^] <> p2^)) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{ ********** }

function CompStr (const s1, s2: AnsiString): Integer;
label
 Zero, One, Two, Three, Match;
var
 p1, p2             : PAnsiChar;
 l, l1, l2          : Integer;
begin
 if Pointer (s1) = Pointer (s2) then goto Match;
 
 if Pointer (s1) = nil then
  begin
   Result := -Byte (Pointer (s2)^); Exit;
  end;
 
 if Pointer (s2) = nil then
  begin
   Result := Byte (Pointer (s1)^); Exit;
  end;
 
 l1 := Integer (Pointer (Cardinal (s1) - 4)^);
 l2 := Integer (Pointer (Cardinal (s2) - 4)^);
 if l1 > l2 then
  l := l2
 else
  l := l1;
 
 p1 := Pointer (s1);
 p2 := Pointer (s2);
 
 while l >= 4 do
  begin
   if (p1^ <> p2^) then goto Zero;
   if (p1[1] <> p2[1]) then goto One;
   if (p1[2] <> p2[2]) then goto Two;
   if (p1[3] <> p2[3]) then goto Three;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p1^ <> p2^) then goto Zero;
    if (p1[1] <> p2[1]) then goto One;
    if (p1[2] <> p2[2]) then goto Three;
   end;
  2:
   begin
    if (p1^ <> p2^) then goto Zero;
    if (p1[1] <> p2[1]) then goto One;
   end;
  1:
   begin
    if (p1^ <> p2^) then goto Zero;
   end;
 end;
 
 Result := l1 - l2;
 Exit;
 
 Match:
 Result := 0;
 Exit;
 
 Zero:
 Result := Byte (p1^) - Byte (p2^);
 Exit;
 
 One:
 Result := Byte (p1[1]) - Byte (p2[1]);
 Exit;
 
 Two:
 Result := Byte (p1[2]) - Byte (p2[2]);
 Exit;
 
 Three:
 Result := Byte (p1[3]) - Byte (p2[3]);
end;

{ ********** }

function SameStr (const s1, s2: AnsiString): Boolean;
label
 Fail, Match;
var
 p1, p2             : PAnsiChar;
 l1, l2             : Integer;
begin
 if Pointer (s1) = Pointer (s2) then goto Match;
 
 if (Pointer (s1) = nil) or (Pointer (s2) = nil) then goto Fail;
 
 l1 := Integer (Pointer (Cardinal (Pointer (s1)) - 4)^);
 l2 := Integer (Pointer (Cardinal (Pointer (s2)) - 4)^);
 
 if l1 <> l2 then goto Fail;
 
 p1 := Pointer (s1);
 p2 := Pointer (s2);
 
 while l1 >= 4 do
  begin
   if (p1^ <> p2^) then goto Fail;
   if (p1[1] <> p2[1]) then goto Fail;
   if (p1[2] <> p2[2]) then goto Fail;
   if (p1[3] <> p2[3]) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l1, 4);
  end;
 
 case l1 of
  3:
   begin
    if (p1^ <> p2^) then goto Fail;
    if (p1[1] <> p2[1]) then goto Fail;
    if (p1[2] <> p2[2]) then goto Fail;
   end;
  2:
   begin
    if (p1^ <> p2^) then goto Fail;
    if (p1[1] <> p2[1]) then goto Fail;
   end;
  1:
   begin
    if (p1^ <> p2^) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{ ********** }

function CompText (const s1, s2: AnsiString): Integer;
label
 Zero, One, Two, Three, Match;
var
 p1, p2             : PAnsiChar;
 l, l1, l2          : Integer;
begin
 if Pointer (s1) = Pointer (s2) then goto Match;
 
 if Pointer (s1) = nil then
  begin
   Result := -Byte (Pointer (s2)^); Exit;
  end;
 
 if Pointer (s2) = nil then
  begin
   Result := Byte (Pointer (s1)^); Exit;
  end;
 
 l1 := Integer (Pointer (Cardinal (s1) - 4)^);
 l2 := Integer (Pointer (Cardinal (s2) - 4)^);
 
 if l1 > l2 then
  l := l2
 else
  l := l1;
 
 p1 := Pointer (s1);
 p2 := Pointer (s2);
 
 while l >= 4 do
  begin
   if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Zero;
   if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto One;
   if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Two;
   if (p1[3] <> p2[3]) and (p1[3] <> AsciiReverseChars[p2[3]]) then goto Three;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Zero;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto One;
    if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Two;
   end;
  2:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Zero;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto One;
   end;
  1:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Zero;
   end;
 end;
 
 Result := l1 - l2;
 Exit;
 
 Match:
 Result := 0;
 Exit;
 
 Zero:
 Result := Byte (AsciiUpperChars[p1^]) - Byte (AsciiUpperChars[p2^]);
 Exit;
 
 One:
 Result := Byte (AsciiUpperChars[p1[1]]) - Byte (AsciiUpperChars[p2[1]]);
 Exit;
 
 Two:
 Result := Byte (AsciiUpperChars[p1[2]]) - Byte (AsciiUpperChars[p2[2]]);
 Exit;
 
 Three:
 Result := Byte (AsciiUpperChars[p1[3]]) - Byte (AsciiUpperChars[p2[3]]);
end;

{ ********** }

function SameText (const s1, s2: AnsiString): Boolean;
label
 Fail, Match;
var
 p1, p2             : PAnsiChar;
 l1, l2             : Integer;
begin
 if Pointer (s1) = Pointer (s2) then goto Match;
 
 if (Pointer (s1) = nil) or (Pointer (s2) = nil) then goto Fail;
 
 l1 := Integer (Pointer (Cardinal (s1) - 4)^);
 l2 := Integer (Pointer (Cardinal (s2) - 4)^);
 
 if l1 <> l2 then goto Fail;
 
 p1 := Pointer (s1);
 p2 := Pointer (s2);
 
 while l1 >= 4 do
  begin
   if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
   if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
   if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Fail;
   if (p1[3] <> p2[3]) and (p1[3] <> AsciiReverseChars[p2[3]]) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l1, 4);
  end;
 
 case l1 of
  3:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
    if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Fail;
   end;
  2:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
   end;
  1:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{ ********** }

function SameTextL (const s1, s2: AnsiString; const MaxLen: Integer): Boolean;
label
 Fail, Match;
var
 p1, p2             : PAnsiChar;
 l1, l2             : Integer;
begin
 if Pointer (s1) = Pointer (s2) then goto Match;
 
 if (Pointer (s1) = nil) or (Pointer (s2) = nil) then goto Fail;
 
 l1 := Integer (Pointer (Cardinal (s1) - 4)^);
 l2 := Integer (Pointer (Cardinal (s2) - 4)^);
 
 if l1 > l2 then l1 := l2;
 if l1 > MaxLen then l1 := MaxLen;
 
 p1 := Pointer (s1);
 p2 := Pointer (s2);
 
 while l1 >= 4 do
  begin
   if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
   if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
   if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Fail;
   if (p1[3] <> p2[3]) and (p1[3] <> AsciiReverseChars[p2[3]]) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l1, 4);
  end;
 
 case l1 of
  3:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
    if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Fail;
   end;
  2:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
   end;
  1:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{******************************************************************************}
{ Changing Case }
{******************************************************************************}

procedure PUpperCase (Run: PAnsiChar);
{ Do not call this function unless you are absolutely sure you are
  passing a reference to a unique string which resides entirely in memory. }
begin
 if Run <> nil then
  while Run^ <> #0 do
   begin
    Run^ := AsciiUpperChars[Run^];
    if Run[1] <> #0 then
     begin
      Run[1] := AsciiUpperChars[Run[1]];
      if Run[2] <> #0 then
       begin
        Run[2] := AsciiUpperChars[Run[2]];
        if Run[3] <> #0 then
         begin
          Run[3] := AsciiUpperChars[Run[3]];
          if Run[4] <> #0 then
           begin
            Run[4] := AsciiUpperChars[Run[4]];
            Inc (Run, 4);
           end
          else
           Break;
         end
        else
         Break;
       end
      else
       Break;
     end
    else
     Break;
   end;
end;

{ ********** }

procedure PLowerCase (Run: PAnsiChar);
{ Do not call this function unless you are absolutely sure you are
  passing a reference to a unique string which resides entirely in memory. }
begin
 if Run <> nil then
  while Run^ <> #0 do
   begin
    Run^ := AsciiLowerChars[Run^];
    if Run[1] <> #0 then
     begin
      Run[1] := AsciiLowerChars[Run[1]];
      if Run[2] <> #0 then
       begin
        Run[2] := AsciiLowerChars[Run[2]];
        if Run[3] <> #0 then
         begin
          Run[3] := AsciiLowerChars[Run[3]];
          if Run[4] <> #0 then
           begin
            Run[4] := AsciiLowerChars[Run[4]];
            Inc (Run, 4);
           end
          else
           Break;
         end
        else
         Break;
       end
      else
       Break;
     end
    else
     Break;
   end;
end;

{ ********** }

procedure UpperCase (var s: AnsiString);
var
 p                  : PAnsiChar;
 l                  : Integer;
begin
 if Pointer (s) = nil then Exit;
 UniqueString (s);
 if Pointer (s) = nil then Exit;
 
 p := Pointer (s);
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   p^ := AsciiUpperChars[p^];
   p[1] := AsciiUpperChars[p[1]];
   p[2] := AsciiUpperChars[p[2]];
   p[3] := AsciiUpperChars[p[3]];
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    p^ := AsciiUpperChars[p^];
    p[1] := AsciiUpperChars[p[1]];
    p[2] := AsciiUpperChars[p[2]];
   end;
  2:
   begin
    p^ := AsciiUpperChars[p^];
    p[1] := AsciiUpperChars[p[1]];
   end;
  1:
   begin
    p^ := AsciiUpperChars[p^];
   end;
 end;
end;

{ ********** }

function UpperCaseFunc (const s: AnsiString): AnsiString;
var
 p1, p2             : PAnsiChar;
 l                  : Integer;
begin
 if Pointer (s) = nil then Exit;
 
 l := Integer (Pointer (Cardinal (s) - 4)^);
 
 SetLength (Result, l);
 if Pointer (Result) = nil then Exit;
 
 p1 := Pointer (s);
 p2 := Pointer (Result);
 
 while l >= 4 do
  begin
   p2^ := AsciiUpperChars[p1^];
   p2[1] := AsciiUpperChars[p1[1]];
   p2[2] := AsciiUpperChars[p1[2]];
   p2[3] := AsciiUpperChars[p1[3]];
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    p2^ := AsciiUpperChars[p1^];
    p2[1] := AsciiUpperChars[p1[1]];
    p2[2] := AsciiUpperChars[p1[2]];
   end;
  2:
   begin
    p2^ := AsciiUpperChars[p1^];
    p2[1] := AsciiUpperChars[p1[1]];
   end;
  1:
   begin
    p2^ := AsciiUpperChars[p1^];
   end;
 end;
end;

{ ********** }

procedure LowerCase (var s: AnsiString);
var
 p                  : PAnsiChar;
 l                  : Integer;
begin
 if Pointer (s) = nil then Exit;
 UniqueString (s);
 if Pointer (s) = nil then Exit;
 
 p := Pointer (s);
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   p^ := AsciiLowerChars[p^];
   p[1] := AsciiLowerChars[p[1]];
   p[2] := AsciiLowerChars[p[2]];
   p[3] := AsciiLowerChars[p[3]];
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    p^ := AsciiLowerChars[p^];
    p[1] := AsciiLowerChars[p[1]];
    p[2] := AsciiLowerChars[p[2]];
   end;
  2:
   begin
    p^ := AsciiLowerChars[p^];
    p[1] := AsciiLowerChars[p[1]];
   end;
  1:
   begin
    p^ := AsciiLowerChars[p^];
   end;
 end;
end;

{ ********** }

function LowerCaseFunc (const s: AnsiString): AnsiString;
var
 p1, p2             : PAnsiChar;
 l                  : Integer;
begin
 if Pointer (s) = nil then Exit;
 
 l := Integer (Pointer (Cardinal (s) - 4)^);
 
 SetLength (Result, l);
 if Pointer (Result) = nil then Exit;
 
 p1 := Pointer (s);
 p2 := Pointer (Result);
 
 while l >= 4 do
  begin
   p2^ := AsciiLowerChars[p1^];
   p2[1] := AsciiLowerChars[p1[1]];
   p2[2] := AsciiLowerChars[p1[2]];
   p2[3] := AsciiLowerChars[p1[3]];
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    p2^ := AsciiLowerChars[p1^];
    p2[1] := AsciiLowerChars[p1[1]];
    p2[2] := AsciiLowerChars[p1[2]];
   end;
  2:
   begin
    p2^ := AsciiLowerChars[p1^];
    p2[1] := AsciiLowerChars[p1[1]];
   end;
  1:
   begin
    p2^ := AsciiLowerChars[p1^];
   end;
 end;
end;

{ **************************************************************************** }

function ProperCase (const s: AnsiString): AnsiString;
label
 ReturnEmptyString;
var
 l                  : Integer;
 p1, p2             : PAnsiChar;
 LastWasAlphaNum    : Boolean;
begin
 if Pointer (s) = nil then goto ReturnEmptyString;
 l := Integer (Pointer (Cardinal (s) - 4)^);
 if l = 0 then goto ReturnEmptyString;
 
 LastWasAlphaNum := False;
 SetLength (Result, l);
 Integer (p1) := Integer (s) + l;
 Integer (p2) := Integer (Result) + l;
 l := -l;
 
 while l <> 0 do
  begin
   if p1[l] = '''' then                 // Special Case. TODO: Are there other characters like them?
    p2[l] := p1[l]
   else
    if p1[l] in AnsiWordSeparators then
     begin
      p2[l] := p1[l];
      LastWasAlphaNum := False;
     end
    else
     if LastWasAlphaNum then
      p2[l] := AsciiLowerChars[p1[l]]   // Lowercase the character.
     else
      begin
       p2[l] := AsciiUpperChars[p1[l]]; // Uppercase the character.
       LastWasAlphaNum := True;
      end;
   Inc (l);
  end;
 
 Exit;
 
 ReturnEmptyString:
 Result := '';
end;

{ **************************************************************************** }

function LoadStringFromResource (const Instance: THandle; const ResourceName: AnsiString): AnsiString;
var
 HResInfo           : HRSRC;
 p                  : PAnsiChar;
begin
 HResInfo := FindResource (Instance, PAnsiChar (ResourceName), RT_RCDATA);
 p := LockResource (LoadResource (Instance, HResInfo));
 SetString (Result, p, SizeOfResource (Instance, HResInfo));
end;

{ **************************************************************************** }
{ Conditional String Assignment
{ **************************************************************************** }

procedure AssignIfEmpty (var Dest: AnsiString; const Source: AnsiString);
begin
 if IsEmptyString (Dest) then Dest := Source;
end;

{ **************************************************************************** }
{ Character & Co.
{ **************************************************************************** }

function CharExists (const s: AnsiString; const c: AnsiChar): Boolean;
{ Returns True if AnsiChar c exists in AnsiString s. }
label
 Fail, Match;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 if Pointer (s) = nil then goto Fail;
 
 p := Pointer (s);
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   if (p^ = c) or (p[1] = c) or (p[2] = c) or (p[3] = c) then goto Match;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   if (p^ = c) or (p[1] = c) or (p[2] = c) then goto Match;
  2:
   if (p^ = c) or (p[1] = c) then goto Match;
  1:
   if (p^ = c) then goto Match;
 end;
 
 Fail:
 Result := False;
 Exit;
 
 Match:
 Result := True;
end;

{ ********** }

function FirstCharsFromWords (const s: AnsiString; const MaxCharCount: Integer; const WordSeparators: TAnsiCharSet): AnsiString;
var
 i, l, LengthResult : Integer;
 p                  : PChar;
begin
 Result := '';
 if Pointer (s) = nil then Exit;
 if MaxCharCount <= 0 then Exit;
 
 p := Pointer (s);
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 if l > 0 then
  begin
   LengthResult := 0;

   repeat

    while (p^ in WordSeparators) and (l > 0) do
     begin
      Inc (p);
      Dec (l);
     end;

    i := MaxCharCount;
    while not (p^ in WordSeparators) and (i > 0) and (l > 0) do
     begin
      TomCat (p^, Result, LengthResult);
      Inc (p);
      Dec (i);
      Dec (l);
     end;

    while not (p^ in WordSeparators) and (l > 0) do
     begin
      Inc (p);
      Dec (l);
     end;

   until l = 0;

   SetLength (Result, LengthResult);
  end;
 
end;

procedure AppendChar (var s: AnsiString; const c: AnsiChar);
{ Adds a character to a string. }
begin
 SetLength (s, succ (Length (s)));
 s[Length (s)] := c;
end;

{ **************************************************************************** }
{ Routines for skipping sets and characters in buffers.
{ **************************************************************************** }

procedure SkipChar (var Source: PAnsiChar; const Search: Char);
var
 Run                : PAnsiChar;
begin
 Run := Source;
 while Run^ = Search do
  if Run[1] = Search then
   if Run[2] = Search then
    if Run[3] = Search then
     begin
      Inc (Run, 4);
     end
    else
     begin
      Inc (Run, 3);
      Break;
     end
   else
    begin
     Inc (Run, 2);
     Break;
    end
  else
   Inc (Run, 1);
 Source := Run;
end;

{ ********** }

procedure SkipCharBack (var Source: PAnsiChar; const Search: Char);
var
 Run                : PAnsiChar;
begin
 Run := Source;
 while Run^ = Search do
  if Run[-1] = Search then
   if Run[-2] = Search then
    if Run[-3] = Search then
     begin
      Dec (Run, 4);
     end
    else
     begin
      Dec (Run, 3);
      Break;
     end
   else
    begin
     Dec (Run, 2);
     Break;
    end
  else
   Dec (Run, 1);
 Source := Run;
end;

{ ********** }

procedure SkipNotChar (var Source: PAnsiChar; const Search: Char);
var
 Run                : PAnsiChar;
begin
 Run := Source;
 while not (Run^ = Search) do
  if not (Run[1] = Search) then
   if not (Run[2] = Search) then
    if not (Run[3] = Search) then
     begin
      Inc (Run, 4);
     end
    else
     begin
      Inc (Run, 3);
      Break;
     end
   else
    begin
     Inc (Run, 2);
     Break;
    end
  else
   Inc (Run, 1);
 Source := Run;
end;

{ ********** }

procedure SkipNotCharBack (var Source: PAnsiChar; const Search: Char);
var
 Run                : PAnsiChar;
begin
 Run := Source;
 while Run^ <> Search do
  if Run[-1] <> Search then
   if Run[-2] <> Search then
    if Run[-3] <> Search then
     begin
      Dec (Run, 4);
     end
    else
     begin
      Dec (Run, 3);
      Break;
     end
   else
    begin
     Dec (Run, 2);
     Break;
    end
  else
   Dec (Run, 1);
 Source := Run;
end;

{ **************************************************************************** }

procedure SkipSet (var p: PAnsiChar; const Search: TAnsiCharSet);
var
 Run                : PAnsiChar;
begin
 Run := p;
 while Run^ in Search do
  if Run[1] in Search then
   if Run[2] in Search then
    if Run[3] in Search then
     begin
      Inc (Run, 4);
     end
    else
     begin
      Inc (Run, 3);
      Break;
     end
   else
    begin
     Inc (Run, 2);
     Break;
    end
  else
   Inc (Run, 1);
 p := Run;
end;

{ ********** }

procedure SkipSetBack (var p: PAnsiChar; const Search: TAnsiCharSet);
var
 Run                : PAnsiChar;
begin
 Run := p;
 while Run^ in Search do
  if Run[-1] in Search then
   if Run[-2] in Search then
    if Run[-3] in Search then
     begin
      Dec (Run, 4);
     end
    else
     begin
      Dec (Run, 3);
      Break;
     end
   else
    begin
     Dec (Run, 2);
     Break;
    end
  else
   Dec (Run, 1);
 p := Run;
end;

{ ********** }

procedure SkipNotSet (var p: PAnsiChar; const Search: TAnsiCharSet);
var
 Run                : PAnsiChar;
begin
 Run := p;
 while not (Run^ in Search) do
  if not (Run[1] in Search) then
   if not (Run[2] in Search) then
    if not (Run[3] in Search) then
     begin
      Inc (Run, 4);
     end
    else
     begin
      Inc (Run, 3);
      Break;
     end
   else
    begin
     Inc (Run, 2);
     Break;
    end
  else
   Inc (Run, 1);
 p := Run;
end;

{ ********** }

procedure SkipNotSetBack (var p: PAnsiChar; const Search: TAnsiCharSet);
var
 Run                : PAnsiChar;
begin
 Run := p;
 while not (Run^ in Search) do
  if not (Run[-1] in Search) then
   if not (Run[-2] in Search) then
    if not (Run[-3] in Search) then
     begin
      Dec (Run, 4);
     end
    else
     begin
      Dec (Run, 3);
      Break;
     end
   else
    begin
     Dec (Run, 1);
     Break;
    end
  else
   Dec (Run, 1);
 p := Run;
end;

{ **************************************************************************** }
{ Scanning for Sets in PChars...
{ **************************************************************************** }

function PScanSetL (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start, BufferLength: Integer): Integer;
{ Scans Source for the first Character in Set Search. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 if (Source = nil) or (Start < 0) or (Start > BufferLength) then goto Fail;
 
 Integer (p) := Integer (Source) + Start;
 l := BufferLength - Start;
 
 while l >= 4 do
  begin
   if p^ in Search then goto Zero;
   if p[1] in Search then goto One;
   if p[2] in Search then goto Two;
   if p[3] in Search then goto Three;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p^ in Search) then goto Zero;
    if (p[1] in Search) then goto One;
    if (p[2] in Search) then goto Two;
   end;
  2:
   begin
    if (p^ in Search) then goto Zero;
    if (p[1] in Search) then goto One;
   end;
  1:
   if (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := -1;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 3;
end;

{ ********** }

function PScanNotSetL (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start, BufferLength: Integer): Integer;
{ Scans Source for the first Character in Set Search. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 if (Source = nil) or (Start <= 0) or (Start > BufferLength) then goto Fail;
 
 Integer (p) := Integer (Source) + Start;
 l := BufferLength - Start;
 
 while l >= 4 do
  begin
   if not (p^ in Search) then goto Zero;
   if not (p[1] in Search) then goto One;
   if not (p[2] in Search) then goto Two;
   if not (p[3] in Search) then goto Three;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[1] in Search) then goto One;
    if not (p[2] in Search) then goto Two;
   end;
  2:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[1] in Search) then goto One;
   end;
  1:
   if not (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := -1;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 3;
end;

{ ********** }

function PScanSetBack (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start: Integer): Integer;
label
 Zero, One, Two, Three, Fail;
var
 p                  : PAnsiChar;
 l                  : Integer;
begin
 if (Source = nil) or (Start < 0) then goto Fail;
 
 p := Source + Start;
 l := Start + 1;
 while l >= 4 do
  begin
   if p^ in Search then goto Zero;
   if p[-1] in Search then goto One;
   if p[-2] in Search then goto Two;
   if p[-3] in Search then goto Three;
   Dec (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p^ in Search) then goto Zero;
    if (p[-1] in Search) then goto One;
    if (p[-2] in Search) then goto Two;
   end;
  2:
   begin
    if (p^ in Search) then goto Zero;
    if (p[-1] in Search) then goto One;
   end;
  1:
   if (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := -1;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) - 1;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) - 2;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) - 3;
end;

{ ********** }

function PScanNotSetBack (const Source: PAnsiChar; const Search: TAnsiCharSet; const Start: Integer): Integer;
label
 Zero, One, Two, Three, Fail;
var
 p                  : PAnsiChar;
 l                  : Integer;
begin
 if (Source = nil) or (Start < 0) then goto Fail;
 
 p := Source + Start;
 l := Start + 1;
 while l >= 4 do
  begin
   if not (p^ in Search) then goto Zero;
   if not (p[-1] in Search) then goto One;
   if not (p[-2] in Search) then goto Two;
   if not (p[-3] in Search) then goto Three;
   Dec (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[-1] in Search) then goto One;
    if not (p[-2] in Search) then goto Two;
   end;
  2:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[-1] in Search) then goto One;
   end;
  1:
   if not (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := -1;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) - 1;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) - 2;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) - 3;
end;

{ **************************************************************************** }

function IsEmptyString (const s: AnsiString): Boolean;
label
 Fail, Match;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (s);
 if p = nil then goto Match;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   if not (p^ in ANSI_WHITE_SPACE) or
    not (p[1] in ANSI_WHITE_SPACE) or
    not (p[2] in ANSI_WHITE_SPACE) or
    not (p[3] in ANSI_WHITE_SPACE) then goto Fail;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   if not (p^ in ANSI_WHITE_SPACE) or
    not (p[1] in ANSI_WHITE_SPACE) or
    not (p[2] in ANSI_WHITE_SPACE) then goto Fail;
  2:
   if not (p^ in ANSI_WHITE_SPACE) or
    not (p[1] in ANSI_WHITE_SPACE) then goto Fail;
  1:
   if not (p^ in ANSI_WHITE_SPACE) then goto Fail;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{ **************************************************************************** }

function PScanChar (const Source: PAnsiChar; const c: Char): Integer;
label
 Zero, One, Two, Three, Fail;
var
 p                  : PAnsiChar;
begin
 if Source = nil then goto Fail;
 p := Source;
 while (p^ <> #0) do
  begin
   if p^ = c then goto Zero;

   if p[1] = #0 then goto Fail;
   if p[1] = c then goto One;

   if p[2] = #0 then goto Fail;
   if (p[2] = c) then goto Two;

   if p[3] = #0 then goto Fail;
   if p[3] = c then goto Three;

   Inc (p, 4)
  end;
 
 Fail:
 Result := -1;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 3;
end;

{ **************************************************************************** }
{ Looking for Sets of AnsiChars in a AnsiStrings.
{ **************************************************************************** }

function ScanSet (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ Scans Source for the first Character in Set Search. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto Fail;
 
 if Start <= 0 then goto Fail;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if Start > l then goto Fail;
 
 Inc (p, Start - 1);
 Dec (l, Start - 1);
 
 while l >= 4 do
  begin
   if p^ in Search then goto Zero;
   if p[1] in Search then goto One;
   if p[2] in Search then goto Two;
   if p[3] in Search then goto Three;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p^ in Search) then goto Zero;
    if (p[1] in Search) then goto One;
    if (p[2] in Search) then goto Two;
   end;
  2:
   begin
    if (p^ in Search) then goto Zero;
    if (p[1] in Search) then goto One;
   end;
  1:
   if (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := 0;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 3;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 4;
end;

{ ********** }

function ScanSetBack (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ Backward/reverse scan from Start location (1 = First Char., 0 = String End) looking for first char in set Search.
  Returns: Position where/if found; otherwise, 0. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto Fail;
 
 if (Start < 0) then goto Fail;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if Start > l then goto Fail;
 
 if Start <> 0 then l := Start;
 
 Inc (p, l - 1);
 
 while l >= 4 do
  begin
   if p^ in Search then goto Zero;
   if p[-1] in Search then goto One;
   if p[-2] in Search then goto Two;
   if p[-3] in Search then goto Three;
   Dec (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p^ in Search) then goto Zero;
    if (p[-1] in Search) then goto One;
    if (p[-2] in Search) then goto Two;
   end;
  2:
   begin
    if (p^ in Search) then goto Zero;
    if (p[-1] in Search) then goto One;
   end;
  1:
   if (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := 0;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) - 1;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) - 2;
end;

{ ********** }

function ScanNotSet (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ Scans Source for the first Character in Set Search. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto Fail;
 
 if Start <= 0 then goto Fail;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if Start > l then goto Fail;
 
 Inc (p, Start - 1);
 Dec (l, Start - 1);
 
 while l >= 4 do
  begin
   if not (p^ in Search) then goto Zero;
   if not (p[1] in Search) then goto One;
   if not (p[2] in Search) then goto Two;
   if not (p[3] in Search) then goto Three;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[1] in Search) then goto One;
    if not (p[2] in Search) then goto Two;
   end;
  2:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[1] in Search) then goto One;
   end;
  1:
   if not (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := 0;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 3;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 4;
end;

{ ********** }

function PScanNotSet (const Source: PAnsiChar; const Search: TAnsiCharSet): Integer;
{ Scans Source for the first Character not in Set Search. }
label
 Zero, One, Two, Three, Fail;
var
 p                  : PAnsiChar;
begin
 if Source = nil then goto Fail;
 p := Source;
 while (p^ <> #0) do
  begin
   if not (p^ in Search) then goto Zero;

   if p[1] = #0 then goto Fail;
   if not (p[1] in Search) then goto One;

   if p[2] = #0 then goto Fail;
   if not (p[2] in Search) then goto Two;

   if p[3] = #0 then goto Fail;
   if not (p[3] in Search) then goto Three;

   Inc (p, 4)
  end;
 
 Fail:
 Result := -1;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 3;
end;

{ ********** }

function ScanNotSetBack (const Source: AnsiString; const Search: TAnsiCharSet; const Start: Integer): Integer;
{ Backward/reverse scan from Start location (1 = First Char., 0 = String End) looking for first char not in set Search.
  Returns: Position where/if found; otherwise, 0. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto Fail;
 
 if (Start < 0) then goto Fail;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if Start > l then goto Fail;
 
 if Start <> 0 then l := Start;
 
 Inc (p, l - 1);
 
 while l >= 4 do
  begin
   if not (p^ in Search) then goto Zero;
   if not (p[-1] in Search) then goto One;
   if not (p[-2] in Search) then goto Two;
   if not (p[-3] in Search) then goto Three;
   Dec (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[-1] in Search) then goto One;
    if not (p[-2] in Search) then goto Two;
   end;
  2:
   begin
    if not (p^ in Search) then goto Zero;
    if not (p[-1] in Search) then goto One;
   end;
  1:
   if not (p^ in Search) then goto Zero;
 end;
 
 Fail:
 Result := 0;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) - 1;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) - 2;
end;

{ **************************************************************************** }

function ScanChar (const Source: AnsiString; const c: AnsiChar; const Start: Integer): Integer;
{ Scans Source for the first Character in Set Search. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto Fail;
 
 if Start <= 0 then goto Fail;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if Start > l then goto Fail;
 
 Inc (p, Start - 1);
 Dec (l, Start - 1);
 
 { Alternatively compare four bytes / characters at a time:
 
 Compare := Byte (c);
 Compare := (Compare shl 8) or Compare;
 Compare := (Compare shl 16) or Compare;
 
 while l >= 4 do
  begin
   if (((not (p^ xor Compare)) and (p^ - $01010101)) and $80808080) <> 0 then Break;
   Inc (p);
   Dec (l, 4);
  end;}
 
 while l >= 4 do
  begin
   if p^ = c then goto Zero;
   if p[1] = c then goto One;
   if p[2] = c then goto Two;
   if p[3] = c then goto Three;
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p^ = c) then goto Zero;
    if (p[1] = c) then goto One;
    if (p[2] = c) then goto Two;
   end;
  2:
   begin
    if (p^ = c) then goto Zero;
    if (p[1] = c) then goto One;
   end;
  1:
   if (p^ = c) then goto Zero;
 end;
 
 Fail:
 Result := 0;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source) + 2;
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) + 3;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) + 4;
end;

{ ********** }

function ScanCharBack (const Source: AnsiString; const c: AnsiChar; const Start: Integer): Integer;
{ Backward/reverse scan from Start location (1 = First Char., 0 = String End) looking for single character, X.
  Returns: Position where/if found; otherwise, 0. }
label
 Zero, One, Two, Three, Fail;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 p := Pointer (Source);
 if p = nil then goto Fail;
 
 if (Start < 0) then goto Fail;
 
 l := Integer (Pointer (Cardinal (p) - 4)^);
 if Start > l then goto Fail;
 
 if Start <> 0 then l := Start;
 
 Inc (p, l - 1);
 
 while l >= 4 do
  begin
   if p^ = c then goto Zero;
   if p[-1] = c then goto One;
   if p[-2] = c then goto Two;
   if p[-3] = c then goto Three;
   Dec (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    if (p^ = c) then goto Zero;
    if (p[-1] = c) then goto One;
    if (p[-2] = c) then goto Two;
   end;
  2:
   begin
    if (p^ = c) then goto Zero;
    if (p[-1] = c) then goto One;
   end;
  1:
   if (p^ = c) then goto Zero;
 end;
 
 Fail:
 Result := 0;
 Exit;
 
 Zero:
 Result := Integer (p) - Integer (Source) + 1;
 Exit;
 
 One:
 Result := Integer (p) - Integer (Source);
 Exit;
 
 Two:
 Result := Integer (p) - Integer (Source) - 1;
 Exit;
 
 Three:
 Result := Integer (p) - Integer (Source) - 2;
end;

{ **************************************************************************** }

procedure ReplaceStr (var Str: string; const sub1, sub2: string);
var
 iStr, jstr, StrEnd, pSub1, Sub2End, pTemp: PAnsiChar;
 k, l, lenS1, lenS2 : Integer;
 newStr             : string;
 c                  : Char;
begin
 pSub1 := PAnsiChar (sub1);
 lenS1 := Length (sub1);
 lenS2 := Length (sub2);
 StrEnd := @Str[Length (Str) + 1];
 Sub2End := @sub2[lenS2 + 1];
 Assert (lenS1 > 0, 'Sub1 - must not be empty');
 // make sure newStr is long enough
 if lenS2 > lenS1 then
  SetLength (newStr, ((Length (Str) div lenS1) + 1) * lenS2)
 else
  SetLength (newStr, Length (Str));
 iStr := @Str[1];
 jstr := @newStr[1];
 c := sub1[1];
 StrEnd[0] := c;                        // set to stop loop (could be a problem if ref count >1 )
 while True do
  begin
   k := 0;
   while (iStr[k] <> c) do
    begin                               // copy until possible match
     jstr[k] := iStr[k];
     Inc (k);
    end;
   Inc (jstr, k);
   Inc (iStr, k);
   if iStr = StrEnd then                // check end
    Break;                              // bail out
   l := StrEnd - iStr;
   k := 1;                              // First char is already matched.
   if l > lenS1 then
    begin                               // don't need to check l since the zero at end of Sub1 will stop loop
     l := lenS1;
     pTemp := pSub1;                    // increases "priority" of psub1
     while iStr[k] = pTemp[k] do
      Inc (k);
    end
   else                                 // here we have to check
    while ((k < l) and (iStr[k] = pSub1[k])) do
     Inc (k);
   if (k = l) then
    begin                               // copy // match
     Inc (iStr, lenS1);
     k := -lenS2;
     Inc (jstr, lenS2);
     while k < 0 do
      begin
       jstr[k] := Sub2End[k];
       Inc (k);
      end;
    end
   else
    begin                               // no match
     jstr^ := iStr^;
     Inc (jstr);
     Inc (iStr);
    end;
  end;
 StrEnd[0] := #0;                       // return to original state
 SetLength (newStr, jstr - PAnsiChar (newStr));
 Str := newStr;
end;

{ **************************************************************************** }

function GetLastErrorString (const MessageID: DWORD): AnsiString;
var
 Len                : Integer;
 Buffer             : array[0..255] of Char;
begin
 Len := FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, nil, MessageID, 0, Buffer, SizeOf (Buffer), nil);
 while (Len > 0) and (Buffer[Len - 1] in [#0] + ANSI_WHITE_SPACE) do
  Dec (Len);
 SetString (Result, Buffer, Len);
end;

{ **************************************************************************** }
{ Funktioniert auch mit LongMonthNames [i] etc., in SysUtils definiert.
  Diese Funktionen nutzen also nur dann,
  wenn SysUtils im Programm nicht gebraucht wird.
  Bei Bedarf kann die Sprache mit ThreadLocale angepat werden.
{ **************************************************************************** }

function LongMonthName (const i: Integer): AnsiString;
var
 l                  : Integer;
 Buffer             : array[0..255] of Char;
begin
 l := GetLocaleInfo (GetThreadLocale, LOCALE_SMONTHNAME1 + i - 1, Buffer, SizeOf (Buffer));
 if l > 0 then
  SetString (Result, Buffer, l - 1)
 else
  Result := '';
end;

{ ********** }

function ShortMonthName (const i: Integer): AnsiString;
var
 l                  : Integer;
 Buffer             : array[0..255] of Char;
begin
 l := GetLocaleInfo (GetThreadLocale, LOCALE_SABBREVMONTHNAME1 + i - 1, Buffer, SizeOf (Buffer));
 if l > 0 then
  SetString (Result, Buffer, l - 1)
 else
  Result := '';
end;

{ ********** }

function LongDayName (const i: Integer): AnsiString;
var
 l                  : Integer;
 Buffer             : array[0..255] of Char;
begin
 l := GetLocaleInfo (GetThreadLocale, LOCALE_SDAYNAME1 + i - 1, Buffer, SizeOf (Buffer));
 if l > 0 then
  SetString (Result, Buffer, l - 1)
 else
  Result := '';
end;

{ ********** }

function ShortDayName (const i: Integer): AnsiString;
var
 l                  : Integer;
 Buffer             : array[0..255] of Char;
begin
 l := GetLocaleInfo (GetThreadLocale, LOCALE_SABBREVDAYNAME1 + i - 1, Buffer, SizeOf (Buffer));
 if l > 0 then
  SetString (Result, Buffer, l - 1)
 else
  Result := '';
end;

{ **************************************************************************** }
{ String Hash Functions
{ **************************************************************************** }

function HashRJ (const s: AnsiString): Cardinal;
var
 l                  : Integer;
 p                  : PChar;
begin
 Result := 0;
 if Pointer (s) = nil then Exit;
 
 l := Integer (Pointer (Cardinal (s) - 4)^);
 p := Pointer (s);
 
 while l > 0 do
  begin
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
   Inc (p);
   Dec (l);
  end;
end;

{ ********** }

function HashPJW (const s: AnsiString): Cardinal;
{Note: this hash function is described in "Practical Algorithms For
       Programmers" by Andrew Binstock and John Rex, Addison Wesley}
const
 BitsInInteger      = SizeOf (Integer) * 8;
 ThreeQuarters      = (BitsInInteger * 3) div 4;
 OneEighth          = BitsInInteger div 8;
 HighBits           : Integer = (not Integer (0)) shl (BitsInInteger - OneEighth);
var
 i                  : Integer;
 test               : Cardinal;
begin
 Result := 0;
 for i := 1 to Length (s) do
  begin
   Result := (Result shl OneEighth) + Ord (s[i]);
   test := Result and HighBits;
   if (test <> 0) then
    Result := (Result xor (test shr ThreeQuarters)) and
     not HighBits;
  end;
end;

{ ********** }

function HashElf (const s: AnsiString): Cardinal;
{Note: this hash function is described in "Practical Algorithms For
       Programmers" by Andrew Binstock and John Rex, Addison Wesley,
       with modifications in Dr Dobbs Journal, April 1996}
var
 g                  : Cardinal;
 i                  : Integer;
begin
 Result := 0;
 for i := 1 to Length (s) do
  begin
   Result := (Result shl 4) + Ord (s[i]);
   g := Result and $F0000000;
   if (g <> 0) then
    Result := Result xor (g shr 24);
   Result := Result and (not g);
  end;
end;

{ ********** }

function HashBKDR (const s: AnsiString): Cardinal;
{Note: this hash function is described in "The C Programming Language"
       by Brian Kernighan and Donald Ritchie, Prentice Hall}
var
 i                  : Integer;
begin
 Result := 0;
 for i := 1 to Length (s) do
  begin
   Result := (Result * 31) + UpperCharArray[s[i]];
  end;
end;

{ **************************************************************************** }

function SameBufferText (const Buffer: Pointer; const BufferLength: Integer; const s2: AnsiString): Boolean;
label
 Fail, Match;
var
 p1, p2             : PAnsiChar;
 l2                 : Integer;
begin
 if (Buffer = nil) or (Pointer (s2) = nil) then goto Fail;
 
 l2 := Integer (Pointer (Cardinal (s2) - 4)^);
 
 if BufferLength <> l2 then goto Fail;
 
 p1 := Buffer;
 p2 := Pointer (s2);
 
 while l2 >= 4 do
  begin
   if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
   if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
   if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Fail;
   if (p1[3] <> p2[3]) and (p1[3] <> AsciiReverseChars[p2[3]]) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l2, 4);
  end;
 
 case l2 of
  3:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
    if (p1[2] <> p2[2]) and (p1[2] <> AsciiReverseChars[p2[2]]) then goto Fail;
   end;
  2:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
    if (p1[1] <> p2[1]) and (p1[1] <> AsciiReverseChars[p2[1]]) then goto Fail;
   end;
  1:
   begin
    if (p1^ <> p2^) and (p1^ <> AsciiReverseChars[p2^]) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

{ ********** }

function SameBufferStr (const Buffer: Pointer; const BufferLength: Integer; const s2: AnsiString): Boolean;
label
 Fail, Match;
var
 p1, p2             : PAnsiChar;
 l2                 : Integer;
begin
 if (Buffer = nil) or (Pointer (s2) = nil) then goto Fail;
 
 l2 := Integer (Pointer (Cardinal (Pointer (s2)) - 4)^);
 
 if BufferLength <> l2 then goto Fail;
 
 p1 := Buffer;
 p2 := Pointer (s2);
 
 while l2 >= 4 do
  begin
   if (p1^ <> p2^) then goto Fail;
   if (p1[1] <> p2[1]) then goto Fail;
   if (p1[2] <> p2[2]) then goto Fail;
   if (p1[3] <> p2[3]) then goto Fail;
   Inc (p1, 4);
   Inc (p2, 4);
   Dec (l2, 4);
  end;
 
 case l2 of
  3:
   begin
    if (p1^ <> p2^) then goto Fail;
    if (p1[1] <> p2[1]) then goto Fail;
    if (p1[2] <> p2[2]) then goto Fail;
   end;
  2:
   begin
    if (p1^ <> p2^) then goto Fail;
    if (p1[1] <> p2[1]) then goto Fail;
   end;
  1:
   begin
    if (p1^ <> p2^) then goto Fail;
   end;
 end;
 
 Match:
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

end.

