{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000
  http://www.zeitungsjunge.de/delphi/ }

unit rjHashVectors;

{$I rj.inc}

interface

uses
 Classes,
 
 rjTypes;

type
 THashVectorObject = class
 protected
  class procedure Error (const Msg: AnsiString; const Data: array of const);
 end;
 
 THashVector = class (THashVectorObject)
 protected
  FDeleted: Integer;

  FHashTable: PIntegerArray;
  FHashTableSize: Integer;

  FVector: Pointer;
  FVectorCount: Integer;

  FCount: Integer;

  procedure FinalizeItemRange (const AIndex, ACount: Integer); virtual; abstract;
  procedure RebuildHashTable; virtual; abstract;
  procedure ResetHashTable;
  procedure SetHashTableSize (const NewHashTableSize: Integer);
 public
  constructor Create;
  constructor CreateCopy (const Source: THashVector);
  destructor Destroy; override;

  procedure Assign (const ASource: THashVector); virtual; abstract;

  procedure Clear; virtual;

  property Count: Integer read FCount;
  property VectorCount: Integer read FVectorCount;

  function IsEmpty: Boolean;
  function IsNotEmpty: Boolean;

  property HashSize: Integer read FHashTableSize write SetHashTableSize;

  property Vector: Pointer read FVector;

  function ReadFromBinaryFile (const FileName: AnsiString): Boolean;
  function ReadFromBinaryStream (const Stream: TStream): Boolean; virtual; abstract;
  procedure WriteToBinaryFile (const FileName: AnsiString);
  procedure WriteToBinaryStream (const Stream: TStream); virtual; abstract;
 end;
 
function HashAnsiStringCaseInSens (const s: AnsiString): Cardinal;
function HashAnsiStringCaseSens (const s: AnsiString): Cardinal;

function HashBufferCaseInSens (const Buffer: Pointer; const BufferLength: Integer): Cardinal;
function HashBufferCaseSens (Buffer: PAnsiChar; const BufferLength: Integer): Cardinal;

const
 MIN_HASH_SIZE      = 11;
 
 HASH_END           = 1 shl 31;
 HASH_DELETED       = 1 shl 30;
 
implementation

uses
 {$IFDEF CheckFree}
 rjCheckFree,
 {$ENDIF}
 rjExceptions,
 rjPrimes,
 rjStrings;

{$DEFINE HashRJ}
{.$DEFINE HashBKDR}

(*
function HashAnsiStringCaseInSens (const s: AnsiString): Cardinal;
var
 p, pStop           : PAnsiChar;
begin
 Result := 0;
 if Pointer (s) = nil then Exit;

 p := Pointer (s);
 pStop := p + Integer (Pointer (Cardinal (p) - 4)^);

 while p < pStop do
  begin
   {$IFDEF HashRJ}Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^]; {$ENDIF}
   {$IFDEF HashBKDR}Result := (Result * 31) + UpperCharArray[p^]; {$ENDIF}
   Inc (p);
  end;
end;
*)

{ ********** }

function HashAnsiStringCaseInSens (const s: AnsiString): Cardinal;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 Result := 0;
 if Pointer (s) = nil then Exit;
 
 p := Pointer (s);
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   {$IFDEF HashRJ}
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[1]];
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[2]];
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[3]];
   {$ENDIF}
   {$IFDEF HashBKDR}
   Result := (Result * 31) + UpperCharArray[p^];
   Result := (Result * 31) + UpperCharArray[p[1]];
   Result := (Result * 31) + UpperCharArray[p[2]];
   Result := (Result * 31) + UpperCharArray[p[3]];
   {$ENDIF}
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[1]];
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[2]];
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + UpperCharArray[p^];
    Result := (Result * 31) + UpperCharArray[p[1]];
    Result := (Result * 31) + UpperCharArray[p[2]];
    {$ENDIF}
   end;
  2:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[1]];
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + UpperCharArray[p^];
    Result := (Result * 31) + UpperCharArray[p[1]];
    {$ENDIF}
   end;
  1:
   begin
    {$IFDEF HashRJ}Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^]; {$ENDIF}
    {$IFDEF HashBKDR}Result := (Result * 31) + UpperCharArray[p^]; {$ENDIF}
   end;
 end;
end;

{ ********** }

function HashBufferCaseInSens (const Buffer: Pointer; const BufferLength: Integer): Cardinal;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 Result := 0;
 if Buffer = nil then Exit;
 p := Buffer;
 l := BufferLength;
 
 while l >= 4 do
  begin
   {$IFDEF HashRJ}
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[1]];
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[2]];
   Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[3]];
   {$ENDIF}
   {$IFDEF HashBKDR}
   Result := (Result * 31) + UpperCharArray[p^];
   Result := (Result * 31) + UpperCharArray[p[1]];
   Result := (Result * 31) + UpperCharArray[p[2]];
   Result := (Result * 31) + UpperCharArray[p[3]];
   {$ENDIF}
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[1]];
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[2]];
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + UpperCharArray[p^];
    Result := (Result * 31) + UpperCharArray[p[1]];
    Result := (Result * 31) + UpperCharArray[p[2]];
    {$ENDIF}
   end;
  2:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^];
    Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p[1]];
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + UpperCharArray[p^];
    Result := (Result * 31) + UpperCharArray[p[1]];
    {$ENDIF}
   end;
  1:
   begin
    {$IFDEF HashRJ}Result := ((Result shl 5) xor (Result shr 27)) xor UpperCharArray[p^]; {$ENDIF}
    {$IFDEF HashBKDR}Result := (Result * 31) + UpperCharArray[p^]; {$ENDIF}
   end;
 end;
end;

{ ********** }

function HashAnsiStringCaseSens (const s: AnsiString): Cardinal;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 Result := 0;
 if Pointer (s) = nil then Exit;
 
 p := Pointer (s);
 l := Integer (Pointer (Cardinal (p) - 4)^);
 
 while l >= 4 do
  begin
   {$IFDEF HashRJ}
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[1]);
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[2]);
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[3]);
   {$ENDIF}
   {$IFDEF HashBKDR}
   Result := (Result * 31) + Cardinal (p^);
   Result := (Result * 31) + Cardinal (p[1]);
   Result := (Result * 31) + Cardinal (p[2]);
   Result := (Result * 31) + Cardinal (p[3]);
   {$ENDIF}
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[1]);
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[2]);
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + Cardinal (p^);
    Result := (Result * 31) + Cardinal (p[1]);
    Result := (Result * 31) + Cardinal (p[2]);
    {$ENDIF}
   end;
  2:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[1]);
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + Cardinal (p^);
    Result := (Result * 31) + Cardinal (p[1]);
    {$ENDIF}
   end;
  1:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + Cardinal (p^);
    {$ENDIF}
   end;
 end;
end;

{ ********** }

function HashBufferCaseSens (Buffer: PAnsiChar; const BufferLength: Integer): Cardinal;
var
 l                  : Integer;
 p                  : PAnsiChar;
begin
 Result := 0;
 if Buffer = nil then Exit;
 p := Buffer;
 l := BufferLength;
 
 while l >= 4 do
  begin
   {$IFDEF HashRJ}
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[1]);
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[2]);
   Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[3]);
   {$ENDIF}
   {$IFDEF HashBKDR}
   Result := (Result * 31) + Cardinal (p^);
   Result := (Result * 31) + Cardinal (p[1]);
   Result := (Result * 31) + Cardinal (p[2]);
   Result := (Result * 31) + Cardinal (p[3]);
   {$ENDIF}
   Inc (p, 4);
   Dec (l, 4);
  end;
 
 case l of
  3:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[1]);
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[2]);
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + Cardinal (p^);
    Result := (Result * 31) + Cardinal (p[1]);
    Result := (Result * 31) + Cardinal (p[2]);
    {$ENDIF}
   end;
  2:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p[1]);
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + Cardinal (p^);
    Result := (Result * 31) + Cardinal (p[1]);
    {$ENDIF}
   end;
  1:
   begin
    {$IFDEF HashRJ}
    Result := ((Result shl 5) xor (Result shr 27)) xor Cardinal (p^);
    {$ENDIF}
    {$IFDEF HashBKDR}
    Result := (Result * 31) + Cardinal (p^);
    {$ENDIF}
   end;
 end;

end;

{ ********** }

{ **************************************************************************** }
{ Class THashObject
{ **************************************************************************** }

class procedure THashVectorObject.Error (const Msg: AnsiString; const Data: array of const);

 function ReturnAddr: Pointer;
 asm
  mov eax, [ebp + 4]
 end;
 
begin
 raise EHash.CreateFmt ('Class ' + ClassName + ': ' + Msg, Data)At ReturnAddr;
end;

{ **************************************************************************** }
{ THashVector
{ **************************************************************************** }

constructor THashVector.Create;
begin
 {$IFDEF CheckFree}
 RegisterObjectCreate (Self);
 {$ENDIF}
 FDeleted := HASH_END;
 { Initialize the HashTable }
 GetMem (FHashTable, MIN_HASH_SIZE * SizeOf (Integer));
 FHashTableSize := MIN_HASH_SIZE;
 ResetHashTable;
end;

constructor THashVector.CreateCopy (const Source: THashVector);
begin
 Create;
 Self.Assign (Source);
end;

{ ********** }

destructor THashVector.Destroy;
begin
 if FHashTable <> nil then FreeMem (FHashTable);
 if FVector <> nil then
  begin
   FinalizeItemRange (0, FVectorCount);
   FreeMem (FVector);
  end;
 {$IFDEF CheckFree}
 RegisterObjectFree (Self);
 {$ENDIF}
end;

{ ********** }

procedure THashVector.Clear;
begin
 // Init FHashTable
 ReallocMem (FHashTable, MIN_HASH_SIZE * SizeOf (Integer));
 FHashTableSize := MIN_HASH_SIZE;
 ResetHashTable;
 // Clear FVector
 if FVector <> nil then
  begin
   FinalizeItemRange (0, FVectorCount);
   FreeMem (FVector);
   FVector := nil;
  end;
 FVectorCount := 0;
 FCount := 0;
end;

{ ********** }

procedure THashVector.SetHashTableSize (const NewHashTableSize: Integer);
var
 i                  : Integer;
begin
 if NewHashTableSize < MIN_HASH_SIZE then
  i := MIN_HASH_SIZE
 else
  i := GetClosestPrime (NewHashTableSize);
 if i <> FHashTableSize then
  begin
   ReallocMem (FHashTable, i * SizeOf (Integer));
   FHashTableSize := i;
   RebuildHashTable;
  end;
end;

{ ********** }

function THashVector.IsEmpty: Boolean;
begin
 Result := FCount <= 0;
end;

{ ********** }

function THashVector.IsNotEmpty: Boolean;
begin
 Result := FCount > 0;
end;

{ ********** }

procedure THashVector.ResetHashTable;
var
 i                  : Integer;
begin
 i := FHashTableSize;
 while i > 0 do
  begin
   Dec (i);
   FHashTable[i] := HASH_END;
  end;
end;

{ ********** }

function THashVector.ReadFromBinaryFile (const FileName: AnsiString): Boolean;
var
 Stream             : TStream;
begin
 Stream := TFileStream.Create (FileName, $0000 {fmOpenRead, defined in SysUtils} or $0020 {fmShareDenyWrite, defined in SysUtils});
 try
  Result := ReadFromBinaryStream (Stream);
 finally
  Stream.Free;
 end;
end;

{ ********** }

procedure THashVector.WriteToBinaryFile (const FileName: AnsiString);
var
 Stream             : TStream;
begin
 Stream := TFileStream.Create (FileName, fmCreate);
 try
  WriteToBinaryStream (Stream);
 finally
  Stream.Free;
 end;
end;

end.

