{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000
  http://www.zeitungsjunge.de/delphi/ }

unit rjVAnsiString;

{$I rj.inc}

interface

uses
 Classes,
 
 rjBase,
 rjConsts,
 rjStreams,
 rjVectors,
 rjVBase32,
 rjTypes;

type
 
 PBaseType = PAnsiString;
 TBaseType = AnsiString;
 PBaseArray = PAnsiStringArray;
 
 {$DEFINE DynamicItems}
 
 TBaseTypeVector = class (ABase32Vector)
  {$I rjVGenericBaseType.def}
 public
  function Add (const Item: TBaseType): Integer;
  function AddEmpty: PBaseType;
  procedure Assign (const Source: TVector); override;
  procedure Insert (const Index: Integer; const Item: TBaseType);

  function ReadFromBinaryStream (const Stream: TStream): Boolean; override;
  procedure WriteToBinaryStream (const Stream: TStream); override;

  function ReadFromTextStream (const Stream: TStream): Boolean;
  procedure WriteToTextStream (const Stream: TStream);

  function ReadFromTextFile (const FileName: AnsiString): Boolean;
  procedure WriteToTextFile (const FileName: AnsiString);
 end;
 TAnsiStringVector = TBaseTypeVector;
 
 TBaseTypeSortedVector = class (TAnsiStringVector)
  {$I rjVGenericBaseTypeSorted.def}
  {$I rjVGenericAllocXSorted.def}
 end;
 TAnsiStringSortedVector = TBaseTypeSortedVector;
 
 {$I rjVCompareDefault.def}
 
implementation

uses
 Windows,
 {$IFDEF RangeChecking}Consts, {$ENDIF}
 
 rjStrings;

{ **************************************************************************** }
{ TAnsiStringVector
{ **************************************************************************** }

{$I rjVGenericBaseType.imp}

function TBaseTypeVector.Add (const Item: TBaseType): Integer;
var
 p                  : PBaseType;
begin
 Result := FCount;
 Grow (1);
 p := @PBaseArray (FVector)^[Result];
 Pointer (p^) := nil;
 p^ := Item;
end;

{ ********** }

function TBaseTypeVector.AddEmpty: PBaseType;
var
 i                  : Integer;
begin
 i := FCount;
 Grow (1);
 Result := @PBaseArray (FVector)^[i];
 Pointer (Result^) := nil;
end;

{ ********** }

procedure TBaseTypeVector.Assign (const Source: TVector);
var
 i                  : Integer;
begin
 Clear;
 i := 0;
 while i < Source.Count do
  begin
   // Add (TBaseType (TBaseTypeVector (Source)[ i])); // Much hidden overhead
   AddEmpty^ := PBaseArray (TBaseTypeVector (Source).FVector)^[i];
   Inc (i);
  end;
end;

{ ********** }

procedure TBaseTypeVector.Insert (const Index: Integer; const Item: TBaseType);
var
 p                  : PBaseType;
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index > FCount) then Error (SRangeError, [Index]);
 {$ENDIF}
 Expand (Index);
 p := @PBaseArray (FVector)^[Index];
 Pointer (p^) := nil;
 p^ := Item;
end;

{ ********** }

{
function TBaseTypeVector.ReadFromFile (const FileName: AnsiString): Boolean;
var
 FileBuffer                   : array[ 0..BUFFER_SIZE - 1] of Char;
 FileHandle                   : THandle;
 BytesRead, BufferPosition, LineEnd: Integer;
 StringBuffer                 : TBuffer;
begin
 Result := False;
 if Pointer (FileName) <> nil then
  begin
   FileHandle := CreateFile (Pointer (FileName), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
   if FileHandle = INVALID_HANDLE_VALUE then
    raise EFileOpen.CreateFmt (sErrorOpeningFile, [ FileName, GetLastErrorString (GetLastError)]);

   LineEnd := 0;

   try
    if not ReadFile (FileHandle, FileBuffer, SizeOf (FileBuffer), Cardinal (BytesRead), nil) then
     raise EFileRead.CreateFmt (sErrorReadingFile, [ FileName, GetLastErrorString (GetLastError)]);

    BufferInit (StringBuffer);

    if BytesRead > 0 then
     begin

      repeat
       BufferPosition := 0;
       if (LineEnd = -2) and (FileBuffer[ BufferPosition] = LF) then Inc (BufferPosition);
       LineEnd := PScanSetL (@FileBuffer, [ CR, LF], BufferPosition, BytesRead);

       while LineEnd >= 0 do
        begin
         BufferAdd (@FileBuffer[ BufferPosition], LineEnd - BufferPosition, StringBuffer);

         SetString (AddEmpty^, StringBuffer.Buffer, StringBuffer.Position);
         StringBuffer.Position := 0;

         if (FileBuffer[ LineEnd] = CR) then Inc (LineEnd);
         if (LineEnd < BytesRead) and (FileBuffer[ LineEnd] = LF) then
          Inc (LineEnd)
         else
          begin
           LineEnd := -2;
           Break;
          end;

         BufferPosition := LineEnd;
         LineEnd := PScanSetL (@FileBuffer, [ CR, LF], BufferPosition, BytesRead);
        end;

       if (LineEnd <> -2) and (BufferPosition < BytesRead) then
        begin
         BufferAdd (@FileBuffer[ BufferPosition], BytesRead - BufferPosition, StringBuffer);
        end;

       if not ReadFile (FileHandle, FileBuffer, SizeOf (FileBuffer), Cardinal (BytesRead), nil) then
        raise EFileRead.CreateFmt (sErrorReadingFile, [ FileName, GetLastErrorString (GetLastError)]);

      until BytesRead = 0;

      SetString (AddEmpty^, StringBuffer.Buffer, StringBuffer.Position);
     end;

    Result := True;
   finally
    CloseHandle (FileHandle);
    BufferFree (StringBuffer);
   end;
  end;
end;
 }

function TBaseTypeVector.ReadFromTextFile (const FileName: AnsiString): Boolean;
var
 Stream             : TStream;
begin
 Stream := TFileStream.Create (FileName, $0000 {fmOpenRead, defined in SysUtils} or $0020 {fmShareDenyWrite, defined in SysUtils});
 try
  Result := ReadFromTextStream (Stream);
 finally
  Stream.Free;
 end;
end;

{ ********** }

function TBaseTypeVector.ReadFromBinaryStream (const Stream: TStream): Boolean;
var
 i                  : Integer;
 Reader             : TrjReader;
begin
 Reader := TrjReader.Create (Stream);
 try
  try
   i := Reader.ReadInteger;
   Clear;
   while i > 0 do
    begin
     AddEmpty^ := Reader.ReadBinaryAnsiString;
     Dec (i);
    end;
   Result := True;
  except
   Result := False;
   if FCount > 0 then DeleteRange (FCount - 1, 1);
  end;
 finally
  Reader.Free;
 end;
end;

function TBaseTypeVector.ReadFromTextStream (const Stream: TStream): Boolean;
var
 FileBuffer         : array[0..BUFFER_SIZE - 1] of Char;
 BytesRead, BufferPosition, LineEnd: Integer;
 StringBuffer       : TBuffer;
begin
 LineEnd := 0;
 
 BytesRead := Stream.Read (FileBuffer, SizeOf (FileBuffer));
 
 BufferInit (StringBuffer);
 
 if BytesRead > 0 then
  begin

   repeat
    BufferPosition := 0;
    if (LineEnd = -2) and (FileBuffer[BufferPosition] = LF) then Inc (BufferPosition);
    LineEnd := PScanSetL (@FileBuffer, [CR, LF], BufferPosition, BytesRead);

    while LineEnd >= 0 do
     begin
      BufferAdd (@FileBuffer[BufferPosition], LineEnd - BufferPosition, StringBuffer);

      SetString (AddEmpty^, StringBuffer.Buffer, StringBuffer.Position);
      StringBuffer.Position := 0;

      if (FileBuffer[LineEnd] = CR) then Inc (LineEnd);
      if (LineEnd < BytesRead) and (FileBuffer[LineEnd] = LF) then
       Inc (LineEnd)
      else
       begin
        LineEnd := -2;
        Break;
       end;

      BufferPosition := LineEnd;
      LineEnd := PScanSetL (@FileBuffer, [CR, LF], BufferPosition, BytesRead);
     end;

    if (LineEnd <> -2) and (BufferPosition < BytesRead) then
     BufferAdd (@FileBuffer[BufferPosition], BytesRead - BufferPosition, StringBuffer);

    BytesRead := Stream.Read (FileBuffer, SizeOf (FileBuffer));

   until BytesRead = 0;

   SetString (AddEmpty^, StringBuffer.Buffer, StringBuffer.Position);
  end;
 Result := True;
end;

{ ********** }
{
procedure TBaseTypeVector.WriteToFile (const FileName: AnsiString);
var
 i                            : Integer;
 FileHandle                   : THandle;
 p                            : Pointer;
 w                            : DWord;
begin
 if Pointer (FileName) <> nil then
  begin
   FileHandle := CreateFile (Pointer (FileName), GENERIC_WRITE, 0, nil, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL or FILE_FLAG_SEQUENTIAL_SCAN, 0);
   if FileHandle = INVALID_HANDLE_VALUE then
    raise EFileCreate.CreateFmt (sErrorCreatingFile, [ FileName, GetLastErrorString (GetLastError)]);

   p := FVector;
   i := FCount;

   try
    while i > 0 do
     begin

      if Pointer (p^) <> nil then
       if not WriteFile (FileHandle, Pointer (p^)^, Integer (Pointer (Integer (p^) - 4)^), w, nil) then
        raise EFileWrite.CreateFmt (sErrorWritingFile, [ FileName, GetLastErrorString (GetLastError)]);

      Dec (i);
      if i > 0 then
       if not WriteFile (FileHandle, PChar (CRLF)^, Length (CRLF), w, nil) then
        raise EFileWrite.CreateFmt (sErrorWritingFile, [ FileName, GetLastErrorString (GetLastError)]);

      Inc (Integer (p), 4);
     end;
   finally
    CloseHandle (FileHandle);
   end;
  end;
end;
 }

procedure TBaseTypeVector.WriteToBinaryStream (const Stream: TStream);
var
 i                  : Integer;
 p                  : PBaseType;
 Writer             : TrjWriter;
begin
 Writer := TrjWriter.Create (Stream);
 try
  i := FCount;
  Writer.WriteInteger (i);
  p := FVector;
  while i > 0 do
   begin
    Writer.WriteBinaryAnsiString (p^);
    Inc (Cardinal (p), SizeOf (TBaseType));
    Dec (i);
   end;
 finally
  Writer.Free;
 end;
end;

{ ********** }

procedure TBaseTypeVector.WriteToTextFile (const FileName: AnsiString);
var
 Stream             : TStream;
begin
 Stream := TFileStream.Create (FileName, fmCreate);
 try
  WriteToTextStream (Stream);
 finally
  Stream.Free;
 end;
end;

{ ********** }

procedure TBaseTypeVector.WriteToTextStream (const Stream: TStream);
var
 i                  : Integer;
 p                  : Pointer;
 Writer             : TrjWriter;
begin
 Writer := TrjWriter.Create (Stream);
 p := FVector;
 i := FCount;
 try
  while i > 0 do
   begin
    if Pointer (p^) <> nil then Writer.Write (Pointer (p^)^, Integer (Pointer (Cardinal (p^) - 4)^));
    Dec (i);
    if i > 0 then Writer.Write (PChar (CRLF)^, Length (CRLF));
    Inc (Cardinal (p), 4);
   end;
 finally
  Writer.Free;
 end;
end;

{ **************************************************************************** }
{ TSortedAnsiStringVector
{ **************************************************************************** }

{$I rjVGenericBaseTypeSorted.imp}
{$I rjVGenericAllocXSorted.imp}

{ **************************************************************************** }
{ Compare / Same Functions
{ **************************************************************************** }

function VCompareDefault (const Item1, Item2): Integer;
begin
 Result := CompText (AnsiString (Item1), AnsiString (Item2));
end;

{ ********** }

function VMatchDefault (const Item1, Item2): Boolean;
begin
 Result := SameText (AnsiString (Item1), AnsiString (Item2));
end;

end.

