{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000 
  http://www.zeitungsjunge.de/delphi/ }

unit rjVAnsiString2;

interface

{$I rj.inc}

uses
 {$IFDEF RangeChecking}
 Consts,
 {$ENDIF}
 Classes,
 
 rjVectors,
 rjVBase64,
 rjStreams,
 rjTypes;

type
 PBaseType = PAnsiString2;
 TBaseType = TAnsiString2;
 TBaseTypeHack = TAnsiString2Hack;
 PBaseArray = PAnsiString2Array;
 
 {$DEFINE DynamicItems}
 
 TBaseTypeVector = class (ABase64Vector)
  {$I rjVBaseType.def}
  {$I rjVDeAlloc.def}
  {$DEFINE Name}{$I rjVHandleRecordDefinedType.def}{$UNDEF Name}
  {$DEFINE Value1}{$I rjVHandleRecordDefinedType.def}{$UNDEF Value1}
  {$I rjVHandleRecordNameValue1.def}
 protected
  function GetItem (const Index: Integer): TBaseType;
  procedure SetItem (const Index: Integer; const Item: TBaseType); virtual;
 public
  function Add (const Item: TBaseType): Integer;
  function AddSorted (const Item: TBaseType): Integer;
  function AddSortedBy (const Item: TBaseType; const CustomCompare: TUntypedCompareFunc): Integer;

  function AddName (const AName: AnsiString): Integer;
  function AddNameSorted (const AName: AnsiString): Integer;
  function AddNameSortedBy (const AName: AnsiString; const CustomCompare: TUntypedCompareFunc): Integer;

  function AddValue1 (const AValue: AnsiString): Integer;
  function AddValue1Sorted (const AValue: AnsiString): Integer;
  function AddValue1SortedBy (const AValue: AnsiString; const CustomCompare: TUntypedCompareFunc): Integer;

  function AddNameValue1 (const AName, AValue: AnsiString): Integer;
  function AddNameValue1Sorted (const AName, AValue: AnsiString): Integer;
  function AddNameValue1SortedBy (const AName, AValue: AnsiString; const CustomCompare: TUntypedCompareFunc): Integer;

  function AddEmpty: PBaseType;

  procedure Assign (const Source: TVector); override;

  procedure Insert (const Index: Integer; const Item: TBaseType);
  procedure InsertName (const Index: Integer; const AName: AnsiString);
  procedure InsertValue1 (const Index: Integer; const AValue: AnsiString);
  procedure InsertNameValue1 (const Index: Integer; const AName, AValue: AnsiString);

  property Items[const AIndex: Integer]: TBaseType read GetItem write SetItem; default;

  function ReadFromBinaryStream (const Stream: TStream): Boolean; override;
  procedure WriteToBinaryStream (const Stream: TStream); override;
 end;
 TAnsiString2Vector = TBaseTypeVector;

 {$I rjVCompareRecordNameValue1.def}
 {$I rjVCompareRecordName.def}
 {$I rjVCompareRecordValue1.def}

const
 VCompareDefault    : TUntypedCompareFunc = VCompareRecordNameValue1;
 VMatchDefault      : TUntypedMatchFunc = VMatchRecordNameValue1;

implementation

uses Windows, rjStrings;

{$I rjVBaseType.imp}
{$I rjVDeAlloc.imp}
{$DEFINE Name}{$I rjVHandleRecordDefinedType.imp}{$UNDEF Name}
{$DEFINE Value1}{$I rjVHandleRecordDefinedType.imp}{$UNDEF Value1}
{$I rjVHandleRecordNameValue1.imp}

function TBaseTypeVector.Add (const Item: TBaseType): Integer;
begin
 Result := FCount;
 Grow (1);
 with PBaseArray (FVector)^[Result] do
  begin
   Pointer (Name) := nil;
   Name := Item.Name;
   Pointer (Value1) := nil;
   Value1 := Item.Value1;
  end;
end;

{ ********** }

function TBaseTypeVector.AddSorted (const Item: TBaseType): Integer;
begin
 if CustomFindInSortedRange (Item, 0, FCount - 1, VCompareDefault, Result) then
  Result := -1
 else
  Insert (Result, Item);
end;

{ ********** }

function TBaseTypeVector.AddSortedBy (const Item: TBaseType; const CustomCompare: TUntypedCompareFunc): Integer;
begin
 if CustomFindInSortedRange (Item, 0, FCount - 1, CustomCompare, Result) then
  Result := -1
 else
  Insert (Result, Item);
end;

{ ********** }

function TBaseTypeVector.AddName (const AName: AnsiString): Integer;
begin
 Result := FCount;
 Grow (1);
 with PBaseArray (FVector)^[Result] do
  begin
   Pointer (Name) := nil;
   Name := AName;
   Pointer (Value1) := nil;
  end;
end;

{ ********** }

function TBaseTypeVector.AddNameSorted (const AName: AnsiString): Integer;
var
 Item               : TBaseTypeHack;
begin
 with Item do
  begin
   Name := Pointer (AName);
   Value1 := nil;
  end;
 if CustomFindInSortedRange (Item, 0, FCount - 1, VCompareDefault, Result) then
  Result := -1
 else
  InsertName (Result, AName);
end;

{ ********** }

function TBaseTypeVector.AddNameSortedBy (const AName: AnsiString; const CustomCompare: TUntypedCompareFunc): Integer;
var
 Item               : TBaseTypeHack;
begin
 with Item do
  begin
   Name := Pointer (AName);
   Value1 := nil;
  end;
 if CustomFindInSortedRange (Item, 0, FCount - 1, CustomCompare, Result) then
  Result := -1
 else
  InsertName (Result, AName);
end;

{ ********** }

function TBaseTypeVector.AddValue1 (const AValue: AnsiString): Integer;
begin
 Result := FCount;
 Grow (1);
 with PBaseArray (FVector)^[Result] do
  begin
   Pointer (Name) := nil;
   Pointer (Value1) := nil;
   Value1 := AValue;
  end;
end;

{ ********** }

function TBaseTypeVector.AddValue1Sorted (const AValue: AnsiString): Integer;
var
 Item               : TBaseTypeHack;
begin
 with Item do
  begin
   Name := nil;
   Value1 := Pointer (AValue);
  end;
 if CustomFindInSortedRange (Item, 0, FCount - 1, VCompareDefault, Result) then
  Result := -1
 else
  InsertValue1 (Result, AValue);
end;

{ ********** }

function TBaseTypeVector.AddValue1SortedBy (const AValue: AnsiString; const CustomCompare: TUntypedCompareFunc): Integer;
var
 Item               : TBaseTypeHack;
begin
 with Item do
  begin
   Name := nil;
   Value1 := Pointer (AValue);
  end;
 if CustomFindInSortedRange (Item, 0, FCount - 1, CustomCompare, Result) then
  Result := -1
 else
  InsertValue1 (Result, AValue);
end;

{ ********** }

function TBaseTypeVector.AddNameValue1 (const AName, AValue: AnsiString): Integer;
begin
 Result := FCount;
 Grow (1);
 with PBaseArray (FVector)^[Result] do
  begin
   Pointer (Name) := nil;
   Name := AName;
   Pointer (Value1) := nil;
   Value1 := AValue;
  end;
end;

{ ********** }

function TBaseTypeVector.AddNameValue1Sorted (const AName, AValue: AnsiString): Integer;
var
 Item               : TBaseTypeHack;
begin
 with Item do
  begin
   Name := Pointer (AName);
   Value1 := Pointer (AValue);
  end;
 if CustomFindInSortedRange (Item, 0, FCount - 1, VCompareDefault, Result) then
  Result := -1
 else
  InsertNameValue1 (Result, AName, AValue);
end;

{ ********** }

function TBaseTypeVector.AddNameValue1SortedBy (const AName, AValue: AnsiString; const CustomCompare: TUntypedCompareFunc): Integer;
var
 Item               : TBaseTypeHack;
begin
 with Item do
  begin
   Name := Pointer (AName);
   Value1 := Pointer (AValue);
  end;
 if CustomFindInSortedRange (Item, 0, FCount - 1, CustomCompare, Result) then
  Result := -1
 else
  InsertNameValue1 (Result, AName, AValue);
end;

{ ********** }

function TBaseTypeVector.AddEmpty: PBaseType;
var
 i                  : Integer;
begin
 i := FCount;
 Grow (1);
 Result := @PBaseArray (FVector)^[i];
 
 Pointer (Result.Name) := nil;
 Pointer (Result.Value1) := nil;
end;

{ ********** }

procedure TBaseTypeVector.Assign (const Source: TVector);
var
 i                  : Integer;
 s                  : PBaseType;
begin
 Clear;
 i := 0;
 while i < Source.Count do
  begin
   // The following lines correspond to, but avoid the hidden overhead caused by the internal use of CopyRecord by compiler.
   // Add (TBaseType (TBaseTypeVector (Source)[ i]));
   s := @PBaseArray (TBaseTypeVector (Source).FVector)^[i];
   with AddEmpty^ do
    begin
     Name := s^.Name;
     Value1 := s^.Value1;
    end;
   Inc (i);
  end;
end;

{ ********** }

function TBaseTypeVector.GetItem (const Index: Integer): TBaseType;
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index >= FCount) then Error (SListIndexError, [Index]);
 {$ENDIF}
 Result := PBaseArray (FVector)^[Index]; // ToDo: Optimize!
end;

{ ********** }

procedure TBaseTypeVector.Insert (const Index: Integer; const Item: TBaseType);
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index > FCount) then Error (SRangeError, [Index]);
 {$ENDIF}
 Expand (Index);
 with PBaseArray (FVector)^[Index] do
  begin
   Pointer (Name) := nil;
   Name := Item.Name;
   Pointer (Value1) := nil;
   Value1 := Item.Value1;
  end;
end;

{ ********** }

procedure TBaseTypeVector.InsertName (const Index: Integer; const AName: AnsiString);
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index > FCount) then Error (SRangeError, [Index]);
 {$ENDIF}
 Expand (Index);
 with PBaseArray (FVector)^[Index] do
  begin
   Pointer (Name) := nil;
   Name := AName;
   Pointer (Value1) := nil;
  end;
end;

{ ********** }

procedure TBaseTypeVector.InsertValue1 (const Index: Integer; const AValue: AnsiString);
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index > FCount) then Error (SRangeError, [Index]);
 {$ENDIF}
 Expand (Index);
 with PBaseArray (FVector)^[Index] do
  begin
   Pointer (Name) := nil;
   Pointer (Value1) := nil;
   Value1 := AValue;
  end;
end;

{ ********** }

procedure TBaseTypeVector.InsertNameValue1 (const Index: Integer; const AName, AValue: AnsiString);
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index > FCount) then Error (SRangeError, [Index]);
 {$ENDIF}
 Expand (Index);
 with PBaseArray (FVector)^[Index] do
  begin
   Pointer (Name) := nil;
   Name := AName;
   Pointer (Value1) := nil;
   Value1 := AValue;
  end;
end;

{ ********** }

function TBaseTypeVector.ReadFromBinaryStream (const Stream: TStream): Boolean;
var
 i                  : Integer;
 p                  : PBaseType;
 Reader             : TrjReader;
begin
 Reader := TrjReader.Create (Stream);
 try
  try
   i := Reader.ReadInteger;
   Clear;
   while i > 0 do
    begin
     p := AddEmpty;
     p^.Name := Reader.ReadBinaryAnsiString;
     p^.Value1 := Reader.ReadBinaryAnsiString;
     Dec (i);
    end;
   Result := True;
  except
   Result := False;
   if FCount > 0 then DeleteRange (FCount - 1, 1);
  end;
 finally
  Reader.Free;
 end;
end;

{ ********** }

procedure TBaseTypeVector.SetItem (const Index: Integer; const Item: TBaseType);
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index >= FCount) then Error (SListIndexError, [Index]);
 {$ENDIF}
 PBaseArray (FVector)^[Index] := Item;  // ToDo: Optimize!
end;

{ ********** }

procedure TBaseTypeVector.WriteToBinaryStream (const Stream: TStream);
var
 i                  : Integer;
 p                  : PBaseType;
 Writer             : TrjWriter;
begin
 Writer := TrjWriter.Create (Stream);
 try
  i := FCount;
  Writer.WriteInteger (i);

  p := FVector;
  while i > 0 do
   begin
    Writer.WriteBinaryAnsiString (p^.Name);
    Writer.WriteBinaryAnsiString (p^.Value1);
    Inc (Cardinal (p), SizeOf (TBaseType));
    Dec (i);
   end;

 finally
  Writer.Free;
 end;
end;

{ **************************************************************************** }
{ Compare / Same Functions
{ **************************************************************************** }

{$I rjVCompareRecordNameValue1.imp}
{$I rjVCompareRecordName.imp}
{$I rjVCompareRecordValue1.imp}

end.

